//= require ./Dialog.js
//= require ./Button.js

AppKit.AlertDialog = AppKit.Dialog.extend({
    classID: 'appkit-alert',
    options: {
        width: 500,
        height: 200,
        resizable: false,
        confirmButton: true,
        cancelButton: true,
        confirmLabel: 'OK',
        cancelLabel: 'Cancel',
        modal: true,
        message: "",
        destroyOnClose: true,
        icon: undefined,
        callback: null
    },
    initialize: function (options) {

        this.$el.addClass('ui-alert');

        var self = this;
        this._ok = false;
        this._confirmBtn = new AppKit.Button({
            type: 'primary',
            className: 'alert-confirm',
            action: function () {
                self._ok = true;
                self.set('visible', false);

            }
        });

        this._cancelBtn = new AppKit.Button({
            className: 'alert-cancel',
            action: function () {
                self._ok = false;
                self.set('visible', false);
            }
        });

        var bbar = $('<div>').addClass('button-bar').appendTo(this.$el);

        this._cancelBtn.$el.appendTo(bbar);
        this._confirmBtn.$el.appendTo(bbar);


        this._msg = $('<div>').addClass('ui-alert-msg').attr('role', 'presentation').appendTo(this.$el);
        this._icon = $('<div>').addClass('ui-alert-icon').appendTo(this.$el);

        this.$el.on('dialogclose', function () {
            if (_.isFunction(self.callback)) {
                self.callback.call(self, self._ok);
            }
            if (self.destroyOnClose) {
                $(this).remove();
            }
        });

        this._super(options);
    },
    confirmButton: function (aBOOL) {
        if (aBOOL === true) {
            this._confirmBtn.set('hidden', false);
        }
        else if (aBOOL === false) {
            this._confirmBtn.set('hidden', true);
        }

        return !this._confirmBtn.get('hidden');

    },
    confirmLabel: function (aString) {
        if (aString !== undefined) {
            this._confirmBtn.set('label', aString);
        }

        return this._confirmBtn.get('label');
    },
    cancelButton: function (aBOOL) {
        if (aBOOL === true) {
            this._cancelBtn.set('hidden', false);
        }
        else if (aBOOL === false) {
            this._cancelBtn.set('hidden', true);
        }

        return !this._cancelBtn.get('hidden');
    },
    cancelLabel: function (aString) {
        if (aString !== undefined) {
            this._cancelBtn.set('label', aString);
        }

        return this._cancelBtn.get('label');

    },
    icon: function (aString) {
        if (aString !== undefined) {
            this._icon.addClass(aString);
            this._iconClass = aString;
        }

        return this._iconClass;
    },
    message: function (aString) {
        if (aString !== undefined) {
            this._msg.html(aString);
        }

        return this._msg.text();
    }
});


AppKit.AlertDialog.info = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-info',
        cancelButton: false,
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();
};


AppKit.AlertDialog.alert = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-warning',
        cancelButton: false,
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();

};

AppKit.AlertDialog.confirm = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-confirm',
        cancelButton: true,
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();

};

AppKit.AlertDialog.yesNo = function (title, msg, callback, config) {

    if (!config) {
        config = {};
    }

    var alertDialog = new AppKit.AlertDialog({
        title: title,
        message: msg,
        icon: 'icon-alert-confirm',
        cancelButton: true,
        cancelLabel: 'NO',
        confirmLabel: 'YES',
        height: config.height ? config.height : 220,
        callback: callback
    });

    alertDialog.center();
    alertDialog.open();
};

AppKit.AlertDialog.notify = function (message, icon) {

    if (!icon) {
        icon = "";
    }

    var popover = $('<div><div class="ui-alert-icon-notification ' + icon + '"></div><table><tr><td>' + message + '</td></tr></table></div>')
        .addClass('ui-tooltip ui-alert-notification');

    popover.appendTo('body');

    popover.fadeIn(300);

    setTimeout(function () {
        popover.fadeOut(300);
    }, 4000)

};