AppKit.PagedTableView = AppKit.TableView.extend({
    classID: 'appkit-paged-tableview',
    options: {
        recordsPerPage: 25
    },
    initialize: function (options) {

        this._super(options);

        this._groupExpandState = {};
        //need to save expanded/collapsed state of grouped datagrids
        this.$el.on('onGroupCollapsed', _.bind(function (evt, groupKey) {

            if (!this._groupExpandState[groupKey]) {
                this._groupExpandState[groupKey] = {};
            }
            this._groupExpandState[groupKey][this._pager.get('selectedPage')] = false;

        }, this));

        this.$el.on('onGroupExpanded', _.bind(function (evt, groupKey) {
            if (!this._groupExpandState[groupKey]) {
                this._groupExpandState[groupKey] = {};
            }

            this._groupExpandState[groupKey][this._pager.get('selectedPage')] = true;
        }, this));

    },
    destroy: function () {
        if (this._pager) {
            this._pager.off('selection');
        }

        this.$el.off('onGroupCollapsed onGroupExpanded');

        if (this._collection) {
            this._collection.off('reset', this._calcNumberOfPages, this);
        }

        this._super();
    },
    recordsPerPage: function (aNumber) {
        if (_.isNumber(aNumber)) {
            this._recordsPerPage = Math.max(1, aNumber);
            this._calcNumberOfPages();
        }

        return this._recordsPerPage;
    },
    source: function (aCollection) {

        if (aCollection !== undefined) {

            this._super(aCollection);

            if (this._collection) {
                this._collection.on('reset', this._calcNumberOfPages, this);
            }

            this._calcNumberOfPages();
        }

        return this._collection;
    },
    _calcNumberOfPages: function () {
        if (this._collection) //calculate the number of pages
        {
            var pageCount = Math.max(1, Math.ceil(this.get('rowCount') / this._recordsPerPage));
            if (this._pager) {
                this._pager.set('pageCount', pageCount);
                this._pager.set('selectedPage', 1);
            }

            this._groupExpandState = {};
            this._cachedData = null;
            this._initData();
        }
    },
    pager: function (aPager) {

        if (aPager !== undefined) {

            if (this._pager) {
                this._pager.off('selection');
            }

            this._pager = aPager;

            if (this._pager) {
                var self = this;
                this._pager.on('selection', function () {
                    self._changePage();
                });
            }

            this._calcNumberOfPages();
        }

        return this._pager;

    },
    _changePage: function () {
        this._initData();
        this.render();
        this.$el.find('.ui-table-scroller').scrollTop(0);
    },
    _update: function () {

        this._super();

        this._calcNumberOfPages();
    },
    displayedRange: function () {
        if (this._pager) {

            var currentPage = this._pager.get('selectedPage');

            if (!currentPage) {
                this._pager.set('selectedPage', 1);
            }

            if (currentPage) {

                var dataStart = (currentPage - 1) * this._recordsPerPage,
                    dataEnd = Math.min(this.get('rowCount'), dataStart + this._recordsPerPage);

                return {start: dataStart, end: dataEnd};
            }
        }

        return undefined;
    },
    _initData: function () {

        var data = this._super();

        var range = this.displayedRange();
        if (range) {

            if (this.get('groupBy') && data.data) { //slice the grouped data

                var slicedData = {data: []};
                var k = 0;
                _.each(data.data, function (item) {
                    var key = _.keys(item)[0],
                        values = item[key],
                        slicedValues = [];

                    if (k + values.length >= range.start && k < range.end) {
                        _.each(values, function (model) {

                            if (k >= range.start && k < range.end) {
                                slicedValues.push(model);
                            }
                            k++;
                        });

                        if (slicedValues.length > 0) {
                            var val = {};
                            val[key] = slicedValues;
                            slicedData.data.push(val);
                        }

                    }
                    else {
                        k += values.length;
                    }
                });

                slicedData.length = range.end - range.start;
                slicedData.expandState = this._groupExpandState;
                slicedData.currentPage = this._pager.get('selectedPage');
                if (this.$el.datatable('instance')) {
                    this.$el.datatable('setData', slicedData, this._delegate);
                }
                return slicedData;
            }
            else //slice non-grouped array
            {
                data = data.slice(range.start, range.end);
                if (this.$el.datatable('instance')) {
                    this.$el.datatable('setData', data, this._delegate);
                }

            }
        }

        return data;
    }
});