/**
 * JLV Notification  = {
    name: null,
    object: null,
    userInfo: null
    }
 *
 **/

JLV.NotificationCenter = {
    _registry: {},
    /* observes a notification on either an object or an action function */
    addObserverForNotification: function (config) // config should specify observer, action, name, object, and userInfo (optional)
    {
        if (this._registry[config.name] === undefined)
            this._registry[config.name] = [];

        this._registry[config.name].push({
            observer: config.observer,
            notification: {
                name: config.name,
                object: config.object,
                userInfo: config.userInfo
            },
            action: config.action
        });
    },
    removeObserver: function (anObserver) //removes observer for all notifications
    {
        for (var notificationName in this._registry) {
            this.removeObserverForNotificationName(notificationName);
        }

    },
    removeObserverForNotificationName: function (anObserver, aNotificationName) {
        var regs = this._registry[aNotificationName];

        if (regs) {
            var count = regs.length,
                index = 0;

            for (; index < count; index++) {
                var reg = regs[index];
                if (reg && reg.observer === anObserver) {
                    regs.splice(index, 1);
                }
            }
        }
    },
    removeObserverForNotificationNameAndObject: function (anObserver, aNotificationName, anObject) {
        var regs = this._registry[aNotificationName];

        if (regs) {
            var count = regs.length,
                index = 0;

            for (; index < count; index++) {
                var reg = regs[index];
                if (reg) {
                    if (reg.observer === anObserver && reg.notification.object === anObject) {
                        regs.splice(index, 1);
                    }
                }

            }
        }
    },
    removeAllNotificationsForNotificationName: function (aNotificationName) {
        delete this._registry[aNotificationName];
    },
    /* posts a notifcation, sent to all observers */
    postNotificationName: function (aName, aSenderObject, userInfo) {

        if (this._registry[aName] !== undefined) {
            var regs = this._registry[aName],
                count = regs.length,
                index = 0;

            for (; index < count; index++) {
                var reg = regs[index],
                    target = reg.observer;

                if (reg) {
                    if (_.isFunction(reg.action) &&
                        (reg.notification.object === aSenderObject || !reg.notification.object)) {
                        if (userInfo)
                            reg.notification.userInfo = userInfo;

                        reg.action(reg.notification);
                    }
                    else if (_.isFunction(target[reg.action])
                        && (reg.notification.object === aSenderObject || !reg.notification.object)) {
                        if (userInfo)
                            reg.notification.userInfo = userInfo;

                        target[reg.action].call(target, reg.notification);
                    }
                }

            }
        }
    }
};
