//= require JLV.js


JLV.Context = new function () {
    // Constants
    var APP_NAME = "JLV";
    var APP_CODE = "bq4Jm8zOaxXbKI63MRi00aJJjlmcrG5twlDLEtZOB39KC1UAoUI32d4WOhtjBWNIeQfAL7H7hYCar" +
        "0yTCbDHPcgxLZDc226fkTwYdNVk0Gs-O6DCXBWzh7A9TQxqo5uUUR7jeaQrihFdG5dNOxPfoGmRg_kaTopn8HF9J" +
        "xhZZHqcXzUP-SwjUSifbMp9_Fz8qt8BJztxnUEoXRJRi1VuffWsi1wrLeM718Tm0J8CjDxlsOJhYpF8XX_x3X94x" +
        "NU9";
    var SURVEYABLE = false;
    var ITEM_NAME_EDIPI = "patient.id.mrn.edi_pn";
    var ITEM_NAME_ICN = "patient.id.mrn.nationalidnumber";
    var ITEM_NAME_VA_IEN = "patient.id.mrn.dfn_";
    var ITEM_NAME_PAT_NAME = "patient.co.patientname";
    var CONNECT_ERROR_MSG = "CV was unable to communicate with the CCOW Context Vault. " +
        "CCOW patient synchronization will be unavailable.";

    var vaultChange = false;    // CCOW vault initiated context change
    var agency = "VA";
    var siteCode = "";

    var status = new function () {
        this.joined = false; // Joined context
        this.active = false; // Currently active

        /**
         * Set context status image.
         * @param status Status image to display.
         */
        this.setImage = function (status) {
            var imageFiles = {
                off: "/../assets/context-off.gif",
                on: "/../assets/context-on.gif",
                changing: "/../assets/context-changing.gif"
            };

            if (document.images["cmStatus"] && imageFiles[status]) {
                document.images["cmStatus"].src = window.location.pathname + imageFiles[status];
            }
        };

        /**
         * Checks to see if context is currently joined and active.
         * @returns {boolean} True if connected or false if not connected.
         */
        this.connected = function () {
            return (this.joined && this.active);
        }

        /**
         * Changes context status. If joined, it will switch between active and not active. If
         * not joined, it will attempt to join context.
         */
        this.change = function () {
            if (this.joined) {
                var self = this;
                if (this.active) {

                    // Suspend context (not active state)
                    AppKit.AlertDialog.confirm("CCOW Warning", "Are you sure you want to break patient link?",
                        function (answer) {
                            if (answer) {
                                try {
                                    ContextorControl.Suspend();
                                    self.setImage("off");
                                    self.active = false;
                                }
                                catch (e) {
                                    handleException(e);
                                }
                            }
                            else {
                                self.setImage("on");
                            }
                        });
                }
                else {

                    // Resume context (active state)
                    AppKit.AlertDialog.confirm("CCOW Warning", "Are you sure you want to rejoin patient link?",
                        function (answer) {
                            if (answer) {
                                try {
                                    ContextorControl.Resume();

                                    self.setImage("on");
                                    self.active = true;

                                    checkVault(ContextorControl.CurrentContext);

                                    if (vaultChange && !vaultIds.compareIds(appIds)) {
                                        updateToVault();
                                    }
                                }
                                catch (e) {
                                    handleException(e);
                                }
                            }
                            else {
                                self.setImage("off");
                            }
                        });
                }
            }
            else {
                // DoD users and DANVSA u ers will not connect to context. Only VA VHA users.
                if (agency.toUpperCase() == "VA" && siteCode != "200CORP") {
                    try {
                        // Connect to context
                        ContextorControl.Run(APP_NAME, APP_CODE, SURVEYABLE);

                        this.setImage("on");

                        this.joined = true;
                        this.active = true;

                        checkVault(ContextorControl.CurrentContext);

                        if (vaultChange && !vaultIds.compareIds(appIds)) {
                            updateToVault();
                        }
                    }
                    catch (e) {
                        handleException(e);
                    }
                }
            }
        }
    };

    /**
     * Handles context exception when trying to communicate with the desktop listener.
     * @param e
     */
    var handleException = function (e) {
        try {
            ContextorControl.UnregisterListener();
            ContextorControl.Stop();
        }
        catch (e) {
            // Attempted to stop (unjoin) context, just continue
        }

        vaultChange = false;

        status.setImage("changing");
        status.joined = false;
        status.active = false;

        setTimeout(function () {
            AppKit.AlertDialog.alert('CCOW Error', CONNECT_ERROR_MSG, function () {
                status.setImage("off");
            });
        }, 2000);


    };

    var identifiers = function () {
        this.edipi = "";
        this.icn = "";
        this.vaIen = "";
        this.vaIenSiteId = "";
        this.name = "";
    };


    identifiers.prototype.compareIds = function (id) {
        // EDIPI identifier matches
        if (this.edipi != "" && this.edipi == id.edipi) {
            return true;
        }
        // ICN identifier matches
        if (this.icn != "" && this.icn == id.icn) {
            return true;
        }
        // VA IEN identifier and site ID matches
        if (this.vaIen != "" && this.vaIen == id.vaIen
            && this.vaIenSiteId != "" && this.vaIenSiteId == id.vaIenSiteId) {
            return true;
        }
        // No identifier matches found
        return false;
    };

    identifiers.prototype.copy = function (id) {
        this.edipi = id.edipi;
        this.icn = id.icn;
        this.vaIen = id.vaIen;
        this.vaIenSiteId = id.vaIenSiteId;
        this.name = id.name;
    };

    identifiers.prototype.set = function (edipi, icn, vaIen, vaIenSiteId, name) {
        this.edipi = edipi;
        this.icn = icn;
        this.vaIen = vaIen;
        this.vaIenSiteId = vaIenSiteId;
        this.name = name;
    };

    var vaultIds = new identifiers();
    var appIds = new identifiers();

    /**
     * Checks vault and stores vault values used by JLV.
     * @param contextItems Vault key-value pair context items.
     */
    var checkVault = function (contextItems) {
        var coll = new Enumerator(contextItems);
        var itemName = "";
        var len = 0;
        var edipi = "";
        var icn = "";
        var vaIen = "";
        var vaIenSiteId = "";
        var name = "";

        // Iterate through vault items and capture relevant values
        for (; !coll.atEnd(); coll.moveNext()) {
            itemName = coll.item().name.toLowerCase();

            // Capture DoD EDIPI if found
            len = ITEM_NAME_EDIPI.length;
            if (itemName.substr(0, len) == ITEM_NAME_EDIPI.substr(0, len)) {
                edipi = coll.item().value;
            }

            // Capture VA ICN if found
            len = ITEM_NAME_ICN.length;
            if (itemName.substr(0, len) == ITEM_NAME_ICN.substr(0, len)) {
                icn = coll.item().value;
            }

            // Capture VA IEN and associated site ID if found
            len = ITEM_NAME_VA_IEN.length;
            if (itemName.substr(0, len) == ITEM_NAME_VA_IEN.substr(0, len)) {
                // Check if site ID is appended to context name for VA IEN
                if (itemName.length > len) {
                    vaIen = coll.item().value;
                    vaIenSiteId = itemName.substr(len, 3);
                }
            }

            // Capture patient name
            len = ITEM_NAME_PAT_NAME.length;
            if (itemName.substr(0, len) == ITEM_NAME_PAT_NAME.substr(0, len)) {
                name = coll.item().value;
            }
        }

        vaultIds.edipi = edipi;
        vaultIds.icn = icn;
        vaultIds.vaIen = vaIen;
        vaultIds.vaIenSiteId = vaIenSiteId;
        vaultIds.name = name;

        vaultChange = (edipi != "" || icn != "" || vaIen != "" || vaIenSiteId != "");
    };

    /**
     * Updates JLV patient to match patient that is set in context vault.
     */
    var updateToVault = function () {
        $.ajax({
            url: "context/getPatientToken",
            type: "POST",
            data: {
                "edipi": vaultIds.edipi,
                "icn": vaultIds.icn,
                "vaIen": vaultIds.vaIen,
                "vaIenSiteId": vaultIds.vaIenSiteId,
                "csrfToken": $('meta[name="csrf-token"]').attr('content')
            },
            success: function (data) {
                var patientName = vaultIds.name;
                if (patientName) {
                    patientName = patientName.replace("^^^^", "");
                    patientName = patientName.replace("^", ", ");
                }
                else {
                    patientName = '';
                }

                JLV.SelectPatient(new Backbone.Model({
                    patientToken: data.patientToken,
                    cmNotify: false,
                    name: patientName,
                    edipi: data.patientToken,
                    fmpssn: '',
                    sponsorSsn: '',
                    gender: '',
                    dob: '',
                    sensitive: false
                }));
            },
            error: function () {
                this.cancelVaultChange();
            }
        });
    };

    /**
     * Attempt to connect to context if user's agency is VA.
     * @param userAgency User's agency.
     */
    this.connect = function (userAgency, userSiteCode) {
        agency = userAgency;
        siteCode = userSiteCode;

        if (agency === JLV.common.AGENCY_VA) {
            status.change();
        }
    };

    /**
     * Change context status.
     */
    this.changeStatus = function () {
        status.change();
    };

    /**
     * Vault notifies JLV that a change is pending.
     * @param contextItems contextItems Vault key-value pair context items.
     */
    this.pending = function (contextItems) {
        //checkVault(contextItems);
    };

    /**
     * Vault changes have been committed and JLV should switch to new context.
     */
    this.committed = function () {
        if (status.connected()) {
            try {
                checkVault(ContextorControl.CurrentContext);

                if (vaultChange && !vaultIds.compareIds(appIds)) {
                    updateToVault();
                }
            }
            catch (e) {
                // Failed to connect to context
                handleException(e);
            }
        }
    };

    /**
     * Vault changes were canceled.
     */
    this.canceled = function () {
        vaultChange = false;
    };

    /**
     * Set context values into the vault.
     * @param edipi Patient's DoD EDIPI
     * @param icn Patient's VA ICN
     * @param vaIen Patient's VA IEN
     * @param vaIenSiteId Patient's VA IEN site code
     * @param name Patient's name
     * @returns {boolean} Returns true if context was set to the vault.
     */
    this.setContext = function (edipi, icn, vaIen, vaIenSiteId, name) {

        if (!document.getElementById("ContextorControl")) {
            return true;
        }

        var success = true;
        var changeIds = new identifiers();

        // Only takes the first part of the full ICN, removing the checksum ending
        if (icn) {
            icn = icn.split("V")[0];
        }

        changeIds.set(edipi, icn, vaIen, vaIenSiteId, name);

        if (status.connected()) {
            if (!vaultChange && !vaultIds.compareIds(changeIds)) {
                try {
                    // Create a context item collection
                    var contextorItems = new ActiveXObject("Sentillion.ContextItemCollection");
                    // Create a context item
                    var item;

                    // Add EDIPI to context items
                    if (edipi != null && edipi != "") {
                        item = new ActiveXObject("Sentillion.ContextItem");
                        item.name = ITEM_NAME_EDIPI;
                        item.value = edipi;
                        contextorItems.add(item);
                    }

                    // Add ICN to context items
                    if (icn != null && icn != "") {
                        item = new ActiveXObject("Sentillion.ContextItem");
                        item.name = ITEM_NAME_ICN;
                        item.value = icn;
                        contextorItems.add(item);

                        item = new ActiveXObject("Sentillion.ContextItem");
                        item.name = ITEM_NAME_ICN + "_test";
                        item.value = icn;
                        contextorItems.add(item);
                    }

                    // Add VA IEN to context items
                    if (vaIen != null && vaIen != "" && vaIenSiteId != null && vaIenSiteId != "") {
                        item = new ActiveXObject("Sentillion.ContextItem");
                        item.name = ITEM_NAME_VA_IEN + vaIenSiteId;
                        item.value = vaIen;
                        contextorItems.add(item);

                        item = new ActiveXObject("Sentillion.ContextItem");
                        item.name = ITEM_NAME_VA_IEN + vaIenSiteId + "_test";
                        item.value = vaIen;
                        contextorItems.add(item);
                    }

                    if (name != null && name != "") {
                        item = new ActiveXObject("Sentillion.ContextItem");
                        item.name = ITEM_NAME_PAT_NAME;
                        item.value = name.replace(/,/g, "^") + "^^^^";
                        contextorItems.add(item);
                    }

                    ContextorControl.StartContextChange();

                    var response = ContextorControl.EndContextChange(true, contextorItems);

                    if (response == 1) {
                        // The context was changed
                        vaultIds.copy(changeIds);
                        appIds.copy(changeIds);
                    } else if (response == 2) {
                        // The user canceled the context change
                        success = false;
                    } else if (response == 3) {
                        // The user chose to break this app away from the context
                        appIds.copy(changeIds);
                        status.setImage("off");
                        status.active = false;
                    } else {
                        // Unknown response
                    }
                } catch (e) {
                    // Failed to connect to context
                    handleException(e);
                }
            } else {
                appIds.copy(changeIds);
            }
        } else {
            appIds.copy(changeIds);
        }

        vaultChange = false;

        return success;
    };

    /**
     * JLV could not be synchronized with the vault for some reason. Break link and shift view to
     * provider portal.
     */
    this.cancelVaultChange = function () {
        if (status.connected() && vaultChange) {
            status.setImage("changing");

            AppKit.AlertDialog.alert("CCOW Error", "CV failed to synchronize with the CCOW Vault." +
            " Patient link will be broken.", function () {
                status.setImage("off");
                status.active = false;

                try {
                    ContextorControl.Suspend();
                }
                catch (e) {
                    handleException(e);
                }
            });
        }

        vaultChange = false;
    };


};
