//= require Widget.js
//= require ./desktop.js

JLV.Desktop.TabView = AppKit.TabView.extend({
    initialize: function () {
        this._super();

        this._addTabBtn = $('<div title="Add Tab" class="jlv-tabview-add" tabindex="0"><div class="icon-plus"></div></div>')
            .button();
        this.$el.append(this._addTabBtn);

        this._addTabBtn.on('click', function (evt) {
            AppKit.get('AddTabDialog').set('visible', true);
        });

        var self = this;
        //508
        this._addTabBtn.on('keydown', function (evt) {
            if (evt.keyCode === $.ui.keyCode.TAB) {

                if (!evt.shiftKey) {
                    evt.preventDefault();
                    var next = self.$el.nextInDOM('.jlv-patient-demographics .ui-tab');
                    if (next.length) {
                        next.focus();
                    }
                    else {
                        next = self.$el.nextInDOM('.jlv-widget');
                        if (next.length) {
                            next.focus()
                        }
                        else {
                            $('#AddWidgetButton').focus();
                        }
                    }
                 }

            }
        });

        this.$el.on('tabsactivate', function (evt, ui) {
            var portal = ui.newPanel.widget();
            if (portal) {
                if (portal.$el.is(":visible")) {
                    portal.layout();
                }
            }
        });

        this.$el.on('tabremoved', function () {
            self._adjustAddButton();
            JLV.Session.saveUIConfigForPortal(self);
        });

        this._portals = [];
    },
    _adjustAddButton: function () {
        if (this._addTabBtn) {
            this._addTabBtn.detach();
            this.$el.children('.ui-tabs-nav').append(this._addTabBtn);
        }
    },
    addTabbedPortal: function (tabName, closable) {

        var portalID = _.uniqueId('portal');

        var portal = new JLV.Desktop.WidgetPortal({
            id: portalID,
            tabView: this,
            name: tabName
        });

        portal.$el.attr('data-id', portalID);
        portal.$el.attr('id', portalID);
        portal.$el.appendTo(this.$el);

        this.addTab(tabName, portalID, closable);

        setTimeout(_.bind(function () {
            this._adjustAddButton();
        }, this), 0);

        this._portals.push(portal);


        return portal;

    },
    addWidgetToPortalAtIndex: function (aWidget, columnIndex, portalIndex) {
        if (portalIndex > -1 && portalIndex < this._portals.length) {
            var portal = this._portals[portalIndex];
            if (portal) {
                portal.addWidget(aWidget, columnIndex);
            }
        }
    },
    getActiveWidgetPortal: function () {
        var active = this.get('activeTab');
        var portalEl = this.$el.children('.jlv-portal').eq(active);
        if (portalEl) {
            return portalEl.widget();
        }
        return undefined;
    },
    portalAtIndex: function (anIndex) {
        if (anIndex > -1 && anIndex < this._portals.length) {
            return this._portals[anIndex];
        }
    },
    layoutActivePortal: function () {
        var portal = this.getActiveWidgetPortal();
        if (portal) {
            portal.layout();
        }
    },
    hidden: function (aBOOL) {
        this._super(aBOOL);
        this._adjustAddButton();
        this.layoutActivePortal();

    },
    onCloseTab: function (tabID) {

        AppKit.AlertDialog.confirm('Confirm Remove Tab', 'Remove tab?', _.bind(function (decision) {
            if (decision) {

                var rmIdx = this.$el.find('#' + tabID).index();

                if (rmIdx > -1) {
                    var portal = this._portals[rmIdx];

                    if (portal && _.isFunction(portal.destroy)) {
                        portal.destroy();
                    }
                    this._portals.splice(rmIdx, 1);
                }

                this.removeTab(tabID);
            }
        }, this));
    },

    destroy: function () {

        var count = this._portals.length,
            i = 0;

        for (; i < count; i++) {
            if (this._portals[i] && _.isFunction(this._portals[i].destroy))
                this._portals[i].destroy();
        }

    }
});


$.widget('ui.portal', $.ui.droppable, {

    dropPlaceholder: $('<div>').addClass('jlv-portlet-placeholder'),

    lastRow: -1,
    lastColumn: -1,


    removePlaceholder: function () {
        this.dropPlaceholder.remove();
        this.lastRow = -1;
        this.lastColumn = -1;
    },

    computeDrop: function (evt, ui, isNew) {

        //compute drop column
        var columns = this.element.find('.jlv-portal-column');

        var x = 0,
            column = 0,
            $dropColumn = null;


        var left =
            ui.helper.offset().left + this.element.scrollLeft() + ui.helper.width() / 2;


        $.each(columns, function (idx, el) {
            if (left > x) {
                column = idx;
                $dropColumn = $(el);
            }
            x += $(el).width();
        });

        if (left > x) {
            column = -1;
            $dropColumn = null;
        }


        if ($dropColumn) {

            var y = 12,
                top =
                    ui.helper.offset().top + this.element.scrollTop() - this.element.offset().top + ui.helper.height() / 2;
            var widgets = $dropColumn.children('.jlv-widget, .jlv-patient-demographics').not('.ui-draggable-dragging');

            //compute row
            var row = 0,
                $elAbove = $dropColumn.children('.jlv-patient-demographics');

            $.each(widgets, function (idx, el) {

                y += ($(el).height() + 12);

                if (top > y && $(el).hasClass('jlv-widget')) {
                    row = idx + 1;
                    $elAbove = $(el);
                }
            });

            if ($elAbove.hasClass('jlv-patient-demographics')) {
                row = 1;
            }

            if (column != this.lastColumn || row != this.lastRow) {

                this.dropPlaceholder.css({
                    width: $dropColumn.width(),
                    height: isNew ? 300 : ui.helper.height()
                });


                if ($elAbove.length) {
                    this.dropPlaceholder.insertAfter($elAbove);
                } else {
                    this.dropPlaceholder.prependTo($dropColumn);
                }

                this.lastColumn = column;
                this.lastRow = row;


                ui.helper.data({
                    rowIndex: this.lastRow,
                    columnIndex: this.lastColumn
                });
            }
        }
    }

});

JLV.Desktop.WidgetPortal = AppKit.View.extend({
    classID: 'jlv-portal',
    tabView: null,
    name: "",
    initialize: function (options) {
        for (var i = 0; i < 3; i++)
            this.$el.append($('<div class="jlv-portal-column">'));

        var self = this;

        this.$el.portal({
            accept: '.jlv-widget, .jlv-widget-menu-item',
            greedy: true,
            over: function (evt, ui) {
                ui.draggable.data('portal', $(this));
            },

            drop: function (evt, ui) {

                if (ui.draggable.hasClass('jlv-widget')) {
                    ui.draggable.insertBefore($(this).find('.jlv-portlet-placeholder'));

                }
                else if (ui.draggable.hasClass('jlv-widget-menu-item')) {

                    var columnIndex = ui.helper.data('columnIndex');
                    var rowIndex = ui.helper.data('rowIndex');

                    var widgetId = ui.helper.data('widget-id');
                    if (widgetId) {
                        //drop the represented widget
                        var theWidget = JLV.createJLVWidget(widgetId, {
                            patient: JLV.ActivePatientModel,
                            portal: self
                        }, true);
                        if (theWidget) {
                            self.insertWidgetInColumnAtIndex(theWidget, columnIndex, rowIndex);
                        }
                        }
                }

                JLV.Session.saveUIConfigForPortal(self.tabView);
            }
        });


        this._super(options);

    },
    layout: function () {

        setTimeout(_.bind(function () {

            if (!this._portalLoaded) {
                this.load();
                this._portalLoaded = true;
            }

        }, this), 0);

        var w = Math.max(350, (this.$el.width() - 16) / 3.0);


        var x = 16;

        var portals = this.$el.children('.jlv-portal-column'),
            count = portals.length,
            idx = 0;

        for (; idx < count; idx++) {
            var column = portals.eq(idx);
            column.css({
                width: w,
                top: 16,
                left: x
            });

            var widgets = column.children('.jlv-widget'),
                wCount = widgets.length,
                jdx = 0;

            for (; jdx < wCount; jdx++) {
                var widgetEl = widgets.eq(jdx);
                var id = widgetEl.data('id');
                if (id) {
                    var widget = AppKit.get(id);
                    if (widget) {
                        widget.sizer();
                    }
                }
            }

            x += (w + 8);
        }


    },
    load: function () {

        var $widgets = this.$el.find('.jlv-widget');
        _.each($widgets, function (w) {
            var widget = $(w).widget();
            if (_.isFunction(widget.loadData)) {
                widget.loadData();
            }
        });
    },
    insertWidgetInColumnAtIndex: function (aWidget, columnIndex, rowIndex) {
        var $column = $(this.$el.children('.jlv-portal-column').get(columnIndex));
        var $widgets = $column.children('.jlv-widget, .jlv-patient-demographics');

        if (rowIndex < $widgets.length) {
            var $before = $($widgets.get(rowIndex));
            aWidget.$el.insertBefore($before);

        }
        else {

            aWidget.$el.appendTo($column);
        }

        if (_.isFunction(aWidget.sizer)) {
            aWidget.sizer();
        }

        aWidget.set('portal', this);


    },
    addWidget: function (aWidget, columnIndex) {
        this.insertWidgetInColumnAtIndex(aWidget, columnIndex, 0);
    },
    removeWidget: function (aWidget) {
        if (aWidget) {
            aWidget.close();
            aWidget = null;
        }
    },

    //cleans up the widgets
    destroy: function () {

        var col = 0;

        for (; col < 3; col++) {
            var widgets = this.widgetsInColumn(col),
                count = widgets.length,
                i = 0;

            for (; i < count; i++) {
                var widget = widgets[i];
                if (widget && _.isFunction(widget.destroy)) {
                    widget.destroy();
                }

                widget = null;
            }
        }

    },
    widgetsInColumn: function (columnIndex) {
        var column = this.$el.children('.jlv-portal-column').get(columnIndex);
        var widgetsEl = $(column).children('.jlv-widget');
        var widgets = [];
        $.each(widgetsEl, function (idx, w) {
            widgets.push($(w).data('widget'));
        });

        return widgets.reverse();
    }

});


JLV.Desktop.SaveProviderPortalUINotification = "JLV.Desktop.SaveProviderPortalUI";
JLV.Desktop.SavePatientPortalUINotification = "JLV.Desktop.SavePatientPortalUI";

JLV.Desktop.ProviderPortal = JLV.Desktop.TabView.extend({
    classID: 'jlv-provider-portal',
    className: 'jlv-tab-view',
    initialize: function (options) {
        this._super(options);

        this.addTabbedPortal('Provider Portal');

        var self = this;
        JLV.NotificationCenter.addObserverForNotification({
            name: JLV.Desktop.SaveProviderPortalUINotification,
            action: function () {
                JLV.Session.saveUIConfigForPortal(self);
            }
        });
    },

    destroy: function () {

        this._super();

        JLV.NotificationCenter.removeAllNotificationsForNotificationName(
            JLV.Desktop.SaveProviderPortalUINotification
        );
    }
});


JLV.Desktop.PatientPortal = JLV.Desktop.TabView.extend({
    classID: 'jlv-patient-portal',
    className: 'jlv-tab-view',
    initialize: function (options) {
        this._super(options);
        this.addTabbedPortal('Patient Portal');

        var self = this;

        JLV.NotificationCenter.addObserverForNotification({
            name: JLV.Desktop.SavePatientPortalUINotification,
            action: function () {
                JLV.Session.saveUIConfigForPortal(self);
            }
        });
    },
    displayActivePatient: function () {
        if (JLV.ActivePatientModel) {
            this._dem = new JLV.PatientDemographics({
                patient: JLV.ActivePatientModel
            });
            this.portalAtIndex(0).addWidget(this._dem, 0);
            this._dem.render();
            this.selectTabAtIndex(0);

        }
    },
    destroy: function () {

        this._super();

        JLV.NotificationCenter.removeAllNotificationsForNotificationName(
            JLV.Desktop.SavePatientPortalUINotification
        );


        if (this._dem && _.isFunction(this._dem.destroy)) {
            this._dem.destroy();
        }

    }
});

//gets the current active portal
JLV.Desktop.WidgetPortal.getActive = function () {
    return AppKit.get(JLV.Desktop.ScreenLaunchablesController.activeScreen);
};