//= require lib/AppKit.js
//= require ./desktop.js
//= require ./WidgetMenu.js
//= require ./WindowManager.js

JLV.Desktop.ScreenLaunchablesController = {
    activeScreen: null,
    addScreen: function (data) {

        var item = $('<li tabindex="1">')
            .addClass("AppToolbarButton")
            .attr("data-screen-id", data.screenID)
            .attr('title', data.title)
            .html(data.content)
            .appendTo($('#AppToolbarLaunchables'));


        var self = this;
        item.mousedown(function (evt) {
            evt.preventDefault();
        });
        item.on('click', function () {
            self.selectScreen(data.screenID);
        });


        item.on('keyup', function (evt) {
            if (evt.keyCode === $.ui.keyCode.SPACE)
                self.selectScreen(data.screenID);
            else if (evt.keyCode == $.ui.keyCode.ESCAPE) {
                if (item.find('.icon-close').length) {
                    JLV.Desktop.WidgetMenu.hideActive(false);
                    self.removeScreen(data.screenID);
                }
            }
        });

        item.on('focus', function (evt) {
            $(this).addClass("ui-state-focus");
        });

        item.on('blur', function () {
            $(this).removeClass('ui-state-focus');
        });

        if (data.closable !== false) {
            var closeBtn = $('<div class="close-launchable">')
                .append('<div class="icon-close">')
                .button()
                .prependTo(item);

            closeBtn.on('click', function (evt) {
                JLV.Desktop.WidgetMenu.hideActive(false);
                self.removeScreen(data.screenID);
            }).mousedown(function (evt) {
                evt.stopPropagation();
            });
        }

        if (data.setActive === true) {
            this.selectScreen(data.screenID);
        }

        //vertically center the label
        var label = item.children('label');

        var fontSize = 14.0;
        while (label.height() > 49.0 && fontSize > 8.0) {
            label.css('font-size', --fontSize);
        }

        label.css('margin-top', (44.0 - label.height()) / 2.0);

    },
    selectScreen: function (screenId) {

        var view = AppKit.get(screenId);

        JLV.Desktop.WidgetMenu.hideActive(false);
        JLV.Desktop.ClearOpenWindows();


        if (view) {
            if (this.activeScreen) {
                AppKit.get(this.activeScreen).set('hidden', true);
            }

            $('.AppToolbarButton').removeClass('ui-state-active');

            this.activeScreen = screenId;
            view.set('hidden', false);

            $('[data-screen-id="' + screenId + '"]').addClass('ui-state-active');
        }


    },
    removeScreen: function (screenId) {

        $('[data-screen-id="' + screenId + '"]').remove();

        var view = AppKit.get(screenId);

        if (view) {

            if (_.isFunction(view.destroy)) {
                view.destroy();
            }

            setTimeout(function () {
                view.$el.remove();
            }, 0);

        }

        if (this.activeScreen === screenId) {
            this.activeScreen = null;
        }

        this.selectScreen('ProviderPortal');
    }
};


