//= require ./PatientWidget.js
//= require ./windows/PatientAppointmentsWidgetWindow.js

JLV.PatientAppointmentsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'clinic',
    url: 'patientAppointments/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['appointmentDate'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientAppointments = JLV.PatientWidget.extend({
    options: {
        widgetId: 'appointments_widget',
        name: 'Appointments',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientAppointmentsWidgetWindow',
        startDay: -120,
        endDay: 30,
        columns: [
            {
                title: 'Date',
                id: 'appointmentDate',
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkDateFormatter(value, model);
                    }
                    else {
                        return JLV.common.dateFormatter(value);

                    }
                },
                sortable: true,
                width: 80
            },
            {
                title: 'Clinic',
                id: 'clinic',
                width: 80,
                sortable: true

            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        try {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: {{-taxonomy}}<br/>' +
                                'NUCC Taxonomy Code: {{-taxonomyCode}}<br/>' +
                                '<b>Provider Address: </b>{{-provider.street1}} {{-provider.street2}} {{-provider.street3}}' +
                                '{{-provider.city}}, {{-provider.stateProvince}} {{-provider.postalCode}} <br/>' +
                                '<b>Provider Phone: </b> {{-provider.phone}}'
                            )(model.toJSON());
                        } catch (e) {

                        }
                    }
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'appointmentDate',
            asc: false
        },
        data: JLV.PatientAppointmentsCollection
    }, /*
    toggleDateFilterPanel: function () {
        if (!this._dateFilterPanel) {
            var data = this.get('data');
            if (data) {
                if (data.startDate !== undefined && data.endDate !== undefined) {
                    this._dateFilterPanel = new JLV.Widget.AppointmentsDateFilterPanel({
                        widget: this,
                        startDate: data.startDate.toDate(),
                        endDate: data.endDate.toDate()
                    });

                    this._dateFilterPanel.$el.appendTo(this._body);
                    this._dateFilterPanel.$el.hide();
                }
            }
        }

        this._super();
     },*/
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {

            var detailStartDate = model.get('appointmentDate'),
                detailEndDate = model.get('appointmentDate');

            var ONE_DAY = 24 * 60 * 60 * 1000;
            detailStartDate.setTime(detailStartDate.getTime() - ONE_DAY);
            detailEndDate.setTime(detailEndDate.getTime() + ONE_DAY);

            var dialog = new JLV.DetailsWindow({
                title: String.format("Appointment Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('clinic')),
                url: 'patientAppointments/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken'),
                    startDate: moment(detailStartDate).format("M/D/YY"),
                    endDate: moment(detailEndDate).format("M/D/YY")
                },
                fail: function (response) {
                    return 'Error: Unable to open HFS file';
                },
                renderer: function (response) {
                    return '<pre>' + response.visitDetails + '</pre>';
                }
            });

            dialog.center();
            dialog.open();
        }
    }

});


JLV.registerJLVWidget('appointments_widget', JLV.PatientAppointments);

