//= require ./PatientWidget.js
//= require ./windows/PatientClinicalRemindersWidgetWindow.js

JLV.PatientClinicalRemindersCollection = JLV.PatientDateFilteredDataCollection.extend({
    url: 'patientClinicalReminders/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: []
    })
});


JLV.PatientClinicalReminders = JLV.PatientWidget.extend({
    options: {
        widgetId: 'clinicalreminders_widget',
        name: 'Clinical Reminders - VA Only',
        dateFilter: false,
        status: false,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientClinicalRemindersWidgetWindow',
        columns: [
            {
                title: 'Due',
                id: 'dueDate',
                sortable: true,
                width: 80,
                formatter: function (value, model) {
                    if (model && model.get('dueDate') && model.get('dueDate').length) {
                        return value;
                    }
                    else {
                        return model.get('dueDateString');
                    }
                }
            },
            {
                title: 'Last Done',
                id: 'lastDate',
                width: 100,
                sortable: true

            },
            {
                title: 'Reminder',
                id: 'name',
                width: 200,
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'dueDate',
            asc: false
        },
        data: JLV.PatientClinicalRemindersCollection
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Clinical Reminder Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('name')),
                url: 'patientClinicalReminders/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                renderer: function (response) {
                    return '<pre>' + response.clinicalReminderText + '</pre>';
                }
            });

            dialog.center();
            dialog.open();
        }

    }

});


JLV.registerJLVWidget('clinicalreminders_widget', JLV.PatientClinicalReminders);