//= require ./PatientWidget.js
//= require ./windows/PatientConsultsWidgetWindow.js

JLV.PatientConsultsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'service',
    url: 'patientConsults/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['requestDate'],
        parse: function (data) {

            if (data.outCode) {
                if (data.outCode.indexOf("<") === 0) {
                    data.outCode = data.outCode.replace("<", "").replace(">", "");
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientConsults = JLV.PatientWidget.extend({
    options: {
        widgetId: 'consults_widget',
        name: 'Consults',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientConsultsWidgetWindow',
        columns: [
            {
                title: 'Due',
                id: 'requestDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 80
            },
            {
                title: 'Consult Order',
                id: 'outCode',
                width: 210,
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Status',
                id: 'outStatus',
                width: 100,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'requestDate',
            asc: false
        },
        data: JLV.PatientConsultsCollection
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {

            if (model.get('noteAsBinaryStream')) {
                var consultURL = String.format("patientConsults/getNoteDataStream?patToken={0}&recToken={1}",
                    JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

                JLV.common.openAuxBrowserWindow(consultURL, "JLV_CONSULT_REPORT");
            }
            else {
                var dialog = new JLV.DetailsWindow({
                    title: String.format("Consult Report ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('outCode')),
                    url: 'patientConsults/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken')
                    },
                    renderer: function (response) {
                        return '<pre>' + response.report + '</pre>';
                    }
                });

                dialog.center();
                dialog.set('visible', true);
            }


        }

    }

});


JLV.registerJLVWidget('consults_widget', JLV.PatientConsults);