//= require ./PatientWidget.js
//= require ./windows/PatientDischargeNotesWidgetWindow.js

JLV.PatientDischargeNotesCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'noteType',
    url: 'patientDischargeSummaries/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['noteDate']

    })
});


JLV.PatientDischargeNotes = JLV.PatientWidget.extend({
    options: {
        widgetId: 'inpatientnotes_widget',
        name: 'Discharge/Essentris Notes',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientDischargeNotesWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Note Type / Title',
                id: 'noteType',
                width: 200,
                sortable: true,
                formatter: function (value, model) {
                    if (!model || value === 'Connection unavailable.' || (model && !model.get('recordDetailsToken'))) {
                        if(model && model.get('noteTitle')){
                            return value + ' / ' + model.get('noteTitle');
                        }

                        return value;
                    }

                    var noteType = value;
                    if (!noteType || noteType.length < 1) {
                        noteType = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if(model.get('noteTitle')){
                        noteType = noteType + ' / ' + model.get('noteTitle');
                    }

                    return JLV.common.linkFormatter(noteType, model);
                }

            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        },
        data: JLV.PatientDischargeNotesCollection
    },
    itemClicked: function (model) {

        if (model.get('noteAsBinaryStream')) {
            var noteURL = String.format("patientDischargeSummaries/details?patToken={0}&recToken={1}",
                JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

            JLV.common.openAuxBrowserWindow(noteURL, "Discharge/Essentris Note Details");
        }
        else {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Discharge/Essentris Note Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('noteType')),
                url: 'patientDischargeSummaries/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                renderer: function (response) {
                    if(!response.images){
                        response.images = "";
                    }
                    return '<pre>' + response.noteText + response.images + '</pre>';
                }
            });

            dialog.center();
            dialog.set('visible', true);

        }
    },
    toggleDateFilterPanel: function () {

        this._super();

        this.$el.find('.jlv-filter-date-msg').html('* Using expanded search dates is recommended in order to display notes after a specific visit or discharge date.');

    }
});


JLV.registerJLVWidget('inpatientnotes_widget', JLV.PatientDischargeNotes);