//= require ./PatientWidget.js
//= require ./PatientAppointments.js
//= require ./windows/PatientEncounterDetailsWindow.js
//= require ./windows/PatientEncountersWidgetWindow.js

JLV.PatientEncountersCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'clinic',
    url: 'patientEncounters/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['apptDate'],
        parse: function (data) {

            if (data.diagnosis) {
                if (data.diagnosis[0].description) {
                    data.diagnosisList = data.diagnosis[0].description;
                }
            }


            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientEncounters = JLV.PatientWidget.extend({
    options: {
        widgetId: 'encounters_widget',
        name: 'Encounters',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientEncountersWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'apptDate',
                sortable: true,
                width: 100,
                formatter: function (value, model) {

                    if (model && model.get('clinic') === 'Connection unavailable.') {
                        return JLV.common.dateFormatter(value);
                    }
                    if (model) {

                        if (model.get('complexDataUrl') || model.get('noteText') || model.get('vaNote'))
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model) + ' ' + JLV.common.linkFormatter("Note", model);
                        else
                            return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm') + '<br/>' + JLV.common.linkFormatter("Details", model) + ' ' + "Note"



                    }
                    return value;
                }
            },
            {
                title: 'Clinic',
                id: 'clinic',
                width: 100,
                sortable: true

            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Diagnosis',
                width: 100,
                id: 'diagnosisList',
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'apptDate',
            asc: false
        },
        data: JLV.PatientEncountersCollection
    },
    initialize: function (options) {
        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });
        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
                if (this._moreWindow) {
                    this._moreWindow.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
                }

            }, this)
        });
    },
    itemClicked: function (model, item) {

        //if (model.get('sensitive')) {
        //    AppKit.AlertDialog.alert('Sensitive Encounter', JLV.common.SENSITIVE_ENCOUNTER_MSG);
        //    return;
        //}

        if (model.get('sensitive')) {
            var jsonData = {
                providerName: JLV.ActiveProvider.name,
                ipAddress: model.get('guiIpAddress'),
                loginAgency: JLV.Session.loginAgency,
                loginSite: JLV.Session.loginSite,
                eventId: model.get('eventId')
            }

            var self = this;

            JLV.BreakTheGlass.CheckSensitiveRecordAccess(jsonData, JLV.ActivePatientModel.get('patientToken'), JLV.ActivePatientModel.get('name'), JLV.ActivePatientModel.get('edipi'), JLV.ActivePatientModel.get('dob'), function(agree, auditText) {
                if (agree) {
                    self._handleDetails(model, item, auditText);
                }
            });
        } else {
            this._handleDetails(model, item);
        }
    },
    _handleDetails: function (model, item, auditText) {
        if (item.text() === 'Note') {

            var auditUrl;

            if (model.get('noteAsBinaryStream')) {
                if (auditText) {
                    auditUrl = String.format("patientEncounters/getNoteDataStream?patToken={0}&recToken={1}&auditText={2}",
                        JLV.ActivePatientModel.get('patientToken'),
                        model.get('recordDetailsToken'),
                        auditText
                    );
                } else {
                    auditUrl = String.format("patientEncounters/getNoteDataStream?patToken={0}&recToken={1}",
                        JLV.ActivePatientModel.get('patientToken'),
                        model.get('recordDetailsToken'));
                }

                var noteURL = auditUrl;

                JLV.common.openAuxBrowserWindow(noteURL, "JLV_ENCOUNTER_NOTE");
            }
            else {
                auditUrl = 'patientEncounters/details.json';
                if (auditText) {
                    auditUrl = String.format(auditUrl + "?auditText={0}", auditText);
                }

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Encounters Note Details ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        JLV.common.dateFormatter(model.get('apptDate'))),
                    url: auditUrl,
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken')
                    },
                    renderer: function (response) {
                        return '<pre class="report-details">' +
                            response.noteText +
                            response.images + '</pre>'
                    }
                });

                dialog.center();
                dialog.open();
            }
        }
        else {

            if (model.get('sensitive')) {
                var auditUrl = String.format('patientEncounters/sensitiveEncounter.json?auditText={0}', auditText);

                // need to retrieve the encounter from the cache
                $.post(auditUrl, {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                    },
                    function (response) {

                        model = new JLV.WidgetDataModel(response, {parse: true});

                        if (model.attributes.codes) {
                            for (var i in model.attributes.codes) {
                                if (model.attributes.codes[i].system) {
                                    if (model.attributes.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {

                                        model.attributes.normCode = model.attributes.codes[i].code;
                                        model.attributes.normDisplay = model.attributes.codes[i].display;
                                        break;
                                    }
                                }
                            }
                        }

                        var dialog = new JLV.PatientEncounterDetailsWindow({
                            title: String.format("Encounter Details ~ {0} {1}",
                                JLV.common.formatSiteMonikerDisplay(model.get('site')),
                                model.get('clinic')),
                            encounter: model,
                            patientToken: JLV.ActivePatientModel.get('patientToken')
                        });

                        dialog.center();
                        dialog.open();

                })
            } else {
                var dialog = new JLV.PatientEncounterDetailsWindow({
                    title: String.format("Encounter Details ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('clinic')),
                    encounter: model,
                    patientToken: JLV.ActivePatientModel.get('patientToken')
                });

                dialog.center();
                dialog.open();

            }

        }
    }

});




JLV.registerJLVWidget('encounters_widget', JLV.PatientEncounters);