//= require ./PatientWidget.js
//= require ./windows/PatientHistoriesWidgetWindow.js

JLV.PatientHistoriesCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'type',
    url: 'patientHistories/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['dateReported'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("SNOMED") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (data.findings) {
                var list = "";

                for (var i in data.findings) {
                    if (list.length > 0 && typeof(data.findings[i]) === undefined) {
                        list += "; ";
                    }
                    if (data.findings[i].item) {
                        list += data.findings[i].item;
                    }
                }
                data.findingList = list;
            }


            if (data.comments && data.comments.length > 0) {
                var list = "";

                for (var i in data.comments) {
                    if (list.length > 0 && typeof(data.comments[i]) === undefined) {
                        list += "; ";
                    }
                    if (data.comments[i]) {
                        list += data.comments[i];
                    }
                }
                data.commentList = list;
            }



            return this._super(data);
        }
    }),
    doFetch: function (patient, callback) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                status: 'A'
            },
            reset: true,
            type: 'POST',
            success: callback
        });
    }
});

JLV.PatientHistories = JLV.PatientWidget.extend({
    options: {
        widgetId: 'histories_widget',
        name: 'Social, Family, and Other Past Histories',
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientHistoriesWidgetWindow',
        columns: [
            {
                width: 80,
                title: 'Date Reported',
                sortable: true,
                id: 'dateReported',
                formatter: JLV.common.dateFormatter
            },
            {
                id: 'type',
                width: 80,
                title: 'Type'
            },
            {
                width: 80,
                title: 'Findings',
                sortable: true,
                id: 'findingList',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                width: 40,
                sortable: true,
                id: 'status'
            },
            {
                title: 'Site',
                id: 'site',
                width: 70,
                sortable: true,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'dateReported',
            asc: false
        },
        data: JLV.PatientHistoriesCollection
    }

});


JLV.registerJLVWidget('histories_widget', JLV.PatientHistories);