//= require ./PatientWidget.js
//= require ./windows/PatientImmunizationsWidgetWindow.js

JLV.PatientImmunizationsCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'vaccine',
    url: 'patientImmunizations/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['adminDate', 'exemptionRefusalDate'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("CVX") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});

JLV.PatientImmunizations = JLV.PatientWidget.extend({
    options: {
        widgetId: 'immunizations_widget',
        name: 'Immunizations',
        status: true,
        vlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientImmunizationsWidgetWindow',
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Vaccine Administered <br /> Date',
                id: 'adminDate',
                formatter: function (aDate, model) {
                    return model.get('dateTime');
                },
                sortable: true,
                width: 140
            },
            {
                title: 'Vaccine Administered <br/> Product Type',
                id: 'vaccine',
                width: 140,
                sortable: true,
                formatter: JLV.common.linkFormatter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Vaccine<br>' +
                                'Administered Code<br>' +
                                '(CVX)</b><br>' +
                                'National Name: ' + normDisplay + '<br>' +
                                'National Code: ' + normCode;
                        }
                    }

                    return '<b>National Vaccine<br>' +
                        'Administered Code<br>' +
                        '(CVX)</b><br>' +
                        'National Name: none<br>' +
                        'National Code: none';
                }
            },
            {
                title: 'Adverse Vaccine<br/>Reaction',
                id: 'reaction',
                width: 100,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 70,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'adminDate',
            asc: false
        },
        data: JLV.PatientImmunizationsCollection
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.open();
        }
        else {

            var dialog = new JLV.DetailsWindow({
                title: String.format("Immunization Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('vaccine')),
                renderer: function () {

                    var data = model.toJSON();
                    data.adminDate = moment(data.adminDate).format("M/D/YY");
                    try {
                        return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                        '<td>Vaccine Product Manufacturer:</td><td>{{-manufacturerName}}</td></tr><tr>' +
                        '<td>Vaccine Lot Number:</td><td>{{-lot}}</td></tr><tr>' +
                        '<td>Dose:</td><td>{{-dose}}</td></tr><tr>' +
                        '<td>Units:</td><td>{{-units}}</td></tr><tr>' +
                        '<td>Vaccine Administered Date:</td><td>{{-dateTime}}</td></tr><tr>' +
                        '<td>Vaccine Administered Product Type:</td><td>{{-vaccine}}</td></tr><tr>' +
                        '<td>Vaccine Route of Administration:</td><td>{{-route}}</td></tr><tr>' +
                        '<td>Vaccine Site of Administration:</td><td>{{-bodySite}}</td></tr><tr>' +
                        '<td>Vaccine Ordering Provider:</td><td>{{-orderingProvider}}</td></tr>' +
                        '</tr></table>')(data);
                    } catch (e) {
                        console.info('template error');
                    }
                }
            });

            dialog.center();
            dialog.open();
        }
    },
    initialize: function (options) {
        this._super(options);

        //add banner that VA data is not available for DoD user
        if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {

            if (this._body) {
                $('<div class="jlv-immunizations-no-va-banner">')
                    .text('VA data is not complete and will not be displayed at this time.')
                    .appendTo(this._body);
                if (this._datagrid) {
                    this._datagrid.$el.css('top', 26);
                }
            }


        }
    }

});

JLV.registerJLVWidget('immunizations_widget', JLV.PatientImmunizations);