//= require ./PatientWidget.js
//= require ./windows/PatientOrdersWidgetWindow.js

JLV.PatientOrdersCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'description',
    url: 'patientOrders/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['orderDate'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("RXNORM") >= 0 ||
                            data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {

                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            data.normCodeType = data.codes[i].system;
                            break;
                        }
                    }
                }
            }
            if (data.status) {
                if (data.status === null || data.status === '') {
                    data.status = 'Unknown';
                }
            } else {
                data.status = 'Unknown';
            }
            if (data.priority && data.priority != null && data.priority !== '') {
                data.status = data.status + " / " + data.priority;
            }

            return this._super(data);
        }
    })
});


JLV.PatientOrders = JLV.PatientWidget.extend({
    options: {
        widgetId: 'orders_widget',
        name: 'Orders',
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientOrdersWidgetWindow',
        columns: [
            {
                title: 'Order Date',
                id: 'orderDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Description',
                id: 'description',
                width: 100,
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            normCodeType = model.get('normCodeType');

                        if (normCodeType && normCodeType.toUpperCase().indexOf('RXNORM') >= 0) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                        else if (normCodeType && normCodeType.toUpperCase().indexOf('LOINC') >= 0) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status / Priority',
                id: 'status',
                width: 100,
                sortable: true,
                formatter: function (value, model) {
                    if (value)
                        return value.toUpperCase();
                }
            },
            {
                title: 'Type',
                id: 'type',
                width: 100,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'orderDate',
            asc: true
        },
        data: JLV.PatientOrdersCollection
    },
    itemClicked: function (model) {

        if (model.get('recordDetailsToken')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Order Details ~ {0} {1}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site')),
                    model.get('description')),
                url: 'patientOrders/details.json',
                queryParams: {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recToken: model.get('recordDetailsToken')
                },
                renderer: function (response) {
                    return '<pre>' + response.orderReport + '</pre>';
                }
            });

            dialog.center();
            dialog.open();

        }
    }
});


JLV.registerJLVWidget('orders_widget', JLV.PatientOrders);