//= require ./PatientWidget.js
//= require ./windows/PatientPrescriptionFillsWindow.js
//= require ./windows/PatientOutpatientMedicationsWidgetWindow.js

JLV.PrescriptionFillsModel = JLV.PatientDataModel.extend({
    dateFields: ['lastFilledDate', 'expirationDate']
});


JLV.OutpatientMedicationsCollection = JLV.PatientDateFilteredDataCollection.extend({
    dodAdapterSourceReportField: 'drugName',
    url: 'patientOutpatientMedications/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['lastFilledDate', 'stopDate', 'dispenseDate', 'startDate', 'orderDate'],
        parse: function (data) {
            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("RXNORM") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            if (data.prescriptionFills) {
                var count = data.prescriptionFills.length,
                    i = 0;
                for (; i < count; i++) {
                    data.prescriptionFills[i] = new JLV.PrescriptionFillsModel(
                        _.extend(data.prescriptionFills[i], {sig: data.sig}),
                        {parse: true}
                    );
                }
            }

            if (data.childResistant === true) {
                data.childResistant = "YES";
            } else if (data.childResistant === false) {
                data.childResistant = "NO";
            }

            return this._super(data);
        }
    })
});


JLV.PatientOutpatientMedications = JLV.PatientWidget.extend({
    options: {
        widgetId: 'outpatientmedications_widget',
        name: 'Outpatient Medications',
        dateFilter: true,
        status: true,
        clipboard: true,
        copyActiveMeds: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientOutpatientMedicationsWidgetWindow',
        startDay: -365,
        endDay: 0,
        columns: [
            {
                title: 'Last Fill',
                id: 'lastFilledDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 150,
                sortable: true,
                formatter: function (value, model) {
                    return JLV.common.linkFormatter(value, model, null, "report");
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 100,
                sortable: true
            },
            {
                title: 'Expires',
                id: 'stopDate',
                width: 100,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Site',
                id: 'site',
                width: 100,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'fillOrderDate',
            asc: false
        },
        data: JLV.OutpatientMedicationsCollection
    },
    initialize: function (options) {
        this._super(options);

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshConfirmNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
                if (this._moreWindow) {
                    this._moreWindow.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
                }

            }, this)
        });
    },
    itemClicked: function (model, item) {

        var route = item.data('route');
        if (route === "report") {
            if (model.get('recordDetailsToken')) {

                var dialog = new JLV.DetailsWindow({
                    title: String.format("Outpatient Medication Detail View ~ {0}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                    url: 'patientOutpatientMedications/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken')
                    },
                    renderer: function (response) {
                        return '<pre>' + response.medDetail + '</pre>';
                    }
                });

                dialog.center();
                dialog.open();

            }
            else {

                var dialog = new JLV.DetailsWindow({
                    height: 500,
                    title: String.format("Outpatient Medication Detail View ~ {0}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                    renderer: function () {

                        var data = model.toJSON();

                        if (data.orderDate) {
                            data.orderDate = moment(data.orderDate).format("M/D/YY");
                        }

                        data.ndcCode = "";
                        if (data.codes && data.codes.length > 0) {
                            for (var i in data.codes) {
                                if (data.codes[i].system && data.codes[i].code && "NDC" === data.codes[i].system) {
                                    data.ndcCode = data.codes[i].code
                                }
                            }

                        }

                        if (data.dispenseDate) {
                            data.dispenseDate = moment(data.dispenseDate).format("M/D/YY");
                        }

                        if (data.stopDate) {
                            data.stopDate = moment(data.stopDate).format("M/D/YY");
                        }

                        try {
                            if (JLV.ActiveProvider.agency !== JLV.common.AGENCY_DOD) {
                                return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                                    '<td>Drug Name:</td><td>{{drugName}}</td></tr><tr>' +
                                    '<td>Units:</td><td>{{units}}</td></tr><tr>' +
                                    '<td>Dispensing Pharmacy:</td><td>{{pharmacy}}</td></tr><tr>' +
                                    '<td>Child Resistant Container:</td><td>{{childResistant}}</td></tr><tr>' +
                                    '<td>Current Supply End Date:</td><td>{{stopDate}}</td></tr><tr>' +
                                    '<td>NDC:</td><td>{{ndcCode}}</td></tr><tr>' +
                                    '<td>RX Number:</td><td>{{rxNumber}}</td></tr><tr>' +
                                    '<td>Number of Refills:</td><td>{{refills}}</td></tr><tr>' +
                                    //'<td>Pharmacy Comments:</td><td></td></tr><tr>' +
                                    '<td>Dispense Date:</td><td>{{dispenseDate}}</td></tr><tr>' +
                                    '<td>Refills left:</td><td>{{refillsRemaining}}</td></tr><tr>' +
                                    '<td>Ordering Provider Comments:</td><td>{{comment}}</td></tr><tr>' +
                                    '<td>Order Date:</td><td>{{orderDate}}</td></tr><tr>' +
                                    '<td>Status:</td><td><pre>{{status}}</pre></td></tr><tr>' +
                                    '<td>Order Number:</td><td><pre>{{orderId}}</pre></td></tr></table>')(data);
                            } else {
                                return _.template('<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                                    '<td>Drug Name:</td><td>{{drugName}}</td></tr><tr>' +
                                    '<td>Units:</td><td>{{units}}</td></tr><tr>' +
                                    '<td>Dispensing Pharmacy:</td><td>{{pharmacy}}</td></tr><tr>' +
                                    '<td>Child Resistant Container:</td><td>{{childResistant}}</td></tr><tr>' +
                                    '<td>Current Supply End Date:</td><td>{{stopDate}}</td></tr><tr>' +
                                    '<td>NDC:</td><td>{{ndcCode}}</td></tr><tr>' +
                                    '<td>RX Number:</td><td>{{rxNumber}}</td></tr><tr>' +
                                    '<td>Number of Refills:</td><td>{{refills}}</td></tr><tr>' +
                                        //'<td>Pharmacy Comments:</td><td></td></tr><tr>' +
                                        //'<td>Dispense Date:</td><td>{{dispenseDate}}</td></tr><tr>' +
                                    '<td>Refills left:</td><td>{{refillsRemaining}}</td></tr><tr>' +
                                    '<td>Prescription Comments:</td><td>{{comment}}</td></tr><tr>' +
                                    '<td>Order Date:</td><td>{{orderDate}}</td></tr><tr>' +
                                    '<td>Status:</td><td><pre>{{status}}</pre></td></tr><tr>' +
                                    '<td>Order Number:</td><td><pre>{{orderId}}</pre></td></tr></table>')(data);
                            }

                        } catch (e) {
                        }

                        return '<table cellpadding="5" cellspacing="0" class="jlv-detail-report-table"><tr>' +
                            '<td>Drug Name:</td><td></td></tr><tr>' +
                            '<td>Units:</td><td></td></tr><tr>' +
                            '<td>Dispensing Pharmacy:</td><td></td></tr><tr>' +
                            '<td>Child Resistant Container:</td><td></td></tr><tr>' +
                            '<td>Current Supply End Date:</td><td></td></tr><tr>' +
                            '<td>NDC:</td><td></td></tr><tr>' +
                            '<td>RX Number:</td><td></td></tr><tr>' +
                            '<td>Number of Refills:</td><td></td></tr><tr>' +
                                //'<td>Pharmacy Comments:</td><td></td></tr><tr>' +
                                //'<td>Dispense Date:</td><td></td></tr><tr>' +
                            '<td>Refills left:</td><td></td></tr><tr>' +
                            '<td>Prescription Comments:</td><td></td></tr><tr>' +
                            '<td>Order Date:</td><td></td></tr><tr>' +
                            '<td>Status:</td><td><pre></pre></td></tr><tr>' +
                            '<td>Order Number:</td><td><pre></pre></td></tr></table>';
                    }
                });

                dialog.center();
                dialog.open();
            }
        }
        else {

            if (model.get('prescriptionFills') &&
                model.get('lastFilledDate')) {

                //send ajax rx fills class
                jQuery.post('patientOutpatientMedications/auditRxFillsQuery', {
                    patToken: JLV.ActivePatientModel.get('patientToken'),
                    recordDate: model.get('lastFilledDate').toString(),
                    recordSiteMoniker: model.get('site').moniker,
                    recordSiteAgency: model.get('site').agency
                });

                var dialog = new JLV.PatientPrescriptionFillsWindow({
                    data: new Backbone.Collection(model.get('prescriptionFills'))
                });

                dialog.center();
                dialog.open();
            }
        }

    },
    copyActiveMeds: function (aBool) {
        // Do not show/create copy active Rxs to clipboard button for outside providers (VACV)
        if (!JLV.common.OUTSIDE_PROVIDER_FLAG) {
            if (aBool === true) {

                if (!this._copyActiveMedsBtn) {
                    var self = this;
                    this._copyActiveMedsBtn = $('<div title="Copy Active Rxs to clipboard" tabindex="0" class="jlv-widget-toolbutton">')
                        .append('<div class="icon-med-drugs">')
                        .button()
                        .insertBefore(self.$el.find('.clipboard'))
                        .on('click', function () {
                            self._doCopyActiveMeds();
                        });
                }

                this._copyActiveMedsBtn.show();
            }
            else if (aBool === false) {
                this._copyActiveMedsBtn.hide();
            }

            return this._copyActiveMedsBtn.is(':visible');
        }
    },
    _doCopyActiveMeds: function () {

        if (window.clipboardData && window.clipboardData.setData) {

            var returnData = "For Official Use Only\r\nLast Fill\tPrescription\tStandardized Prescription\tSig\tQuantity\tDays Supply\tRefills\tStatus\tOrdering HCP\tExpires\tSite\r\n";

            var data = this._datagrid.get('source');
            data.each(function (model) {
                if (model.get('status').toUpperCase() === 'ACTIVE') {
                    returnData += JLV.common.dateFormatter(model.get('lastFilledDate')) + "\t";
                    returnData += model.get('drugName') + "\t";
                    returnData += model.get('normDisplay') + "\t";
                    returnData += model.get('sig') ? model.get('sig') : "" + "\t";
                    returnData += model.get('quantity') + "\t";
                    returnData += model.get('daysSupply') + "\t";
                    returnData += model.get('refills') + "\t";
                    returnData += model.get('active') + "\t";
                    returnData += model.get('providerName') + "\t";
                    returnData += JLV.common.dateFormatter(model.get('stopDate')) + "\t";
                    returnData += model.get('site').moniker + "\r\n";
                }
            });

            window.clipboardData.setData('text', returnData);
        }
        else {
            console.info("Copying to clipboard only works in IE8+");
        }
    }
});

JLV.registerJLVWidget('outpatientmedications_widget', JLV.PatientOutpatientMedications);