//= require ./PatientWidget.js
//= require ./windows/PatientQuestionnairesWidgetWindow.js

JLV.PatientQuestionnairesCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'documentTitle',
    url: 'patientForms/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['formDate'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("LOINC") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            return this._super(data);
        }
    })
});


JLV.PatientQuestionnaires = JLV.PatientWidget.extend({
    options: {
        widgetId: 'questionnaires_widget',
        name: 'Questionnaires & Deployment Assessments - AHLTA Only',
        dateFilter: false,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        moreWindowClass: 'JLV.PatientQuestionnairesWidgetWindow',
        columns: [
            {
                title: 'Date',
                id: 'formDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Document Title',
                id: 'documentTitle',
                width: 180,
                sortable: true,
                formatter: function (value, model) {
                    if (!model || value === 'Connection unavailable.' || !model.get('recordDetailsToken')) {
                        return value;
                    }

                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 100,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'formDate',
            asc: false
        },
        data: JLV.PatientQuestionnairesCollection
    },
    itemClicked: function (model) {

        if (model && model.get('recordDetailsToken')) {
            if (model.get('noteAsBinaryStream')) {

                var noteURL = String.format("patientForms/getNoteDataStream?patToken={0}&recToken={1}",
                    JLV.ActivePatientModel.get('patientToken'), model.get('recordDetailsToken'));

                JLV.common.openAuxBrowserWindow(noteURL, 'JLV_DEPLOYMENT_FORMS');
            }
            else {
                var dialog = new JLV.DetailsWindow({
                    title: String.format("Questionnaire ~ {0} {1}",
                        JLV.common.formatSiteMonikerDisplay(model.get('site')),
                        model.get('documentTitle')),
                    width: 700,
                    height: 500,
                    url: 'patientForms/details.json',
                    queryParams: {
                        patToken: JLV.ActivePatientModel.get('patientToken'),
                        recToken: model.get('recordDetailsToken')
                    },
                    renderer: function (response) {
                        return '<div class="report-details">' + response.reportText + '</div>';
                    }
                });

                dialog.center();
                dialog.set('visible', true);

            }

        }

    }

});


JLV.registerJLVWidget('questionnaires_widget', JLV.PatientQuestionnaires);