//= require ../Widget.js

JLV.PatientDataModel = JLV.WidgetDataModel.extend({});

JLV.PatientDataCollection = Backbone.Collection.extend({
    initialize: function (models, options) {

        this.dodAdapterSourceReport = [];
        this.dodAdapterStatusRec = [];

    },
    doFetch: function (patient, callback, error) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                includeVLER: JLV.Session.dodVLERReady === true,
                forceRefresh: JLV.Session.forceRefresh === true
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    },
    parse: function (response) {

        this.dodAdapterSourceReport = [];
        this.dodAdapterStatusRec = [];

        if (this.dodAdapterSourceReportField) {

            var count = response.length,
                i = 0;

            while (i < count) {
                var model = response[i];
                var site = model.site;
                if (site && site.moniker === JLV.common.DODADAPTER_STATUS_REPORT_FLAG) {
                    this.dodAdapterSourceReport.push(model[this.dodAdapterSourceReportField]);
                    this.dodAdapterStatusRec.push(model);
                    response.splice(i, 1);
                    count = response.length;
                }
                else {
                    i++;
                }


            }
        }
        return response;
    }
});

JLV.PatientDateFilteredDataCollection = JLV.PatientDataCollection.extend({
    doFetch: function (patient, callback, error) {
        this.fetch({
            data: {
                patToken: patient.get('patientToken'),
                startDate: moment(this.startDate).format("M/D/YY"),
                endDate: moment(this.endDate).format("M/D/YY"),
                includeVLER: JLV.Session.dodVLERReady === true
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});


JLV.PatientWidget = JLV.Widget.extend({
    /**
     *
     * Fetch the data from the server using the Collection
     *
     **/
    loadData: function (callback) {

        var data = this.get('data'),
            patient = this.get('patient');

        if (!data && this._datagrid) {
            console.warn(this.get('name') + ' widget data not set.');
        }

        if (!patient) {
            console.warn(this.get('name') + ' widget patient not set.');
        }

        if (data && this._datagrid &&
            patient) {

            if (!_.isFunction(data.doFetch)) {
                console.warn('Widget data sources must be a Backbone collection implementing a doFetch method.');
                return;
            }

            data.reset();

            this._super();

            var self = this;
            data.doFetch(patient, function () {

                    if (self._datagrid.get('rowCount') === 0) {
                        self._datagrid.showMessage("No Data");
                    }
                    else if (typeof self._defaultSort !== 'undefined' && self._defaultSort !== null) {

                        var column = _.findWhere(self.options.columns, {id: self._defaultSort.id});
                        if (column) {
                            column.index = _.indexOf(self.options.columns, column);

                            var sortInfo = {
                                asc: self._defaultSort.asc,
                                column: column
                            };

                            self._datagrid.sortByColumn(sortInfo);
                            self._datagrid.render();

                            if (self.get('moreWindowClass')) {
                                self._moreBtn.show();
                            }


                        }
                        else {
                            self._datagrid.render();

                            if (self.get('moreWindowClass')) {
                                self._moreBtn.show();
                            }
                        }


                    }
                    else {
                        self._datagrid.render();

                        if (self.get('moreWindowClass')) {
                            self._moreBtn.show();
                        }
                    }

                    //render the expanded view
                    if (self._moreWindow) {
                        self._moreWindow.renderData();
                    }

                    //update the widget title to reflect the number of records
                    self.set('title', self.get('name') + ' (' + self._datagrid.get('rowCount') + ')');

                    if (_.isFunction(callback)) {
                        callback();
                    }

                    self._updatePageInfoDisplay();

                },
                /* handle error */
                function (collection, err) {

                    console.info(err)
                    if (err.status !== 200) {

                        self._datagrid.showMessage(_.template(
                                '<div class="server-error-msg" style="color:red;cursor:pointer;text-decoration:underline;">Something went wrong: {{statusText}} ({{status}})</div>')(err)
                        );


                    }

                });
        }
    },
    doPrint: function () {

        if (this._datagrid) {
            //fit columns to 775 width

            var dataHTML = '<html><head><title>{{title}}</title>\n{{cssLink1}}\n{{cssLink2}}</head>';
            dataHTML += '<body><div style="text-align: center;font-weight: bold;">For Official Use Only</div>';

            if (JLV.common.isDoDProvider(JLV.ActiveProvider)) {
                dataHTML += '<div><b>Patient Name: </b>{{name}} &nbsp;<b>DOB: </b>{{dob}} &nbsp;<b>Gender: </b>{{gender}} &nbsp;<b>DoD ID: </b>{{edipi}}</div>';
            }
            else {
                dataHTML += '<div><b>Patient Name: </b>{{name}} &nbsp;<b>DOB: </b>{{dob}} &nbsp;<b>Gender: </b>{{gender}} &nbsp;<b>SSN: </b>{{fmpssn}}</div>';
            }

            dataHTML += '<div class="ui-table">{{tableHTML}}</div></body></html>';

            var clone = this._datagrid.$el.find('.ui-table-body');
            clone.find('.appkit-table-column, .ui-datatable-loading-rows, .dummy-row-1, .dummy-column').remove();

            try {

                var printData = {
                    name: JLV.ActivePatientModel.get('name') || 'NA',
                    dob: JLV.ActivePatientModel.get('dob') || 'NA',
                    gender: JLV.ActivePatientModel.get('gender') || 'NA',
                    fmpssn: JLV.ActivePatientModel.get('fmpssn') || 'NA',
                    edipi: JLV.ActivePatientModel.get('edipi') || 'NA'
                };


                var tableHTML = '<table style=""><tr>';

                var count = this._columns.length,
                    i = 0;

                for (; i < count; i++) {
                    tableHTML += '<th>' + this.options.columns[i].title + '</th>';
                }

                tableHTML += '</tr>';

                tableHTML += clone.html();

                tableHTML += '</table>';

                var printHTML = _.template(dataHTML)(_.extend({
                    title: this.get('name') + " Print out - " + JLV.ActivePatientModel.get('name'),
                    cssLink1: '<link rel="stylesheet" type="text/css" href="assets/AppKit/datatable.css">',
                    cssLink2: '<link rel="stylesheet" type="text/css" href="assets/view/print.css">',
                    tableHTML: tableHTML
                }, printData));


                var printWindow = window.open("", "Print");
                printWindow.document.write(printHTML);

                printWindow.document.close();

                setTimeout(function () {
                    printWindow.print();
                    printWindow.close();
                }, 200);

            } catch (e) {

            }
        }

    },
    savePortalUI: function () {
        JLV.NotificationCenter.postNotificationName(JLV.Desktop.SavePatientPortalUINotification);
    }
});


