//= require ./PatientWidgetWindow.js

JLV.PatientAllergiesWidgetWindow = JLV.PatientWidgetWindow.extend({
    dodAdapterSourceReportField: 'allergyName',
    normCodeSystems: [JLV.common.CODING_SYSTEM_UMLS],
    options: {
        name: 'Allergies Expanded',
        status: true,
        vlerDomain: true,
        filter: false,
        clipboard: true,
        print: true,
        refresh: true,
        columns: [
            {
                id: 'enteredDate',
                title: 'Date Recorded',
                width: 130,
                sortable: true,
                formatter: JLV.common.dateFormatter
            },
            {
                id: 'allergyName',
                title: 'Allergen',
                width: 100,
                sortable: true,
                formatter: function (value, model) {
                    if (value == undefined || value == null || value == "") {
                        value = "Allergen name was not supplied"
                    }
                    return JLV.common.linkFormatter(value, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Unified Medical <br>' +
                                'Language System (UMLS)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Unified Medical<br>' +
                        'Language System (UMLS)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                id: 'normDisplay',
                title: "Standardized Allergen",
                width: 200,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Unified Medical <br>' +
                                'Language System (UMLS)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Unified Medical<br>' +
                        'Language System (UMLS)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                id: 'reaction',
                title: 'Reaction',
                width: 100,
                sortable: true
            },
            {
                id: 'severity',
                title: "Severity",
                width: 100,
                sortable: true
            },
            {
                id: 'comment',
                title: 'Comments',
                width: 120,
                sortable: true
            },
            {
                title: 'Site',
                width: 50,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'allergyName',
            asc: true
        }
    }


});