//= require ./PatientWidgetWindow.js

JLV.PatientImmunizationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Immunizations',
        width: 1000,
        status: true,
        vlerDomain: true,
        filter: true,
        dateFilter: false,
        clipboard: true,
        print: true,
        refresh: true,
        gridHeaderHeight: 30,
        filters: ['vaccine', 'normDisplay'],
        columns: [
            {
                title: 'Vaccine Administered <br /> Date',
                id: 'adminDate',
                formatter: function (aDate, model) {
                    return model.get('dateTime');
                },
                sortable: true,
                width: 120
            },
            {
                title: 'Vaccine Administered <br /> Product Type',
                id: 'vaccine',
                width: 160,
                sortable: true,
                formatter: JLV.common.linkFormatter,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Vaccine<br>' +
                                'Administered Code<br>' +
                                '(CVX)</b><br>' +
                                'National Name: ' + normDisplay + '<br>' +
                                'National Code: ' + normCode;
                        }
                    }

                    return '<b>National Vaccine<br>' +
                        'Administered Code<br>' +
                        '(CVX)</b><br>' +
                        'National Name: none<br>' +
                        'National Code: none';
                }

            },
            {
                title: 'Standardized Vaccine <br/> Product Type',
                width: 180,
                sortable: true,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Vaccine<br>' +
                                'Administered Code<br>' +
                                '(CVX)</b><br>' +
                                'National Name: ' + normDisplay + '<br>' +
                                'National Code: ' + normCode;
                        }
                    }

                    return '<b>National Vaccine<br>' +
                        'Administered Code<br>' +
                        '(CVX)</b><br>' +
                        'National Name: none<br>' +
                        'National Code: none';
                }
            },
            {
                id: 'series',
                width: 80,
                title: 'Series',
                sortable: true
            },
            {
                id: 'result',
                width: 100,
                title: 'Immunization <br/> Result',
                sortable: true,
                formatter: function (value, model) {
                    if (model && (model.get('size'))) {
                        if (value == null || value == "") {
                            return model.get('size');
                        } else {
                            return String.format('{0}/{1}', model.get('size'), value);
                        }
                    }
                    return value;
                },
            },
            {
                title: 'Adverse Vaccine<br/>Reaction',
                id: 'reaction',
                width: 100,
                sortable: true
            },
            {
                title: 'Exemption /<br/>Refusal Reason',
                id: 'exemptionRefusalReason',
                width: 120,
                sortable: true
            },
            {
                title: 'Exemption /<br/>Refusal Date',
                id: 'exemptionRefusalDate',
                width: 120,
                sortable: true,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 50,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'adminDate',
            asc: false
        }
    },
    initialize: function (options) {
        this._super(options);

        //add banner that VA data is not available for DoD user
        if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {

            if (this._body) {
                $('<div class="jlv-immunizations-no-va-banner">')
                    .text('VA data is not complete and will not be displayed at this time.')
                    .appendTo(this._body);

            }


        }
    },
    toggleFilterPanel: function () {

        if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {
            if (this.get('filter')) {
                if (!this._filterPanel) {

                    var filterIds = this.get('filters');
                    var filters = _.toArray(_.pick(_.indexBy(this.get('columns'), 'id'), filterIds));

                    var data = this.get('data');

                    if (data) {

                        if (this.get('dateFilter')) {
                            if (data.startDate !== undefined && data.endDate !== undefined) {

                                this._filterPanel = new JLV.WidgetWindow.DateFilterPanel({
                                    window: this,
                                    startDate: data.startDate.toDate(),
                                    endDate: data.endDate.toDate(),
                                    filters: filters,
                                    data: data
                                });
                            }
                        }
                        else {

                            this._filterPanel = new JLV.WidgetWindow.FilterPanel({
                                window: this,
                                filters: filters,
                                data: data
                            });
                        }

                        if (this._filterPanel) {
                            this._filterPanel.$el.css('top', 26);
                            this._filterPanel.$el.prependTo(this._body);
                            this._filterPanel.$el.hide();
                        }
                    }
                }

                if (this._filterPanel) {
                    if (this._filterPanel.get('hidden')) {
                        this._filterPanel.set('hidden', false);
                        if (this.get('filter')) {
                            if (this._datagrid)
                                this._datagrid.$el.css('top', this._filterPanel.el.offsetTop + this._filterPanel.el.offsetHeight);
                        }
                        else {
                            if (this._datagrid) {
                                if (this._datagrid.el.offsetTop - this._filterPanel.el.offsetHeight < 0) {
                                    this._datagrid.$el.css('top', 26);
                                } else {
                                    this._datagrid.$el.css('top', this._filterPanel.el.offsetTop - this._filterPanel.el.offsetHeight);
                                }
                            }

                        }
                    }
                    else {
                        this._filterPanel.set('hidden', true);
                        if (this._datagrid)
                            this._datagrid.$el.css('top', 26);

                    }
                }
            }
        } else {
            this._super();

        }

    }
});
