//= require ./PatientWidgetWindow.js

JLV.PatientInpatientMedicationsWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Inpatient Medications',
        width: 1000,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        groupBy: 'orderId',
        groupFormatter: function (value) {
            if (value && (value !== 'null')) {
                return 'Order Number: &nbsp;' + value;
            } else {
                return 'Order Number: none';
            }
        },
        filters: ['drugName', 'normDisplay', 'taxonomy'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Order Number',
                id: 'orderId',
                sortable: true,
                width: 110
            },
            {
                title: 'Drug Name',
                id: 'drugName',
                width: 110,
                sortable: true,
                formatter: function (value, model) {
                    return JLV.common.linkFormatter(value, model, null, "report");
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized<br/>Drug Name',
                id: 'normDisplay',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                    }

                    return '<b>National Library of Medicine<br>' +
                        'RxNorm Concept Unique Identifiers</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Status',
                id: 'status',
                width: 65,
                sortable: true
            },
            {
                width: 110,
                title: 'Ordering HCP',
                sortable: true,
                id: 'providerName',
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 95,
                title: 'Ordering<br/>HCP Specialty',
                sortable: true,
                id: 'taxonomy',
                tooltip: function (row, model) {
                    try {
                        if (model && model.attributes.taxonomy) {
                            return _.template('<b>National Uniform Claim Committee -<br/>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: {{-taxonomy}}<br/>' +
                                'NUCC Taxonomy Code: {{-taxonomyCode}}'
                            )(model.toJSON());
                        }
                    } catch (e) {
                        console.info('template error');
                    }
                }
            },
            {
                title: 'Start Date',
                id: 'startDate',
                width: 90,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Stop Date',
                id: 'stopDate',
                width: 90,
                formatter: JLV.common.dateFormatter,
                sortable: true
            },
            {
                title: 'Schedule',
                id: 'sig',
                sortable: true,
                width: 100
            },
            {
                width: 60,
                title: 'Quantity',
                sortable: true,
                id: 'quantity'
            },
            {
                title: 'Site',
                id: 'site',
                width: 50,
                stretchable: false,
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'orderId',
            asc: false
        }
        , data: JLV.InpatientMedicationsCollection
    },
    initialize: function (options) {
        this._super(options);

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            if (sortInfo.column.id === 'orderId') {
                if (this._datagrid.get('groupBy')) {
                    if (sortInfo.asc) {
                        this.set('groupSorter', function (val1, val2) {
                            return (val1 >= val2) ? 1 : -1;
                        });
                    }
                    else {
                        this.set('groupSorter', function (val1, val2) {
                            return (val1 <= val2) ? 1 : -1;
                        });
                    }
                }
            }
        }, this));

    }
});