//= require ./PatientWidgetWindow.js


JLV.PatientInsuranceCollection = JLV.PatientDataCollection.extend({
    dodAdapterSourceReportField: 'planName',
    url: 'patientInsurances/list.json',
    model: JLV.PatientDataModel.extend({
        dateFields: ['effectiveDate', 'expirationDate', 'subscriberDOB'],
        parse: function (data) {

            if (data.codes) {
                for (var i in data.codes) {
                    if (data.codes[i].system) {
                        if (data.codes[i].system.toUpperCase().indexOf("X12") >= 0) {
                            data.normCode = data.codes[i].code;
                            data.normDisplay = data.codes[i].display;
                            break;
                        }
                    }
                }
            }

            var addr = "";
            if (data.companyAddressLine1) {
                addr += data.companyAddressLine1;
            }
            if (data.companyAddressLine2) {
                addr += ("<br/>" + data.companyAddressLine2);
            }

            if (data.companyAddressLine3) {
                addr += ("<br/>" + data.companyAddressLine3);
            }

            if (data.companyCity) {
                addr += ("<br/>" + data.companyCity);
            }

            if (data.companyStateProvince) {
                addr += (", " + data.companyStateProvince);
            }

            if (data.companyPostalCode) {
                addr += (" " + data.companyPostalCode);
            }

            data.companyAddress = addr;

            return this._super(data);
        }
    })
});


JLV.PatientInsuranceWidgetWindow = JLV.PatientWidgetWindow.extend({
    classID: 'jlv-insurance-widget',
    options: {
        name: 'Insurance Information',
        width: 1000,
        filter: false,
        dateFilter: false,
        vlerDomain: true,
        status: false,
        clipboard: true,
        print: true,
        refresh: true,
        fitColumns: false,
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Health Plan Type',
                id: 'insuranceType',
                sortable: true,
                width: 130,
                formatter: function (value, model) {
                    if (value == undefined || value == null || value == "") {
                        value = "Health Plan Type was not supplied"
                    }
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                }
            },
            {
                title: 'Health Plan Name',
                id: 'companyName',
                sortable: true,
                width: 140

            },
            {
                title: 'Standardized <br/> Insurance Type',
                id: 'normDisplay',
                width: 120,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>X12 Health Insurance Type' +
                                '</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>X12 Health Insurance Type' +
                        '</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Plan Effective Date',
                id: 'effectiveDate',
                sortable: true,
                width: 160,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Plan Expiration Date',
                id: 'expirationDate',
                sortable: true,
                width: 160,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            },
            {
                title: 'Group Number',
                id: 'groupNumber',
                sortable: true,
                width: 120
            },
            {
                title: 'Member ID',
                id: 'memberId',
                sortable: true,
                width: 120
            },
            {
                title: 'Subscriber ID',
                id: 'subscriberId',
                sortable: true,
                width: 120
            },
            {
                title: 'RxBIN',
                id: 'rxBin',
                sortable: true,
                width: 60
            },
            {
                title: 'RxPCN',
                id: 'rxPcn',
                sortable: true,
                width: 60
            },
            {
                title: 'Notes',
                id: 'textDescription',
                sortable: true,
                width: 120
            },
            {
                title: 'Comments',
                id: 'comment',
                sortable: true,
                width: 120
            },
            {
                title: 'Subscriber <br/> Date of Birth',
                id: 'subscriberDOB',
                sortable: true,
                width: 110,
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Subscriber&apos;s <br/> Relationship to Insurer',
                id: 'relationShip',
                sortable: true,
                width: 150
            },
            {
                title: 'Health Plan <br/> Mailing Address',
                id: 'companyAddress',
                sortable: true,
                width: 150
            },
            {
                title: 'Health Plan <br/> Contact Information',
                id: 'companyPhone',
                sortable: true,
                width: 160
            }
        ],
        defaultSort: {
            id: 'planName',
            asc: true
        },
        data: JLV.PatientInsuranceCollection
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    var details = '<table cellspacing="0" cellpadding="0">' +
                    '<tr><td style="font-weight:bold;">Plan Type:</td>' +
                    '<td>' +
                    model.get('insuranceType') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Plan Name:</td>' +
                    '<td>' +
                    model.get('planName') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Group Number:</td>' +
                    '<td>' +
                    model.get('groupNumber') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Rx BIN:</td>' +
                    '<td>' +
                    model.get('rxBin') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Rx PCN:</td>' +
                    '<td>' +
                    model.get('rxPcn') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Source:</td>' +
                    '<td>' +
                    model.get('site').moniker +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Member Id:</td>' +
                    '<td>' +
                    model.get('memberId') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Plan Address:</td>' +
                    '<td>' +
                    model.get('companyAddress') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Contact Info:</td>' +
                    '<td>' +
                    model.get('companyPhone') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Comments:</td>' +
                    '<td>' +
                    model.get('comment') +
                    '</td></tr>' +
                    '<tr><td style="font-weight:bold;">Notes:</td>' +
                    '<td>' +
                    model.get('textDescription') +
                    '</td></tr>' +
                    '</table><br/>';

                    details += '<h2" style="font-weight:bold;">Narrative:</h2><br/>'
                    details += response;

                    return details;
                }
            });

            dialog.center();
            dialog.open();
        }
    },
    initialize: function (options) {
        this._super(options);

        this._datagrid.on('onSort', _.bind(function (evt, sortInfo) {
            this.options.defaultSort = {
                asc: sortInfo.asc,
                id: sortInfo.column.id
            };

            this.$el.trigger('sortChange', [this.options.defaultSort]);
        }, this));

        var dg = this._datagrid;
        var self = this;
        this._datagrid.on('afterRender.click', function () {
            this.$el.find('.jlv-clickable-link').on('click', function () {
                var modelId = $(this).data('model-id'),
                    model = dg.get('source').get(modelId);

                self.itemClicked(model, $(this));
            });
        });

        this.$el.find('.jlv-show-all-btn').hide();

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFetchedNotification,
            action: _.bind(function () {
                this.vlerLoadMessage(JLV.common.DOD_VLER_LOADING_MSG);
                this.$el.find('.jlv-vler-loading').addClass('jlv-vler-loading-img');
                if (this._moreWindow) {
                    this._moreWindow.vlerLoadMessage(JLV.common.DOD_VLER_LOADING_MSG);
                    this._moreWindow.$el.find('.jlv-vler-loading').addClass('jlv-vler-loading-img');
                }
            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataRefreshNotification,
            action: _.bind(function () {

                if (this._moreWindow) {
                    this.loadData(_.bind(function () {
                        this._moreWindow.renderData();
                    }, this));
                    this._moreWindow.hideVlerLoadMessage();
                } else {
                    this.loadData();
                }

                this.hideVlerLoadMessage();

            }, this)
        });

        JLV.NotificationCenter.addObserverForNotification({
            observer: this,
            name: JLV.VlerDataFailNotification,
            action: _.bind(function () {
                this.vlerLoadMessage(JLV.common.DOD_VLER_NOT_AVAILABLE_MSG);
            }, this)
        });

    },
    loadData: function () {

        //load the data
        var data = this.get('data'),
            patient = this.get('patient');

        if (!data) {
            console.warn(this.get('name') + ' widget data not set.');
        }

        if (!patient) {
            console.warn(this.get('name') + ' widget patient not set.');
        }

        if (data &&
            patient) {

            if (!_.isFunction(data.doFetch)) {
                console.warn('Widget data sources must be a Backbone collection implementing a doFetch method.');
                return;
            }

            data.reset();

            this.set('title', this.get('name'));
            this.dodAdapterSourceReport = [];
            this.dodAdapterStatusRec = [];
            this._updatePageInfoDisplay();

            this._datagrid.showMessage('<div class="jlv-table-data-loading"></div>');

            var self = this;
            data.doFetch(patient, function () {

                    if (self._datagrid.get('rowCount') === 0) {
                        self._datagrid.showMessage("No Data");
                    }
                    else if (self.options.defaultSort !== null) {

                        var column = _.findWhere(self.options.columns, {id: self.options.defaultSort.id});
                        if (column) {
                            column.index = _.indexOf(self.options.columns, column);
                            var sortInfo = {
                                asc: self.options.defaultSort.asc,
                                column: column
                            };

                            self._datagrid.sortByColumn(sortInfo);
                            self._datagrid.render();


                        }
                        else {
                            self._datagrid.render();
                        }

                    }
                    else {
                        self._datagrid.render();

                    }
                    //update the widget title to reflect the number of records
                    self.set('title', self.get('name') + ' (' + self._datagrid.get('rowCount') + ')');

                    self._updatePageInfoDisplay();

                },
                // handle error
                function (collection, err) {

                    console.info(err)


                    self._datagrid.showMessage(_.template(
                            '<div class="server-error-msg" style="color:red;cursor:pointer;text-decoration:underline;">{{statusText}} ({{status}})</div>')(err)
                    );

                    self._datagrid.$el.find('.server-error-msg').click(function () {
                        var alertDialog = new AppKit.AlertDialog({
                            title: err.statusText,
                            message: err.responseText,
                            icon: 'icon-frown-o',
                            cancelButton: false,
                            height: 500
                        });

                        alertDialog.center();
                        alertDialog.set('visible', true);
                    });
                });
        }
    },
    renderData: function () {
        this.loadData();
    }
});

