//= require ./PatientWidgetWindow.js

JLV.PatientOrdersWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Orders',
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['type', 'normDisplay', 'taxonomy'],
        columns: [
            {
                title: 'Order Date',
                id: 'orderDate',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                },
                sortable: true,
                width: 100
            },
            {
                title: 'Order Number',
                id: 'orderId',
                sortable: true,
                width: 100
            },
            {
                title: 'Description',
                id: 'description',
                width: 100,
                sortable: true,
                formatter: function (value, model) {
                    if (model && model.get('recordDetailsToken')) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            normCodeType = model.get('normCodeType');

                        if (normCodeType && normCodeType.toUpperCase().indexOf('RXNORM') >= 0) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                        else if (normCodeType && normCodeType.toUpperCase().indexOf('LOINC') >= 0) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }

            },
            {
                title: 'Status / Priority',
                id: 'status',
                width: 80,
                sortable: true,
                formatter: function (value) {
                    if (value)
                        return value.toUpperCase();
                }
            },
            {
                title: 'Type',
                id: 'type',
                width: 80,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            normCodeType = model.get('normCodeType');

                        if (normCodeType && normCodeType.toUpperCase().indexOf('RXNORM') >= 0) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                        else if (normCodeType && normCodeType.toUpperCase().indexOf('LOINC') >= 0) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                width: 120,
                title: 'Standardized Type',
                sortable: true,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay'),
                            normCodeType = model.get('normCodeType');

                        if (normCodeType && normCodeType.toUpperCase().indexOf('RXNORM') >= 0) {
                            return '<b>National Library of Medicine<br>' +
                                'RxNorm Concept Unique Identifiers</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }
                        else if (normCodeType && normCodeType.toUpperCase().indexOf('LOINC') >= 0) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }

                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider Specialty',
                id: 'taxonomy',
                width: 120,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }

                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'orderDate',
            asc: false
        }
    }
});