//= require ./PatientWidgetWindow.js

JLV.PatientProblemListWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Problem List',
        filter: true,
        status: true,
        vlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        width: 900,
        filters: ['description', 'normDisplay', 'status'],
        columns: [
            {
                title: 'ICD',
                width: 52,
                stretchable: false,
                id: 'icdCode'
            },
            {
                title: 'Onset',
                width: 70,
                stretchable: false,
                id: 'onsetDate',
                formatter: function (aDate, model) {
                    return model.get('dateTime');
                }
            },
            {
                title: 'Description',
                width: 100,
                id: 'description',
                formatter: function (value, model) {
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'Standardized Description',
                width: 160,
                id: 'normDisplay',
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Systematized Nomenclature<br>' +
                                'of Medicine - Clinical Terms<br>' +
                                '(SNOMED CT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Systematized Nomenclature<br>' +
                        'of Medicine - Clinical Terms<br>' +
                        '(SNOMED CT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none'
                }
            },
            {
                title: 'Updated',
                width: 100,
                stretchable: false,
                id: 'lastUpdated',
                formatter: JLV.common.dateFormatter
            },
            {
                title: 'Severity',
                width: 100,
                sortable: true,
                id: 'priority'
            },
            {
                title: 'Status',
                width: 60,
                stretchable: false,
                id: 'status'
            },
            {
                title: 'Site',
                width: 50,
                stretchable: false,
                id: 'site',
                sorter: JLV.common.siteSorter,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'lastUpdated',
            asc: false
        },
        defaultFilter: {
            id: 'status',
            value: 'ACTIVE'
        }
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.open();
        }
    }
});