//= require ./PatientWidgetWindow.js

JLV.PatientProceduresWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Procedures',
        width: 900,
        gridHeaderHeight: 30,
        filter: true,
        dateFilter: true,
        status: true,
        vlerDomain: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['taxonomy'],
        columns: [
            {
                title: 'Date/Time',
                id: 'date',
                formatter: function (value, model) {
                    return JLV.common.dateFormatter(value, model, 'MMM DD, YYYY HH:mm');
                },
                sortable: true,
                width: 100
            },
            {
                width: 100,
                title: 'Location',
                sortable: true,
                id: 'location'
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider Specialty',
                id: 'taxonomy',
                width: 120,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 90,
                title: 'CPT Code',
                sortable: true,
                id: 'normCode'
            },
            {
                title: 'Procedure <br/> Description',
                sortable: true,
                id: 'description',
                formatter: function (value, model) {
                    if (model && (model.get('recordDetailsToken') || model.get('narrativeId'))) {
                        return JLV.common.linkFormatter(value, model);
                    }
                    else {
                        return value;
                    }
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Current Procedural Terminology<br>' +
                                '(CPT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Current Procedural Terminology<br>' +
                        '(CPT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized <br/> Procedure Description',
                sortable: true,
                id: 'normDisplay',
                width: 160,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Current Procedural Terminology<br>' +
                                '(CPT)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Current Procedural Terminology<br>' +
                        '(CPT)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'date',
            asc: false
        }
    },
    itemClicked: function (model) {

        if (model.get('narrativeId')) {
            var dialog = new JLV.DetailsWindow({
                title: String.format("Narrative ~ {0}",
                    JLV.common.formatSiteMonikerDisplay(model.get('site'))),
                url: 'patientDODVLER/getNarrative',
                queryParams: {
                    narrativeId: model.get('narrativeId')
                },
                renderer: function (response) {
                    return response;
                }
            });

            dialog.center();
            dialog.set('visible', true);
        }
    },
    initialize: function (options) {

        //add coding status for dod user
        if (JLV.ActiveProvider.agency === JLV.common.AGENCY_DOD) {
            var columns = this.options.columns;
            if (_.findIndex(columns, {id: 'codingStatus'}) < 0) {
                columns.splice(5, 0, {
                    title: 'Coding Status',
                    id: 'codingStatus',
                    sortable: true,
                    width: 110
                });
            }
        }

        this._super(options);

    }
});