//= require ./PatientWidgetWindow.js

JLV.PatientProgressNotesWidgetWindow = JLV.PatientWidgetWindow.extend({
    options: {
        name: 'Progress Notes',
        width: 900,
        minWidth: 840,
        filter: true,
        dateFilter: true,
        status: true,
        clipboard: true,
        print: true,
        refresh: true,
        filters: ['noteTitle', 'normDisplay', 'providerName', 'taxonomy'],
        gridHeaderHeight: 30,
        columns: [
            {
                title: 'Date',
                id: 'noteDate',
                formatter: JLV.common.dateFormatter,
                sortable: true,
                width: 100
            },
            {
                title: 'Document Type/Title',
                id: 'noteTitle',
                width: 160,
                sortable: true,
                formatter: function (value, model) {
                    if (!model || value === 'Connection unavailable.' || !model.get('recordDetailsToken')) {
                        return value;
                    }

                    var noteTitle = value;
                    if (!noteTitle || noteTitle.length < 1) {
                        noteTitle = JLV.common.NOTE_TITLE_UNAVAILABLE_TXT;
                    }

                    if (model.get('sensitive')) {
                        noteTitle = noteTitle.replace(/\n/g, '<br />');
                    }

                    return JLV.common.linkFormatter(noteTitle, model);
                },
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Standardized <br/>Document Type',
                id: 'normDisplay',
                width: 200,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var normCode = model.get('normCode'),
                            normDisplay = model.get('normDisplay');

                        if (normCode) {
                            return '<b>Logical Observation Identifiers<br>' +
                                'Names and Codes<br>' +
                                '(LOINC)</b><br>' +
                                'Standardized Name: ' + normDisplay + '<br>' +
                                'Standardized Code: ' + normCode;
                        }

                    }

                    return '<b>Logical Observation Identifiers<br>' +
                        'Names and Codes<br>' +
                        '(LOINC)</b><br>' +
                        'Standardized Name: none<br>' +
                        'Standardized Code: none';
                }
            },
            {
                title: 'Provider',
                id: 'providerName',
                width: 110,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                title: 'Provider <br/> Specialty',
                id: 'taxonomy',
                width: 100,
                sortable: true,
                tooltip: function (row, model) {
                    if (model) {
                        var taxonomyCode = model.get('taxonomyCode'),
                            taxonomy = model.get('taxonomy');

                        if (taxonomyCode && taxonomy) {
                            return '<b>National Uniform Claim Committee -<br>' +
                                'Taxonomy<br>' +
                                '(NUCC)</b><br>' +
                                'NUCC Taxonomy Description: ' + taxonomy + '<br>' +
                                'NUCC Taxonomy Code: ' + taxonomyCode;
                        }

                    }
                    return '<b>National Uniform Claim Committee -<br>' +
                        'Taxonomy<br>' +
                        '(NUCC)</b><br>' +
                        'NUCC Taxonomy Description: none<br>' +
                        'NUCC Taxonomy Code: none';
                }
            },
            {
                width: 150,
                title: 'Clinic',
                sortable: true,
                id: 'location'
            },
            {
                title: 'Site',
                id: 'site',
                sorter: JLV.common.siteSorter,
                width: 60,
                formatter: JLV.common.formatSiteMonikerDisplay
            }
        ],
        defaultSort: {
            id: 'noteDate',
            asc: false
        }
    }
});