

JLV.ProviderJLVAdministrativeModule = JLV.ProviderWidget.extend({
    initialize: function (options) {
        this._super(options);

        var b = $('<div class="jlv-jam-link-button">').appendTo(this._body);


        $('<a href="#">Community Care Provider Provisioning</a>')
            .on('click', function () {
                JLV.ProviderJLVAdministrativeModule.UserProvisioningController.show();

            })
            .appendTo(b)


    },
    options: {
        widgetId: 'prov_jam_widget',
        name: 'Community Care Provider Management',
        plus: false,
        dateFilter: false,
        status: false,
        settings: false,
        clipboard: false,
        print: false,
        refresh: false
    }
});


JLV.registerJLVWidget('prov_jam_widget', JLV.ProviderJLVAdministrativeModule);


/* JAM UI controllers */


JLV.ProviderJLVAdministrativeModule.PatientSearchResults = Backbone.Collection.extend({
    url: 'community/patientSearch.json',
    model: Backbone.Model.extend({
        parse: function (data) {
            if (data.name) {
                data.name = data.name.replace(/,/g, ', ');
            }
            return data;
        }
    }),
    doFetch: function (patientSearchString, callback, error) {
        this.fetch({
            data: {
                patientSearchString: patientSearchString
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});


JLV.ProviderJLVAdministrativeModule.PatientConsultResults = Backbone.Collection.extend({
    url: 'community/patientConsults.json',
    model: Backbone.Model.extend({
        parse: function (data) {
            if (data.provider && data.provider.name) {
                data.providerName = data.provider.name.replace(/,/g, ', ');
            }

            if (data.requestDate) {
                data.requestDate = new Date(data.requestDate);
            }

            return data;
        }
    }),
    doFetch: function (patientIen, callback, error) {
        this.fetch({
            data: {
                ien: patientIen
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});


JLV.ProviderJLVAdministrativeModule.PatientSearchController = {
    patientSearchResults: new JLV.ProviderJLVAdministrativeModule.PatientSearchResults(),
    patientConsultResults: new JLV.ProviderJLVAdministrativeModule.PatientConsultResults(),
    init: function () {

        var self = this;
        this.searchField.on('valueChanged', function (evt, value) {
            self.searchBtn.set('enabled', value.trim().length > 0);
        });


        this.patientSearchTable.on('afterRender', function () {
            var checkboxes = this.$el.find('.jlv-jam-patient-check');
            checkboxes.on('click', function () {

                self.selectedConsult = null;
                self.assignBtn.set('enabled', false);

                checkboxes.not($(this)).prop('checked', false);

                if ($(this).prop('checked')) {
                    var modelId = $(this).data('model');
                    self.selectedPatient = self.patientSearchResults.get(modelId);
                    if (self.selectedPatient) {
                        self.fetchConsults();
                    }
                }
                else {
                    self.selectedPatient = null;
                    self.assignBtn.set('enabled', false);
                }
            });
        });


        this.consultTable.on('afterRender', function () {
            var checkboxes = this.$el.find('.jlv-jam-consult-check');
            checkboxes.on('click', function () {

                checkboxes.not($(this)).prop('checked', false);

                if ($(this).prop('checked')) {
                    var modelId = $(this).data('model');
                    self.selectedConsult = self.patientConsultResults.get(modelId);
                    if (self.selectedConsult) {
                        var requestDate = moment(self.selectedConsult.get('requestDate'));
                        self.startDate.set('date', requestDate.subtract(3, 'months').toDate());
                        self.assignBtn.set('enabled', true);
                    }
                }
                else {
                    self.selectedConsult = null;
                    self.assignBtn.set('enabled', false);
                }
            });

        });
    },
    fetchConsults: function () {

        if (this.selectedPatient) {

            var patIens = this.selectedPatient.get('patientIens');

            if (patIens.length > 0) {
                this.consultTable.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');
                var self = this;
                this.patientConsultResults.doFetch(JSON.stringify(
                        patIens[0]
                    ),
                    function (collection) {

                        if (collection.length === 0) {
                            return self.consultTable.showMessage('No Consults');
                        }
                        else {
                            self.consultTable.render();
                        }
                    });
            }
        }
    },
    onDialogOpen: function () {

        this.dialog.$el.find('.appkit-textfield').val('');
        this.searchBtn.set('enabled', false);

        setTimeout(_.bind(function () {
            this.searchField.$el.focus();
        }, this), 0);

        this.selectedPatient = null;
        this.assignBtn.set('enabled', false);

        this.patientSearchResults.reset();
        this.patientConsultResults.reset();

    },
    show: function () {
        this.dialog.center();
        this.dialog.open();
    },
    close: function () {
        this.dialog.close();
    },
    search: function () {

        var searchString = this.searchField.get('value').trim();

        if (searchString.length > 0) {
            this.patientSearchTable.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');
            this.patientSearchResults.doFetch(searchString, _.bind(function (collection) {
                if (collection.length === 0) {
                    return this.patientSearchTable.showMessage('No Results');
                }
                else {
                    this.patientSearchTable.render();
                }
            }, this));
        }
    },
    assign: function () {
        if (this.selectedPatient) {
            JLV.ProviderJLVAdministrativeModule.UserProvisioningController.assignPatient(this.selectedPatient, this.selectedConsult, this.startDate.get('date'));
            this.close();
        }
    },
    nameFormatter: function (value, model) {
        return String.format('<div><input data-model="{0}" type="checkbox" class="jlv-jam-patient-check"/><div style="display:inline-block;margin-top:3px;">{1}</div></div>', model.cid, value);
    },
    nameSorter: function (value1, value2) {
        return value1 > value2 ? 1 : -1;
    },
    consultNoFormatter: function (value, model) {
        return String.format('<div><input data-model="{0}" type="checkbox" class="jlv-jam-consult-check"/><div style="display:inline-block;margin-top:3px;">{1}</div></div>', model.cid, value);
    }
};




JLV.ProviderJLVAdministrativeModule.ProviderSearchResults = Backbone.Collection.extend({
    url: 'community/providerSearch.json',
    model: Backbone.Model.extend({
        parse: function (data) {
            if (data.name) {
                data.name = data.name.replace(/,/g, ', ');
            }

            if (data.dataTypeAccess) {
                data.dataTypeAccess = JSON.parse(data.dataTypeAccess);
            }

            return data;
        }
    }),
    doFetch: function (providerName, callback, error) {
        this.fetch({
            data: {
                providerName: providerName
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});


JLV.ProviderJLVAdministrativeModule.AssignedPatientsCollection = Backbone.Collection.extend({
    url: 'community/assignedPatients.json',
    model: Backbone.Model.extend({
        parse: function (data) {
            if (data.name) {
                data.name = data.name.replace(/,/g, ', ');
            }

            if (data.dateFilter && data.dateFilter.time) {
                data.dateFilter = new Date(data.dateFilter.time);
            }

            return data;
        }
    }),
    doFetch: function (providerId, callback, error) {
        this.fetch({
            data: {
                providerId: providerId
            },
            reset: true,
            type: 'POST',
            success: callback,
            error: error
        });
    }
});




JLV.ProviderJLVAdministrativeModule.ProviderInformationPanelController = {
    init: function () {
        this.phoneField.$el.mask('(999) 999-9999');

        this.emailField.on('valueChanged', function (evt, value) {
            $('#JAMProviderUsernameField').text(value);
        });


    },
    show: function () {
        this.dialog.center();
        this.dialog.open();
    },
    close: function () {
        this.dialog.close();
    },
    onDialogOpen: function () {
        //reset the fields
        this.dialog.$el.find('.appkit-textfield').val('');

        $('#JAMProviderAccountTypeOutside').prop('checked', true);
        $('#JAMProviderVAAccess').prop('checked', true);
        this.passwordField.$el.parents('tr').show();
        this.generatePassword();

        this.editMode = false;
    },
    edit: function (providerModel) {

        this.show();

        this.editMode = true;

        var names = providerModel.get('name').split(',');

        if (names.length > 1)
            this.firstNameField.set('value', names[1].trim());
        if (names.length > 0)
            this.lastNameField.set('value', names[0].trim());

        this.npiField.set('value', providerModel.get('npi').trim());
        this.phoneField.set('value', providerModel.get('phone').trim());
        this.emailField.set('value', providerModel.get('email').trim());
        this.organizationField.set('value', providerModel.get('organization').trim());

        this.passwordField.$el.parents('tr').hide();
        this.passwordField.set('resetOn', false);
        this.passwordResetBtn.$el.parents('tr').show();


        $('#JAMProviderUsernameField').text(providerModel.get('username').trim())

        var dataTypeAccess = providerModel.get('dataTypeAccess');

        if (dataTypeAccess.length == 4) {
            $('#JAMProviderVAAccess').prop('checked', dataTypeAccess[0]);
            $('#JAMProviderDoDAccess').prop('checked', dataTypeAccess[1]);
            $('#JAMProviderVAVlerAccess').prop('checked', dataTypeAccess[2]);
            $('#JAMProviderDoDVlerAccess').prop('checked', dataTypeAccess[3]);
        }
    },
    confirm: function () {

        if (!this.editMode) {

            if (this.validate()) {

                $.post('community/usernameCheck.json', {
                    name: _.escape(this.emailField.get('value'))
                }, _.bind(function (response) {
                    if (!response.exists) {

                        var dataTypeAccess = [
                            $('#JAMProviderVAAccess').prop('checked'),
                            $('#JAMProviderDoDAccess').prop('checked'),
                            $('#JAMProviderVAVlerAccess').prop('checked'),
                            $('#JAMProviderDoDVlerAccess').prop('checked')
                        ];

                        this.close();

                        $.post('community/addProvider.json', {

                                name: _.escape(this.lastNameField.get('value') + "," + this.firstNameField.get('value')),
                                email: _.escape(this.emailField.get('value')),
                                npi: _.escape(this.npiField.get('value')),
                                organization: _.escape(this.organizationField.get('value')),
                                dataTypeAccess: JSON.stringify(dataTypeAccess),
                                phone: _.escape(this.phoneField.get('value')),
                                username: _.escape(this.emailField.get('value')),
                                password: _.escape(this.passwordField.get('value'))

                            },
                            function () {
                                AppKit.AlertDialog.notify('Provider created.', 'icon-alert-ok');
                            });

                    }
                    else {
                        this.emailField.validate(function () {
                            return "Email/username already exits.";
                        });
                    }

                }, this));
            }
        }
        else {

            var dataTypeAccess = [
                $('#JAMProviderVAAccess').prop('checked'),
                $('#JAMProviderDoDAccess').prop('checked'),
                $('#JAMProviderVAVlerAccess').prop('checked'),
                $('#JAMProviderDoDVlerAccess').prop('checked')
            ];

            this.close();

            $.post('community/updateProvider.json', {
                    id: JLV.ProviderJLVAdministrativeModule.UserProvisioningController.editProvider.get('id'),
                    name: _.escape(this.lastNameField.get('value') + "," + this.firstNameField.get('value')),
                    email: _.escape(this.emailField.get('value')),
                    npi: _.escape(this.npiField.get('value')),
                    organization: _.escape(this.organizationField.get('value')),
                    dataTypeAccess: JSON.stringify(dataTypeAccess),
                    phone: _.escape(this.phoneField.get('value')),
                    username: _.escape(this.emailField.get('value')),
                    resetPassword: (this.passwordField.get('resetOn') === true),
                    password: _.escape(this.passwordField.get('value'))

                },
                _.bind(function () {

                    JLV.ProviderJLVAdministrativeModule.UserProvisioningController.editProvider.set({
                        name: _.escape(this.lastNameField.get('value') + "," + this.firstNameField.get('value')),
                        email: _.escape(this.emailField.get('value')),
                        npi: _.escape(this.npiField.get('value')),
                        organization: _.escape(this.organizationField.get('value')),
                        dataTypeAccess: JSON.stringify(dataTypeAccess),
                        phone: _.escape(this.phoneField.get('value')),
                        username: _.escape(this.emailField.get('value')),
                        password: _.escape(this.passwordField.get('value'))

                    });


                    AppKit.AlertDialog.notify('Provider updated.', 'icon-alert-ok');
                }, this));
        }
    },
    validate: function () {

        var valid = true;

        valid = this.firstNameField.validate(function (value) {
            if (value.trim().length === 0) {
                return "Required Field"
            }
        });

        valid = (this.lastNameField.validate(function (value) {
            if (value.trim().length === 0) {
                return "Required Field"
            }
        }) && valid);

        valid = (this.phoneField.validate(function (value) {
            if (value.trim().length === 0) {
                return "Required Field"
            }
        }) && valid);

        valid = (this.emailField.validate(function (value) {
            if (value.trim().length === 0) {
                return "Required Field"
            }
        }) && valid);


        valid = (this.passwordField.validate(function (value) {
            if (value.trim().length === 0) {
                return "Required Field"
            }
        }) && valid);

        return valid;
    },
    generatePassword: function () {

        $.post('community/generateDefaultPassword', {

        }, _.bind(function (response) {
            this.passwordField.set('value', response.pwd);
        }, this));

    },
    resetPassword: function () {

        this.passwordField.$el.parents('tr').show();
        this.passwordField.set('resetOn', true);
        this.passwordResetBtn.$el.parents('tr').hide();

    }

};


JLV.ProviderJLVAdministrativeModule.UserProvisioningController = {
    providerSearchResults: new JLV.ProviderJLVAdministrativeModule.ProviderSearchResults(),
    assignedPatients: new JLV.ProviderJLVAdministrativeModule.AssignedPatientsCollection(),
    init: function () {

        var self = this;

        $('#JAMEditProviderProfileLink').on('click', function () {
            if (self.editProvider)
                JLV.ProviderJLVAdministrativeModule.ProviderInformationPanelController.edit(self.editProvider);
        });

        this.providerSearchTable.on('afterRender', function (evt) {
            this.$el.find('.jlv-jam-provider-search-remove-entry a').on('click', function () {
                var providerId = "" + $(this).data('providerid');
                AppKit.AlertDialog.confirm('Remove Provider', "Do you want to remove the provider form the list of authorized providers? " +
                "This will permanently remove the provider's account from the system.", function (yes) {
                    if (yes) {
                        self.remove(providerId);
                    }
                });
            });

            this.$el.find('.jlv-jam-provider-search-edit-entry a').on('click', function () {
                var providerId = '' + $(this).data('providerid');
                self.editProvider = self.providerSearchResults.findWhere({id: providerId});
                $('#JAMProviderName').text(self.editProvider.get('name'));
                self.showProviderPatientList();
                self.fetchAssignedPatients();
            });
        });

        this.assignedPatientsTable.set('filter', function (value) {
            return value.get('assigned');
        });

        this.assignedPatientsTable.on('afterRender', function () {
            this.$el.find('.jlv-jam-usage-link').on('click', function () {
                var modelid = $(this).data('modelid');
                var model = self.assignedPatients.get(modelid);
                if (model) {
                    var usage = model.get('usage');

                    if (usage && usage.length > 0) {
                        var msg = '<table>',
                            count = usage.length,
                            i = 0;

                        for (; i < count; i++) {
                            msg += String.format('<tr><td>{0}</td></tr>', usage[i]);
                        }

                        AppKit.AlertDialog.info("JLV Usage", msg, null, {height: 400});
                    }
                }
            });
        });

        this.prevAssignedPatientsTable.set('filter', function (value) {
            return !value.get('assigned');
        });

        this.prevAssignedPatientsTable.on('afterRender', function () {
            this.$el.find('.jlv-jam-usage-link').on('click', function () {
                var modelid = $(this).data('modelid');
                var model = self.assignedPatients.get(modelid);
                if (model) {
                    var usage = model.get('usage');

                    if (usage && usage.length > 0) {
                        var msg = '<table>',
                            count = usage.length,
                            i = 0;


                        for (; i < count; i++) {
                            msg += String.format('<tr><td>{0}</td></tr>', usage[i]);
                        }

                        AppKit.AlertDialog.info("JLV Usage", msg, null, {height: 400});
                    }
                }
            });

        });

        this.searchField.on('valueChanged', function (evt, value) {
            self.searchBtn.set('enabled', value.trim().length > 0);
        });

        $('#JAMAssignNewPatientButton').on('click', function () {
            JLV.ProviderJLVAdministrativeModule.PatientSearchController.show();
        });

        $('#JAMUnassignPatientButton').on('click', function () {
            self.unassignPatients();
        });

        JLV.NotificationCenter.addObserverForNotification({
            name: 'closeNotification',
            observer: this,
            action: 'close'
        });


    },
    show: function () {
        this.dialog.center();
        this.dialog.open();
    },
    close: function () {
        this.dialog.close();
    },
    onDialogOpen: function () {

        this.providerSearchTable.$el.find('a').off('click');

        this.providerSearchResults.reset();
        this.assignedPatients.reset();

        this.providerSearchTable.showMessage("Search Results");
        this.showProviderSearch();
        this.searchField.set('value', '');

        setTimeout(_.bind(function () {
            this.searchField.$el.focus();
        }, this), 0);
    },
    showProviderSearch: function () {
        $('#JAMProviderPatientList').hide();
        $('#JAMProviderSearch').show();
    },
    showProviderPatientList: function () {
        $('#JAMProviderSearch').hide();
        $('#JAMProviderPatientList').show();
    },
    nameFormatter: function (value, model) {
        return String.format('<div>{0} <div style="float:right;margin-right:10px;"><span class="jlv-jam-provider-search-remove-entry"><a data-providerId="{1}" href="#">{2}</a></span><span class="jlv-jam-provider-search-edit-entry"><a data-providerId="{1}" href="#">{3}</a></span></div></div>',
            value, model.get('id'), "Remove", "Edit")
    },
    patientNameFormatter: function (value, model) {
        return String.format('<div><input data-model="{0}" type="checkbox" class="jlv-jam-patient-check"/><div style="display:inline-block;margin-top:3px;">{1}</div></div>', model.cid, value);
    },
    usageFormatter: function (value, model) {
        return String.format('<div style="padding:2px 2px 0 2px;"><a class="jlv-jam-usage-link" data-modelid="{0}" href="#">{1}</a></div>', model.cid, value.length);
    },
    nameSorter: function (value1, value2) {
        return value1 > value2 ? 1 : -1;
    },
    search: function () {

        var providerNameQuery = this.searchField.get('value').trim();

        if (providerNameQuery.length > 0) {
            this.providerSearchTable.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');

            this.providerSearchResults.doFetch(providerNameQuery, _.bind(function (collection) {
                if (collection.length === 0) {
                    return this.providerSearchTable.showMessage('No Results');
                }
                else {
                    this.providerSearchTable.render();
                }
            }, this));
        }

    },
    remove: function (providerId) {

        $.post('community/deleteProvider.json', {
                providerId: providerId
            },
            _.bind(function () {
                this.providerSearchResults.remove({id: providerId});
                this.providerSearchTable.render();
                if (this.providerSearchResults.length === 0) {
                    return this.providerSearchTable.showMessage('No Results');
                }
            }, this));

    },
    assignPatient: function (patient, consult, startDate) {

        if (patient && this.editProvider && consult) {
            var self = this;
            $.post('community/assignPatient.json', {
                patient: JSON.stringify(patient),
                consult: JSON.stringify(consult),
                startDate: moment(startDate).format("M/D/YY"),
                providerId: this.editProvider.get('id')
            }, function () {
                self.fetchAssignedPatients();
            });
        }
    },
    fetchAssignedPatients: function () {

        if (this.editProvider) {
            this.assignedPatientsTable.showMessage('<div class="jlv-table-data-loading" alt="loading indicator"></div>');
            this.assignedPatients.doFetch(this.editProvider.get('id'), _.bind(function (collection) {

                this.assignedPatientsTable.render();
                this.prevAssignedPatientsTable.render();

                if (this.assignedPatientsTable.get('rowCount') === 0) {
                    this.assignedPatientsTable.showMessage('No Patients Assigned.');
                }

                if (this.prevAssignedPatientsTable.get('rowCount') === 0) {
                    this.prevAssignedPatientsTable.showMessage('No Previously Assigned Patients.');
                }

            }, this));

        }
    },
    unassignPatients: function () {

        if (this.editProvider) {

            var patients = [];
            var self = this;

            //get the checked patients
            this.assignedPatientsTable.$el.find('.jlv-jam-patient-check').each(function (idx, el) {

                if ($(el).prop('checked')) {
                    var modelId = $(el).data('model');
                    var patient = self.assignedPatients.get(modelId);
                    if (patient) {
                        patients.push(patient.get('id'));
                    }
                }
            });

            if (patients.length > 0) {
                $.post('community/deleteAssignedPatients.json', {
                    patients: JSON.stringify(patients)
                }, function () {
                    self.fetchAssignedPatients();
                });
            }
        }
    }

};


AppKit.launch(function () {
    JLV.ProviderJLVAdministrativeModule.ProviderInformationPanelController.init();
    JLV.ProviderJLVAdministrativeModule.UserProvisioningController.init();
    JLV.ProviderJLVAdministrativeModule.PatientSearchController.init();
});