/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUISite
import gov.va.med.DNS   .common.Utils
import grails.converters.JSON
import grails.converters.XML

class BaseController {

    def appConfig
    def tokenUtils

    protected sendResponse(data) {
        withFormat
                {
                    json { render data as JSON }
                    xml { render data as XML }
                }
    }

    /**
     * Date Range Helper method will provide controller with valid start and end dates if
     * they were not passed in by request. Start and end dates are placed into the request object.
     */
    protected void dateRangeCheck() {
        String startDateParam = params[HttpKeys.START_DATE];
        String endDateParam = params[HttpKeys.END_DATE];

        //if no start date param, set default to T-3 months
        if (startDateParam == null || startDateParam.length() < 2) {
            Calendar startCal = Calendar.getInstance();
            startCal.add(Calendar.MONTH, -3);
            startDateParam = Utils.formatDate(startCal.getTime());
        }

        //if no end date param, set default to today's date
        if (endDateParam == null || endDateParam.length() < 2) {
            Calendar endCal = Calendar.getInstance();
            endDateParam = Utils.formatDate(endCal.getTime());
        }

        //push start and end date params into request object for controller use
        request[HttpKeys.START_DATE] = startDateParam;
        request[HttpKeys.END_DATE] = endDateParam;
    }

    protected generateRecordDetailsToken(String patToken, String recordControllerName, Map recordQuery,
                                         GUISite recordSite, String recordSourceProtocol) {
        return generateRecordDetailsToken(patToken, recordControllerName, recordQuery, recordSite, recordSourceProtocol, false)
    }

    protected generateRecordDetailsToken(String patToken, String recordControllerName,
                                         Map recordQuery, GUISite recordSite,
                                         String recordSourceProtocol, boolean isSessionCached) {
        return tokenUtils.generateRecordDetailsToken(
                session.appUser.securityKey, patToken,
                recordControllerName, recordQuery,
                recordSite, recordSourceProtocol,
                isSessionCached, session.appUser.getCacheCounter());
    }

    protected getCachedRecordKey(recordDetailsToken) {
        Map recordParamsMap = tokenUtils.decryptTokenToMap(session.appUser.securityKey, recordDetailsToken)

        return recordParamsMap.get(HttpKeys.RECORD_CACHE_KEY)
    }

    protected GUISite getStatusReportSite() {
        GUISite statusSite = new GUISite();
        statusSite.setMoniker("DODADAPTER_SOURCE_STATUS_REPORT");
        statusSite.setName("DODADAPTER_SOURCE_STATUS_REPORT");
        statusSite.setAgency("DOD");
        statusSite.setSiteCode("DOD");

        return statusSite;
    }
}
