package jlv

import gov.va.med.guibeans.GUIAssignedPatient
import gov.va.med.guibeans.GUIKeyValuePair
import gov.va.med.guibeans.GUIOutsideProvider
import gov.va.med.guibeans.GUIPatient
import gov.va.med.DNS   .common.Utils
import grails.converters.JSON

class CommunityController extends BaseController {

    def communityDao;

    static allowedMethods = [patientSearch   : 'POST',
                             providerSearch  : 'POST',
                             addProvider     : 'POST',
                             deleteProvider  : 'POST',
                             assignedPatients: 'POST',
                             patientConsults: 'POST',
                             updateProvider : 'POST',
                             usernameCheck  : 'POST'
    ]


    def patientSearch = {
        def results = communityDao.lookupPatient(session.appUser, params.patientSearchString)
        sendResponse(results)
    }


    def providerSearch = {
        def results = communityDao.lookupOutsideProvider(session.appUser, params.providerName)
        sendResponse(results)
    }

    def addProvider = {

        def provider = new GUIOutsideProvider();

        provider.setName(params.name);
        provider.setEmail(params.email)
        provider.setNpi(params.npi)
        provider.setOrganization(params.organization)
        provider.setDataTypeAccess(params.dataTypeAccess)
        provider.setPhone(params.phone)
        provider.setUsername(params.username);
        provider.setDataTypeAccess(params.dataTypeAccess)

        String hashedPassword = params.password.encodeAsMD5();
        provider.setPassword(hashedPassword)

        communityDao.addOutsideProvider(session.appUser, provider)

        sendResponse([message: "OK"])

    }

    def deleteProvider = {

        communityDao.deleteOutsideProvider(session.appUser, params.providerId);
        sendResponse([message: "OK"])
    }

    def assignedPatients = {

        def providerId = params.providerId
        if (session.guiOutsideProvider != null) {
            providerId = session.guiOutsideProvider.getId();
        }

        def results = communityDao.getAssignedPatientList(session.appUser, providerId, session.appUser.securityKey)
        println results
        sendResponse(results)
    }

    def assignPatient = {

        def rawPatient = JSON.parse(params.patient)
        def rawConsult = JSON.parse(params.consult)
        def providerId = params.providerId
        def startDate = Utils.getCal(params.startDate)

        def guiAssignedPatient = new GUIAssignedPatient();

        guiAssignedPatient.setAssigned(true)
        guiAssignedPatient.setName(rawPatient['name'])
        guiAssignedPatient.setOutsideProviderId(providerId)
        guiAssignedPatient.setStatus(rawPatient['status'])
        guiAssignedPatient.setConsultNo(rawConsult['consultId'])
        guiAssignedPatient.setConsultName(rawConsult['service'])
        guiAssignedPatient.setProvider(rawConsult['providerName'])
        guiAssignedPatient.setStatus(rawConsult['outStatus'])

        def iens = rawPatient['patientIens']

        if (iens.size() > 0) {
            def ien = iens[0]['value']
            guiAssignedPatient.setIen("" + ien)
        }

        def sites = rawPatient['vistaSites']
        if (sites.size() > 0) {
            def siteId = sites[0]['siteCode']
            guiAssignedPatient.setSiteId("" + siteId)
        }

        guiAssignedPatient.setDateFilter(startDate);

        communityDao.addAssignedPatient(session.appUser, startDate, guiAssignedPatient)

        sendResponse([message: "OK"])

    }

    def deleteAssignedPatients = {

        def patientIds = JSON.parse(params.patients)
        patientIds.each { patientId ->
            communityDao.deleteAssignedPatient(session.appUser, patientId)
        }

        sendResponse([message: "OK"])

    }

    def patientConsults = {

        def ien = JSON.parse(params.ien);

        def patient = new GUIPatient()

        def guiKeyValuePair = new GUIKeyValuePair();

        guiKeyValuePair.setKey(ien['key'])
        guiKeyValuePair.setValue(ien['value'])

        patient.getPatientIens().add(guiKeyValuePair)
        patient.setEDIPI("NA")
        patient.setICN("NA")

        patient.getVistaSites().add(session.appUser.getHostSite())

        def results = communityDao.getPatientActiveConsultRequest(session.appUser, patient)
        sendResponse(results)

        //mock code
        /*def results = []

        GUIProvider provider = new GUIProvider();
        provider.setName("Doctor, Beta")
        provider.setPhone("222-222-2222")

        GUIConsult consult = new GUIConsult();
        consult.setProvider(provider)
        consult.setConsultId("321211")
        consult.setOutCode("MRI")
        consult.setOutStatus("ACTIVE")

        results.add(consult)


        GUIProvider provider2 = new GUIProvider();
        provider2.setName("Doctor, Omega")
        provider2.setPhone("333-333-3333")

        GUIConsult consult2 = new GUIConsult();
        consult2.setProvider(provider2)
        consult2.setConsultId("003181")
        consult2.setOutCode("C&P Exam")
        consult2.setOutStatus("ACTIVE")

        results.add(consult2)

        sendResponse(results) */
    }


    def updateProvider = {

        def provider = new GUIOutsideProvider();

        provider.setId(params.id)
        provider.setName(params.name)
        provider.setEmail(params.email)
        provider.setNpi(params.npi)
        provider.setOrganization(params.organization)
        provider.setDataTypeAccess(params.dataTypeAccess)
        provider.setPhone(params.phone)
        provider.setUsername(params.username);
        provider.setDataTypeAccess(params.dataTypeAccess)

        if (Boolean.valueOf(params.resetPassword)) {
            provider.setPassword(params.password.encodeAsMD5());
        }
        else {
            provider.setPassword(null);
        }

        communityDao.updateOutsideProvider(session.appUser, provider)

        sendResponse([message: "OK"])


    }

    def usernameCheck = {

        def username = params.name;
        def exists = communityDao.doesUsernameExist(session.appUser, username);

        def response = ["exists": exists]

        sendResponse(response)
    }

    def generateDefaultPassword = {

        def pwd = communityDao.generateDefaultPassword(10);

        sendResponse(["pwd": pwd])
    }

}
