/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIAllergy
import gov.va.med.guibeans.GUISite
import gov.va.med.jmeadows.webservice.NarrativeV4


class PatientAllergiesController extends BaseController {

    def allergyDao

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        }
    }

    def list = {
        def includeVLER = false;

        if(params["includeVLER"] == "true"){
            includeVLER = true;
        }

        //will contain unique narrative Ids for this domain
        HashSet narrativeIds = new HashSet();

        def guiAllergies = allergyDao.getPatientAllergies(session.appUser, request.patient, includeVLER )

        //generate record details token
        if (guiAllergies != null) {
            for (def guiAllergy : guiAllergies) {

                if (guiAllergy.getNarrativeId() != null && !"".equalsIgnoreCase(guiAllergy.getNarrativeId())) {
                    narrativeIds.add(guiAllergy.getNarrativeId());

                    if(guiAllergy.getSite() != null && guiAllergy.getSite().getName() != null
                            && "VLER".equalsIgnoreCase(guiAllergy.getSite().getName())
                            && guiAllergy.getSite().getSiteCode() != null){
                        String siteName = session.appUser.partnersMap.get(guiAllergy.getSite().getSiteCode());
                        if(siteName != null){
                            guiAllergy.getSite().setName(siteName);
                        }
                        guiAllergy.getSite().setAgency("VLER");

                    }

                }

                //set record details token
                if (!guiAllergy.getAllergyId()) {
                    continue;
                }
                def queryMap = [(HttpKeys.ALLERGY_ID): guiAllergy.getAllergyId()]
                guiAllergy.setRecordDetailsToken(
                        generateRecordDetailsToken(
                                params[HttpKeys.PATIENT_TOKEN],
                                controllerName,
                                queryMap,
                                guiAllergy.getSite(),
                                guiAllergy.getSourceProtocol()))
            }
        }
        if (includeVLER && session.appUser.narrativesMap != null
                && !session.appUser.narrativesMap.isEmpty()) {
            Iterator narrativeIter = session.appUser.narrativesMap.entrySet().iterator();

            //loop through all narratives, check the allergy ones to see if they are referenced in this call
            //if they are not referenced, then create a record for them so the user is able to view this narrative
            while (narrativeIter.hasNext()) {
                Map.Entry narrative = (Map.Entry) narrativeIter.next();
                if (narrative.getKey().toString().contains("52472-8")) {
                    //check to see if record using this narrative exists
                    if (!narrativeIds.contains(narrative.getKey().toString())) {
                        NarrativeV4 narrativeItem = session.appUser.narrativesMap.get(narrative.getKey().toString());
                        String siteName = "VLER";
                        String siteMoniker = "VLER Partner";
                        if (narrativeItem != null)
                            if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                    && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                                siteName = session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier());
                            }
                        if (narrativeItem.getNarrativeHomeCommunityIdentifier() != null
                                && session.appUser.partnersMap.get(narrativeItem.getNarrativeHomeCommunityIdentifier()) != null) {
                            siteMoniker = session.appUser.partnersMap.get(narrativeItem.getNarrativeAssigningAuthorityIdentifier());
                        }
                        GUISite narrativeSite = new GUISite();
                        narrativeSite.setAgency("VLER");
                        narrativeSite.setName(siteName);
                        narrativeSite.setMoniker(siteMoniker);
                        narrativeSite.setSiteCode("VLER");

                        GUIAllergy narrativeAllergy = new GUIAllergy();
                        narrativeAllergy.setNarrativeId(narrative.getKey().toString());
                        narrativeAllergy.setSite(narrativeSite);
                        narrativeAllergy.setAllergyName("Narrative");
                        guiAllergies.add(narrativeAllergy);

                        narrativeIds.add(narrative.getKey().toString());
                    }

                }
            }

        }

        sendResponse(guiAllergies)
    }

    def details = {

        def guiAllergyDetail = allergyDao.getAllergyDetail(
                session.appUser,
                request.patient,
                request.recordQuery[HttpKeys.ALLERGY_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_CODE])

        sendResponse(guiAllergyDetail)
    }
}
