package jlv

import gov.va.med.jmeadows.webservice.NarrativeV4
import gov.va.med.jmeadows.webservice.SiteStatusV4
import gov.va.med.jmeadows.webservice.Vlerccda

import javax.crypto.*
import javax.crypto.spec.IvParameterSpec
import javax.crypto.spec.SecretKeySpec
import java.security.InvalidAlgorithmParameterException
import java.security.InvalidKeyException
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException
import java.text.SimpleDateFormat

class PatientDODVLERController extends BaseController {

    def DODVLERDao;
    def auditDao;

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {


    }


    def list = {
        def forceRefresh = false;

        if (params["forceRefresh"] == "true") {
            forceRefresh = true;
        }

        session.appUser.partnersMap.clear();
        session.appUser.narrativesMap.clear();
        session.appUser.vlerSiteStatus = "";

        def vlerccdas = DODVLERDao.getDODVLERPatientCCDA(session.appUser, request.patient, forceRefresh);
        //one vler ccda is the error report
        if (vlerccdas != null && vlerccdas.size() > 1) {

            List<String> vlerAvailability = new ArrayList<String>();

            //cache narratives in session
            for (Vlerccda ccda : vlerccdas) {
                if (ccda == null) continue;
                if (ccda.getNarratives() != null && ccda.getNarratives().size() > 0) {
                    for (NarrativeV4 narrative : ccda.getNarratives()) {
                        if (narrative == null) continue;
                        if (narrative.getNarrativeIdentifier() != null && narrative.getNarrativeText() != null)
                            session.appUser.narrativesMap.put(narrative.getNarrativeIdentifier(), narrative);
                    }
                }
                if (ccda.getStatusList() != null && ccda.getStatusList().size() > 0) {
                    String vlerSiteStatus = DODVLERDao.generateBhieStatusReportV4(ccda.getStatusList(), ccda.getErrorList())
                    for (SiteStatusV4 status : ccda.getStatusList()) {
                        if (status.getPartner() != null
                                && status.getPartner().getPartnerIdentifier() != null
                                && status.getPartner().getPartnerName() != null) {
                            session.appUser.partnersMap.put(status.getPartner().getPartnerIdentifier(), status.getPartner().getPartnerName());
                            if (status.getPartner().getPartnerSubIdentifier() != null) {
                                session.appUser.partnersMap.put(status.getPartner().getPartnerSubIdentifier(), status.getPartner().getPartnerSubIdentifierName());
                            }
                        }
                    }
                    if (vlerSiteStatus != null && vlerSiteStatus.length() > 0) {
                        session.appUser.vlerSiteStatus = vlerSiteStatus;
                    }
                }

                if (ccda.getResults() != null
                        && (ccda.getResults().size() > 0)
                    && ccda.getResults().get(0).getSite() != null
                    && ccda.getResults().get(0).getSite().getMoniker() != null
                    && !"DODADAPTER_SOURCE_STATUS_REPORT".equalsIgnoreCase(ccda.getResults().get(0).getSite().getMoniker()))
                 {
                    vlerAvailability.add("RESULTS");
                }
                if (ccda.getEncounters() != null && (ccda.getEncounters().size() > 0)
                        && ccda.getEncounters().get(0).getSite() != null
                        && ccda.getEncounters().get(0).getSite().getMoniker() != null
                        && !"DODADAPTER_SOURCE_STATUS_REPORT".equalsIgnoreCase(ccda.getEncounters().get(0).getSite().getMoniker())) {
                    vlerAvailability.add("ENCOUNTERS");
                }
                if (ccda.getMedications() != null && (ccda.getMedications().size() > 0)
                        && ccda.getMedications().get(0).getSite() != null
                        && ccda.getMedications().get(0).getSite().getMoniker() != null
                        && !"DODADAPTER_SOURCE_STATUS_REPORT".equalsIgnoreCase(ccda.getMedications().get(0).getSite().getMoniker())) {
                    vlerAvailability.add("MEDICATIONS");
                }

            }

            response.status = 200
            sendResponse(vlerAvailability);
//            render "SUCCESS!"
        } else {
            response.status = 500
            render "Failed to retrieve DoD VLER Records."
        }
        //sendResponse("place your return item(s) here")
    }

    def getNarrative = {
        NarrativeV4 narrative = session.appUser.narrativesMap.get(params.get("narrativeId"));

        if (narrative != null && narrative.getNarrativeText() != null) {
            response.status = 200
            render narrative.getNarrativeText()
        } else {
            response.status = 200
            render "This DoD VLER Narrative is no longer available. Please refresh this widget and try again."
        }
    }

    def getWidgetUrl = {
        //  https://172.18.20.71/VlerDocumentViewer/jlv.jsp?payload={encrypted value}
        // {encrypted value} = dt=20150513073033&providerId=JLV&patientId=95000015
        def theme = "black";

        if (params["theme"] == "blue") {
            theme = "jlv";
        } else if (params["theme"] == "gray") {
            theme = "gray";
        }

        String widgetURL = appConfig.getDodVlerURL() + "/jlv.jsp?payload=";

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));

        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.add(Calendar.MINUTE, 0);
        String dateString = dateFormat.format(cal.getTime())

        //todo: melcolm debug hardcoded for Dev.com test
        String ahltaUnitNum = "";
        if (appConfig.getDodVlerTestId() != null && !appConfig.getDodVlerTestId().isEmpty()
                && !"".equalsIgnoreCase(appConfig.getDodVlerTestId())) {
            ahltaUnitNum = appConfig.getDodVlerTestId()
        } else {

            if (request.patient.getAhltaUnitNumber() == null || request.patient.getAhltaUnitNumber().isEmpty()) {
                response.status = 500
                render "This patient does not have an AHLTA Unit Number."
            } else {
                ahltaUnitNum = request.patient.getAhltaUnitNumber();
            }

        }

        String payload = "dt=" + dateString + "Z&providerId=JLV&patientId=" + ahltaUnitNum + "&theme=" + theme;
        String encPayload = URLEncoder.encode(encrypt(payload), "UTF-8"); ;

        widgetURL += encPayload;

        response.status = 200
        render widgetURL
    }

    def getWindowUrl = {

        def theme = "black";

        if (params["theme"] == "blue") {
            theme = "jlv";
        } else if (params["theme"] == "gray") {
            theme = "gray";
        }

        String widgetURL = appConfig.getDodVlerURL() + "/vler.htm?payload=";

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));

        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date());
        cal.add(Calendar.MINUTE, 0);
        String dateString = dateFormat.format(cal.getTime())

        //todo: melcolm debug hardcoded for Dev.com test
        String ahltaUnitNum = "";
        if (appConfig.getDodVlerTestId() != null && !appConfig.getDodVlerTestId().isEmpty()
                && !"".equalsIgnoreCase(appConfig.getDodVlerTestId())) {
            ahltaUnitNum = appConfig.getDodVlerTestId()
        } else {

            if (request.patient.getAhltaUnitNumber() == null || request.patient.getAhltaUnitNumber().isEmpty()) {
                response.status = 500
                render "This patient does not have an AHLTA Unit Number."
            } else {
                ahltaUnitNum = request.patient.getAhltaUnitNumber();
            }

        }

        String payload = "dt=" + dateString + "Z&providerId=JLV&patientId=" + ahltaUnitNum + "&theme=" + theme;
        String encPayload = URLEncoder.encode(encrypt(payload), "UTF-8"); ;

        widgetURL += encPayload;

        response.status = 200
        render widgetURL
    }

    /**
     * Encrypt.
     *
     * @param message input
     *
     * @return output
     */
    public String encrypt(String message) {
        String rtc;
        final MessageDigest digest;

        try {
            digest = MessageDigest.getInstance("SHA-512");
        } catch (NoSuchAlgorithmException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] digestOfPassword;

        try {
            digestOfPassword = digest.digest("ZZ58YZ3CR9".getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] keyBytes = Arrays.copyOf(digestOfPassword, 16);
        int index = 0;
        int kindex = 8;
        while (index < 8) {
            keyBytes[kindex++] = keyBytes[index++];
        }

        final SecretKey key = new SecretKeySpec(keyBytes, "AES");
        final IvParameterSpec ivspec = new IvParameterSpec(new byte[16]);
        final Cipher cipher;

        try {
            cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
        } catch (NoSuchAlgorithmException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        } catch (NoSuchPaddingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        try {
            cipher.init(Cipher.ENCRYPT_MODE, key, ivspec);
        } catch (InvalidKeyException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        } catch (InvalidAlgorithmParameterException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] plainTextBytes;

        try {
            plainTextBytes = message.getBytes("utf-8");
        } catch (UnsupportedEncodingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] cipherText;

        try {
            cipherText = cipher.doFinal(plainTextBytes);
        } catch (IllegalBlockSizeException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        } catch (BadPaddingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        rtc = new sun.misc.BASE64Encoder().encode(cipherText);

        return rtc;
    }

    /**
     * Decrypt.
     *
     * @param enc input
     *
     * @return output
     */
    public String decrypt(String enc) {
        String rtc;

        byte[] message;

        try {
            message = new sun.misc.BASE64Decoder().decodeBuffer(enc);
        } catch (IOException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final MessageDigest digest;

        try {
            digest = MessageDigest.getInstance("SHA-512");
        } catch (NoSuchAlgorithmException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] digestOfPassword;

        try {
            digestOfPassword = digest.digest("ZZ58YZ3CR9".getBytes("utf-8"));
        } catch (UnsupportedEncodingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        final byte[] keyBytes = Arrays.copyOf(digestOfPassword, 16);
        int index = 8;
        int jindex = 0;
        while (jindex < 8) {
            keyBytes[index++] = keyBytes[jindex++];
        }

        final SecretKey key = new SecretKeySpec(keyBytes, "AES");
        final IvParameterSpec ivspec = new IvParameterSpec(new byte[16]);
        final Cipher decipher;

        try {
            decipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
        } catch (NoSuchAlgorithmException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        } catch (NoSuchPaddingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        try {
            decipher.init(Cipher.DECRYPT_MODE, key, ivspec);
        } catch (InvalidKeyException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        } catch (InvalidAlgorithmParameterException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        // final byte[] encData = new
        // sun.misc.BASE64Decoder().decodeBuffer(message);
        final byte[] plainText;

        try {
            plainText = decipher.doFinal(message);
        } catch (IllegalBlockSizeException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        } catch (BadPaddingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        try {
            rtc = new String(plainText, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            //LOGGER.error(e.getMessage(), e);
            return null;
        }

        return rtc;
    }

}
