/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.DNS   .common.Utils
import org.apache.commons.lang.StringUtils

class PatientOrdersController extends BaseController {
    def orderDao

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }


        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        } else dateRangeCheck();
    }

    def list = {

        def guiOrders = orderDao.getPatientOrders(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        //generate record details token
        if (guiOrders != null) {
            for (def guiOrder : guiOrders) {
                if (appConfig.getProtocolRPC() == guiOrder.getSourceProtocol() &&
                        StringUtils.isNotEmpty(guiOrder.getOrderId())) {
                    def queryMap = [(HttpKeys.ORDER_ID): guiOrder.getOrderId()]
                    guiOrder.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiOrder.getSite(),
                                    guiOrder.getSourceProtocol()))
                }
            }
        }

        sendResponse(guiOrders)
    }

    def details = {

        def guiOrderDetail = orderDao.getOrderDetail(
                session.appUser,
                request.patient,
                request.recordQuery[HttpKeys.ORDER_ID],
                request.recordQuery[HttpKeys.RECORD_SITE_CODE]);

        sendResponse(guiOrderDetail)
    }
}
