/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.common.HttpKeys
import gov.va.med.guibeans.GUIEndpoint
import gov.va.med.guibeans.GUIRadiologyExam
import gov.va.med.DNS   .common.Utils
import org.apache.commons.beanutils.BeanUtils
import org.apache.commons.lang.StringUtils

class PatientRadiologyExamsController extends BaseController {
    def radiologyDao
    def auditDao

    static allowedMethods = [list: 'POST', details: 'POST']

    def beforeInterceptor = {

        if (request.patient == null) {
            response.sendError 500
            return false;
        }

        if (actionName == 'details') {
            if (request.recordQuery == null) {
                response.sendError 500
                return false;
            }
        } else dateRangeCheck();
    }

    def list = {

        def guiRads = radiologyDao.getPatientRads(
                session.appUser,
                request.patient,
                Utils.getCal(request[HttpKeys.START_DATE]),
                Utils.getCal(request[HttpKeys.END_DATE]))

        //generate record details token
        if (guiRads != null) {
            for (def guiRad : guiRads) {
                if (StringUtils.isNotEmpty(guiRad.getCacheId())) {
                    def queryMap = [(HttpKeys.RAD_CACHE_ID)               : guiRad.getCacheId(),
                                    (HttpKeys.RAD_EXAM_ID)               : guiRad.getExamId(),
                                    (HttpKeys.RADIOLOGY_ACCESSION_NUMBER): guiRad.getAccessionNumber(),
                                    (HttpKeys.RECORD_SITE_MONIKER)       : guiRad.site.moniker,
                                    (HttpKeys.RECORD_SITE_AGENCY)        : guiRad.site.agency]

                    guiRad.setRecordDetailsToken(
                            generateRecordDetailsToken(
                                    params[HttpKeys.PATIENT_TOKEN],
                                    controllerName,
                                    queryMap,
                                    guiRad.getSite(),
                                    guiRad.getSourceProtocol(),
                                    guiRad.isSessionCachedRecord()))

                    if (guiRad.isMedWebAvailable()) {
                        GUIEndpoint medwebEndpoint = null;

                        String hostFQDN = "";
                        try {
                            hostFQDN = new URL(request.getRequestURL().toString()).getHost();
                        } catch (Exception e) {
                            //cannot extract HostName
                            //logger.error("get HostName exception: " + e.getMessage());
                            hostFQDN = "";
                        };

                        for (GUIEndpoint endpoint : guiRad.getSite().getEndpoints()) {
                            String proto = appConfig.getProtocolMedweb() + "_" + hostFQDN;

                            if (proto.equalsIgnoreCase(endpoint.getProtocol())) {
                                medwebEndpoint = endpoint;
                                break;
                            }
                        }

                        if (medwebEndpoint != null) {
                            queryMap.put(HttpKeys.MEDWEB_URL, medwebEndpoint.getHost());
                            queryMap.put(HttpKeys.MEDWEB_USR, medwebEndpoint.getUsername());
                            queryMap.put(HttpKeys.MEDWEB_PWD, medwebEndpoint.getPassword());
                            queryMap.put(HttpKeys.MEDWEB_AETITLE, medwebEndpoint.getAeTitle());
                            queryMap.put(HttpKeys.MEDWEB_ACCESSION_PREFIX, medwebEndpoint.getAccessionPrefix());

                            guiRad.setMedWebToken(
                                    generateRecordDetailsToken(
                                            params[HttpKeys.PATIENT_TOKEN],
                                            "patientMedWeb",
                                            queryMap,
                                            guiRad.getSite(), guiRad.getSourceProtocol()))
                        }
                    }

                    if (guiRad.isSessionCachedRecord()) {
                        //cache radiology report in session
                        def guiRadDetail = BeanUtils.cloneBean(guiRad)

                        session.appUser.radiologyReportsMap.put(
                                getCachedRecordKey(guiRad.getRecordDetailsToken()), guiRadDetail)

                        //don't send report text with rad exam listings
                        guiRad.setReportText("")
                    }
                }
            }
        }

        if (session.appUser.vlerSiteStatus != null && session.appUser.vlerSiteStatus.length() > 0) {
            GUIRadiologyExam statusRecord = new GUIRadiologyExam();
            statusRecord.setStudy(session.appUser.vlerSiteStatus);
            statusRecord.setSite(getStatusReportSite());
            guiRads.add(statusRecord);
        }

        sendResponse(guiRads)
    }

    def details = {
        def guiRadReport = null

        if (request.isSessionCachedRecord) {

            //audit details query

            auditDao.auditDetailsQuery("RadiologyReport", session.appUser, request.patient,
                    "accNum:" + request.recordQuery[HttpKeys.RADIOLOGY_ACCESSION_NUMBER],
                    request.recordQuery[HttpKeys.RECORD_SITE_MONIKER],
                    request.recordQuery[HttpKeys.RECORD_SITE_AGENCY])

            guiRadReport = session.appUser.radiologyReportsMap.get(
                    getCachedRecordKey(params[HttpKeys.RECORD_DETAILS_TOKEN]))
        } else {
            guiRadReport = radiologyDao.getRadiologyReport(
                    session.appUser,
                    request.patient,
                    request.recordQuery[HttpKeys.RAD_EXAM_ID],
                    request.recordQuery[HttpKeys.RECORD_SITE_CODE])
            guiRadReport.setPriority(params[HttpKeys.RAD_PRIORITY]);
        }

        if (guiRadReport.isMedWebAvailable()) {
            def queryMap = [(HttpKeys.RADIOLOGY_ACCESSION_NUMBER): guiRadReport.getAccessionNumber()]

            GUIEndpoint medwebEndpoint = null;
            for (GUIEndpoint endpoint : guiRadReport.getSite().getEndpoints()) {
//                if (appConfig.getProtocolMedweb().equalsIgnoreCase(endpoint.getProtocol()))
                if (endpoint.getProtocol().startsWith(appConfig.getProtocolMedweb())) {
                    medwebEndpoint = endpoint;
                    break;
                }
            }

            if (medwebEndpoint != null) {
                queryMap.put(HttpKeys.MEDWEB_URL, medwebEndpoint.getHost());
                queryMap.put(HttpKeys.MEDWEB_USR, medwebEndpoint.getUsername());
                queryMap.put(HttpKeys.MEDWEB_PWD, medwebEndpoint.getPassword());
                queryMap.put(HttpKeys.MEDWEB_AETITLE, medwebEndpoint.getAeTitle());
                queryMap.put(HttpKeys.MEDWEB_ACCESSION_PREFIX, medwebEndpoint.getAccessionPrefix());

                guiRadReport.setMedWebToken(
                        generateRecordDetailsToken(
                                params[HttpKeys.PATIENT_TOKEN],
                                "patientMedWeb",
                                queryMap,
                                guiRadReport.getSite(), guiRadReport.getSourceProtocol()))
            }
        }

        sendResponse(guiRadReport)
    }
}
