/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.enterprise.bean.ResponseGUIPatientQuery
import gov.va.med.jmeadows.webservice.PdwsQueryBean

class PatientSearchController extends BaseController {

    def patientDao
    def userDao

    //static allowedMethods = [list:'POST']

    def list = {

        def patients = null;

        def queryBean = new PdwsQueryBean();
        queryBean.setPatientId(params.queryPatId);
        queryBean.setPatientIdType(params.queryPatIdType);
        queryBean.setLastName(params.queryLastName);
        queryBean.setFirstName(params.queryFirstName);
        queryBean.setMiddleName(params.queryMI);
        queryBean.setDob(params.queryDoB);
        queryBean.setGender(params.queryGender);
        queryBean.setAddress(params.queryAddress);
        queryBean.setCity(params.queryCity);
        queryBean.setState(params.queryState);
        queryBean.setTel(params.queryTel);
        queryBean.setZip(params.queryZip);
        queryBean.setInquiryType(params.queryInquiryType);

        try {
            ResponseGUIPatientQuery respGUIPatQuery =
                    patientDao.lookupPatientPDWS(session.appUser, queryBean, session.appUser.securityKey);

            if (respGUIPatQuery != null) {
                if (respGUIPatQuery.isSuccess()) {
                    patients = respGUIPatQuery.getGuiPatients();
                } else {
                    response.setStatus(500);
                    patients = ["userMsg": respGUIPatQuery.getErrorMsg()];
                }
            }
        } catch (Exception e) {
            patients = e.getMessage()
            response.setStatus(500);
        }

        sendResponse(patients);
    }

    def getRecentlyViewedPatients = {

        def patients = [];

        try {
            patients = userDao.getRecentlyViewedPatients(session.appUser, session.appUser.securityKey)
        }
        catch (Exception e) {
            response.setStatus(500)
            patients = []
        }

        sendResponse(patients)
    }
}
