/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package jlv

import gov.va.med.guibeans.GUIEndpoint
import grails.converters.JSON
import groovy.json.JsonBuilder

class ViewController {
    def index = {

        def siteIcon = "VA"
        def cfg = session.appUser.cfg
        def theme = "default"
        def jsonCfg = ""
        if (cfg != null && cfg.length() > 0) {
            jsonCfg = JSON.parse(cfg)
            for (item in jsonCfg) {
                if (item.cfgItem == "UI_CONF") {
                    for (option in item.options) {
                        if (option.theme) {
                            theme = option.theme
                        }
                    }
                }
            }
        }


        def hostSite = session.appUser.hostSite

        if (hostSite.agency == grailsApplication.config.grails.agencyDOD) {
            siteIcon = "DOD"
        }
        else if (hostSite.agency == grailsApplication.config.grails.agencyOP) {
            siteIcon = "VLER"
        }

        //only enable the provider profile for sites that include a cache site
        def enableProvProfile = false

        for (GUIEndpoint endpoint : hostSite.endpoints) {
            if (endpoint.protocol == 'CACHE' && endpoint.status == 'active') {
                enableProvProfile = true
                break
            }
        }

        //compute session timeout in minutes
        def sessionTimeout = session.maxInactiveInterval / 60

        def allowMedwebPlugin = false

        hostSite.permissions.each() {
            if (it.description == 'medweb_plugin') {
                allowMedwebPlugin = true
            }
        }

        // Ensure AHLTA link flag and object are valid if not coming from AHLTA link
        if (session.ahltaLoginFlag == null) {
            session.ahltaLoginFlag = false
            session.ahltaLoginPatient = []
        }

        // Ensure outside provider flag are valid if not coming from community login
        if (session.outsideProviderFlag == null) {
            session.outsideProviderFlag = false
            session.outsideProvider = []
        }

        //generate and set csrf token
        def csrfToken = "";
        if (session.getAttribute("X-CSRF-Token") == null) {
            csrfToken = UUID.randomUUID().toString()
            session.setAttribute("X-CSRF-Token", csrfToken)
        } else {
            csrfToken = session.getAttribute("X-CSRF-Token")
        }

        // User JAM widget access flag
        def jamWidgetAccessFlag = false;
        if (session.jamWidgetAccessFlag != null) {
            jamWidgetAccessFlag = session.jamWidgetAccessFlag
        }

        log.debug(new JsonBuilder(session.appUser).toPrettyString())

        [user             : session.appUser,
         theme            : theme,
         allowMedwebPlugin: allowMedwebPlugin,
         sessionTimeout   : sessionTimeout,
         csrfToken: csrfToken,
         siteIcon         : siteIcon,
         enableProvProfile: enableProvProfile,
         grailsConfig     : grailsApplication.config.grails,
         appVersion       : grailsApplication.metadata['app.version'],
         cfg              : cfg, theme: theme,
         ahltaLoginFlag   : session.ahltaLoginFlag,
         ahltaLoginPatient: session.ahltaLoginPatient as JSON,
         outsideProviderFlag: session.outsideProviderFlag,
         guiOutsideProvider: session.guiOutsideProvider as JSON,
         jamWidgetAccessFlag : jamWidgetAccessFlag,
        ]
    }

    def refresh = {
        def user = session.appUser
        render("OK");
    }

}
