/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.Encounter;
import gov.va.med.jmeadows.webservice.FreeTextReport;
import gov.va.med.jmeadows.webservice.PatientAppointments;
import gov.va.med.DNS   .common.Utils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 *
 *
 */
@Component
public class AppointmentDao extends DaoBase {

    private static final Logger logger = Logger.getLogger(AppointmentDao.class.getName());

    public List<GUIAppointment> getPatientAppointments(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            PatientAppointments[] appts = getServiceHandler().getPatientAppointments(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIAppointment> guiAppts = new ArrayList<GUIAppointment>();

            for (PatientAppointments appt : appts) {
                GUIAppointment guiAppt = new GUIAppointment();

                String apptDate = "";
                String clinic = "";
                String clinicId = "";
                String status = "";
                String type = "";
                String providerName = "";
                String specialty = "";
                String reason = "";
                String appointmentId = "";

                if (appt.getProvider() != null && appt.getProvider().getName() != null) {
                    guiAppt.setProvider(dozerBeanMapper.map(appt.getProvider(), GUIProvider.class));
                    providerName = appt.getProvider().getName();
                    if (appt.getProvider() != null)
                        specialty = appt.getProvider().getSpecialization();
                }

                if (appt.getClinic() != null) {
                    clinic = appt.getClinic();
                }

                if (appt.getClinicId() != null) {
                    clinicId = appt.getClinicId();
                }

                if (appt.getStatus() != null) {
                    status = appt.getStatus();
                }

                if (appt.getApptId() != null) {
                    appointmentId = appt.getApptId();
                }

                if (appt.getApptDate() != null) {
                    apptDate = Utils.formatDisplayDateTime(appt.getApptDate());
                }

                if (appt.getApptType() != null) {
                    type = appt.getApptType();
                }

                if (appt.getReason() != null) {
                    reason = appt.getReason();
                }

                guiAppt.setAppointmentDate(apptDate);
                guiAppt.setAppointmentId(appointmentId);
                guiAppt.setClinic(clinic);
                guiAppt.setClinicId(clinicId);
                guiAppt.setProviderName(providerName);
                guiAppt.setReason(reason);
                guiAppt.setStatus(status);
                guiAppt.setType(type);

                if (appt.getProvider() != null) {
                    guiAppt.setProvider(dozerBeanMapper.map(appt.getProvider(), GUIProvider.class));
                }


                addSiteDataToGUIBean(guiAppt, appt);

                guiAppts.add(guiAppt);
            }

            return guiAppts;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIAppointmentDetail getAppointmentDetails(GUIUser user, GUIPatient patient, String appointmentId,
                                                      String recordSiteCode, Calendar startDate, Calendar endDate) {
        try {
            FreeTextReport report = getServiceHandler().getPatientEncountersReport(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .itemId(appointmentId)
                            .recordSiteCode(recordSiteCode)
                            .build());

            GUIAppointmentDetail apptDetail = new GUIAppointmentDetail();
            if (report != null) {
                apptDetail.setVisitDetails(report.getReportText());
                addSiteDataToGUIBean(apptDetail, report);
            }

            return apptDetail;
        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public List<GUIAppointment> getProviderAppointments(
            GUIUser user,
            Calendar startDate,
            Calendar endDate,
            byte[] encryptSecurityKey) {
        try {
            PatientAppointments[] appts = getServiceHandler().getProviderAppointments(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIAppointment> guiAppts = new ArrayList<GUIAppointment>();

            for (PatientAppointments appt : appts) {
                GUIAppointment guiAppt = new GUIAppointment();

                String apptDate = "";
                String clinic = "";
                String status = "";
                String type = "";
                String providerName = "";
                String reason = "";
                String appointmentId = "";
                String patName = "";
                String patToken = "";

                if (appt.getProvider() != null && appt.getProvider().getName() != null) {
                    providerName = appt.getProvider().getName();
                }

                if (appt.getClinic() != null) {
                    clinic = appt.getClinic();
                }

                if (appt.getStatus() != null) {
                    status = appt.getStatus();
                }

                if (appt.getApptId() != null) {
                    appointmentId = appt.getApptId();
                }

                if (appt.getApptDate() != null) {
                    apptDate = Utils.formatDisplayDateTime(appt.getApptDate());
                }

                if (appt.getApptType() != null) {
                    type = appt.getApptType();
                }

                if (appt.getReason() != null) {
                    reason = appt.getReason();
                }

                if (appt.getPatientName() != null) {
                    patName = appt.getPatientName();
                }

                patToken = getProvQueryPatientToken(encryptSecurityKey, appt);

                guiAppt.setAppointmentDate(apptDate);
                guiAppt.setAppointmentId(appointmentId);
                guiAppt.setClinic(clinic);
                guiAppt.setProviderName(providerName);
                guiAppt.setReason(reason);
                guiAppt.setStatus(status);
                guiAppt.setType(type);
                addSiteDataToGUIBean(guiAppt, appt);
                guiAppt.setPatientName(patName);
                guiAppt.setPatientToken(patToken);

                guiAppts.add(guiAppt);
            }

            return guiAppts;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    /**
     * Retrieves patient encounters from AHLTA (only returns DoD data for now)
     *
     * @param user
     * @param patient
     * @param startDate
     * @param endDate
     * @return a list of patient encounters from AHLTA.
     */
    public List<GUIPatientEncounter> getPatientEncounters(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            Encounter[] patientEncounters = getServiceHandler().getPatientEncounters(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIPatientEncounter> guiEncounters = new ArrayList<GUIPatientEncounter>();
            StringBuilder encounterId = new StringBuilder();
            String newEncounterId;
            int i = 0;

            for (Encounter patientEncounter : patientEncounters) {
                GUIPatientEncounter guiEncounter = dozerBeanMapper.map(patientEncounter, GUIPatientEncounter.class);

                encounterId.setLength(0);
                newEncounterId = (patientEncounter.getEncounterId() != null) ? patientEncounter.getEncounterId() : "";

                //check for sensitive DoD adapter encounter
                if (guiEncounter.isSensitive()) {
                    guiEncounter.setSensitive(true);
                    guiEncounter.setClinic(guiEncounter.getClinic() + "  ** Sensitive **");
                    guiEncounter.setReason(guiEncounter.getReason() + "  ** Sensitive **");
//                    guiEncounter.setDiagnosis(null);
//                    guiEncounter.setProcedures(null);
//                    guiEncounter.setEncounterNumber("");
//                    guiEncounter.setEncounterId("");
//                    guiEncounter.setComplexDataUrl("");
//                    guiEncounter.setReasonNarrative("");
//                    guiEncounter.setDischargeDisposition("");
//                    guiEncounter.setDocuments(null);
//                    guiEncounter.setReasonCode("");
//                    guiEncounter.setCodes(null);
                }

                //if rpc or dod adapter protocol then flag record for caching
                if (appConfig.getProtocolDODAdapter().equals(patientEncounter.getSourceProtocol())) {
                    guiEncounter.setSessionCachedRecord(true);
                    encounterId.append(appConfig.getCachedTag());
                    encounterId.append(i);
                } else {
                    encounterId.append(newEncounterId);
                }

                guiEncounter.setEncounterId(encounterId.toString());

                if (StringUtils.isNotBlank(guiEncounter.getComplexDataUrl())) {
                    guiEncounter.setNoteAsBinaryStream(true);
                }
                guiEncounter.setApptDate(Utils.formatDisplayDateTime(patientEncounter.getApptDate()));
                guiEncounter.setEventId(patientEncounter.getCdrEventId());

                guiEncounters.add(guiEncounter);
                i++;
            }

            return guiEncounters;
        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }
}
