/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.GUICode;
import gov.va.med.guibeans.GUIImmunization;
import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.Code;
import gov.va.med.jmeadows.webservice.Immunization;
import gov.va.med.DNS   .common.Utils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class ImmunizationDao extends DaoBase {
    private static final Logger logger = Logger.getLogger(ImmunizationDao.class.getName());

    public List<GUIImmunization> getPatientImmunizations(GUIUser user, GUIPatient patient, boolean includeVLER) {
        try {
            Immunization[] imms = getServiceHandler().getPatientImmunizations(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .includeVLER(includeVLER)
                            .build());

            List<GUIImmunization> guiImms = new ArrayList<GUIImmunization>();

            for (Immunization im : imms) {
                GUIImmunization guiIm = dozerBeanMapper.map(im, GUIImmunization.class);

                String adminDate = "";
                String vaccine = "";
                String exemptionRefusalDate = "";

                if (im.getDateTime() != null) {
                    adminDate = im.getDateTime();
                }

                if (im.getExemptionRefusalDate() != null) {
                    exemptionRefusalDate = Utils.formatDisplayDate(im.getExemptionRefusalDate());
                }

                if (im.getName() != null) {
                    vaccine = im.getName();
                }

                guiIm.setAdminDate(adminDate);
                guiIm.setExemptionRefusalDate(exemptionRefusalDate);
                guiIm.setVaccine(vaccine);
                guiIm.setNarrativeId(im.getNarrativeId());
                addSiteDataToGUIBean(guiIm, im);

                List<GUICode> guiCodes = new ArrayList<GUICode>();
                for (Code code : im.getCodes()) {
                    guiCodes.add(dozerBeanMapper.map(code, GUICode.class));
                }
                guiIm.setCodes(guiCodes);

                if (appConfig.getFilterOutVAImmunizations()
                        && "DOD".equalsIgnoreCase(user.getAgency())
                        && "VA".equalsIgnoreCase(guiIm.getSite().getAgency())) {
                    // DoD users not allowed to see VA records
                    // do nothing
                } else {
                    guiImms.add(guiIm);
                }


            }

            return guiImms;

        } catch (Exception e) {
            throw new DaoException(e);
        }


    }
}
