package gov.va.med.dao;


import gov.va.med.guibeans.GUIInsurance;
import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.InsuranceBean;
import gov.va.med.DNS   .common.Utils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
public class InsuranceDao extends DaoBase {

    /**
     * Retrieves a list of patient allergies from all available patient locations.
     *
     * @param user    {@link gov.va.med.guibeans.GUIUser} making the request.
     * @param patient {@link gov.va.med.guibeans.GUIPatient} patient retrieving allergies from this patient record.
     * @return A list of patient allergies.
     */
    public List<GUIInsurance> getPatientInsurances(GUIUser user, GUIPatient patient, boolean includeVLER) {
        try {
            InsuranceBean[] insuranceBeans = getServiceHandler().getPatientInsurances(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .includeVLER(includeVLER)
                            .build());

            List<GUIInsurance> guiInsurances = new ArrayList<GUIInsurance>();

            for (InsuranceBean ib : insuranceBeans) {
                GUIInsurance guiIb = dozerBeanMapper.map(ib, GUIInsurance.class);

                if (ib.getEffectiveDate() != null)
                    guiIb.setEffectiveDate(Utils.formatDisplayDate(ib.getEffectiveDate()));
                if (ib.getExpirationDate() != null)
                    guiIb.setExpirationDate(Utils.formatDisplayDate(ib.getExpirationDate()));
                if (ib.getCompanyTelecom() != null && ib.getCompanyTelecom().size() > 0) {
                    guiIb.setCompanyPhone(ib.getCompanyTelecom().get(0).getValue());
                }

                if ("VLER".equalsIgnoreCase(ib.getSite().getName())) {
                    guiIb.setCompanyName(ib.getPlanName());
                }

                guiInsurances.add(guiIb);

            }

            return guiInsurances;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }
}