/*
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *             Honorable Senator Daniel K. Inouye
 *             VA Pacific Islands Health Care System
 *             Tripler Army Medical Center
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at:
 *
 *            http://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and limitations under the License.
 */


package gov.va.med.dao;

import gov.va.med.guibeans.*;
import gov.va.med.jmeadows.webservice.*;
import gov.va.med.DNS   .common.Utils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 *
 *
 */
@Component
public class NoteDao extends DaoBase {

    private static final Logger logger = Logger.getLogger(NoteDao.class.getName());

    public List<GUIProgressNote> getPatientDischargeSummaries(GUIUser user,
                                                              GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            ProgressNote[] inpatientNotes = getServiceHandler()
                    .getPatientDischargeSummaries(
                            jMeadowsQueryBuilderFactory.getBuilder()
                                    .user(user)
                                    .patient(patient)
                                    .startDate(startDate)
                                    .endDate(endDate)
                                    .build());

            List<GUIProgressNote> guiProgressNotes = new ArrayList<GUIProgressNote>();

            for (int i = 0; i < inpatientNotes.length; i++) {
                ProgressNote note = inpatientNotes[i];
                GUIProgressNote guiNote = mapToGUIProgressNote(note);


                //remove location until we get proper location from DES ** 20151016 see emails from this date
                //we already show DES data locations in the Site hover-over
                guiNote.setLocation("");

                if ("UNKNOWN".equalsIgnoreCase(guiNote.getNoteType())) {
                    String noteType = guiNote.getLocation();
                    guiNote.setLocation("");
                    guiNote.setNoteType(noteType);
                }

                //cache notes from the DoD adapter
                if (appConfig.getProtocolDODAdapter().equals(guiNote.getSourceProtocol())) {
//                    if ("CDA".equalsIgnoreCase(guiNote.getStatus()))
//                    {

                    if("plaintext".equalsIgnoreCase(guiNote.getStatus())){
                        guiNote.setNoteAsBinaryStream(false);
                        guiNote.setNoteAsPlainText(true);
                    } else {
                        guiNote.setNoteAsBinaryStream(true);
                    }
//                        //map essentris note title to location value
//                        guiNote.setLocation(guiNote.getNoteTitle());
//                    }


                }

                guiNote.setNoteId(appConfig.getCachedTag() + i);
                guiNote.setSessionCachedRecord(true);

                guiProgressNotes.add(guiNote);
            }

            return guiProgressNotes;

        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public List<GUIProgressNote> getPatientProgressNotes(GUIUser user,
                                                         GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            ProgressNote[] notes = getServiceHandler().getPatientProgressNotes(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());
            List<GUIProgressNote> guiNotes = new ArrayList<GUIProgressNote>();

            for (int i = 0; i < notes.length; i++) {
                ProgressNote note = notes[i];
                GUIProgressNote guiNote = mapToGUIProgressNote(note);

                //cache notes from the DoD adapter
                if (appConfig.getProtocolDODAdapter().equals(guiNote.getSourceProtocol())) {
                    guiNote.setSessionCachedRecord(true);
                    guiNote.setNoteId(appConfig.getCachedTag() + i);

                    if (StringUtils.isNotBlank(guiNote.getComplexDataUrl())) {
                        guiNote.setNoteAsBinaryStream(true);

                        guiNote.setNoteAsPlainText(StringUtils.isNotBlank(guiNote.getNoteText()) ||
                                StringUtils.isNotBlank(guiNote.getImages()));
                    }
                } else {
                    guiNote.setNoteId(note.getNoteId());
                }

                guiNotes.add(guiNote);
            }

            return guiNotes;

        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    public GUIProgressNote getProgressNote(GUIUser user, GUIPatient patient,
                                           String noteId, String recordSiteCode) {
        try {
            FreeTextReport report = getServiceHandler().getProgressNote(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .itemId(noteId)
                            .recordSiteCode(recordSiteCode)
                            .build());
            GUIProgressNote progressNote = new GUIProgressNote();
            if (report != null) {
                progressNote.setNoteText(report.getReportText());
                addSiteDataToGUIBean(progressNote, report);
            }


            return progressNote;

        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    private GUIProgressNote mapToGUIProgressNote(ProgressNote note) {

        GUIProgressNote guiNote = dozerBeanMapper.map(note, GUIProgressNote.class);

        guiNote.setNoteDate(Utils.formatDisplayDateTime(note.getNoteDate()));
        guiNote.setVisitAdm(note.getVisitDate());

        //check for DoD adapter sensitive note flag
        if (guiNote.isSensitive() || (guiNote.getNoteTitle() != null &&
                appConfig.getProtocolDODAdapter().equalsIgnoreCase(note.getSourceProtocol()) &&
                guiNote.getNoteTitle().startsWith("**"))) {
            guiNote.setSensitive(true);
            guiNote.setNoteTitle("** Sensitive **");
//            guiNote.setNoteText("");
//            guiNote.setImages("");
//            guiNote.setNoteType("");
//            guiNote.setComplexDataUrl("");
//            guiNote.setNoteId("");
//            guiNote.setCodes(null);
        }

        String providerName = "";

        if (note.getProvider() != null) {
            providerName = note.getProvider().getName();

        }

        guiNote.setProviderName(providerName);
        if (note.getProvider() != null) {
            guiNote.setProvider(dozerBeanMapper.map(note.getProvider(), GUIProvider.class));
        }


        guiNote.setNoteAsPlainText(true);

        //if DOD complex note
        if (appConfig.getProtocolDODAdapter().equalsIgnoreCase(note.getSourceProtocol()) &&
                StringUtils.isNotBlank(guiNote.getComplexDataUrl())) {
            guiNote.setNoteAsBinaryStream(true);

            //DOD complex notes should not display plain text note
            guiNote.setNoteAsPlainText(false);
        }

        guiNote.setEventId(note.getCdrEventId());

        return guiNote;
    }

    public List<GUIConsult> getPatientConsults(GUIUser user,
                                               GUIPatient patient, Calendar startDate, Calendar endDate) {
        try {
            Consult[] consults = getServiceHandler().getPatientConsultRequests(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());
            List<GUIConsult> guiConsults = new ArrayList<GUIConsult>();
            String requestDate;
            StringBuilder consultId = new StringBuilder();
            StringBuilder outCode = new StringBuilder();
            String outDescription;
            String outStatus;
            String consultReport;
            String service;
            String newConsultId;
            String authNum;
            Consult consult;

            for (int i = 0; i < consults.length; i++) {
                consult = consults[i];
                requestDate = (consult.getRequestDate() != null) ? Utils.formatDisplayDate(consult.getRequestDate()) : "";
                consultReport = (consult.getReport() != null) ? consult.getReport() : "";
                outDescription = (consult.getProcedureConsult() != null) ? consult.getProcedureConsult() : "";
                service = (consult.getService() != null) ? consult.getService() : "";

                consultId.setLength(0);
                outCode.setLength(0);
                if (consult.getService() != null) {
                    if (StringUtils.isBlank(service)) {
                        outCode.append("click to view consult");
                    } else {
                        outCode.append(consult.getService());
                    }
                }
                outStatus = (consult.getStatus() != null) ? consult.getStatus() : "";
                newConsultId = (consult.getId() != null) ? consult.getId() : "";

                authNum = Utils.getPiece(consult.getId(), "^", 2);
                if (StringUtils.isNotEmpty(authNum)) {
                    outCode.append("  #");
                    outCode.append(authNum);
                }

                GUIConsult guiConsult = new GUIConsult();
                //if rpc or dod adapter protocol then flag record for caching
                if (appConfig.getProtocolDODAdapter().equals(consult.getSourceProtocol())) {
                    guiConsult.setSessionCachedRecord(true);
                    consultId.append(appConfig.getCachedTag());
                    consultId.append(i);
                } else {
                    consultId.append(newConsultId);
                }

                guiConsult.setConsultId(consultId.toString());
                guiConsult.setOutCode(outCode.toString());
                guiConsult.setOutDescription(outDescription);
                guiConsult.setOutStatus(outStatus);
                guiConsult.setRequestDate(requestDate);
                guiConsult.setReport(consultReport);
                guiConsult.setService(service);
                guiConsult.setComplexDataUrl(consult.getComplexDataUrl());

                if (consult.getProvider() != null) {

                    guiConsult.setProvider(dozerBeanMapper.map(consult.getProvider(), GUIProvider.class));
                }


                //if (consultReport.contains("CISURL"))
                if (appConfig.getProtocolDODAdapter().equalsIgnoreCase(consult.getSourceProtocol()) &&
                        StringUtils.isNotBlank(guiConsult.getComplexDataUrl())) {
                    guiConsult.setNoteAsBinaryStream(true);
                }
                addSiteDataToGUIBean(guiConsult, consult);
                guiConsults.add(guiConsult);
            }
            return guiConsults;
        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public GUIConsult getConsultReport(GUIUser user, GUIPatient patient,
                                       String consultId, String recordSiteCode) {
        try {
            FreeTextReport[] reports = getServiceHandler()
                    .getConsultReport(

                            jMeadowsQueryBuilderFactory.getBuilder()
                                    .user(user)
                                    .patient(patient)
                                    .itemId(consultId)
                                    .recordSiteCode(recordSiteCode)
                                    .build());
            GUIConsult consult = new GUIConsult();

            if (reports != null) {

                StringBuilder reportResultSB = new StringBuilder("");
                //int i = 0;
                for (FreeTextReport report : reports) {
                    String line = report.getReportText();
                    if (line != null) {
                        line = line.replaceAll("<", "&lt;");
                        line = line.replaceAll(">", "&gt;");
                    } else {
                        line = "";
                    }
                    reportResultSB.append(line + "\n");
                }

                consult.setReport(reportResultSB.toString());
                if (reports.length > 0) {
                    addSiteDataToGUIBean(consult, reports[0]);
                }
            }

            return consult;

        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw new DaoException(e);
        }
    }

    public List<GUIConsult> getProviderConsultsRequested(
            GUIUser user,
            Calendar startDate,
            Calendar endDate,
            byte[] encryptSecurityKey) {
        try {
            Consult[] consults = getServiceHandler().getProviderConsultsRequested(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIConsult> guiConsults = new ArrayList<GUIConsult>();
            StringBuilder consultId = new StringBuilder();
            StringBuilder outCode = new StringBuilder();

            for (int i = 0; i < consults.length; i++) {
                Consult consult = consults[i];

                String requestDate = "";
                consultId.setLength(0);
                outCode.setLength(0);
                String outDescription = "";
                String outStatus = "";
                String consultReport = "";
                String service = "";
                String patName = "";
                String patToken = "";

                if (consult.getRequestDate() != null) {
                    requestDate = Utils.formatDisplayDate(consult
                            .getRequestDate());
                }

                if (consult.getService() != null) {
                    outCode.append(consult.getService());
                }

                if (consult.getReport() != null) {
                    consultReport = consult.getReport();
                }

                if (consult.getProcedureConsult() != null) {
                    outDescription = consult.getProcedureConsult();
                }

                if (consult.getService() != null) {
                    service = consult.getService();
                }

                if (consult.getStatus() != null) {
                    outStatus = consult.getStatus();
                }

                String newConsultId = "";
                if (consult.getId() != null) {
                    newConsultId = consult.getId();
                }

                String authNum = Utils.getPiece(consult.getId(), "^", 2);
                if (StringUtils.isNotEmpty(authNum)) {
                    outCode.append("  #");
                    outCode.append(authNum);
                }

                if (consult.getPatientName() != null) {
                    patName = consult.getPatientName();
                }

                if (appConfig.getAgencyDOD().equalsIgnoreCase(consult.getSite().getAgency())) {
                    patToken = generatePatientToken(encryptSecurityKey, appConfig.getIdTypeEDIPI(), consult.getPatientId());
                } else if (appConfig.getAgencyVA().equalsIgnoreCase(consult.getSite().getAgency())) {
                    patToken = tokenUtils.generatePatientToken(encryptSecurityKey, appConfig.getIdTypeVAIen(),
                            consult.getPatientId(), consult.getSite().getSiteCode());
                }

                GUIConsult guiConsult = new GUIConsult();
                if (appConfig.getAgencyDOD().equalsIgnoreCase(consult.getSite().getAgency()) &&
                        appConfig.getProtocolCache().equals(consult.getSourceProtocol())) {
                    guiConsult.setSessionCachedRecord(true);
                    consultId.append(appConfig.getCachedTag());
                    consultId.append(i);
                } else {
                    consultId.append(newConsultId);
                }

                guiConsult.setConsultId(consultId.toString());
                guiConsult.setOutCode(outCode.toString());
                guiConsult.setOutDescription(outDescription);
                guiConsult.setOutStatus(outStatus);
                guiConsult.setRequestDate(requestDate);
                guiConsult.setReport(consultReport);
                guiConsult.setService(service);

                if (consultReport.contains("CISURL")) {
                    guiConsult.setNoteAsBinaryStream(true);
                }

                addSiteDataToGUIBean(guiConsult, consult);

                guiConsult.setPatientName(patName);
                guiConsult.setPatientToken(patToken);

                guiConsults.add(guiConsult);
            }

            return guiConsults;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public List<GUIConsult> getProviderConsultsReceived(
            GUIUser user,
            Calendar startDate,
            Calendar endDate,
            byte[] encryptSecurityKey) {
        try {
            Consult[] consults = getServiceHandler().getProviderConsultsReceived(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());

            List<GUIConsult> guiConsults = new ArrayList<GUIConsult>();
            StringBuilder consultId = new StringBuilder();
            StringBuilder outCode = new StringBuilder();

            for (int i = 0; i < consults.length; i++) {
                Consult consult = consults[i];

                String requestDate = "";
                consultId.setLength(0);
                outCode.setLength(0);
                String outDescription = "";
                String outStatus = "";
                String consultReport = "";
                String service = "";
                String patName = "";
                String patToken = "";

                if (consult.getRequestDate() != null) {
                    requestDate = Utils.formatDisplayDate(consult
                            .getRequestDate());
                }

                if (consult.getService() != null) {
                    outCode.append(consult.getService());
                }

                if (consult.getReport() != null) {
                    consultReport = consult.getReport();
                }

                if (consult.getProcedureConsult() != null) {
                    outDescription = consult.getProcedureConsult();
                }

                if (consult.getService() != null) {
                    service = consult.getService();
                }

                if (consult.getStatus() != null) {
                    outStatus = consult.getStatus();
                }

                String newConsultId = "";
                if (consult.getId() != null) {
                    newConsultId = consult.getId();
                }

                String authNum = Utils.getPiece(consult.getId(), "^", 2);
                if (StringUtils.isNotEmpty(authNum)) {
                    outCode.append("  #");
                    outCode.append(authNum);
                }

                if (consult.getPatientName() != null) {
                    patName = consult.getPatientName();
                }

                if (appConfig.getAgencyDOD().equalsIgnoreCase(consult.getSite().getAgency())) {
                    patToken = generatePatientToken(encryptSecurityKey, appConfig.getIdTypeEDIPI(), consult.getPatientId());
                } else if (appConfig.getAgencyVA().equalsIgnoreCase(consult.getSite().getAgency())) {
                    patToken = tokenUtils.generatePatientToken(encryptSecurityKey, appConfig.getIdTypeVAIen(),
                            consult.getPatientId(), consult.getSite().getSiteCode());
                }

                GUIConsult guiConsult = new GUIConsult();
                if (appConfig.getAgencyDOD().equalsIgnoreCase(consult.getSite().getAgency()) &&
                        appConfig.getProtocolCache().equals(consult.getSourceProtocol())) {
                    guiConsult.setSessionCachedRecord(true);
                    consultId.append(appConfig.getCachedTag());
                    consultId.append(i);
                } else {
                    consultId.append(newConsultId);
                }

                guiConsult.setConsultId(consultId.toString());
                guiConsult.setOutCode(outCode.toString());
                guiConsult.setOutDescription(outDescription);
                guiConsult.setOutStatus(outStatus);
                guiConsult.setRequestDate(requestDate);
                guiConsult.setReport(consultReport);
                guiConsult.setService(service);

                if (consultReport.contains("CISURL")) {
                    guiConsult.setNoteAsBinaryStream(true);
                }

                addSiteDataToGUIBean(guiConsult, consult);

                guiConsult.setPatientName(patName);
                guiConsult.setPatientToken(patToken);

                guiConsults.add(guiConsult);
            }

            return guiConsults;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

    public List<GUIProgressNote> getProviderUnsignedNotes(
            GUIUser user,
            Calendar startDate,
            Calendar endDate,
            byte[] encryptSecurityKey) {
        try {
            ProgressNote[] notes = getServiceHandler().getProviderUnsignedNotes(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .startDate(startDate)
                            .endDate(endDate)
                            .build());
            List<GUIProgressNote> guiNotes = new ArrayList<GUIProgressNote>();

            for (int i = 0; i < notes.length; i++) {
                ProgressNote note = notes[i];
                GUIProgressNote guiNote = mapToGUIProgressNote(note);

                String patToken = getProvQueryPatientToken(encryptSecurityKey, note);

                guiNote.setPatientToken(patToken);

                boolean isVistaCache = appConfig.getAgencyVA().equalsIgnoreCase(guiNote.getSite().getAgency()) &&
                        appConfig.getProtocolCache().equals(guiNote.getSourceProtocol());

                // all non-vista cache notes are delivered with initial call
                // need to be cached
                if (!isVistaCache) {
                    guiNote.setSessionCachedRecord(true);

                    guiNote.setNoteId(appConfig.getCachedTag() + i);

                    //if CHCS record
                    if (appConfig.getAgencyDOD().equalsIgnoreCase(guiNote.getSite().getAgency()) &&
                            appConfig.getProtocolCache().equals(guiNote.getSourceProtocol())) {
                        String format = guiNote.getStatus();

                        //if format is null, assume format is text
                        if (format != null && !(format.equalsIgnoreCase("text") || format.equalsIgnoreCase("txt"))) {
                            guiNote.setNoteAsBinaryStream(true);
                        }
                    }
                }


                guiNotes.add(guiNote);
            }

            return guiNotes;

        } catch (Exception e) {
            throw new DaoException(e);
        }
    }

    /**
     * Fetches a BHIE note image.
     *
     * @param user    GUI user
     * @param eventId Note event id
     * @return Note Image
     */
    public NoteImage getBHIENoteImage(GUIUser user, String eventId) {
        try {
            return getServiceHandler().getBHIENoteImage(

                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .itemId(eventId)
                            .build());
        } catch (Exception e) {
            throw new DaoException(e);
        }
    }
}
