package gov.va.med.dao;


import gov.va.med.guibeans.GUIPatient;
import gov.va.med.guibeans.GUIProcedure;
import gov.va.med.guibeans.GUIUser;
import gov.va.med.jmeadows.webservice.Procedure;
import gov.va.med.DNS   .common.Utils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

@Component
public class ProcedureDao extends DaoBase {

    /**
     * Retrieves a list of patient procedures from all available patient locations.
     *
     * @param user    {@link gov.va.med.guibeans.GUIUser} making the request.
     * @param patient {@link gov.va.med.guibeans.GUIPatient} patient retrieving procedures from this patient record.
     * @return A list of patient procedures.
     */
    public List<GUIProcedure> getPatientProcedures(GUIUser user, GUIPatient patient, Calendar startDate, Calendar endDate, boolean includeVLER) {
        try {
            Procedure[] procedures = getServiceHandler().getPatientProcedures(
                    jMeadowsQueryBuilderFactory.getBuilder()
                            .user(user)
                            .patient(patient)
                            .startDate(startDate)
                            .endDate(endDate)
                            .includeVLER(includeVLER)
                            .build());

            List<GUIProcedure> guiProcedures = new ArrayList<GUIProcedure>();

            for (Procedure procedure : procedures) {
                GUIProcedure guiProcedure = dozerBeanMapper.map(procedure, GUIProcedure.class);
                if (procedure.getDate() != null)
                    guiProcedure.setDate(Utils.formatDisplayDate(procedure.getDate()));


                guiProcedures.add(guiProcedure);


            }


            return guiProcedures;

        } catch (Exception ex) {
            throw new DaoException(ex);
        }
    }

}
