package gov.va.med.jmeadows;

import gov.va.med.jmeadows.webservice.*;
import org.springframework.stereotype.Component;

import javax.xml.soap.SOAPException;
import java.util.List;

@Component
public interface JMeadowsServiceHandler {

    //User loginEnterprise(String cardID, String siteCode, String accessCode, String verifyCode, String ipAddress, String subjectDN) throws SOAPException;
    User loginEnterprise(String cardID, String siteCode, String accessCode, String verifyCode, String ipAddress, String subjectDN, String emailAddress) throws SOAPException;

    AdmissionDetail getAdmissionDetails(JMeadowsQuery queryBean) throws SOAPException;

    Diagnosis[] getAdmissionDiagnosis(JMeadowsQuery queryBean) throws SOAPException;

    Procedure[] getAdmissionProcedures(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getClinicalReminderDetail(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport[] getConsultReport(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getMedicationDetails(JMeadowsQuery queryBean) throws SOAPException;

    LabResult[] getLabOrderResult(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getOrderDetail(JMeadowsQuery queryBean) throws SOAPException;

    PatientAdmission[] getPatientAdmissions(JMeadowsQuery queryBean) throws SOAPException;

    Allergy[] getPatientAllergies(JMeadowsQuery queryBean) throws SOAPException;

    AllergyDetail getAllergyDetail(JMeadowsQuery queryBean) throws SOAPException;

    PatientAppointments[] getPatientAppointments(JMeadowsQuery queryBean) throws SOAPException;

    Encounter[] getPatientEncounters(JMeadowsQuery queryBean) throws SOAPException;

    ClinicalReminder[] getPatientClinicalReminders(JMeadowsQuery queryBean) throws SOAPException;

    Consult[] getPatientConsultRequests(JMeadowsQuery queryBean) throws SOAPException;

    Vitals[] getPatientCurrentVitals(JMeadowsQuery queryBean) throws SOAPException;

    List<PatientDemographicsDetail> getAllPatientDemographicDetails(JMeadowsQuery queryBean) throws SOAPException;

    List<PatientDemographics> getPatientDemographics(JMeadowsQuery queryBean) throws SOAPException;

    ProgressNote[] getPatientDischargeSummaries(JMeadowsQuery queryBean) throws SOAPException;

    Immunization[] getPatientImmunizations(JMeadowsQuery queryBean) throws SOAPException;

    InsuranceBean[] getPatientInsurances(JMeadowsQuery queryBean) throws SOAPException;

    LabOrder[] getPatientLabs(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getPatientLabReport(JMeadowsQuery queryBean) throws SOAPException;

    LabResult[] getPatientLabTestResults(JMeadowsQuery queryBean) throws SOAPException;

    LabResult[] getPatientLabResults(JMeadowsQuery queryBean) throws SOAPException;

    Medication[] getPatientMedications(JMeadowsQuery queryBean) throws SOAPException;

    Order[] getPatientOrders(JMeadowsQuery queryBean) throws SOAPException;

    Problem[] getPatientProblemList(JMeadowsQuery queryBean) throws SOAPException;

    ProgressNote[] getPatientProgressNotes(JMeadowsQuery queryBean) throws SOAPException;

    Procedure[] getPatientProcedures(JMeadowsQuery queryBean) throws SOAPException;

//    QuestionnaireBean[] getPatientQuestionnaires(JMeadowsQuery queryBean) throws SOAPException;

    /**
     * Fetches a BHIE note image.
     *
     * @param queryBean required params: provider, itemid (containing bhie note event id)
     * @return Note image
     * @throws javax.xml.soap.SOAPException if an error occurs.
     */
    NoteImage getBHIENoteImage(JMeadowsQuery queryBean) throws SOAPException;

    VlerDocument getVLERDocument(JMeadowsQuery queryBean, VlerDocument vDoc) throws SOAPException;

    VlerDocument[] getVLERDocumentList(JMeadowsQuery queryBean) throws SOAPException;

    RadiologyReport[] getPatientRads(JMeadowsQuery queryBean) throws SOAPException;

    Vitals[] getPatientVitals(JMeadowsQuery queryBean) throws SOAPException;

    ProblemDetail getProblemDetail(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getProgressNote(JMeadowsQuery queryBean) throws SOAPException;

    User getProviderProfile(JMeadowsQuery queryBean) throws SOAPException;

    PatientHistory[] getPatientHistories(JMeadowsQuery queryBean) throws SOAPException;

    RadiologyReport getRadiologyReport(JMeadowsQuery queryBean) throws SOAPException;

    Alert[] getUserAlerts(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getVisitNotes(JMeadowsQuery queryBean) throws SOAPException;

    FreeTextReport getPatientEncountersReport(JMeadowsQuery queryBean) throws SOAPException;

    Patient[] lookupPatient(JMeadowsQuery queryBean) throws SOAPException;

    ResponsePatientQuery lookupPatientPDWS(PdwsQueryBean queryBean) throws SOAPException;

    Patient selectPatient(JMeadowsQuery queryBean) throws SOAPException;

    ResponsePatientSelect selectPatientMVI(JMeadowsQuery queryBean) throws SOAPException;

    void selectPatientForVASensitive(JMeadowsQuery queryBean) throws SOAPException;

    void auditVARestrictedAccess(JMeadowsQuery queryBean) throws SOAPException;

    void auditSensitiveRecordAccess(JMeadowsQuery queryBean) throws SOAPException;

    void logAudit(String siteId, String userId, String providerId, String userNPI,
                  String userName, String patId, String category,
                  String startDate, String endDate,String info, String cardId, String emailAddress) throws SOAPException;

//    void logAudit(String siteId, String userId, String providerId, String userNPI,
//                  String userName, String patId, String category,
//                  String startDate, String endDate) throws SOAPException;

    PatientAdmission[] getProviderAdmissions(JMeadowsQuery queryBean) throws SOAPException;

    PatientAppointments[] getProviderAppointments(JMeadowsQuery queryBean) throws SOAPException;

    Consult[] getProviderConsultsReceived(JMeadowsQuery queryBean) throws SOAPException;

    Consult[] getProviderConsultsRequested(JMeadowsQuery queryBean) throws SOAPException;

    LabResult[] getProviderLabAbnormalResults(JMeadowsQuery queryBean) throws SOAPException;

    Order[] getProviderOrdersPending(JMeadowsQuery queryBean) throws SOAPException;

    Order[] getProviderOrdersResulted(JMeadowsQuery queryBean) throws SOAPException;

    ProgressNote[] getProviderUnsignedNotes(JMeadowsQuery queryBean) throws SOAPException;

    boolean setJanusGUIConfig(JMeadowsQuery queryBean, String cfg) throws SOAPException;

    String getJanusGUIConfig(JMeadowsQuery queryBean) throws SOAPException;

    boolean setProviderFlags(JMeadowsQuery queryBean, String flags) throws SOAPException;

    String getProviderFlags(JMeadowsQuery queryBean) throws SOAPException;

//    IehrUserProfile getIehrUserProfile(String smartCardID, String smartCardAgency) throws SOAPException;

    IehrUserProfile getIehrUserProfile(String smartCardID, String smartCardAgency, String emailAddress) throws SOAPException;

    boolean setIehrUserProfile(IehrUserProfile userProfile) throws SOAPException;

    String getAppName();

    void setAppName(String appName);

    List<Site> getSites() throws SOAPException;

    List<AuditEntry> getLoginInfo(String cardID, String emailAddress) throws SOAPException;

    String getServiceErrors() throws SOAPException;

    AuthUserInfo getAuthUser(AuthUserInfo authUserInfoQuery) throws SOAPException;

    List<Patient> getRecentlyViewedPatients(String userId) throws SOAPException;

    Form[] getPatientForms(JMeadowsQuery queryBean) throws SOAPException;

    Vlerccda[] getDODVLERPatientCCDA(JMeadowsQuery queryBean) throws SOAPException;

    List<OutsideProvider> getOutsideProviderList(JMeadowsQuery queryBean, String searchName) throws SOAPException;

    OutsideProvider getOutsideProvider(JMeadowsQuery queryBean, String username, String password) throws SOAPException;

    void addOutsideProvider(JMeadowsQuery queryBean, OutsideProvider outsideProvider) throws SOAPException;

    void deleteOutsideProvider(JMeadowsQuery queryBean, String id) throws SOAPException;


    List<AssignedPatient> getAssignedPatientList(JMeadowsQuery queryBean, String providerId) throws SOAPException;

    void addAssignedPatient(JMeadowsQuery queryBean, AssignedPatient assignedPatient) throws SOAPException;

    void deleteAssignedPatient(JMeadowsQuery queryBean, String assignedPatientId) throws SOAPException;

    void addAssignedPatientUsage(JMeadowsQuery queryBean, String providerId, String assignedPatientId) throws SOAPException;

    List<Consult> getPatientActiveConsultRequest(JMeadowsQuery queryBean) throws SOAPException;

    void updateOutsideProvider(JMeadowsQuery queryBean, OutsideProvider provider) throws SOAPException;

    OutsideProvider getOutsideProviderExist(JMeadowsQuery queryBean, String username) throws SOAPException;





}
