/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.base.dao.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.VistaQueryFactory;
import gov.va.med.rpc.VistaQueryFactoryUtil;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.vistabeans.DataBean;
import gov.va.med.vds.vistabeans.PatientDemographics;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.io.StringReader;

/**
 *
 *
 */
public abstract class BaseRPC {
    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();

    protected static final String PROTOCOL_RPC = APP_CONFIG.getProtocolRPC();
    protected static final String AGENCY_VA = APP_CONFIG.getAgencyVA();

    private static final Logger LOGGER = LoggerFactory.getLogger(BaseRPC.class.getName());

    protected RPCConnection cxn;

    protected VistaQueryFactory vistaQueryFactory;

    ErrorHandler xmlParserErrorHandler = new ErrorHandler() {
        public void fatalError(SAXParseException exception) throws SAXException {
            LOGGER.debug(exception.getLocalizedMessage());
        }

        public void error(SAXParseException exception) throws SAXException {
            LOGGER.debug(exception.getLocalizedMessage());
        }

        public void warning(SAXParseException exception) throws SAXException {
            LOGGER.debug(exception.getLocalizedMessage());
        }
    };

    public BaseRPC(RPCConnection cxn) {
        this.cxn = cxn;

        if (!cxn.isConnected() || cxn.getErrorMsg() != null) {
            String siteStr = "";

            if (cxn.getSite() != null) {
                siteStr = "SiteCode: " + cxn.getSite().getSiteCode() + ", SiteName: " + cxn.getSite().getName();
            }

            String errMsg = "Bad RPC Connection";

            if (!siteStr.isEmpty()) {
                errMsg += " (" + siteStr + ")";
            }

            if (cxn.getErrorMsg() != null) {
                errMsg += ", Error Message: " + cxn.getErrorMsg();
            }

            throw new IllegalArgumentException(errMsg);
        }

        vistaQueryFactory = VistaQueryFactoryUtil.getVistaQueryFactory(cxn.getConnectionStrategy());
    }

    protected void fillCommonDataBeanValues(DataBean bean) {
        bean.setSourceProtocol(PROTOCOL_RPC);
        bean.setSite(cxn.getSite());
    }

    protected void fillCommonDataBeanValues(PatientDemographics bean) {
        bean.setSourceProtocol(PROTOCOL_RPC);
        bean.setSite(cxn.getSite());
    }

    protected Document parseXMLDocument(String xmlResponse) throws ParserConfigurationException, IOException, SAXException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(true);
        factory.setFeature("http://javax.xml.XMLConstants/feature/secure-processing", true);
        factory.setFeature("http://xml.org/sax/features/external-general-entities", false);
        factory.setFeature("http://xml.org/sax/features/external-parameter-entities", false);

        DocumentBuilder builder = factory.newDocumentBuilder();
        builder.setErrorHandler(xmlParserErrorHandler);

        InputSource ins = new InputSource(new StringReader(xmlResponse));

        return builder.parse(ins);
    }
}
