/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.base.dataservice;

import gov.va.med.foundations.utilities.Messages;
import gov.va.med.rpc.KeyValuePair;
import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.VistaConstants;
import gov.va.med.rpc.site.VistaSiteException;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.vistabeans.Endpoint;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 *
 *
 */
public abstract class DataServiceBase {

    public static final String CONNECTION_UNAVAILABLE = "Connection unavailable.";
    public static final String UNABLE_TO_RETRIEVE = "Currently, this type of record retrieval is unavailable at this site.";
    public static final String UNKNOWN_PATIENT_MSG = "Unknown, Patient";

    protected static final AppConfig APP_CONFIG = AppConfig.getInstance();
    protected static final String AGENCY_VA = APP_CONFIG.getAgencyVA();


//    private static final Logger LOGGER = LoggerFactory.getLogger(DataServiceBase.class);
    

    protected static VistaRPCConnectionManager connectToSite(Site site) throws VistaSiteException, RPCException
    {
        if (site == null) {
            throw new VistaSiteException("Site is null");
        }

        if (site == null) {
            throw new VistaSiteException("Site code is invalid");
        }

        VistaRPCConnectionManager cxnMgr = new VistaRPCConnectionManager(site);
        cxnMgr.connect();

        return cxnMgr;
    }

    protected static VistaRPCConnectionManager setupMultiSourceQuery(User user, List<Site> sites,
            String incontext) throws RPCException, VistaSiteException {
        String context = incontext;
        if (context == null) {
            context = VistaConstants.CPRS_CONTEXT;
        }

        VistaRPCConnectionManager cxnMgr = new VistaRPCConnectionManager(sites);
        cxnMgr.connect();

        VistaRPCConnectionManager msq = new VistaRPCConnectionManager(cxnMgr.getConnections());
        msq.execute(VistaRPCConnectionManager.EXECUTION_CLASS_USER_RPC, VistaRPCConnectionManager.EXECUTION_METHOD_USER_RPC, new Object[] {user, context });

        return msq;
    }

    protected static VistaRPCConnectionManager setupMultiSourceQuery(QueryBean queryBean) throws RPCException, ClassNotFoundException, VistaSiteException {
        return setupMultiSourceQuery(queryBean, VistaConstants.CPRS_CONTEXT);
    }

    protected static VistaRPCConnectionManager setupVPRContextMultiSourceQuery(QueryBean queryBean) throws RPCException, ClassNotFoundException, VistaSiteException {
        return setupMultiSourceQuery(queryBean, VistaConstants.VPR_CONTEXT);
    }

    private static VistaRPCConnectionManager setupMultiSourceQuery(QueryBean queryBean, String context) throws RPCException,
            ClassNotFoundException, VistaSiteException {

        User user = queryBean.getUser();

        Patient patient = queryBean.getPatient();

        VistaRPCConnectionManager cxnMgr = new VistaRPCConnectionManager(patient.getVistaSites());
        cxnMgr.connect();

        VistaRPCConnectionManager msq = new VistaRPCConnectionManager(cxnMgr.getConnections());
        msq.execute(VistaRPCConnectionManager.EXECUTION_CLASS_USER_RPC, VistaRPCConnectionManager.EXECUTION_METHOD_USER_RPC, new Object[] {user, context });

        setLocalPids(msq, patient);

        return msq;
    }

    protected static RPCConnection setupVPRContextSingleRemoteSourceQuery(QueryBean queryBean) throws RPCException, ClassNotFoundException, VistaSiteException {
        return setupSingleRemoteSourceQuery(queryBean.getUser(), queryBean.getPatient(), queryBean.getRecordSite(), VistaConstants.VPR_CONTEXT);
    }

    protected static RPCConnection setupSingleRemoteSourceQuery(QueryBean queryBean) throws RPCException, ClassNotFoundException, VistaSiteException {
        return setupSingleRemoteSourceQuery(queryBean.getUser(), queryBean.getPatient(), queryBean.getRecordSite(), VistaConstants.CPRS_CONTEXT);

    }

    protected static RPCConnection setupSingleRemoteSourceQuery(QueryBean queryBean,
            Site site) throws RPCException, ClassNotFoundException, VistaSiteException {
        return setupSingleRemoteSourceQuery(queryBean.getUser(), queryBean.getPatient(), site, VistaConstants.CPRS_CONTEXT);

    }

    private static RPCConnection setupSingleRemoteSourceQuery(User user, Patient patient,
            Site site, String context) throws RPCException, VistaSiteException {
        VistaRPCConnectionManager cxnMgr = new VistaRPCConnectionManager(site);
        cxnMgr.connect();

        VistaRPCConnectionManager msq = new VistaRPCConnectionManager(cxnMgr.getConnections());
        msq.execute(VistaRPCConnectionManager.EXECUTION_CLASS_USER_RPC, VistaRPCConnectionManager.EXECUTION_METHOD_USER_RPC, new Object[] {user, context });

        setLocalPids(msq, patient);

        return msq.getConnection(site);
    }

    protected void checkProviderQueryHostSite(User user) throws VDSException {
        boolean rtc = false;
        String errorText = "User doesn't have a Vista location or NPI.";

        if (!AGENCY_VA.equalsIgnoreCase(user.getHostSite().getAgency()) && StringUtils.isEmpty(user.getNPI())) {
            throw new VDSException(errorText);
        } else {
            //check for cache endpoint
            for (Endpoint endpoint : user.getHostSite().getEndpoints()) {
                if ("CACHE".equalsIgnoreCase(endpoint.getProtocol())) {
                    rtc = true;
                    break;
                }
            }
            if (!rtc) {
                throw new VDSException(errorText);
            }
        }
    }

    protected static RPCConnection setupCPRSContextSingleRemoteSourceQuery(User user, Site site) throws RPCException, VistaSiteException {
        return setupSingleRemoteSourceQuery(user, site, VistaConstants.CPRS_CONTEXT);
    }

    private static RPCConnection setupSingleRemoteSourceQuery(User user, Site site,
            String context) throws RPCException, VistaSiteException {

        VistaRPCConnectionManager cxnMgr = new VistaRPCConnectionManager(site);
        cxnMgr.connect();

        VistaRPCConnectionManager msq = new VistaRPCConnectionManager(cxnMgr.getConnections());
        msq.execute(VistaRPCConnectionManager.EXECUTION_CLASS_USER_RPC, VistaRPCConnectionManager.EXECUTION_METHOD_USER_RPC, new Object[] {user, context });

        return msq.getConnection(site);
    }



    private static void setLocalPids(VistaRPCConnectionManager msq, Patient patient) throws RPCException {
        //set local patient ien from patient iens key-value pair
        if (patient.getPatientIens() != null && !patient.getPatientIens().isEmpty()) {
            for (KeyValuePair<String, String> patientIenKV : patient.getPatientIens()) {
                RPCConnection cxn = msq.getConnection(patientIenKV.getKey());

                //
                if (cxn != null) {
                    cxn.setPatientIen(patientIenKV.getValue());
                }
            }

            return;
        }

        //otherwise set with ICN via RPC if key-value list is null or empty

        String mpiPid = patient.getICN();
        if (mpiPid != null && mpiPid.contains("V")) {
            mpiPid = mpiPid.substring(0, mpiPid.indexOf('V'));
        }

        LinkedHashMap<Site, Object> result = msq.execute(VistaRPCConnectionManager.EXECUTION_CLASS_PATIENT_RPC, VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_RPC, new Object[]{mpiPid});
        Set<Map.Entry<Site, Object>> entries = result.entrySet();
        for (Map.Entry<Site, Object> entry : entries) {
            Object value = entry.getValue();
            try {
                if (Class.forName("java.lang.Exception").isAssignableFrom(value.getClass()))
                {
                    continue;
                }
            } catch (ClassNotFoundException e) {
                throw new RPCException(e);
            }
            RPCConnection cxn = msq.getConnection(entry.getKey());
            cxn.setPatientIen((String) value);
        }
    }

//    /**
//     * Helper method to address edge case; when a selected patient does not have
//     * an ICN assigned limit data retrieval from the local vista instance.
//     * @param msq - the prebuilt query
//     * @param patient - the patient selected; ALL patients have an IEN/ID.
//     */
//    private static void convertToLocalQuery(VistaRPCConnectionManager msq, Patient patient)
//    {
//        LinkedHashMap<Site,RPCConnection> connectionTable = msq.getConnectionTable();
//
//        Set<Map.Entry<Site, RPCConnection>> entries = connectionTable.entrySet();
//
//        Iterator<Map.Entry<Site, RPCConnection>> entriesIterator = entries.iterator();
//        Stack<String> garbageCan = new Stack<String>();
//
//        String localVistId = patient.getPatientIdSiteCode();
//
//        while(entriesIterator.hasNext())
//        {
//            Map.Entry<Site, RPCConnection> nextEntry = entriesIterator.next();
//
//            Site site = nextEntry.getKey();
//
//            if(!localVistId.equals(site))
//            {
//                garbageCan.push(site);
//            }
//            else if(localVistId.equals(site))
//            {
//                RPCConnection temp = nextEntry.getValue();
//                temp.setPatientIen(patient.getPatientId());
//            }
//        }
//
//        while(!garbageCan.isEmpty())
//        {
//            RPCConnection temp = connectionTable.get(garbageCan.peek());
//            if(temp.isConnected())
//            {
//                try
//                {
//                    temp.disconnect();
//                } catch(RPCException ex) {
//                    logger.error(ex.getMessage());
//                }
//            }
//            connectionTable.remove(garbageCan.pop());
//        }
//
//        return;
//    }

    protected static boolean isFormedProvider(User user) {
        return user != null 
                && user.getHostSite() != null 
                && !StringUtils.isEmpty(user.getHostSite().getSiteCode()) 
                && !StringUtils.isEmpty(user.getName()) 
                && !StringUtils.isEmpty(user.getUserIen()) 
                && !StringUtils.isEmpty(user.getEncryptedFederatedUid());
    }

    protected static void authorizeProvider(VistaRPCConnectionManager cxnMgr, User user) throws RPCException, UnauthorizedProviderException
    {
        if (cxnMgr == null) {
            throw new IllegalArgumentException("Invalid Site Connection.");
        }

        if (user == null) {
            throw new IllegalArgumentException("Invalid Vista User");
        }

        String context = VistaConstants.CPRS_CONTEXT;
        UserRPC providerDao = new UserRPC(cxnMgr.loginConnection());
        String uid = providerDao.visit(user, context, Boolean.TRUE);

        if (!uid.equals(user.getUserIen())) {
            throw new UnauthorizedProviderException("Unauthorized VistA user");
        }
    }

    protected List<String> validateUser(User user, List<String> ininvalidParams) {
        List<String> invalidParams;
        if (ininvalidParams == null) {
            invalidParams = new ArrayList<String>();
        } else {
            invalidParams = new ArrayList<String>(ininvalidParams);
        }

        if (user == null) {
            invalidParams.add("User");
        } else {
            if (StringUtils.isEmpty(user.getUserIen())) {
                invalidParams.add("User IEN");
            }

            if (user.getHostSite() == null) {
                invalidParams.add("User host site");
            }

            if (StringUtils.isEmpty(user.getEncryptedFederatedUid())) {
                invalidParams.add("Federated user id");
            }
        }

        return invalidParams;
    }

    protected List<String> validatePatient(Patient patient, List<String> ininvalidParams) {
        List<String> invalidParams;
        if (ininvalidParams == null) {
            invalidParams = new ArrayList<String>();
        } else {
            invalidParams = new ArrayList<String>(ininvalidParams);
        }

        if (patient == null) {
            throw new IllegalArgumentException("Patient is null.");
        } else {
            boolean hasICN = !StringUtils.isEmpty(patient.getICN());
            boolean hasVistaPatIENs =
                    patient.getPatientIens() != null && !patient.getPatientIens().isEmpty()
                            && patient.getVistaSites() != null && !patient.getVistaSites().isEmpty();

            if (!hasICN && !hasVistaPatIENs) {
                throw new IllegalArgumentException("Missing patient identifiers");
            } else if (patient.getVistaSites() == null || patient.getVistaSites().size() <= 0) {
                throw new IllegalArgumentException("Patient site array is null or empty");
            }
        }

        return invalidParams;
    }

    protected void validateQueryBean(QueryBean queryBean) {
        List<String> invalidParams = new ArrayList<String>();

        if (queryBean == null) {
            invalidParams.add("queryBean");
        } else {
            invalidParams.addAll(validateUser(queryBean.getUser(), invalidParams));

            invalidParams.addAll(validatePatient(queryBean.getPatient(), invalidParams));
        }

        if (!invalidParams.isEmpty()) {
            throw new IllegalArgumentException(Messages.get("invalid.parameters") + invalidParams.toString());
        }
    }

    protected void validateQueryBeanDateRange(QueryBean queryBean) {
        validateQueryBean(queryBean);
        if (queryBean.getStartDate() == null) {
            throw new IllegalArgumentException("startDate is null.");
        } else if (queryBean.getEndDate() == null) {
            throw new IllegalArgumentException("endDate is null.");
        }
    }

    protected void validateQueryBeanProvider(QueryBean queryBean) {
        List<String> invalidParams = new ArrayList<String>();

        if (queryBean == null) {
            invalidParams.add("queryBean");
        } else {
            invalidParams = validateUser(queryBean.getUser(), invalidParams);
        }

        if (!invalidParams.isEmpty()) {
            throw new IllegalArgumentException(Messages.get("invalid.parameters") + invalidParams.toString());
        }
    }

    protected void queryParamsCheckProviderWithDateRange(QueryBean queryBean) {
        validateQueryBeanProvider(queryBean);
        if (queryBean.getStartDate() == null) {
            throw new IllegalArgumentException("startDate is null.");
        } else if (queryBean.getEndDate() == null) {
            throw new IllegalArgumentException("endDate is null.");
        }
    }
}
