/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.base.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionManager;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.vistabeans.Site;

import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;


/*
 *
 *
 */
public final class VistaRPCConnectionManager extends RPCConnectionManager {
    //---------------------  valid executions --------------------------------
    public static final String EXECUTION_CLASS_USER_RPC = "gov.va.med.vds.provider.dao.rpc.UserRPC";
    public static final String EXECUTION_METHOD_USER_RPC = "visitSite";

    public static final String EXECUTION_CLASS_PATIENT_RPC = "gov.va.med.vds.patient.dao.rpc.PatientRPC";
    public static final String EXECUTION_METHOD_PATIENT_RPC = "getLocalPid";

    public static final String EXECUTION_CLASS_ADMISSIONS = "gov.va.med.vds.patient.dao.AdmissionsDao";
    public static final String EXECUTION_METHOD_ADMISSION_DIAGNOSIS = "getAdmissionDiagnosis";
    public static final String EXECUTION_METHOD_ADMISSION_PROCEDURES = "getAdmissionProcedures";
    public static final String EXECUTION_METHOD_PATIENT_ADMISSIONS = "getPatientAdmissions";
    public static final String EXECUTION_METHOD_PATIENT_ADMISSIONS_ENCOUNTERS = "getPatientAdmissionsEncounters";

    public static final String EXECUTION_CLASS_ALLERGY = "gov.va.med.vds.patient.dao.AllergyDao";
    public static final String EXECUTION_METHOD_PATIENT_ALLERGIES = "getPatientAllergies";

    public static final String EXECUTION_CLASS_APPOINTMENTS = "gov.va.med.vds.patient.dao.AppointmentsDao";
    public static final String EXECUTION_METHOD_PATIENT_APPOINTMENTS = "getPatientAppointments";

    public static final String EXECUTION_CLASS_CLINICAL_REMINDERS = "gov.va.med.vds.patient.dao.ClinicalRemindersDao";
    public static final String EXECUTION_METHOD_PATIENT_CLINICAL_REMINDERS = "getPatientClinicalReminders";

    public static final String EXECUTION_CLASS_IMMUNIZATION = "gov.va.med.vds.patient.dao.ImmunizationDao";
    public static final String EXECUTION_METHOD_PATIENT_IMMUNIZATIONS = "getPatientImmunizations";

    public static final String EXECUTION_CLASS_LABS = "gov.va.med.vds.patient.dao.LabsDao";
    public static final String EXECUTION_METHOD_PATIENT_CHEM_LAB_RESULTS = "getPatientChemLabResults";
    public static final String EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS = "getPatientMicroLabResults";
    public static final String EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS = "getPatientAnaPathLabResults";
    public static final String EXECUTION_METHOD_PATIENT_CHEM_LABS = "getPatientChemLabs";
    public static final String EXECUTION_METHOD_PATIENT_MICRO_LABS = "getPatientMicroLabs";
    public static final String EXECUTION_METHOD_PATIENT_ANAPATH_LABS = "getPatientAnaPathLabs";
    public static final String EXECUTION_METHOD_PATIENT_LABS_VPR = "getPatientLabOrdersVPR";
    public static final String EXECUTION_METHOD_PATIENT_LAB_RESULTS_VPR = "getPatientLabResultsVPR";

    public static final String EXECUTION_CLASS_MEDICATIONS = "gov.va.med.vds.patient.dao.MedicationsDao";
    public static final String EXECUTION_METHOD_PATIENT_MEDICATIONS = "getPatientMedications";
    public static final String EXECUTION_METHOD_PATIENT_MEDICATIONS_NVA = "getPatientMedicationsNVA";

    public static final String EXECUTION_CLASS_NOTES = "gov.va.med.vds.patient.dao.NotesDao";
    public static final String EXECUTION_METHOD_PATIENT_CONSULT_REQUESTS = "getPatientConsultRequests";
    public static final String EXECUTION_METHOD_PATIENT_ACTIVE_CONSULT_REQUESTS = "getPatientActiveConsultRequests";
    public static final String EXECUTION_METHOD_PATIENT_DISCHARGE_SUMMARIES = "getPatientDischargeSummaries";
    public static final String EXECUTION_METHOD_PATIENT_PROGRESS_NOTES = "getPatientProgressNotes";

    public static final String EXECUTION_CLASS_ORDERS = "gov.va.med.vds.patient.dao.OrdersDao";
    public static final String EXECUTION_METHOD_PATIENT_ORDERS = "getPatientOrders";

    public static final String EXECUTION_CLASS_PATIENT_APPT = "gov.va.med.vds.patient.dao.PatientApptDao";
    public static final String EXECUTION_METHOD_VISIT_DIAGNOSIS = "getVisitDiagnosis";
    public static final String EXECUTION_METHOD_VISIT_PROCEDURES = "getVisitProcedures";

    public static final String EXECUTION_CLASS_PATIENT = "gov.va.med.vds.patient.dao.PatientDao";
    public static final String EXECUTION_METHOD_DR_REFERRALS = "getDRReferrals";
    public static final String EXECUTION_METHOD_PATIENT_DEMOGRAPHICS = "getPatientDemographics";
    public static final String EXECUTION_METHOD_PATIENT_DEMOGRAPHICS_DETAIL = "getPatientDemographicsDetail";
    public static final String EXECUTION_METHOD_SENSITIVE_PATIENT = "isSensitivePatient";
    public static final String EXECUTION_METHOD_SELECT_PATIENT = "selectPatient";

    public static final String EXECUTION_CLASS_PROBLEMS = "gov.va.med.vds.patient.dao.ProblemsDao";
    public static final String EXECUTION_METHOD_PATIENT_PROBLEM_LIST = "getPatientProblemList";

    public static final String EXECUTION_CLASS_RADIOLOGY = "gov.va.med.vds.patient.dao.RadiologyDao";
    public static final String EXECUTION_METHOD_PATIENT_RADS = "getPatientRads";

    public static final String EXECUTION_CLASS_VITALS = "gov.va.med.vds.patient.dao.VitalsDao";
    public static final String EXECUTION_METHOD_PATIENT_CURRENT_VITALS = "getPatientCurrentVitals";
    public static final String EXECUTION_METHOD_PATIENT_VITALS = "getPatientVitals";

    public static final String EXECUTION_CLASS_ENCOUNTERS = "gov.va.med.vds.patient.dao.EncountersDao";
    public static final String EXECUTION_METHOD_PATIENT_ENCOUNTERS = "getPatientEncounters";

    public static final String EXECUTION_CLASS_INSURANCE = "gov.va.med.vds.patient.dao.InsuranceDao";
    public static final String EXECUTION_METHOD_PATIENT_INSURANCE = "getPatientInsurance";

    public static final String EXECUTION_CLASS_PROCEDURES = "gov.va.med.vds.patient.dao.ProceduresDao";
    public static final String EXECUTION_METHOD_PATIENT_PROCEDURES = "getPatientProcedures";

    private static final String DOT = ".";
    private static final AppConfig APP_CONFIG = AppConfig.getInstance();
    private static final int MAX_THREAD_POOL_SIZE = APP_CONFIG.getMaxThreadPoolSize();
    private static final long THREAD_TIMEOUT_MS = APP_CONFIG.getThreadTimeoutMS();


    private static Set<String> validExecution;
    static {
        validExecution = new HashSet<String>();
        validExecution.add(EXECUTION_CLASS_USER_RPC.concat(DOT).concat(EXECUTION_METHOD_USER_RPC));
        validExecution.add(EXECUTION_CLASS_PATIENT_RPC.concat(DOT).concat(EXECUTION_METHOD_PATIENT_RPC));
        validExecution.add(EXECUTION_CLASS_ADMISSIONS.concat(DOT).concat(EXECUTION_METHOD_ADMISSION_DIAGNOSIS));
        validExecution.add(EXECUTION_CLASS_ADMISSIONS.concat(DOT).concat(EXECUTION_METHOD_ADMISSION_PROCEDURES));
        validExecution.add(EXECUTION_CLASS_ADMISSIONS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ADMISSIONS));
        validExecution.add(EXECUTION_CLASS_ADMISSIONS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ADMISSIONS_ENCOUNTERS));
        validExecution.add(EXECUTION_CLASS_ALLERGY.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ALLERGIES));
        validExecution.add(EXECUTION_CLASS_APPOINTMENTS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_APPOINTMENTS));
        validExecution.add(EXECUTION_CLASS_CLINICAL_REMINDERS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_CLINICAL_REMINDERS));
        validExecution.add(EXECUTION_CLASS_IMMUNIZATION.concat(DOT).concat(EXECUTION_METHOD_PATIENT_IMMUNIZATIONS));
//        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_CHEM_LAB_RESULTS));
//        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS));
//        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS));
//        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_CHEM_LABS));
//        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_MICRO_LABS));
//        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ANAPATH_LABS));
        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_LABS_VPR));
        validExecution.add(EXECUTION_CLASS_LABS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_LAB_RESULTS_VPR));
        validExecution.add(EXECUTION_CLASS_MEDICATIONS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_MEDICATIONS));
        validExecution.add(EXECUTION_CLASS_MEDICATIONS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_MEDICATIONS_NVA));
        validExecution.add(EXECUTION_CLASS_NOTES.concat(DOT).concat(EXECUTION_METHOD_PATIENT_CONSULT_REQUESTS));
        validExecution.add(EXECUTION_CLASS_NOTES.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ACTIVE_CONSULT_REQUESTS));
        validExecution.add(EXECUTION_CLASS_NOTES.concat(DOT).concat(EXECUTION_METHOD_PATIENT_DISCHARGE_SUMMARIES));
        validExecution.add(EXECUTION_CLASS_NOTES.concat(DOT).concat(EXECUTION_METHOD_PATIENT_PROGRESS_NOTES));
        validExecution.add(EXECUTION_CLASS_ORDERS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ORDERS));
        validExecution.add(EXECUTION_CLASS_PATIENT_APPT.concat(DOT).concat(EXECUTION_METHOD_PATIENT_PROCEDURES));
        validExecution.add(EXECUTION_CLASS_PATIENT_APPT.concat(DOT).concat(EXECUTION_METHOD_VISIT_DIAGNOSIS));
        validExecution.add(EXECUTION_CLASS_PATIENT_APPT.concat(DOT).concat(EXECUTION_METHOD_VISIT_PROCEDURES));
        validExecution.add(EXECUTION_CLASS_PATIENT.concat(DOT).concat(EXECUTION_METHOD_DR_REFERRALS));
        validExecution.add(EXECUTION_CLASS_PATIENT.concat(DOT).concat(EXECUTION_METHOD_PATIENT_DEMOGRAPHICS));
        validExecution.add(EXECUTION_CLASS_PATIENT.concat(DOT).concat(EXECUTION_METHOD_PATIENT_DEMOGRAPHICS_DETAIL));
        validExecution.add(EXECUTION_CLASS_PATIENT.concat(DOT).concat(EXECUTION_METHOD_SENSITIVE_PATIENT));
        validExecution.add(EXECUTION_CLASS_PATIENT.concat(DOT).concat(EXECUTION_METHOD_SELECT_PATIENT));
        validExecution.add(EXECUTION_CLASS_PROBLEMS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_PROBLEM_LIST));
        validExecution.add(EXECUTION_CLASS_RADIOLOGY.concat(DOT).concat(EXECUTION_METHOD_PATIENT_RADS));
        validExecution.add(EXECUTION_CLASS_VITALS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_CURRENT_VITALS));
        validExecution.add(EXECUTION_CLASS_VITALS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_VITALS));
        validExecution.add(EXECUTION_CLASS_ENCOUNTERS.concat(DOT).concat(EXECUTION_METHOD_PATIENT_ENCOUNTERS));
        validExecution.add(EXECUTION_CLASS_INSURANCE.concat(DOT).concat(EXECUTION_METHOD_PATIENT_INSURANCE));
        validExecution.add(EXECUTION_CLASS_PROCEDURES.concat(DOT).concat(EXECUTION_METHOD_PATIENT_PROCEDURES));

    }
    //---------------------------------------------------------------------------


    public VistaRPCConnectionManager(LinkedHashMap<Site, RPCConnection> cxnTable)
    {
        super(cxnTable);

    }

    public VistaRPCConnectionManager(RPCConnection cxn) {
        super(cxn);

        init();
    }

    public VistaRPCConnectionManager(Site site) {
        super(site);

        init();
    }

    public VistaRPCConnectionManager(List<Site> sites) {
        super(sites);

        init();
    }

    private void init() {
        setThreadTimeoutMS(THREAD_TIMEOUT_MS);
        setMaxThreadPoolSize(MAX_THREAD_POOL_SIZE);
    }

    public static LinkedHashMap<Site, Object> execute(LinkedHashMap<Site, RPCConnection> cxnTbl, String daoName, String methodName, Object[] args) throws RPCException {
        if (!isValidExecution(daoName, methodName)) {
            throw new RPCException("Invalid Execution.  class: " + daoName + " method: " + methodName);
        }
        return RPCConnectionManager.execute(cxnTbl, daoName, methodName, args,
                THREAD_TIMEOUT_MS, MAX_THREAD_POOL_SIZE);
    }

    public LinkedHashMap<Site, Object> execute(String className, String methodName, Object[] args) throws RPCException {
        if (!isValidExecution(className, methodName)) {
            throw new RPCException("Invalid Execution.  class: " + className + " method: " + methodName);
        }
        return super.execute(className, methodName, args);
    }

    private static boolean isValidExecution(String className, String methodName) {
        boolean rtc = false;
        if (!StringUtils.isEmpty(className) && !StringUtils.isEmpty(methodName)) {
            rtc = validExecution.contains(className.concat(DOT).concat(methodName));
        }
        return rtc;
    }

}
