/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.common;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * User: DNS   UYEHAC
 * Date: 8/29/11
 * Time: 2:36 PM
 */
public class AppConfig {
    private static final Logger LOGGER = LoggerFactory.getLogger(AppConfig.class);

    private static final String THREAD_TIMEOUT_MS = "appconfig.thread.timeout.ms";
    private static final String MAX_THREAD_POOL_SIZE = "appconfig.thread.maxPoolSize";
    private static final String AGENCY_VA = "appconfig.agency.va";
    private static final String AGENCY_DOD = "appconfig.agency.dod";
    private static final String LOG4J_PROPS = "appconfig.log4j.props";
    private static final String APP_ENVIRONMENT = "app.environment";
    private static final String APP_VERSION = "app.version";
    private static final String NON_BSE_SECURITY_PHRASE = "appconfig.authentication.nonBseSecurityPhrase";

    private static final String REQUEST_TIMEOUT_MS = "appconfig.request.timeout.ms";
    private static final String CONNECTION_TIMEOUT_MS = "appconfig.connection.timeout.ms";

    private static final String VLER_DQ_URL = "appconfig.vler.dq.url";
    private static final String VLER_DR_URL = "appconfig.vler.dr.url";
//    private static final String VLER_PD_URL = "appconfig.vler.pd.url";
//    private static final String VLER_PA_URL = "appconfig.vler.pa.url";

    //constants
    private static final String PROTOCOL_RPC = "RPC";
    private static final String PROTOCOL_CACHE = "CACHE";
    private static final String DATASOURCE = "appconfig.datasource";

    private static final AppConfig INSTANCE = new AppConfig();

    private Properties appConfigProp;

    private AppConfig() {
        appConfigProp = new Properties();
        ClassLoader classLoader = AppConfig.class.getClassLoader();

        if (classLoader == null) {
            throw new RuntimeException("Classloader is null");
        }

        InputStream appMetaStream = classLoader.getResourceAsStream("application.properties");
        try {
            appConfigProp.load(appMetaStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            if (appMetaStream != null) {
                try {
                    appMetaStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        InputStream appConfigStream = classLoader.getResourceAsStream(String.format("appconfig-%s.properties", appConfigProp.getProperty(APP_ENVIRONMENT)));

        try {
            appConfigProp.load(appConfigStream);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            if (appConfigStream != null) {
                try {
                    appConfigStream.close();
                } catch (IOException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }
    }

    public static final AppConfig getInstance() {
        return INSTANCE;
    }

    public final String getProtocolRPC() {
        return PROTOCOL_RPC;
    }

    public final String getProtocolCache() {
        return PROTOCOL_CACHE;
    }

    public final long getThreadTimeoutMS() {
        return Long.parseLong(appConfigProp.getProperty(THREAD_TIMEOUT_MS));
    }

    public final int getMaxThreadPoolSize() {
        return Integer.parseInt(appConfigProp.getProperty(MAX_THREAD_POOL_SIZE));
    }

    public final String getAgencyVA() {
        return appConfigProp.getProperty(AGENCY_VA);
    }

    public final String getAgencyDOD() {
        return appConfigProp.getProperty(AGENCY_DOD);
    }

    public final String getLog4jProperties() {
        return appConfigProp.getProperty(LOG4J_PROPS);
    }

    public final String getAppEnvironment() {
        return appConfigProp.getProperty(APP_ENVIRONMENT);
    }

    public final String getAppVersion() {
        return appConfigProp.getProperty(APP_VERSION);
    }

    public final String getNonBseSecurityPhrase() {
        return appConfigProp.getProperty(NON_BSE_SECURITY_PHRASE);
    }

    public final int getConnectionTimeoutMs() {
        return Integer.parseInt(appConfigProp.getProperty(CONNECTION_TIMEOUT_MS));
    }

    public final int getRequestTimeoutMs() {
        return Integer.parseInt(appConfigProp.getProperty(REQUEST_TIMEOUT_MS));
    }

    public final String getVLER_DQ_URL() {
        return appConfigProp.getProperty(VLER_DQ_URL);
    }

    public final String getVLER_DR_URL() {
        return appConfigProp.getProperty(VLER_DR_URL);
    }

    public final String getDataSource() {
        return appConfigProp.getProperty(DATASOURCE);
    }

//    public final String getVLER_PD_URL()
//    {
//        return appConfigProp.getProperty(VLER_PD_URL);
//    }
//
//    public final String getVLER_PA_URL()
//    {
//        return appConfigProp.getProperty(VLER_PA_URL);
//    }
}
