/*
 * Janus 4.0 (c)
 * Copyright (c) 2016 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.common.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.vistabeans.Site;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class VistaStatusService {

    private static final Logger LOGGER = LoggerFactory.getLogger(VistaStatusService.class.getName());

    /**
     * Connects to the VistAs in the site list and returns with a connectivity status list.
     * @param siteList List of VistA sites.
     * @return A VistA status list.
     */
    public List<VistaStatus> getVistaStatusList(List<Site> siteList) {
        VistaRPCConnectionManager cxnMgr = null;
        try {

            cxnMgr = new VistaRPCConnectionManager(siteList);
            cxnMgr.connect();

            List<VistaStatus> vistaStatusList = new ArrayList<VistaStatus>();

            LinkedHashMap<Site, RPCConnection> connections = cxnMgr.getConnections();

            Set<Map.Entry<Site, RPCConnection>> entries = connections.entrySet();

            Iterator<Map.Entry<Site, RPCConnection>> iterator = entries.iterator();

            while(iterator.hasNext()) {
                Map.Entry<Site, RPCConnection> next = iterator.next();
                Site site = next.getKey();
                RPCConnection connection = next.getValue();
                
                VistaStatus vistaStatus = new VistaStatus();
                vistaStatus.setSite(site);
                vistaStatus.setConnected(connection.isConnected());
                vistaStatus.setErrorMsg(connection.getErrorMsg());

                vistaStatusList.add(vistaStatus);
            }

            return vistaStatusList;
        }
        catch(Throwable e) {
            LOGGER.error(e.getMessage(), e);

            throw new RuntimeException(e);
        }
        finally {
            if (cxnMgr != null) {
                try {
                    cxnMgr.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }
}
