/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.vds.dataservicehandler;

import gov.hhs.fha.nhinc.common.nhinccommon.AssertionType;
import gov.hhs.fha.nhinc.common.nhinccommon.CeType;
import gov.hhs.fha.nhinc.common.nhinccommon.HomeCommunityType;
import gov.hhs.fha.nhinc.common.nhinccommon.PersonNameType;
import gov.hhs.fha.nhinc.common.nhinccommon.UserType;
import gov.va.med.vds.common.NHINHandlerException;
import gov.va.med.vds.vistabeans.QueryBean;

/**
 * Created with IntelliJ IDEA.
 * User: DNS   gotom
 * Date: 5/30/14
 * Time: 9:41 AM
 * To change this template use File | Settings | File Templates.
 */
public abstract class AdapterHandler extends DataServiceHandler {

    public static final String LOCAL_HCID = "2.16.840.1.113883.3.426";
    public static final String TARGET_HCID = "2.16.840.1.113883.4.349";
    public static final String VA_HCID = "2.16.840.1.113883.4.349.1";
//    public static final String LOCAL_HCID = "2.16.840.1.113883.4.349";
//    public static final String TARGET_HCID = "2.16.840.1.113883.4.349.1";
    //2.16.840.1.113883.3.42

    public static final String FIND_DOCUMENTS = "urn:uuid:14d4debf-8f97-4251-9a74-a90016b0af0d";
    public static final String VA_HOME_ID = "urn:oid:2.16.840.1.113883.4.349";
    public static final String SOURCE_PATIENT_ID = "sourcePatientId";
    public static final String REPOSITORY_UNIQUE_ID = "repositoryUniqueId";
    public static final String CREATION_TIME = "creationTime";
    public static final String DOCUMENT_UNIQUE_ID = "XDSDocumentEntry.uniqueId";
    public static final String AUTHOR_INFORMATION = "urn:uuid:93606bcf-9494-43ec-9b4e-a7748d1a838d";
    public static final String AUTHOR_PERSON = "authorPerson";
    public static final String AUTHOR_INSTITUTION = "authorInstitution";
    public static final String CLASS_CODE = "urn:uuid:41a5887f-8865-4c09-adf7-e362475b143a";

    public static final String SSN_OID = "2.16.840.1.113883.4.1";
    public static final String SLOT_CLASS_CODE = "$XDSDocumentEntryClassCode";
    public static final String SLOT_STATUS = "$XDSDocumentEntryStatus";
    public static final String SLOT_PATIENT_ID = "$XDSDocumentEntryPatientId";
    public static final String SLOT_FORMAT_CODE = "$XDSDocumentEntryFormatCode";
    public static final String PATIENT_ICN_STRING = "^^^&2.16.840.1.113883.4.349.1&ISO";
    public static final String C32_CLASS_CODE = "('34133-9^^2.16.840.1.113883.6.1')";
    public static final String RETURN_TYPE = "LeafClass";
    public static final String SLOT_STATUS_STRING = "('urn:ihe:iti:2010:StatusType:DeferredCreation','urn:oasis:names:tc:ebxml-regrep:StatusType:Approved')";
    //public static final String SLOT_STATUS_STRING = "('urn:oasis:names:tc:ebxml-regrep:StatusType:Approved')";
    public AssertionType generateAssertion(QueryBean qbean) {
        gov.hhs.fha.nhinc.common.nhinccommon.ObjectFactory atOf = new gov.hhs.fha.nhinc.common.nhinccommon.ObjectFactory();
        AssertionType atype = atOf.createAssertionType();

        try {

            //HomeCommunityType - required
            HomeCommunityType hct = new HomeCommunityType();
            hct.setName("JLV");
            hct.setHomeCommunityId(VA_HOME_ID);
            hct.setDescription("JLV");
            atype.setHomeCommunity(hct);

            //user info - required
            UserType utype = new UserType();
            PersonNameType pnt = new PersonNameType();
            String lastName = "";
            String firstName = "";
            String fullName =  qbean.getUser().getName();
            if (fullName.contains(",")) {
                String[] parts = fullName.split(",");
                if (parts[0] != null) {
                    lastName = parts[0].trim();
                }
                if (parts[1] != null) {
                    firstName = parts[1].trim();
                }
            } else {
                lastName = fullName;
            }
            pnt.setFullName(fullName);
            pnt.setGivenName(firstName);
            pnt.setFamilyName(lastName);
            utype.setPersonName(pnt);

            StringBuilder userName = new StringBuilder();

            //if VA user, set the sitecode to the user's login site
            //if DoD user, set the sitecode to 200
            //if VBA user, set the sitecode to 200CORP
            if ("VA".equalsIgnoreCase(qbean.getUser().getHostSite().getAgency())
                && !"200CORP".equalsIgnoreCase(qbean.getUser().getHostSite().getSiteCode())) {
                userName.append(qbean.getUser().getHostSite().getSiteCode());

            } else if ("DOD".equalsIgnoreCase(qbean.getUser().getHostSite().getAgency())) {
                userName.append("200");
            } else if ("VA".equalsIgnoreCase(qbean.getUser().getHostSite().getAgency())
                && "200CORP".equalsIgnoreCase(qbean.getUser().getHostSite().getSiteCode())) {
                userName.append("200CORP");
            }
            userName.append(":");
            userName.append(lastName.replaceAll(" ", "").replaceAll(",", "").replaceAll("-", ""));

            utype.setUserName(userName.toString());
            utype.setOrg(hct);

            //setup role
            CeType roleCoded = new CeType();
            roleCoded.setCode("112247003");
            roleCoded.setCodeSystem("2.16.840.1.113883.6.96");
            roleCoded.setCodeSystemName("SNOMED_CT");
            roleCoded.setDisplayName("Medical doctor");
            utype.setRoleCoded(roleCoded);

            atype.setUserInfo(utype);

            //setup purpose
            CeType purpose = new CeType();
            purpose.setCode("TREATMENT");
            purpose.setCodeSystem("2.16.840.1.113883.3.18.7.1");
            purpose.setDisplayName("TREATMENT");
            purpose.setCodeSystemName("nhin-purpose");
//          purpose.setCodeSystemVersion("");
//          purpose.setOriginalText("");
            atype.setPurposeOfDisclosureCoded(purpose);

            //setup patient id
            atype.getUniquePatientId().add(qbean.getPatient().getICN() + PATIENT_ICN_STRING);

            atype.setAuthorized(false);


        } catch (Exception e) {
            throw new NHINHandlerException(e);
        }

        return atype;
    }
}
