/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.patient.dao.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.Timestamp;
import gov.va.med.rpc.VistaQuery;
import gov.va.med.vds.base.dao.rpc.BaseRPC;
import gov.va.med.vds.patient.dao.rpc.utils.RPCUtil;
import gov.va.med.vds.patient.dao.rpc.utils.VPRUtil;
import gov.va.med.vds.vistabeans.Encounter;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.Provider;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 *
 */
public class AppointmentsRPC extends BaseRPC {

    private static final Logger LOGGER = LoggerFactory.getLogger(AppointmentsRPC.class);
//    private static final String START_OF_PAST_APPTS_MARKER = "-------------------------- CVP - Past Clinic Visits --------------------------";
//    private static final String START_OF_FUTURE_APPTS_MARKER = "--------------------------- CVF - Fut Clinic Visits ---------------------------";
//    private static final String END_OF_APPTS_MARKER = "*** END ***************";
//    private static final String DATE_AT_START_OF_INPUT_REGEX = "\\A(0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])[- /.](19|20)\\d\\d.*";

    public AppointmentsRPC(RPCConnection cxn) {
        super(cxn);
    }

    public PatientAppointments[] getAppointments(Calendar startDate, Calendar endDate) throws RPCException {
        return getAppointments(cxn.getPatientIen(), startDate, endDate);
    }

    public PatientAppointments[] getAppointments(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
//        PatientAppointments[] appts;
//
//        String request = buildGetPastApptsReportRequest(patientIen, startDate, endDate);
//        String response = (String) cxn.query(request);
//        List<PatientAppointments>  apptsList = parsePastOrFutureAppointmentsReport(response);
//
//        return apptsList.toArray(new PatientAppointments[0]);
        return  getAppointmentsVPR(patientIen, startDate, endDate);
    }

//    private String buildGetAppointmentsRequest(String patientIen, Calendar startDate, Calendar endDate) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORQQVS VISITS/APPTS");
////        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORWCV VST");
//        vq.addParameter(vq.LITERAL, patientIen);
//
//        String startDateString = Timestamp.fromDateTime(startDate);
//        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
//        startDateString = flds[0];
//        String endDateString = Timestamp.fromDateTime(endDate);
//        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
//        endDateString = flds[0];
//        vq.addParameter(vq.LITERAL, startDateString);
//        vq.addParameter(vq.LITERAL, endDateString);
//        // add a dummy parameter so the Broker won't crash (see Vista RPC doc)
////        vq.addParameter(vq.LITERAL, "");
//
//        return vq.buildMessage();
//    }


    public PatientAppointments[] getPastAppointments(Calendar startDate, Calendar endDate) throws RPCException {
        return getPastAppointments(cxn.getPatientIen(), startDate, endDate);
    }

    public PatientAppointments[] getPastAppointments(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetPastApptsReportRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);
        PatientAppointments[] appts = toPatientAppointments(response/*, patientIen*/);

        return appts;
    }

    private String buildGetPastApptsReportRequest(String patientIen, Calendar startDate, Calendar endDate) {
        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, "OR_CVP:PAST CLINIC VISITS~;;14;999");
//        vq.addParameter(vq.LITERAL, "");
//        vq.addParameter(vq.LITERAL, "550000");
//        vq.addParameter(vq.LITERAL, "");
//        vq.addParameter(vq.LITERAL, "0");
//        vq.addParameter(vq.LITERAL, "0");

        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        String startDateString = Timestamp.fromDateTime(startDate);
        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
        startDateString = flds[0];
        String endDateString = Timestamp.fromDateTime(endDate);
        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
        endDateString = flds[0];
        vquery.addParameter(VistaQuery.LITERAL, startDateString);
        vquery.addParameter(VistaQuery.LITERAL, endDateString);

        return vquery.buildMessage();
    }

//    private String buildGetFutureApptsReportRequest(String patientIen, Calendar startDate, Calendar endDate) {
//        VistaQuery vq = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
//        vq.addParameter(vq.LITERAL, patientIen);
//        vq.addParameter(vq.LITERAL, "OR_CVF:FUTURE CLINIC VISITS~;;9;999");
//        vq.addParameter(vq.LITERAL, "");
//        vq.addParameter(vq.LITERAL, "550000");
//        vq.addParameter(vq.LITERAL, "");
//        vq.addParameter(vq.LITERAL, "0");
//        vq.addParameter(vq.LITERAL, "0");
//
////        vq.addParameter(vq.LITERAL, "");
////        vq.addParameter(vq.LITERAL, "");
////        vq.addParameter(vq.LITERAL, "");
////        String startDateString = Timestamp.fromDateTime(startDate);
////        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
////        startDateString = flds[0];
////        String endDateString = Timestamp.fromDateTime(endDate);
////        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
////        endDateString = flds[0];
////        vq.addParameter(vq.LITERAL, startDateString);
////        vq.addParameter(vq.LITERAL, endDateString);
//
//        return vq.buildMessage();
//    }


    private PatientAppointments[] toPatientAppointments(String response/*, String patientIen*/) {
        PatientAppointments[] rtc = null;
        final int APPT_DATE_OR_VISIT_IEN = 0; // inpatient admissions include "a" suffix, visit "v", scheduled "s"
        final int APPT_LOC_NAME = 1;
        final int APPT_SCHED_OR_VISITED_HDR = 2;
        final int APPT_DATE = 3;
        final int APPT_INVERSE_DATE = 4;

        if ("".equals(response)) {
            rtc = new PatientAppointments[0];
        } else {
            String[] lines = StringUtils.split(response, StringUtils.CRLF);
            ArrayList<PatientAppointments> apptList = new ArrayList<PatientAppointments>();
    
            // check if the first field in the first line is empty, if so, there are no appts.
            if (lines.length == 1) {
                String[] fields = StringUtils.split(lines[0], StringUtils.CARET);
                if ("".equals(fields[0])) {
                    rtc = new PatientAppointments[0];
                }
            }
    
            if (rtc == null) {
                for (String line : lines) {
                    if ("".equals(line)) {
                        continue;
                    }
        
                    String[] fields = StringUtils.split(line, StringUtils.CARET);
        
                    PatientAppointments appointments = new PatientAppointments();
                    fillCommonDataBeanValues(appointments);
        
                    for (int i = 0; i < fields.length; i++) {
                        String field = fields[i];
        
                        if (StringUtils.isEmpty(field)) {
                            continue;
                        }
        
                        switch (i) {
                            case APPT_DATE_OR_VISIT_IEN:
                                String[] subfields = StringUtils.split(field, StringUtils.SEMICOLON);
                                // visit IEN
                                if ("v".equalsIgnoreCase(subfields[1])) {
                                    appointments.setApptId(subfields[0]);
                                }
                                // TODO: get suffix for type?
                                break;
                            case APPT_LOC_NAME:
                                appointments.setClinic(field);
                                break;
                            case APPT_SCHED_OR_VISITED_HDR:
                                // remove the colon from the header (last char)
                                appointments.setApptType(field.substring(0, field.length() - 1));
                                break;
                            case APPT_DATE:
                                appointments.setApptDate(Timestamp.toDateTime(field));
                                break;
                            case APPT_INVERSE_DATE:
                                break;
                            default:
                                break;
                        }
                    }
        
                    apptList.add(appointments);
                }
        
                rtc = apptList.toArray(new PatientAppointments[0]);
            }
        }
        return rtc;
    }

    public FreeTextReport getEncountersReport(Calendar startDate, Calendar endDate) throws RPCException {
        return getEncountersReport(cxn.getPatientIen(), startDate, endDate);
    }


    public FreeTextReport getEncountersReport(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        String request = buildGetEncountersReportRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        FreeTextReport report = new FreeTextReport();
        fillCommonDataBeanValues(report);

        report.setReportText(response);

        return report;
    }

    private String buildGetEncountersReportRequest(String patientIen, Calendar startDate, Calendar endDate) {

        VistaQuery vquery = vistaQueryFactory.getVistaQuery("ORWRP REPORT TEXT");
        vquery.addParameter(VistaQuery.LITERAL, patientIen);
        vquery.addParameter(VistaQuery.LITERAL, "OR_OE:OUTPATIENT ENCOUNTER~;;205;999");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");
        vquery.addParameter(VistaQuery.LITERAL, "");

        String startDateString = Timestamp.fromDateTime(startDate);
        String[] flds = StringUtils.split(startDateString, StringUtils.PERIOD);
        startDateString = flds[0];
        String endDateString = Timestamp.fromDateTime(endDate);
        flds = StringUtils.split(endDateString, StringUtils.PERIOD);
        endDateString = flds[0];

        vquery.addParameter(VistaQuery.LITERAL, startDateString);
        vquery.addParameter(VistaQuery.LITERAL, endDateString);
        return vquery.buildMessage();
    }

//    private List<PatientAppointments> parsePastOrFutureAppointmentsReport(String report) {
//        if ("".equals(report)) {
//            return new ArrayList<PatientAppointments>();
//        }
//
//        // extract the lines that we want
//        boolean noStartIndex = false;
//        int apptsStartIndex = report.indexOf(START_OF_PAST_APPTS_MARKER);
//        if (apptsStartIndex < 0) {
//            apptsStartIndex = report.indexOf(START_OF_FUTURE_APPTS_MARKER);
//            if (apptsStartIndex < 0) {
//                noStartIndex = true;
//            } else {
//                apptsStartIndex += START_OF_FUTURE_APPTS_MARKER.length();
//            }
//        } else {
//            apptsStartIndex += START_OF_PAST_APPTS_MARKER.length();
//        }
//        int apptsEndIndex = report.indexOf(END_OF_APPTS_MARKER);
//
//        if (noStartIndex || apptsEndIndex < 0) {
//            return new ArrayList<PatientAppointments>();
//        }
//
//        String response = report.substring(apptsStartIndex, apptsEndIndex);
//
//        String[] lines = StringUtils.split(response, StringUtils.CRLF);
//        ArrayList<PatientAppointments> apptList = new ArrayList<PatientAppointments>();
//
//        SimpleDateFormat sdf = new SimpleDateFormat("MM/dd/yyyy HH:mm");
//        Pattern dateAtStartOfInputRegexPattern = Pattern.compile(DATE_AT_START_OF_INPUT_REGEX);
//
//        for (String line : lines) {
//
//            // check that this line starts with a date as a valid appointment.
//            Matcher dateAtStartOfInputMatcher = dateAtStartOfInputRegexPattern.matcher(line);
//            if (!dateAtStartOfInputMatcher.matches()) {
//                continue;
//            }
//
//            PatientAppointments appointment = new PatientAppointments();
//            fillCommonDataBeanValues(appointment);
//
//            String dateField = line.substring(0,16);
//            String clinicNameField = line.substring(16, 57).trim();
//            String statusField = line.substring(57).trim();
//
//            Date appointmentDate = null;
//            try {
//                appointmentDate = sdf.parse(dateField);
//            } catch (ParseException pe) {
//                LOGGER.debug(pe.getMessage());
//            }
//            if (appointmentDate != null) {
//                Calendar appointmentCalendar = Calendar.getInstance();
//                appointmentCalendar.setTime(appointmentDate);
//                appointment.setApptDate(appointmentCalendar);
//            }
//
//            appointment.setClinic(clinicNameField);
//            appointment.setStatus(statusField);
//
//            apptList.add(appointment);
//        }
//
//            return apptList;
//    }

    public PatientAppointments[] getAppointmentsVPR(Calendar startDate, Calendar endDate) throws RPCException {
        return getAppointmentsVPR(cxn.getPatientIen(), startDate, endDate);
    }

    public PatientAppointments[] getAppointmentsVPR(String patientIen, Calendar startDate, Calendar endDate) throws RPCException {
        EncountersRPC encountersRPC = new EncountersRPC(cxn);
        Encounter[] visits = encountersRPC.getPatientNonAdmissionsFromEncounters(patientIen, startDate, endDate);

        String request = buildGetAppointmentsVPRRequest(patientIen, startDate, endDate);
        String response = (String) cxn.query(request);

        PatientAppointments[] appointments = parseAppointments(response);

        //TODO: combine the visits and appointments
        ArrayList<PatientAppointments> allAppointmentList = new ArrayList<PatientAppointments>(Arrays.asList(visits));
        allAppointmentList.addAll(new ArrayList<PatientAppointments>(Arrays.asList(appointments)));

        return allAppointmentList.toArray(new PatientAppointments[0]);
    }

    private String buildGetAppointmentsVPRRequest(String patientIen, Calendar startDate, Calendar endDate) {
        return VPRUtil.buildVPRRequest(vistaQueryFactory, patientIen, startDate, endDate, VPRUtil.VPRType.APPOINTMENTS);
    }

    private PatientAppointments[] parseAppointments(String response) throws RPCException {
        PatientAppointments[] rtc;
        if ("".equals(response)) {
            rtc = new PatientAppointments[0];
        } else {
            StringBuilder sbld = new StringBuilder(response);
            sbld.insert(0, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    
            ArrayList<PatientAppointments> appointmentsList = new ArrayList<PatientAppointments>();
    
            final String XPATH_APPOINTMENT = "//appointment";
    
            Document document = null;
            XPathExpression exprAppointment = null;
    
            try {
                document = parseXMLDocument(sbld.toString());
                XPathFactory xPathFactory = XPathFactory.newInstance();
                XPath xpath = xPathFactory.newXPath();
                exprAppointment = xpath.compile(XPATH_APPOINTMENT);
            } catch (Exception e) {
                throw new RPCException(e);
            }
    
            NodeList appointmentsNodeList = null;
            try {
                appointmentsNodeList = (NodeList) exprAppointment.evaluate(document, XPathConstants.NODESET);
            } catch (XPathExpressionException e) {
                throw new RPCException(e);
            }
    
            List<VPRAppointmentsParser> appointmentsParserList = new ArrayList<VPRAppointmentsParser>();
            int appointmentsNodeListLength = appointmentsNodeList.getLength();
            for (int apptNum = 0; apptNum < appointmentsNodeListLength; apptNum++) {
                Node apptNode = appointmentsNodeList.item(apptNum);
                apptNode.getParentNode().removeChild(apptNode);
    
                appointmentsParserList.add(new VPRAppointmentsParser(apptNode));
            }
    
            ExecutorService executorService = Executors.newFixedThreadPool(5);
            List<Future<PatientAppointments>> futures = null;
            try {
                futures = executorService.invokeAll(appointmentsParserList, 2, TimeUnit.MINUTES);
    
                if (futures == null) {
                    throw new RPCException("Failed to parse appointments results : appt node parser futures list is null.");
                }
            } catch (InterruptedException e) {
                RPCUtil.logErrorMessage(LOGGER, e);
                throw new RPCException(e);
            } finally {
                executorService.shutdown();
            }
    
            Iterator<Future<PatientAppointments>> futureIterator = futures.iterator();
    
            while (futureIterator.hasNext()) {
                try {
                    Future<PatientAppointments> future = futureIterator.next();
                    if (future.isCancelled()) {
                        throw new RPCException("Failed to parse VPR Appointment Data");
                    } else {
                        PatientAppointments appointment = future.get();
    
                        // exclude null results
                        if (appointment != null) {
                            appointmentsList.add(appointment);
                        }
                    }
                } catch (InterruptedException ie) {
                    RPCUtil.logErrorMessage(LOGGER, ie);
                } catch (ExecutionException ee) {
                    RPCUtil.logErrorMessage(LOGGER, ee);
                }
            }
    
            rtc = appointmentsList.toArray(new PatientAppointments[0]);
        }
        return rtc;
    }

    private class VPRAppointmentsParser implements Callable<PatientAppointments> {
        private Node appointmentNode;

        public VPRAppointmentsParser(Node appointmentNode) {
            this.appointmentNode = appointmentNode;
        }

        public PatientAppointments call() throws Exception {
            final String XPATH_APPT_STATUS = "./apptStatus/@value";
            final String XPATH_CLINIC_STOP_CODE = "./clinicStop/@code";
            final String XPATH_CLINIC_STOP_NAME = "./clinicStop/@name";
            final String XPATH_DATE_TIME = "./dateTime/@value";
//            final String XPATH_FACILITY_CODE = "./facility/@code";
//            final String XPATH_FACILITY_NAME = "./facility/@name";
            final String XPATH_ID = "./id/@value";
            final String XPATH_LOCATION = "./location/@value";
            final String XPATH_PATIENT_CLASS = "./patientClass/@value";
            final String XPATH_PROVIDER_CODE = "./provider/@code";
            final String XPATH_PROVIDER_NAME = "./provider/@name";
            final String XPATH_PROVIDER_ROLE = "./provider/@role";
            final String XPATH_PROVIDER_TAXOMOMY_CODE = "./provider/@taxonomyCode";
            final String XPATH_PROVIDER_PROVIDER_TYPE = "./provider/@providerType";
            final String XPATH_PROVIDER_CLASSIFICATION = "./provider/@classification";
            final String XPATH_PROVIDER_SPECIALIZATION = "./provider/@specialization";
            final String XPATH_PROVIDER_PRIMARY = "./provider/@primary";
            final String XPATH_SERVICE = "./service/@value";
            final String XPATH_SERVICE_CATEGORY_CODE = "./serviceCategory/@code";
            final String XPATH_SERVICE_CATEGORY_NAME = "./serviceCategory/@name";
//            final String XPATH_TYPE_CODE = "./type/@code";
            final String XPATH_TYPE_NAME = "./type/@name";
            final String XPATH_VISIT_STRING = "./visitString/@value";

            XPathExpression exprApptStatus = null;
            XPathExpression exprClinicStopCode = null;
            XPathExpression exprClinicStopName = null;
//                    exprFacilityCode = null, exprFacilityName = null,
            XPathExpression exprId = null; 
            XPathExpression exprDateTime = null;
            XPathExpression exprLocation = null;
            XPathExpression exprPatientClass = null;
            XPathExpression exprProviderCode = null;
            XPathExpression exprProviderName = null;
            XPathExpression exprProviderRole = null;
            XPathExpression exprProviderTaxonomyCode = null;
            XPathExpression exprProviderProviderType = null;
            XPathExpression exprProviderClassification = null;
            XPathExpression exprProviderSpecialization = null;
            XPathExpression exprProviderPrimary = null;
            XPathExpression exprService = null;
            XPathExpression exprServiceCategoryCode = null;
            XPathExpression exprServiceCategoryName = null;
//                    exprTypeCode = null,
            XPathExpression exprTypeName = null;
            XPathExpression exprVisitString = null;

            XPathFactory xPathFactory = XPathFactory.newInstance();
            XPath xpath = xPathFactory.newXPath();

            try {
                exprApptStatus = xpath.compile(XPATH_APPT_STATUS);
                exprClinicStopCode = xpath.compile(XPATH_CLINIC_STOP_CODE);
                exprClinicStopName = xpath.compile(XPATH_CLINIC_STOP_NAME);
//                exprFacilityCode = xpath.compile(XPATH_FACILITY_CODE);
//                exprFacilityName = xpath.compile(XPATH_FACILITY_NAME);
                exprId = xpath.compile(XPATH_ID);
                exprDateTime = xpath.compile(XPATH_DATE_TIME);
                exprLocation = xpath.compile(XPATH_LOCATION);
                exprPatientClass = xpath.compile(XPATH_PATIENT_CLASS);
                exprProviderCode = xpath.compile(XPATH_PROVIDER_CODE);
                exprProviderName = xpath.compile(XPATH_PROVIDER_NAME);
                exprProviderRole = xpath.compile(XPATH_PROVIDER_ROLE);
                exprProviderTaxonomyCode = xpath.compile(XPATH_PROVIDER_TAXOMOMY_CODE);
                exprProviderProviderType = xpath.compile(XPATH_PROVIDER_PROVIDER_TYPE);
                exprProviderClassification = xpath.compile(XPATH_PROVIDER_CLASSIFICATION);
                exprProviderSpecialization = xpath.compile(XPATH_PROVIDER_SPECIALIZATION);
                exprProviderPrimary = xpath.compile(XPATH_PROVIDER_PRIMARY);
                exprService = xpath.compile(XPATH_SERVICE);
                exprServiceCategoryCode = xpath.compile(XPATH_SERVICE_CATEGORY_CODE);
                exprServiceCategoryName = xpath.compile(XPATH_SERVICE_CATEGORY_NAME);
//                exprTypeCode = xpath.compile(XPATH_TYPE_CODE);
                exprTypeName = xpath.compile(XPATH_TYPE_NAME);
                exprVisitString = xpath.compile(XPATH_VISIT_STRING);

            } catch (XPathExpressionException e) {
                throw  new RPCException(e);
            }

            String dateTimeString = null;
            Calendar dateTime = null;
            dateTimeString = RPCUtil.evaluateNodeToString(appointmentNode, exprDateTime, LOGGER);
            if (dateTimeString != null) {
                dateTime = Timestamp.toDateTime(dateTimeString);
            }

            String apptStatus = RPCUtil.evaluateNodeToString(appointmentNode, exprApptStatus, LOGGER);
            String clinicStopCode = RPCUtil.evaluateNodeToString(appointmentNode, exprClinicStopCode, LOGGER);
            String clinicStopName = RPCUtil.evaluateNodeToString(appointmentNode, exprClinicStopName, LOGGER);
//            String facilityCode = RPCUtil.evaluateNodeToString(appointmentNode, exprFacilityCode, logger);
//            String facilityName = RPCUtil.evaluateNodeToString(appointmentNode, exprFacilityName, logger);
            String id = RPCUtil.evaluateNodeToString(appointmentNode, exprId, LOGGER);
            String location = RPCUtil.evaluateNodeToString(appointmentNode, exprLocation, LOGGER);
            String patientClass = RPCUtil.evaluateNodeToString(appointmentNode, exprPatientClass, LOGGER);
            String providerCode = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderCode, LOGGER);
            String providerName = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderName, LOGGER);
            String providerRole = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderRole, LOGGER);
            String providerTaxonomyCode = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderTaxonomyCode, LOGGER);
            String providerProviderType = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderProviderType, LOGGER);
            String providerClassification = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderClassification, LOGGER);
            String providerSpecialization = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderSpecialization, LOGGER);
            String providerPrimary = RPCUtil.evaluateNodeToString(appointmentNode, exprProviderPrimary, LOGGER);
            String service = RPCUtil.evaluateNodeToString(appointmentNode, exprService, LOGGER);
            String serviceCategoryCode = RPCUtil.evaluateNodeToString(appointmentNode, exprServiceCategoryCode, LOGGER);
            String serviceCategoryName = RPCUtil.evaluateNodeToString(appointmentNode, exprServiceCategoryName, LOGGER);
//            String typeCode = RPCUtil.evaluateNodeToString(appointmentNode, exprTypeCode, logger);
            String typeName = RPCUtil.evaluateNodeToString(appointmentNode, exprTypeName, LOGGER);
            String visitString = RPCUtil.evaluateNodeToString(appointmentNode, exprVisitString, LOGGER);

            PatientAppointments appointment = new PatientAppointments();
            fillCommonDataBeanValues(appointment);

            appointment.setApptDate(dateTime);
            Provider provider = new Provider();
            provider.setName(providerName);
            provider.setRole(providerRole);
            provider.setCode(providerCode);
            provider.setTaxonomyCode(providerTaxonomyCode);
            provider.setProviderType(providerProviderType);
            provider.setClassification(providerClassification);
            provider.setSpecialization(providerSpecialization);
            if (providerPrimary != null && "1".equals(providerPrimary)) {
                provider.setPrimary(true);
            }
            appointment.setProvider(provider);
            appointment.setClinic(location);
            appointment.setClinicStopName(clinicStopName);
            appointment.setClinicStopCode(clinicStopCode);
            appointment.setService(service);
            appointment.setServiceCategoryCode(serviceCategoryCode);
            appointment.setServiceCategoryName(serviceCategoryName);
            appointment.setApptType(typeName);
            appointment.setVisitString(visitString);
            appointment.setApptId(id);
            appointment.setStatus(apptStatus);
            appointment.setPatientClass(patientClass);

            return appointment;
        }
    }
}
