/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.Timestamp;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.vistabeans.Immunization;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.DNS   .common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class ImmunizationsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(ImmunizationsDataService.class.getName());

    public List<Immunization> getPatientImmunizations(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> immsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_IMMUNIZATION,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_IMMUNIZATIONS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = immsMap.entrySet();
            List<Immunization> immsList = new ArrayList<Immunization>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object imms = entry.getValue();
                if (imms instanceof Exception) {
                    imms = generateConnectionUnavailableBean(entry.getKey(), (Exception) imms);
                }
                immsList.addAll(Arrays.asList((Immunization[]) imms));
            }

            return immsList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private Immunization[] generateConnectionUnavailableBean(Site site, Exception except) {
        
        Immunization immun = new Immunization();
        immun.setName(CONNECTION_UNAVAILABLE);
        immun.setDateTime(Utils.formatDisplayDate(Utils.backDate().toGregorianCalendar()));
        immun.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            immun.setName(UNABLE_TO_RETRIEVE);
        } else {
            LOGGER.error(except.getMessage(), except);        //log if any other exception type
        }
        return new Immunization[] {immun};
    }
}
