/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.rpc.site.VistaSiteException;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.LabsDao;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.LabOrder;
import gov.va.med.vds.vistabeans.LabResult;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.DNS   .common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class LabsDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(LabsDataService.class.getName());

    public List<LabResult> getLabOrderResult(QueryBean queryBean) throws VDSException {
        List<LabResult> rtc;
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            cxn = setupVPRContextSingleRemoteSourceQuery(queryBean);
            LabsDao labsDao = new LabsDao(cxn);
            LabResult[] labs = labsDao.getLabOrderResult(queryBean);

            if (labs == null) {
                rtc = null;
            } else {
                rtc = Arrays.asList(labs);
            }
            return rtc;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }

    }

    public LabResult[] getPatientLabTestResults(QueryBean queryBean) throws VDSException
    {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
            cxn = setupVPRContextSingleRemoteSourceQuery(queryBean);
            LabsDao labsDao = new LabsDao(cxn);
            LabResult[] labs = labsDao.getPatientLabTestResults(queryBean);

            return labs;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }

    }

    public List<LabOrder> getPatientLabs(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LABS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabOrder> labsList = new ArrayList<LabOrder>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labsList.addAll(Arrays.asList((LabOrder[]) labs));
            }



//            // Get micro labs
//            msq = setupMultiSourceQuery(queryBean);
//            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
//                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
//                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LABS, new Object[]{queryBean});
//
//            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
//            List<LabOrder> labsList = new ArrayList<LabOrder>();
//
//            for (Map.Entry<Site, Object> entry : entries)
//            {
//                Object labs = entry.getValue();
//                if (labs instanceof Exception)
//                {
//                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
//                }
//                labsList.addAll(Arrays.asList((LabOrder[]) labs));
//            }
//
//            // Get AP labs
//            labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
//                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
//                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LABS, new Object[]{queryBean});
//
//            entries = labsMap.entrySet();
//
//            for (Map.Entry<Site, Object> entry : entries)
//            {
//                Object labs = entry.getValue();
//                if (labs instanceof Exception)
//                {
//                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
//                }
//                labsList.addAll(Arrays.asList((LabOrder[]) labs));
//            }
//
//            // get Chem labs
//            labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
//                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
//                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_CHEM_LABS, new Object[]{queryBean});
//
//            entries = labsMap.entrySet();
//
//            List<LabOrder> chemLabsList = new ArrayList<LabOrder>();
//            for (Map.Entry<Site, Object> entry : entries)
//            {
//                Object labs = entry.getValue();
//                if (labs instanceof Exception)
//                {
//                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
//                }
//                chemLabsList.addAll(Arrays.asList((LabOrder[]) labs));
//            }
//
//            // Remove all the micro and ap labs from the chem orders if they are repeated in there
//            HashSet<String> microOrApAccessions = new HashSet<String>();
//            for (LabOrder labOrder : labsList) {
//                if (labOrder.getAccessionNumber() != null && labOrder.getAccessionNumber().length() > 0) {
//                    microOrApAccessions.add(labOrder.getAccessionNumber());
//                }
//            }
//            for (LabOrder chemLabOrder : chemLabsList) {
//                if (chemLabOrder.getAccessionNumber() == null) {
//                    labsList.add(chemLabOrder);
//                } else {
//                    String chemAccession = chemLabOrder.getAccessionNumber();
//                    if (!microOrApAccessions.contains(chemAccession)) {
//                        labsList.add(chemLabOrder);
//                    }
//                }
//            }

            return labsList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public List<LabResult> getPatientLabResults(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            // Get lab results
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_LAB_RESULTS_VPR, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
            List<LabResult> labResultList = new ArrayList<LabResult>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object labs = entry.getValue();
                if (labs instanceof Exception) {
                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
                }
                labResultList.addAll(Arrays.asList((LabResult[]) labs));
            }
//            // Get micro lab results
//            msq = setupMultiSourceQuery(queryBean);
//            LinkedHashMap<Site, Object> labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
//                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
//                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_MICRO_LAB_RESULTS, new Object[]{queryBean});
//
//            Set<Map.Entry<Site, Object>> entries = labsMap.entrySet();
//            List<LabResult> labResultList = new ArrayList<LabResult>();
//
//            for (Map.Entry<Site, Object> entry : entries)
//            {
//                Object labs = entry.getValue();
//                if (labs instanceof Exception)
//                {
//                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
//                }
//                labResultList.addAll(Arrays.asList((LabResult[]) labs));
//            }
//
//            // Get AP lab results
//            labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
//                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
//                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_ANAPATH_LAB_RESULTS, new Object[]{queryBean});
//
//            entries = labsMap.entrySet();
//
//            for (Map.Entry<Site, Object> entry : entries)
//            {
//                Object labs = entry.getValue();
//                if (labs instanceof Exception)
//                {
//                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
//                }
//                labResultList.addAll(Arrays.asList((LabResult[]) labs));
//            }
//
//            // Get Chem lab results
//            msq = setupVPRContextMultiSourceQuery(queryBean);
//            labsMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
//                    VistaRPCConnectionManager.EXECUTION_CLASS_LABS,
//                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_CHEM_LAB_RESULTS, new Object[]{queryBean});
//
//            entries = labsMap.entrySet();
//
//            for (Map.Entry<Site, Object> entry : entries)
//            {
//                Object labs = entry.getValue();
//                if (labs instanceof Exception)
//                {
//                    labs = generateConnectionUnavailableBeanLabOrder(entry.getKey(), (Exception) labs);
//                }
//                labResultList.addAll(Arrays.asList((LabResult[]) labs));
//            }

            return labResultList;
        } catch (ClassNotFoundException ex) {
            throw new VDSException(ex);
        } catch (RPCException ex) {
            throw new VDSException(ex);
        } catch (VistaSiteException ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public FreeTextReport getPatientLabReport(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        String itemId = queryBean.getItemId();
        String siteCode = queryBean.getRecordSite().getSiteCode();

        RPCConnection cxn = null;

        if (itemId == null || itemId.length() <= 0) {
            throw new IllegalArgumentException("Item Id cannot be null or empty. This is the accession number");
        } else if (queryBean.getStartDate() == null) {
            throw new IllegalArgumentException("Start Date cannot be null. This is the collectedDate");
        } else if (StringUtils.isEmpty(siteCode)) {
            throw new IllegalArgumentException("Missing record site code");
        }
        try {
            cxn = setupSingleRemoteSourceQuery(queryBean);
            LabsDao labsDao = new LabsDao(cxn);
            return labsDao.getPatientLabReport(queryBean);
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }

    }

    private LabOrder[] generateConnectionUnavailableBeanLabOrder(Site site, Exception except) {
        LabOrder lab = new LabOrder();
        lab.setTestName(CONNECTION_UNAVAILABLE);
        lab.setCollectionDate(Utils.backDate().toGregorianCalendar());
        lab.setSite(site);
        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            lab.setTestName(UNABLE_TO_RETRIEVE);
        } else {
            LOGGER.error(except.getMessage(), except);        //log if any other exception type
        }

        
        return new LabOrder[] {lab};
    }
}
