/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCException;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.vistabeans.Procedure;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.DNS   .common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class ProceduresDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProceduresDataService.class.getName());

    public List<Procedure> getPatientProcedures(QueryBean queryBean) throws VDSException {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> proceduresMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_PROCEDURES,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_PROCEDURES, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = proceduresMap.entrySet();
            List<Procedure> proceduresList = new ArrayList<Procedure>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object procedures = entry.getValue();
                if (procedures instanceof Exception) {
                    procedures = generateConnectionUnavailableBean(entry.getKey(), (Exception) procedures);
                }
                proceduresList.addAll(Arrays.asList((Procedure[]) procedures));
            }

            return proceduresList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private Procedure[] generateConnectionUnavailableBean(Site site, Exception except) {
        Procedure procedure = new Procedure();
        procedure.setDescription(CONNECTION_UNAVAILABLE);
        procedure.setDate(Utils.backDate().toGregorianCalendar());
        procedure.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            procedure.setStatus(UNABLE_TO_RETRIEVE);
        } else {
            LOGGER.error(except.getMessage(), except);        //log if any other exception type
        }
        return new Procedure[]{procedure};
    }
}
