/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.patient.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dao.UnsupportedRPCMethodException;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.base.dataservice.VistaRPCConnectionManager;
import gov.va.med.vds.patient.dao.RadiologyDao;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.RadiologyReport;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.DNS   .common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
public class RadiologyDataService extends DataServiceBase {
    private static final Logger LOGGER = LoggerFactory.getLogger(RadiologyDataService.class.getName());

    public List<RadiologyReport> getPatientRads(QueryBean queryBean) throws VDSException
    {
        validateQueryBeanDateRange(queryBean);

        VistaRPCConnectionManager msq = null;

        try {
            msq = setupVPRContextMultiSourceQuery(queryBean);
            LinkedHashMap<Site, Object> radiologyMap = VistaRPCConnectionManager.execute(msq.getConnectionTable(),
                    VistaRPCConnectionManager.EXECUTION_CLASS_RADIOLOGY,
                    VistaRPCConnectionManager.EXECUTION_METHOD_PATIENT_RADS, new Object[]{queryBean});

            Set<Map.Entry<Site, Object>> entries = radiologyMap.entrySet();
            List<RadiologyReport> radReportList = new ArrayList<RadiologyReport>();

            for (Map.Entry<Site, Object> entry : entries) {
                Object radReport = entry.getValue();
                if (radReport instanceof Exception) {
                    radReport = generateConnectionUnavailableBean(entry.getKey(), (Exception) radReport);
                }
                radReportList.addAll(Arrays.asList((RadiologyReport[]) radReport));
            }

            return radReportList;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (msq != null) {
                try {
                    msq.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public RadiologyReport getRadiologyReport(QueryBean queryBean) throws VDSException {
        validateQueryBean(queryBean);

        RPCConnection cxn = null;

        try {
//            Patient patient = queryBean.getPatient();
            String itemId = queryBean.getItemId();
            String siteCode = queryBean.getRecordSite().getSiteCode();
            if (itemId == null || itemId.length() <= 0) {
                throw new IllegalArgumentException("Item Id cannot be null or empty");
            } else if (StringUtils.isEmpty(siteCode)) {
                throw new IllegalArgumentException("Missing record site code");
            }
            
            cxn = setupSingleRemoteSourceQuery(queryBean);
            RadiologyDao radiologyDao = new RadiologyDao(cxn);
            RadiologyReport report = radiologyDao.getRadiologyReport(queryBean);

            return report;
        } catch (Exception ex) {
            throw new VDSException(ex);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    private RadiologyReport[] generateConnectionUnavailableBean(Site site, Exception except) {

        RadiologyReport exam = new RadiologyReport();
        exam.setStudy(CONNECTION_UNAVAILABLE);
        exam.setExamDate(Utils.backDate().toGregorianCalendar());
        exam.setSite(site);

        if (except.getCause() instanceof UnsupportedRPCMethodException) {
            exam.setStudy(UNABLE_TO_RETRIEVE);
        } else {
            LOGGER.error(except.getMessage(), except);        //log if any other exception type
        }


        return new RadiologyReport[] {exam};
    }
}
