/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache;

import com.intersys.objects.CacheException;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.handler.AdmissionHandler;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.QueryBean;

public class AdmissionCache extends CacheBase {
    public AdmissionCache(CacheSiteConnection cacheConnection) {
        super(cacheConnection);
    }

    /**
     * Gets an array of patients who are admitted to specific ward(s).
     *
     * @param queryBean - itemId required; refer to calling class AdmissionsDao
     * @return an array of patients who are admitted to specific ward(s).
     * @throws CacheException
     */
    public PatientAdmission[] getWardAdmissions(QueryBean queryBean) throws CacheException {
        PatientAdmission[] admissions = null;

        cacheConnection.open();

        try {
            AdmissionHandler handler = new AdmissionHandler(
                    cacheConnection.getDatabase(), cacheConnection.getSite(),
                    queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            admissions = handler.processWardAdmissions(queryBean.getItemId(), queryBean.getUser().getUserIen(), queryBean.getUser().getNPI());

        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return admissions;
    }

    /**
     * Gets an array of patients who are assigned to the user.
     *
     * @param queryBean - no parameters required. Refer to calling class
     *                  AdmissionDao for more information.
     * @return an array of patient admissions
     * @throws CacheException
     */
    public PatientAdmission[] getProviderAdmissions(QueryBean queryBean) throws CacheException {
        PatientAdmission[] admissions = null;

        cacheConnection.open();
        try {
            AdmissionHandler handler = new AdmissionHandler(
                    cacheConnection.getDatabase(), cacheConnection.getSite(),
                    queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            admissions = handler.processProviderAdmissions(queryBean.getUser().getUserIen(), queryBean.getUser().getNPI());

        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return admissions;
    }
}
