/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache;

import com.intersys.objects.CacheException;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.handler.ConsultHandler;
import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.DNS   .common.Utils;

public class ConsultCache extends CacheBase {
    public ConsultCache(CacheSiteConnection cacheConnection) {
        super(cacheConnection);
    }

    public Consult[] getConsultsRequested(QueryBean queryBean) throws CacheException {
        Consult[] consults = null;
        String startDate = null;
        String endDate = null;
        String userIen = null;

        userIen = queryBean.getUser().getUserIen();
        if (queryBean.getEndDate() != null) {
            endDate = Utils.formatDate(queryBean.getEndDate().getTime());
        }
        if (queryBean.getStartDate() != null) {
            startDate = Utils.formatDate(queryBean.getStartDate().getTime());
        }
        
        cacheConnection.open();

        try {
            ConsultHandler handler = new ConsultHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), userIen, queryBean.getUser().getHostSite().getSiteCode());
            consults = handler.processConsultsRequested(userIen, queryBean.getUser().getNPI(), startDate, endDate);
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return consults;
    }

    public Consult[] getConsultsReceived(QueryBean queryBean) throws CacheException {
        Consult[] consults;
        String startDate = null;
        String endDate = null;
        String userIen = null;

        userIen = queryBean.getUser().getUserIen();
        if (queryBean.getEndDate() != null) {
            endDate = Utils.formatDate(queryBean.getEndDate().getTime());
        }
        if (queryBean.getStartDate() != null) {
            startDate = Utils.formatDate(queryBean.getStartDate().getTime());
        }

        cacheConnection.open();

        try {
            ConsultHandler handler = new ConsultHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), userIen, queryBean.getUser().getHostSite().getSiteCode());
            consults = handler.processConsultsReceived(userIen, queryBean.getUser().getNPI(), startDate, endDate);
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }
        return consults;
    }
}
