/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache;

import com.intersys.objects.CacheException;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.handler.OrderHandler;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.DNS   .common.Utils;

public class OrderCache extends CacheBase {
    public OrderCache(CacheSiteConnection cacheConnection) {
        super(cacheConnection);
    }

    public Order[] getOrdersPending(QueryBean queryBean) throws CacheException {
        Order[] orders = null;
        String startDate = null;
        String endDate = null;

        if (queryBean.getEndDate() != null) {
            endDate = Utils.formatDate(queryBean.getEndDate().getTime());
        }
        if (queryBean.getStartDate() != null) {
            startDate = Utils.formatDate(queryBean.getStartDate().getTime());
        }

        cacheConnection.open();

        try {
            OrderHandler handler = new OrderHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            orders = handler.processOrdersPending(queryBean.getUser().getUserIen(), queryBean.getUser().getNPI(), startDate, endDate);

        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return orders;
    }

    public Order[] getOrdersResulted(QueryBean queryBean) throws CacheException {
        Order[] orders = null;
        String startDate = null;
        String endDate = null;

        if (queryBean.getEndDate() != null) {
            endDate = Utils.formatDate(queryBean.getEndDate().getTime());
        }
        if (queryBean.getStartDate() != null) {
            startDate = Utils.formatDate(queryBean.getStartDate().getTime());
        }

        cacheConnection.open();
        try {
            OrderHandler handler = new OrderHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            orders = handler.processOrdersResulted(queryBean.getUser().getUserIen(), queryBean.getUser().getNPI(), startDate, endDate);

        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return orders;
    }

    public Order getOrderDetails(QueryBean queryBean) throws CacheException {
        String userIen = queryBean.getUser().getUserIen();
        String orderId = queryBean.getItemId();
        Order order = null;

        cacheConnection.open();

        try {
            OrderHandler handler = new OrderHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            //TODO: pass the providerNpi as well?
            order = handler.processOrderDetails(userIen, orderId);
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return order;
    }

}
