/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache;

import VHAJAVA.NEWPERSON;
import com.intersys.objects.CacheException;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.handler.UserHandler;
import gov.va.med.vds.vistabeans.Alert;
import gov.va.med.vds.vistabeans.User;
import gov.va.med.vds.vistabeans.ProviderAppointmentBean;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.DNS   .common.Utils;

/**
 *
 */
public class UserCache extends CacheBase {
    public UserCache(CacheSiteConnection cacheConnection) {
        super(cacheConnection);
    }

    public String login(String siteCode, String accessCode, String verifyCode) throws CacheException
    {


//        Handler audit;
        String isVistALink = "NO";

        String userId = null;

        //ResourceBundle prop = ResourceBundle.getBundle("gov.va.med.DNS   .VistaData");
        isVistALink = "NO";
        /*
         * Get Connection
         */

        if ("NO".equals(isVistALink)) {
            /*
             * use Cache Objects to authenticate
             */

            cacheConnection.open();
            String aCode = VistaKernelHash.decrypt(accessCode);
            String vCode = VistaKernelHash.decrypt(verifyCode);
            userId = NEWPERSON.validLogin(cacheConnection.getDatabase(), aCode, vCode);

            cacheConnection.close();
        }

        return userId;
    }

    public User[] lookupUser(String siteCode, String lookupstr, String userId, int maxReturn) throws CacheException {
        User[] pList = null;
        cacheConnection.open();

        try {
            UserHandler handler = new UserHandler(cacheConnection.getDatabase(), cacheConnection.getSite(), userId, siteCode);
            pList = handler.processLookupUsers(lookupstr);
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }
        return pList;
    }

    public User getProviderProfile(String siteCode, String userIen, String providerNpi) throws CacheException {

        User user = null;
        cacheConnection.open();

        try {
            UserHandler handler = new UserHandler(cacheConnection.getDatabase(), cacheConnection.getSite(), userIen, siteCode);
            user = handler.processProviderProfile(userIen, providerNpi);
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }
        return user;
    }

    public ProviderAppointmentBean[] getProviderWorkload(QueryBean queryBean) throws CacheException
    {

        ProviderAppointmentBean[] retBean = null;

        String startDate = null;
        String endDate = null;
        if (queryBean.getEndDate() != null) {
            endDate = Utils.formatDate(queryBean.getEndDate().getTime());
        }
        if (queryBean.getStartDate() != null) {
            startDate = Utils.formatDate(queryBean.getStartDate().getTime());
        }
        cacheConnection.open();

        try {
            UserHandler handler = new UserHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(), queryBean.getUser().getUserIen(),
                    queryBean.getUser().getName(), queryBean.getUser().getHostSite().getSiteCode());
            retBean = handler.processProviderWorkload(queryBean.getItemId(), startDate, endDate);

        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return retBean;
    }

    public Alert[] getUserAlerts(QueryBean queryBean) throws CacheException {

        Alert[] alerts = null;
        cacheConnection.open();
        try {
            UserHandler handler = new UserHandler(cacheConnection.getDatabase(),
                    cacheConnection.getSite(),
                    queryBean.getUser().getUserIen(), queryBean.getUser().getHostSite().getSiteCode());
            alerts = handler.processUserAlerts(Utils.formatDate(queryBean.getStartDate().getTime()), Utils.formatDate(queryBean.getEndDate().getTime()));
        } catch (CacheHandlerException e) {
            throw new CacheException(e);
        } finally {
            cacheConnection.close();
        }

        return alerts;
    }

}
