/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.handler;

import com.intersys.objects.Database;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.model.ConsultData;
import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.Site;

import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ConsultHandler extends Handler {
    private static final Logger LOGGER = LoggerFactory.getLogger(ConsultHandler.class);
    public ConsultHandler(Database dbase, Site site, String userId, String userSiteCode) {
        super(dbase, site, userId, userSiteCode);
    }

    public ConsultHandler(Database dbase, Site site, String userId,
                          String username, String userSiteCode) {
        super(dbase, site, userId, username, userSiteCode);
    }

    public Consult[] processConsultsRequested(String userIen, String providerNpi,
                                              String startDate, String endDate)
    {
        Consult[] consults;
        if (userIen == null || m_db == null) {
            consults = null;
        } else {
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_PROVIDER_CONSULTS_REQUESTED(?,?,?,?)}");
                cacheCallable.setString(1, userIen);
                cacheCallable.setString(2, providerNpi);
                cacheCallable.setString(3, startDate);
                cacheCallable.setString(4, endDate);
                tableOfResults = cacheCallable.executeQuery();
    
                ConsultData data = new ConsultData(m_site, tableOfResults);
                consults = data.getConsultsRequested();
            } catch (Exception e) {
                throw new CacheHandlerException("ConsultHandler::processConsultsRequested(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return consults;
    }

    public Consult[] processConsultsReceived(String userIen, String providerNpi,
                                             String startDate, String endDate) {
        Consult[] consults;
        if (userIen == null || m_db == null) {
            consults = null;
        } else {
    
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_PROVIDER_CONSULTS_RECEIVED(?,?,?,?)}");
                cacheCallable.setString(1, userIen);
                cacheCallable.setString(2, providerNpi);
                cacheCallable.setString(3, startDate);
                cacheCallable.setString(4, endDate);
                tableOfResults = cacheCallable.executeQuery();
    
                ConsultData data = new ConsultData(m_site, tableOfResults);
                consults = data.getConsultsReceived();
                cacheCallable.close();
            } catch (Exception e) {
                throw new CacheHandlerException("ConsultHandler::processConsultsReceived(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return consults;
    }

}