/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.handler;

import com.intersys.objects.Database;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.model.NoteData;
import gov.va.med.vds.vistabeans.ProgressNote;
import gov.va.med.vds.vistabeans.Site;

import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class NoteHandler extends Handler {
    private static final Logger LOGGER = LoggerFactory.getLogger(NoteHandler.class);
    public NoteHandler(Database dbase, Site site, String userId, String userSiteCode) {
        super(dbase, site, userId, userSiteCode);
    }

    public NoteHandler(Database dbase, Site site, String userId,
                       String username, String userSiteCode) {
        super(dbase, site, userId, username, userSiteCode);
    }

    public ProgressNote[] processProviderUnsignedNotes(String userIen, String providerNpi)
    {
        ProgressNote[] rtc;
        ProgressNote[] unsignedNotes = null;
        if (userIen == null || m_db == null) {
            rtc = null;
        } else {
            
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_PROVIDER_UNSIGNED_NOTES(?,?)}");
                cacheCallable.setString(1, userIen);
                cacheCallable.setString(2, providerNpi);
                tableOfResults = cacheCallable.executeQuery();
    
                NoteData data = new NoteData(m_site, tableOfResults);
                unsignedNotes = data.getUnsignedNotes();
                cacheCallable.close();
            } catch (Exception e) {
                throw new CacheHandlerException("NotesHandler::processProviderUnsignedNotes(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
            rtc = unsignedNotes;
        }
        return rtc;
    }
}