/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.handler;

import com.intersys.objects.Database;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.provider.dao.cache.model.OrderData;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.Site;

import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OrderHandler extends Handler {
    private static final Logger LOGGER = LoggerFactory.getLogger(OrderHandler.class);
    public OrderHandler(Database dbase, Site site, String userId, String userSiteCode) {
        super(dbase, site, userId, userSiteCode);
    }

    public OrderHandler(Database dbase, Site site, String userId,
                        String username, String userSiteCode) {
        super(dbase, site, userId, username, userSiteCode);
    }

    public Order[] processOrdersPending(String userIen, String providerNpi,
                                        String startDate, String endDate) {
        Order[] orders;
        if (userIen == null || m_db == null) {
            orders = null;
        } else {
    
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_PROVIDER_PENDING_ORDERS(?,?,?,?)}");
                cacheCallable.setString(1, userIen);
                cacheCallable.setString(2, providerNpi);
                cacheCallable.setString(3, startDate);
                cacheCallable.setString(4, endDate);
                tableOfResults = cacheCallable.executeQuery();
    
                OrderData data = new OrderData(m_site, tableOfResults);
                orders = data.getOrdersPending();
            } catch (Exception e) {
                throw new CacheHandlerException("OrderHandler::processOrdersPending(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return orders;
    }

    public Order[] processOrdersResulted(String userIen, String providerNpi,
                                         String startDate, String endDate) {
        Order[] orders = null;
        if (userIen == null || m_db == null) {
            orders = null;
        } else {
    
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_PROVIDER_RESULTED_ORDERS(?,?,?,?)}");
                cacheCallable.setString(1, userIen);
                cacheCallable.setString(2, providerNpi);
                cacheCallable.setString(3, startDate);
                cacheCallable.setString(4, endDate);
                tableOfResults = cacheCallable.executeQuery();
    
                OrderData data = new OrderData(m_site, tableOfResults);
                orders = data.getOrdersResulted();
            } catch (Exception e) {
                throw new CacheHandlerException("OrderHandler::processOrdersResulted(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return orders;
    }

    public Order processOrderDetails(String userIen, String orderId) {
        Order order;
        if (m_db == null) {
            order = null;
        } else {
    
            ResultSet tableOfResults = null;
            CallableStatement cacheCallable = null;
    
            try {
                cacheCallable = m_db.prepareCall("{CALL EHRW.GET_ORDER_DETAILS(?)}");
                cacheCallable.setString(1, orderId);
                tableOfResults = cacheCallable.executeQuery();
    
                OrderData data = new OrderData(m_site, tableOfResults);
                order = data.getOrderDetails();
            } catch (Exception e) {
                throw new CacheHandlerException("OrderHandler::processOrderDetails(...);"
                        + " Exception - " + e.getMessage());
            } finally {
                if (tableOfResults != null) {
                    try {
                        tableOfResults.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
                if (cacheCallable != null) {
                    try {
                        cacheCallable.close();
                    } catch (SQLException e) {
                        LOGGER.error(e.getMessage());
                    }
                }
            }
        }
        return order;
    }
}
