/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package gov.va.med.vds.provider.dao.cache.handler;

import VHAJAVA.ALERT;
import VHAJAVA.RPCNEWPERSON;
import com.intersys.objects.Database;
import gov.va.med.vds.common.CacheHandlerException;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.provider.dao.cache.model.ProceduresData;
import gov.va.med.vds.provider.dao.cache.model.ProviderAppointmentData;
import gov.va.med.vds.provider.dao.cache.model.ProviderProfileData;
import gov.va.med.vds.vistabeans.Alert;
import gov.va.med.vds.vistabeans.User;
import gov.va.med.vds.vistabeans.ProviderAppointmentBean;
import gov.va.med.DNS   .common.Utils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

/**
 * @author DNS   gotom
 */
public class UserHandler extends Handler {
    private static final Logger LOGGER = LoggerFactory.getLogger(UserHandler.class);

    public UserHandler(Database dbase, Site site, String userId, String userSiteCode) {
        super(dbase, site, userId, userSiteCode);
    }

    public UserHandler(Database dbase, Site site, String userId, String username, String userSiteCode) {
        super(dbase, site, userId, username, userSiteCode);
    }

    public User processProviderProfile(String userIen, String providerNpi) {
        User user = null;

        if (m_db != null && (userIen != null || providerNpi != null)) {
            ResultSet table = null;
            CallableStatement callableStatement = null;
    
            try {
                if (userIen != null) {
                    callableStatement = m_db.prepareCall("CALL VHAJAVA.NEWPERSON_getProviderByIen(?)");
                    callableStatement.setString(1, userIen);
                    table = callableStatement.executeQuery();
                } else if (providerNpi != null) {
                    callableStatement = m_db.prepareCall("CALL VHAJAVA.NEWPERSON_getProviderByNpi(?)");
                    callableStatement.setString(1, providerNpi);
                    table = callableStatement.executeQuery();
                } else {
                    throw new CacheHandlerException("UserHandler::processProviderProfile(): Exception - failed to retrieve user profile");
                }
    
                ProviderProfileData providerProfileData = new ProviderProfileData(m_site, table);
                user = providerProfileData.getProvider();
            } catch (Exception e) {
                throw new CacheHandlerException("UserHandler::processProviderProfile(); Exception - " + e.getMessage());
            } finally {
                try {
                    if (table != null) {
                        table.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
    
                try {
                    if (callableStatement != null) {
                        callableStatement.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }
        return user;
    }

    public Alert[] processUserAlerts(String startDate, String endDate) {
        Alert[] alerts = null;
        String filemanStartDate = "";
        String filemanEndDate = "";
        if (m_userId != null) {
            if (startDate.length() > 0) {
                filemanStartDate = Utils.getFilemanDate(Utils.formatDate(startDate));
            }
            if (endDate.length() > 0) {
                filemanEndDate = Utils.getFilemanDate(Utils.formatDate(endDate));
            }

            if (m_db != null) {
                List list = null;
                try {
                    list = ALERT.getUserAlerts(m_db, m_userId, filemanStartDate, filemanEndDate);
                } catch (Exception e) {
                    throw new CacheHandlerException("UserHandler::processUserAlerts(); Exception - " + e.getMessage());
                }

                ProviderProfileData data = new ProviderProfileData(m_site, list);
                alerts = data.getAlerts();
            }
        }

        return alerts;
    }

    public User[] processLookupUsers(String lookupString) {
        User[] users = null;

        if (m_db != null && lookupString != null) {
            List table = null;

            try {
                //SB: Use default max
                table = RPCNEWPERSON.findNewPerson(m_db, lookupString, m_userId, "");

                ProviderProfileData data = new ProviderProfileData(m_site, table);
                users = data.getLookupNewPerson();
            } catch (Exception e) {
                throw new CacheHandlerException("UserHandler::processLookupUsers(); Exception - " + e.getMessage());
            }
        }

        return users;
    }

    public ProviderAppointmentBean[] processProviderWorkload(String userIen, String startDate, String endDate) {
        ProviderAppointmentBean[] providerAppointmentBeans = null;

//        if (userIen != null) {
//            String filemanStartDate = "";
//            String filemanEndDate = "";
//
//            filemanStartDate = Utils.getFilemanDate(Utils.formatDate(startDate));
//            filemanEndDate = toFilemanEndDate(Utils.formatDate(endDate));
//
//            if (m_db != null) {
//                ResultSet table = null;
//                CallableStatement stmt = null;
//
//                try {
//                    stmt = m_db.prepareCall("{ call VHAJAVA.GET_PROVIDER_WORKLOAD(?,?,?)}");
//                    stmt.setString(1, userIen);
//                    stmt.setString(2, filemanStartDate);
//                    stmt.setString(3, filemanEndDate);
//                    table = stmt.executeQuery();
//
//                    ProviderAppointmentData data = new ProviderAppointmentData(m_site, table);
//                    providerAppointmentBeans = data.getProviderAppointment();
//                    stmt.close();
//
//                    String encounterId = "";
//                    String visitProcedures = "";
//                    String visitDiagnosis = "";
//                    if (providerAppointmentBeans != null) {
//                        for (int i = 0; i < providerAppointmentBeans.length; i++)
//                        {
//                            encounterId = providerAppointmentBeans[i].getEncounterId();
//                            if (encounterId != null) {
//                                visitProcedures = processVisitProcedures(encounterId);
//                                visitDiagnosis = processVisitDiagnosis(encounterId);
//                                providerAppointmentBeans[i].setCPTs(visitProcedures);
//                                providerAppointmentBeans[i].setICDs(visitDiagnosis);
//                            }
//                        }
//                    }
//                } catch (Exception e) {
//                    throw new CacheHandlerException("UserHandler::processProviderWorkload(); Exception - " + e.getMessage());
//                } finally {
//                    try {
//                        if (table != null) {
//                            table.close();
//                        }
//                    } catch (SQLException e) {
//                        LOGGER.error(e.getMessage());
//                    }
//
//                    try {
//                        if (stmt != null) {
//                            stmt.close();
//                        }
//                    } catch (SQLException e) {
//                        LOGGER.error(e.getMessage());
//                    }
//                }
//            }
//        }

        // Cache call to processVisitDiagnosis and processVisitProcedures is causing Fortify (Access Control: Database)
        // high findings. Since provider portal is currently unsupported by RPC and JLV is currently not using this
        // method, the call to processVisitDiagnosis and processVisitProcedures is commented out (above) to mitigate
        // the high finding. -Jimi

        return providerAppointmentBeans;
    }

    public String processVisitDiagnosis(String visitId) {
        String diagnosis = "";

        if (visitId != null) {
            ResultSet table = null;
            CallableStatement callableStatement = null;

            try {
                callableStatement = m_db.prepareCall("CALL VHAJAVA.VCPT_getVisitDiagnosis(?)");
                callableStatement.setString(1, visitId);

                table = callableStatement.executeQuery();

                ProceduresData proceduresData = new ProceduresData(m_site, table);
                diagnosis = proceduresData.getProcedures();
            } catch (Exception e) {
                throw new CacheHandlerException("UserHandler::processVisitDiagnosis(); Exception - " + e.getMessage());
            } finally {
                try {
                    if (table != null) {
                        table.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }

                try {
                    if (callableStatement != null) {
                        callableStatement.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        return diagnosis;
    }

    public String processVisitProcedures(String visitId) {
        String procedures = "";

        if (visitId != null) {
            ResultSet table = null;
            CallableStatement callableStatement = null;

            try {
                callableStatement = m_db.prepareCall("CALL VHAJAVA.VCPT_getVisitProcedures(?)");
                callableStatement.setString(1, visitId);
                table = callableStatement.executeQuery();

                ProceduresData proceduresData = new ProceduresData(m_site, table);
                procedures = proceduresData.getProcedures();

            } catch (Exception e) {
                throw new CacheHandlerException("UserHandler::processVisitProcedures(); Exception - " + e.getMessage());
            } finally {
                try {
                    if (table != null) {
                        table.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }

                try {
                    if (callableStatement != null) {
                        callableStatement.close();
                    }
                } catch (SQLException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }

        return procedures;
    }
}
