/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.model;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.SQLException;
import java.util.Iterator;

/** Iterator class for ModelData objects
 *  May be extended (hasNext in mind) to allow for
 *  partial row data, i.e. 1st 100, 2nd 100, etc.
 */
public class DataModelIterator implements Iterator {
    private static final Logger LOGGER = LoggerFactory.getLogger(DataModelIterator.class);

    protected int iterRow = -1;
    protected ModelData mData = null;

    // For iterating over the data one item at a time
    protected DataModelIterator(ModelData data) {
        mData = data;
        iterRow = -1;
    }

    // expose the row number
    public int getIndex() {
        return iterRow;
    }
        
    // Implementation of Iterator
    public boolean hasNext() {
        boolean rtc = false;
        try {
            if (mData.mTable.next()) {
                rtc = true;
            }
        } catch (SQLException e) {
            LOGGER.error(e.getMessage());
        }
        return rtc;
    }
    
    public Object next() {
        if (hasNext()) {
            try {
                iterRow = mData.mTable.getRow();
                return this;
            } catch (Exception e) {
                throw new java.util.NoSuchElementException("Past end of table");
            }
        }
        throw new java.util.NoSuchElementException("Past end of table");
    }
    
    public void remove() {
        // Nothing to remove.
    }

    /**
     * Below is the accessor used by the JSPs to retrieve data stored in tables.
     * These wrap the details regarding how to access via "getCell()"
     */     
    public String getItem(int column) {
        String item = mData.getCell(iterRow, column);
        if (item == null) {
            item = "";
        }
        return item;
    }


}