/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.model;

import VHAJAVA.ALERT;
import VHAJAVA.RPCNEWPERSON;
import com.intersys.objects.CacheException;
import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.Alert;
import gov.va.med.vds.vistabeans.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.ResultSet;
import java.util.ArrayList;




public class ProviderProfileData extends ModelData {
    public static final int NAME = 0;
    public static final int CHCSID = 1;
    public static final int CLASS = 2;
    public static final int LOCATION = 3;
    public static final int IEN = 4;
    public static final int LOCATION_ID = 5;
    public static final int GENDER = 6;
    public static final int DUTY_PHONE_1 = 7;
    public static final int PAGER = 8;
    public static final int PROVIDER_FLAG = 9;
    public static final int SIG_CLASS = 10;
    public static final int USER_IEN = 11;
    public static final int SSN = 12;
    public static final int NPI = 13;

    private static final Logger LOGGER = LoggerFactory.getLogger(ProviderProfileData.class);

    public ProviderProfileData(Site site, ResultSet table) {
        super(site, table);
    }
    public ProviderProfileData(Site site, java.util.List list) {
        super(site, list);
    }
    
    public User getProvider() {
        User prov = null;
        gov.va.med.vds.provider.dao.cache.model.DataModelIterator iter = new DataModelIterator(this);

        if (iter.hasNext()) {

            prov = new User();
            prov.setName(iter.getItem(ProviderProfileData.NAME));
            //chcsId = iter.getItem(ProviderProfileData.CHCSID);
            prov.setProviderClass(iter.getItem(2));
            prov.setGender(iter.getItem(ProviderProfileData.GENDER));
            prov.setDutyPhone(iter.getItem(ProviderProfileData.DUTY_PHONE_1));
            prov.setPager(iter.getItem(ProviderProfileData.PAGER));
            prov.setProviderFlag(iter.getItem(ProviderProfileData.PROVIDER_FLAG));
            prov.setSigClass(iter.getItem(ProviderProfileData.SIG_CLASS));
            prov.setUserIen(iter.getItem(ProviderProfileData.USER_IEN));

            prov.setEncryptedFederatedUid(EncryptUtil.encrypt(iter.getItem(ProviderProfileData.SSN)));
            //prov.setHostSiteId(mSite.getSiteCode());
            prov.setNPI(iter.getItem(ProviderProfileData.NPI));
            prov.setAgency(AGENCY_VA);
        }
 

        return prov;
    
    }

    public Alert[] getAlerts() {
        Alert[] alerts = null;
        
        if (mObjectList != null) {
            Object[] resArray = null;
            resArray = mObjectList.toArray();
            int size = resArray.length;
        
            alerts = new Alert[size];
            for (int i = 0; i < size; i++) {
                try {
                    Alert rslt = new Alert();
                    ALERT res = (ALERT) resArray[i];
                    rslt.setAlertId(res.getAlertId()); 
                    rslt.setMessageText(res.getMessageText());
                    fillCommonDataBeanValues(rslt);
                    alerts[i] = rslt;
                   
                } catch (CacheException e) {
                    LOGGER.error(e.getMessage());
                }
            }
        }
     
        return alerts;
      
    }
    
    public User[] getLookupNewPerson() {
        ArrayList<User> prov = null;
        
        Object[] resArray = null;
        resArray = mObjectList.toArray();
    
        prov = new ArrayList<User>();
        
        for (int i = 0; i < resArray.length; i++) {
            try {
    
                RPCNEWPERSON res = (RPCNEWPERSON) resArray[i];
                
                String name = res.getname();
                String ssn = res.getssn();
                String dob = res.getdob();
                if ("".equals(dob)) {
                    dob = "UNK";
                }
                String gender = res.getsex();
                if ("".equals(gender)) {
                    gender = "UNK";
                }
                String userIen = res.get_Id();
                String email = res.getemail();

                User person = new User();
                //p.setHostSiteId(mSiteCode);
                person.setName(name);
                person.setDob(dob);
                person.setGender(gender);
                person.setEncryptedFederatedUid(EncryptUtil.encrypt(ssn));
                person.setUserIen(userIen);
                person.setEmail(email);

                prov.add(person);
                   
            } catch (CacheException e) {
                LOGGER.error(e.getMessage());
            }
        }
    
        return prov.toArray(new User[0]);

    }

}