/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dataservice;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.provider.dao.OrderDao;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.User;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * @author DNS   uyehac
 */
public class ProviderOrdersDataService extends DataServiceBase {

    private static final Logger LOGGER = LoggerFactory.getLogger(ProviderOrdersDataService.class.getName());

    public Order[] getOrdersPending(QueryBean queryBean) throws VDSException {
        User user = queryBean.getUser();
        String userIen = user.getUserIen();
        if (StringUtils.isEmpty(userIen)) {
            throw new IllegalArgumentException("userIen cannot be null or empty.");
        }

        if (user.getHostSite() == null) {
            throw new IllegalArgumentException("hostSite cannot be null");
        }

        checkProviderQueryHostSite(user);

        RPCConnection cxn = null;

        try {
            cxn = setupCPRSContextSingleRemoteSourceQuery(user, user.getHostSite());
            OrderDao orderDao = new OrderDao(cxn);
            Order[] orders = orderDao.getOrdersPending(queryBean);

            return orders;
        } catch (Exception e) {
            throw new VDSException(e);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public Order[] getOrdersResulted(QueryBean queryBean) throws VDSException {
        User user = queryBean.getUser();
        String userIen = user.getUserIen();
        if (StringUtils.isEmpty(userIen)) {
            throw new IllegalArgumentException("userIen cannot be null or empty.");
        }

        if (user.getHostSite() == null) {
            throw new IllegalArgumentException("hostSite cannot be null");
        }

        checkProviderQueryHostSite(user);

        RPCConnection cxn = null;

        try {
            cxn = setupCPRSContextSingleRemoteSourceQuery(user, user.getHostSite());
            OrderDao orderDao = new OrderDao(cxn);
            Order[] orders = orderDao.getOrdersResulted(queryBean);

            return orders;
        } catch (Exception e) {
            throw new VDSException(e);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }

    public Order getOrderDetail(QueryBean queryBean) throws VDSException {
        User user = queryBean.getUser();
        if (StringUtils.isEmpty(queryBean.getItemId())) {
            throw new IllegalArgumentException("QueryBean itemId cannot be null or empty.");
        }

        if (user.getHostSite() == null) {
            throw new IllegalArgumentException("hostSite cannot be null");
        }

        checkProviderQueryHostSite(user);

        RPCConnection cxn = null;

        try {
            cxn = setupCPRSContextSingleRemoteSourceQuery(user, user.getHostSite());
            OrderDao orderDao = new OrderDao(cxn);
            Order order = orderDao.getOrderDetails(queryBean);

            return order;
        } catch (Exception e) {
            throw new VDSException(e);
        } finally {
            if (cxn != null) {
                try {
                    cxn.disconnect();
                } catch (RPCException e) {
                    LOGGER.error(e.getMessage(), e);
                }
            }
        }
    }
}
