/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

public class AdmissionDetail extends PatientAdmission {
    /**
     * Array of procedure objects representing procedures associated with this visit.
     * This will only be populated when requesting admission details.
     */
    private Procedure[] procedures;
    /**
     * Array of procedure Objects representing Diagnoses assoicated with this admission.
     * This will only be populated when requesting admission details.
     */
    private Diagnosis[] diagnoses;

    /**
     * Detail Text in a report form.
     */
    private String detailText;

    public AdmissionDetail() {
        super();
        procedures = null;
        diagnoses = null;
    }

    public AdmissionDetail(PatientAdmission adm) {
        super(adm);
    }


    /**
     * Array of procedures associated with the admission.  This will only be populated
     * when requesting admission details.
     */
    public Procedure[] getProcedures() {
        Procedure[] rtc = null;
        if (procedures != null) {
            rtc = procedures.clone();
        }
        return rtc;
    }

    public void setProcedures(Procedure[] newProcedures) {
        if (newProcedures == null) {
            this.procedures = null;
        } else {
            this.procedures = newProcedures.clone();
        }
    }

    /**
     * Array of diagnoses associated with the admission.  This will only be populated
     * when requesting admission details.
     */
    public Diagnosis[] getDiagnoses() {
        Diagnosis[] rtc = null;
        if (diagnoses != null) {
            rtc = diagnoses.clone();
        }
        return rtc;
    }

    public void setDiagnoses(Diagnosis[] newDiagnoses) {
        if (newDiagnoses == null) {
            this.diagnoses = null;
        } else {
            this.diagnoses = newDiagnoses.clone();
        }
    }

    /**
     * Detail text
     */
    public String getDetailText() {
        return detailText;
    }

    public void setDetailText(String detailText) {
        this.detailText = detailText;
    }
}