/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;


/**
 * ClinicalReminders are associated with a patient based on age, diagnosis, gender and various other parameters to enable preventive care maintanance.  Two type of ClinicalReminders are supported, those that are due or over due, and those that have been completed.
 */
public class ClinicalReminder extends DataBean {
    /**
     * Internal ID of the clinical reminder, used to retrieve the detailed text of a
     * clinical reminder.
     */
    private String id;
    /**
     * Descriptive text of the ClinicalReminder. (Summary)
     */
    private String description;
    /**
     * Date the clinical reminder is next due.
     */
    private Calendar dueDate;
    /**
     * Date the Clincialreminder was last completed.
     */
    private Calendar lastOccurrenceDate;

    private String dueDateString;
    private String lastOccurrenceDateString;
    /**
     * Details
     */
    private String detail;
    private String name;
    private String status;
    /**
     * Class: National, VISN, Local
     */
    private String reminderClass;

    /**
     * Internal Id of the Clinical Reminder.  This is used to retrieve detail information
     * of a clinical reminder.
     */
    public String getId() {
        return id;
    }

    public void setId(String newId) {
        id = newId;
    }

    /**
     * Description of the clinical reminder.
     */
    public String getDescription() {
        return description;
    }

    public void setDescription(String newDescription) {
        description = newDescription;
    }

    /**
     * Date the clinicalReminder is due.  This could be a date in the past if overdue.
     */
    public Calendar getDueDate() {
        return dueDate;
    }

    public void setDueDate(Calendar newDueDate) {
        dueDate = newDueDate;
    }

    /**
     * Last date the Clinical reminder was completed.
     */
    public Calendar getLastOccurrenceDate() {
        return lastOccurrenceDate;
    }

    public void setLastOccurrenceDate(Calendar newDate) {
        lastOccurrenceDate = newDate;
    }

    public String getDueDateString() {
        return dueDateString;
    }

    public void setDueDateString(String dueDateString) {
        this.dueDateString = dueDateString;
    }

    public String getLastOccurrenceDateString() {
        return lastOccurrenceDateString;
    }

    public void setLastOccurrenceDateString(String lastOccurrenceDateString) {
        this.lastOccurrenceDateString = lastOccurrenceDateString;
    }

    public String getDetail() {
        return detail;
    }

    public void setDetail(String detail) {
        this.detail = detail;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getStatus() {
        return status;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    public String getReminderClass() {
        return reminderClass;
    }

    public void setReminderClass(String reminderClass) {
        this.reminderClass = reminderClass;
    }
}