/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * Demographic information regarding a patient. This class extends the Patient class.
 */
public class PatientDemographics extends Patient {

    /**
     * Line 1 of address of patient.
     */
    private String address1;
    /**
     * Line 2 of the patient's address.
     */
    private String address2;
    /**
     * City of the patient's address.
     */
    private String city;
    /**
     * State where the patient resides.
     */
    private String state;
    /**
     * Zip code of the patient's address.
     */
    private String zipCode;
    /**
     * Primary Care Manage start and end dates.  Not populated for vista patients.
     */
    private Date pcmDates;
    /**
     * Primary User assigned to the patient.
     */
    private String primaryProvider;
    /**
     * Clinic name the patient is assigned to.
     */
    private String clinic;
    /**
     * Phone number of the patient's Primary Care Manager.
     */
    private String pcmPhone;
    /**
     * Primary phone number for the patient.
     */
    private String phone1;
    /**
     * Clinic name of the patient's PCM
     */
    private String pcmClinic;
    /**
     * Secondary phone number for patient.
     */
    private String phone2;
    /**
     * Internal if of the admission if the patient is an inpatient.
     */
    private String admissionId;
    /**
     * Admission date if the patient is an inpatient.
     */
    private String admissionDate;
    /**
     * Ward the patient is assigned to if an inpatient.
     */
    private String ward;
    /**
     * Internal id of ward if the patient is an inpatient.
     */
    private String wardId;

    /**
     * Internal lab id of the patient.  Only defined for vista patients.
     */
    private String labId;

    /**
     * Code Green Alert
     */
    private String codeGreen;

    private String percentServiceConnected;

    private Site site;

    private String sourceProtocol;

    /** DoD CDR record event identifier **/
    private String cdrEventId;

    private Code[] race;
    private Code[] ethnicity;

    private String rank;

    private String sponsorName;

    /**
     * Primary care provider bean
     */
    private Provider pcProvider;
    private Calendar pcAssigned;
    private String pcTeam;
    private String eligibility;
    private String eligibilityStatus;

    private Flag[] flags;

    private List<Person> nextOfKin;
    private List<Person> emergencyContact;
    private List<String> languages;
    private List<String> religion;
    private String maritalStatus;

    public Site getSite() {
        return site;
    }

    public void setSite(Site site) {
        this.site = site;
    }

    /**
     * Line 1 of the patient's address.
     */
    public String getAddress1() {
        return address1;
    }

    public void setAddress1(String newAddress1) {
        address1 = newAddress1;
    }

    /**
     * Line 2 of the patient address.
     */
    public String getAddress2() {
        return address2;
    }

    public void setAddress2(String newAddress2) {
        address2 = newAddress2;
    }

    /**
     * City of the patient's address.
     */
    public String getCity() {
        return city;
    }

    public void setCity(String newCity) {
        city = newCity;
    }

    /**
     * State of the patient's address.
     */
    public String getState() {
        return state;
    }

    public void setState(String newState) {
        state = newState;
    }

    /**
     * Zip Code of the patient's address.
     */
    public String getZipCode() {
        return zipCode;
    }

    public void setZipCode(String newZipCode) {
        zipCode = newZipCode;
    }

    /**
     * Start and end dates of PCM assignment. Not defined for Vista data.
     */
    public Date getPcmDates() {
        Date rtc = null;
        if (pcmDates != null) {
            rtc = (Date) pcmDates.clone();
        }
        return rtc;
    }

    public void setPcmDates(Date newPcmDates) {
        if (newPcmDates == null) {
            this.pcmDates = null;
        } else {
            this.pcmDates = (Date) newPcmDates.clone();
        }
    }

    /**
     * Primary User name assigned to the patient.
     */
    public String getPrimaryProvider() {
        return primaryProvider;
    }

    public void setPrimaryProvider(String newPrimaryProvider) {
        primaryProvider = newPrimaryProvider;
    }

    /**
     * Clinic name the patient is assigned to.
     */
    public String getClinic() {
        return clinic;
    }

    public void setClinic(String newClinic) {
        clinic = newClinic;
    }

    /**
     * User's phone number.
     */
    public String getPcmPhone() {
        return pcmPhone;
    }

    public void setPcmPhone(String newpcmPhone) {
        pcmPhone = newpcmPhone;
    }

    /**
     * Primary phone number of the patient.
     */
    public String getPhone1() {
        return phone1;
    }

    /**
     * Primary Phone number of the patient.
     */
    public void setPhone1(String newPhone1) {
        phone1 = newPhone1;
    }

    /**
     * Secondary phone number for the patient.
     */
    public String getPhone2() {
        return phone2;
    }

    public void setPhone2(String newPhone2) {
        phone2 = newPhone2;
    }

    /**
     * Internal id of the admission if the patient is an inpatient.
     */
    public String getAdmissionId() {
        return admissionId;
    }

    public void setAdmissionId(String newAdmissionId) {
        admissionId = newAdmissionId;
    }

    /**
     * Date of admission if the patient is an inpatient.
     */
    public String getAdmissionDate() {
        return admissionDate;
    }

    public void setAdmissionDate(String newAdmissionDate) {
        admissionDate = newAdmissionDate;
    }

    /**
     * Ward name if the patient is an inpatient.
     */
    public String getWard() {
        return ward;
    }

    public void setWard(String newWard) {
        ward = newWard;
    }

    /**
     * Internal id of the patient's ward if an inpatient.
     */
    public String getWardId() {
        return wardId;
    }

    public void setWardId(String newWardId) {
        wardId = newWardId;
    }

    /**
     * Internal lab id of the patient.  Only set for Vista data.
     */
    public String getLabId() {
        return labId;
    }

    public void setLabId(String newLabId) {
        labId = newLabId;
    }

    /**
     * Code Green Alert
     */
    public String getCodeGreen() {
        return codeGreen;
    }

    public void setCodeGreen(String newCodeGreen) {
        codeGreen = newCodeGreen;
    }

    public String getPercentServiceConnected() {
        return percentServiceConnected;
    }

    public void setPercentServiceConnected(String newPercentServiceConnected) {
        percentServiceConnected = newPercentServiceConnected;
    }

    public String getSourceProtocol() {
        return sourceProtocol;
    }

    public void setSourceProtocol(String sourceProtocol) {
        this.sourceProtocol = sourceProtocol;
    }

    public String getCdrEventId() {
        return cdrEventId;
    }

    public void setCdrEventId(String cdrEventId) {
        this.cdrEventId = cdrEventId;
    }

    public Code[] getRace() {
        Code[] rtc = null;
        if (race != null) {
            rtc = race.clone();
        }
        return rtc;
    }

    public void setRace(Code[] race) {
        if (race == null) {
            this.race = null;
        } else {
            this.race = race.clone();
        }
    }

    public Code[] getEthnicity() {
        Code[] rtc = null;
        if (ethnicity != null) {
            rtc = ethnicity.clone();
        }
        return rtc;
    }

    public void setEthnicity(Code[] ethnicity) {
        if (ethnicity == null) {
            this.ethnicity = null;
            
        } else {
            this.ethnicity = ethnicity.clone();            
        }
    }

    public String getRank() { return rank; }

    public void setRank(String rank) { this.rank = rank; }

    /**
     * Clinic name of the patient's PCM
     *
     * @return PCM clinic name
     */
    public String getPcmClinic() {
        return pcmClinic;
    }

    /**
     * Set's the PCM clinic name of the patient
     *
     * @param pcmClinic
     */
    public void setPcmClinic(String pcmClinic) {
        this.pcmClinic = pcmClinic;
    }

    public String getSponsorName() { return sponsorName; }

    public void setSponsorName(String sponsorName) { this.sponsorName = sponsorName; }

    public Provider getPcProvider() {
        return pcProvider;
    }

    public void setPcProvider(Provider pcProvider) {
        this.pcProvider = pcProvider;
    }

    public Calendar getPcAssigned() {
        return pcAssigned;
    }

    public void setPcAssigned(Calendar pcAssigned) {
        this.pcAssigned = pcAssigned;
    }

    public String getPcTeam() {
        return pcTeam;
    }

    public void setPcTeam(String pcTeam) {
        this.pcTeam = pcTeam;
    }

    public String getEligibility() {
        return eligibility;
    }

    public void setEligibility(String eligibility) {
        this.eligibility = eligibility;
    }

    public String getEligibilityStatus() {
        return eligibilityStatus;
    }

    public void setEligibilityStatus(String eligibilityStatus) {
        this.eligibilityStatus = eligibilityStatus;
    }

    public Flag[] getFlags() {
        Flag[] rtc = null;
        if (flags != null) {
            rtc = flags.clone();
        }
        return rtc;
    }

    public void setFlags(Flag[] flags) {
        if (flags == null) {
            this.flags = null;
        } else {
            this.flags = flags.clone();
        }
    }

    public List<Person> getNextOfKin() {
        return nextOfKin;
    }

    public void setNextOfKin(List<Person> nextOfKin) {
        this.nextOfKin = nextOfKin;
    }

    public List<Person> getEmergencyContact() {
        return emergencyContact;
    }

    public void setEmergencyContact(List<Person> emergencyContact) {
        this.emergencyContact = emergencyContact;
    }

    public List<String> getLanguages() {
        return languages;
    }

    public void setLanguages(List<String> languages) {
        this.languages = languages;
    }

    public List<String> getReligion() {
        return religion;
    }

    public void setReligion(List<String> religion) {
        this.religion = religion;
    }

    public String getMaritalStatus() {
        return maritalStatus;
    }

    public void setMaritalStatus(String maritalStatus) {
        this.maritalStatus = maritalStatus;
    }
}