/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */


package gov.va.med.vds.vistabeans;

import java.util.Calendar;

/**
 * Class representing a vital sign value.
 */
public class Vitals extends DataBean {

    /**
     * Internal id of the vital sign.  Used to retrieve vital sign results over time for
     * a particular vital sign.
     */
    private String vitalsIEN;
    /**
     * Type of vital sign. (TEMP, BP, Pulse, RR, Pain)
     */
    private String vitalType;
    /**
     * Value of the vital sign.
     */
    private String rate;
    /**
     * Date/time the vital sign was taken.
     */
    private Calendar dateTimeTaken;

    private String units;

    private String unitsCode;

    private String qualifiers;

    private String comments;

    private String resultComments;

    //private String bmi;

    private VitalModifier[] vitalModifiers;

    /**
     * Internal Id of vital sign type. Used to retrieve vital sign data of a particular
     * type over time.
     */
    public String getVitalsIEN() {
        return vitalsIEN;
    }

    public void setVitalsIEN(String NewVitalsIEN) {
        vitalsIEN = NewVitalsIEN;
    }

    /**
     * Type of vital sign. (T = Temperature, P = Pulse, BP = Blood Pressure,
     * R=Respiratory Rate, PN = Pain)
     */
    public String getVitalType() {
        return vitalType;
    }

    public void setVitalType(String NewVitalType) {
        vitalType = NewVitalType;
    }

    /**
     * Value of vital sign.
     */
    public String getRate() {
        return rate;
    }

    public void setRate(String NewRate) {
        rate = NewRate;
    }

    /**
     * Date/Time vital sign was taken.
     */
    public Calendar getDateTimeTaken() {
        return dateTimeTaken;
    }

    public void setDateTimeTaken(Calendar NewDateTimeTaken) {
        dateTimeTaken = NewDateTimeTaken;
    }

    public String getUnits() {
        return units;
    }

    public void setUnits(String units) {
        this.units = units;
    }

    public String getQualifiers() {
        return qualifiers;
    }

    public void setQualifiers(String qualifiers) {
        this.qualifiers = qualifiers;
    }

    public String getUnitsCode() {
        return unitsCode;
    }

    public void setUnitsCode(String unitsCode) {
        this.unitsCode = unitsCode;
    }

    public String getComments() {
        return comments;
    }

    public void setComments(String comments) {
        this.comments = comments;
    }

    public String getResultComments() {
        return resultComments;
    }

    public void setResultComments(String resultComments) {
        this.resultComments = resultComments;
    }

//    public String getBmi() {
//        return bmi;
//    }
//
//    public void setBmi(String bmi) {
//        this.bmi = bmi;
//    }

    public VitalModifier[] getVitalModifiers() {
        VitalModifier[] rtc = null;
        if (vitalModifiers != null) {
            rtc = vitalModifiers.clone();
        }
        return rtc;
    }

    public void setVitalModifiers(VitalModifier[] vitalModifiers) {
        if (vitalModifiers == null) {
            this.vitalModifiers = null;
        } else {
            this.vitalModifiers = vitalModifiers.clone();
        }
    }
}