/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.webservice;

import gov.va.med.foundations.utilities.ARMLog;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.common.dataservice.VistaStatus;
import gov.va.med.vds.common.dataservice.VistaStatusService;
import gov.va.med.vds.patient.dao.VLERDao;
import gov.va.med.vds.patient.dataservice.AdmissionsDataService;
import gov.va.med.vds.patient.dataservice.AllergyDataService;
import gov.va.med.vds.patient.dataservice.AppointmentsDataService;
import gov.va.med.vds.patient.dataservice.ClinicalRemindersDataService;
import gov.va.med.vds.patient.dataservice.EncountersDataService;
import gov.va.med.vds.patient.dataservice.ImmunizationsDataService;
import gov.va.med.vds.patient.dataservice.InsuranceDataService;
import gov.va.med.vds.patient.dataservice.LabsDataService;
import gov.va.med.vds.patient.dataservice.MedicationsDataService;
import gov.va.med.vds.patient.dataservice.OrdersDataService;
import gov.va.med.vds.patient.dataservice.PatientApptDataService;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.patient.dataservice.ProblemsDataService;
import gov.va.med.vds.patient.dataservice.ProceduresDataService;
import gov.va.med.vds.patient.dataservice.RadiologyDataService;
import gov.va.med.vds.patient.dataservice.VitalsDataService;
import gov.va.med.vds.provider.dataservice.ProviderAdmissionsDataService;
import gov.va.med.vds.provider.dataservice.ProviderAppointmentsDataService;
import gov.va.med.vds.provider.dataservice.ProviderConsultsDataService;
import gov.va.med.vds.provider.dataservice.ProviderLabsDataService;
import gov.va.med.vds.provider.dataservice.ProviderNotesDataService;
import gov.va.med.vds.provider.dataservice.ProviderOrdersDataService;
import gov.va.med.vds.provider.dataservice.UserDataService;
import gov.va.med.vds.vistabeans.AdmissionDetail;
import gov.va.med.vds.vistabeans.Alert;
import gov.va.med.vds.vistabeans.Allergy;
import gov.va.med.vds.vistabeans.AllergyDetail;
import gov.va.med.vds.vistabeans.ClinicalReminder;
import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.DeploymentForm;
import gov.va.med.vds.vistabeans.Diagnosis;
import gov.va.med.vds.vistabeans.Encounter;
import gov.va.med.vds.vistabeans.FreeTextReport;
import gov.va.med.vds.vistabeans.Immunization;
import gov.va.med.vds.vistabeans.InsuranceBean;
import gov.va.med.vds.vistabeans.LabOrder;
import gov.va.med.vds.vistabeans.LabResult;
import gov.va.med.vds.vistabeans.Medication;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.PatientDemographics;
import gov.va.med.vds.vistabeans.PatientDemographicsDetail;
import gov.va.med.vds.vistabeans.Problem;
import gov.va.med.vds.vistabeans.ProblemDetail;
import gov.va.med.vds.vistabeans.Procedure;
import gov.va.med.vds.vistabeans.ProgressNote;
import gov.va.med.vds.vistabeans.ProviderAppointmentBean;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.QuestionnaireBean;
import gov.va.med.vds.vistabeans.RadiologyReport;
import gov.va.med.vds.vistabeans.ReferralBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import gov.va.med.vds.vistabeans.VLERDocument;
import gov.va.med.vds.vistabeans.Vitals;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.jws.HandlerChain;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 *
 */
@WebService
@HandlerChain(file = "SoapHandlerChain.xml")
public class VistaData {
    private static final Logger LOGGER = LoggerFactory.getLogger(VistaData.class.getName());

    private static final AppConfig APP_CONFIG = AppConfig.getInstance();

    /**
     * Web context.
     */
    @Resource
    private WebServiceContext wsContext;

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientAdmissions")
    public List<PatientAdmission> getPatientAdmissions(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionsDataService admService = new AdmissionsDataService();
            List<PatientAdmission> rtc = admService.getPatientAdmissions(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getAdmissionDetails")
    public AdmissionDetail getAdmissionDetails(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionsDataService admService = new AdmissionsDataService();
            AdmissionDetail rtc = admService.getAdmissionDetails(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getAdmissionDiagnosis")
    public List<Diagnosis> getAdmissionDiagnosis(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionsDataService admService = new AdmissionsDataService();
            List<Diagnosis> rtc = admService.getAdmissionDiagnosis(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getAdmissionProcedures")
    public List<Procedure> getAdmissionProcedures(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AdmissionsDataService admService = new AdmissionsDataService();
            List<Procedure> rtc = admService.getAdmissionProcedures(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Retrieves list of patient allergies.
     * @param queryBean Query parameters.
     *                  Required params: User, Patient
     * @return A list of patient allergies.
     * @throws VDSException if there was an error..
     */
    @WebMethod(operationName = "getPatientAllergies")
    public List<Allergy> getPatientAllergies(@WebParam(name = "queryBean")
                        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AllergyDataService allergyService = new AllergyDataService();
            List<Allergy> rtc = allergyService.getPatientAllergies(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Retrieves patient allergy details from the provided location.
     * @param queryBean Query parameters.
     *                  Required params: User, Patient, itemId, recordSiteCode
     * @return AllergyDetail containing allergy report text.
     * @throws VDSException if there was an error..
     */
    @WebMethod(operationName = "getAllergyDetail")
    public AllergyDetail getAllergyDetail(@WebParam(name = "queryBean")
                        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AllergyDataService allergyService = new AllergyDataService();
            AllergyDetail rtc = allergyService.getAllergyDetail(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientAppointments")
    public List<PatientAppointments> getPatientAppointments(@WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            AppointmentsDataService apptService = new AppointmentsDataService();
            List<PatientAppointments> rtc = apptService.getPatientAppointments(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientEncounters")
    public List<Encounter> getPatientEncounters(@WebParam(name = "queryBean")
                                                            QueryBean queryBean) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            EncountersDataService encounterService = new EncountersDataService();
            List<Encounter> rtc = encounterService.getPatientEncounters(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getEncountersReport")
    public FreeTextReport getEncountersReport(@WebParam(name = "queryBean")
                                                            QueryBean queryBean) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            AppointmentsDataService apptService = new AppointmentsDataService();
            FreeTextReport rtc = apptService.getEncountersReport(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientClinicalReminders")
    public List<ClinicalReminder> getPatientClinicalReminders(@WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ClinicalRemindersDataService clinicalRemindersService = new ClinicalRemindersDataService();
            List<ClinicalReminder> rtc = clinicalRemindersService.getPatientClinicalReminders(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getClinicalReminderDetail")
    public FreeTextReport getClinicalReminderDetail(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ClinicalRemindersDataService clinicalRemindersService = new ClinicalRemindersDataService();
            FreeTextReport rtc = clinicalRemindersService.getClinicalReminderDetail(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientDemographics")
    public List<PatientDemographics> getPatientDemographics(@WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientDataService patientService = new PatientDataService();
            List<PatientDemographics> rtc = patientService.getPatientDemographics(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientDemographicsDetail")
    public List<PatientDemographicsDetail> getPatientDemographicsDetail(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientDataService patientService = new PatientDataService();
            List<PatientDemographicsDetail> rtc = patientService.getPatientDemographicsDetail(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Determines if a patient is flagged as sensitive in VistA.
     * @param queryBean Required queryBean params: User, patient
     * @return true if a patient is flagged as a sensitive in VistA.
     * @throws VDSException if an error has occurred.
     */
    @WebMethod(operationName = "isSensitivePatient")
    public Boolean isSensitivePatient(@WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientDataService patientDataService = new PatientDataService();
            Boolean rtc = patientDataService.isSensitivePatient(queryBean);
            logResponse(queryBean, begin, "", 0);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "lookupPatient")
    public Patient[] lookupPatient(@WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientDataService patientService = new PatientDataService();
            Patient[] rtc = patientService.lookupPatient(queryBean);
            if (rtc == null) {
                logResponse(queryBean, begin, "", 0);
            } else {
                logResponse(queryBean, begin, "", rtc.length);
            }
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Calls patient select RPC at each site specified in patient's vistaSites list.
     * @param queryBean QueryBean
     *    Required Params:
     *                  Fully qualified User and Patient objects.
     * @return true if patient select was successfully invoked at EACH vista site,
     *                  false otherwise (even if patient select was successful at some sites).
     * @throws VDSException if an error occurred.
     */
    @WebMethod(operationName = "selectPatient")
    public boolean selectPatient(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientDataService patientService = new PatientDataService();
            boolean rtc = patientService.selectPatient(queryBean);
            logResponse(queryBean, begin, "", 0);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }
    
    /**
     * Web service operation
     */
    @WebMethod(operationName = "login")
    public User login(@WebParam(name = "site")
        Site site, @WebParam(name = "accessCode")
        String accessCode, @WebParam(name = "verifyCode")
        String verifyCode) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            UserDataService providerService = new UserDataService();
            User rtc = providerService.login(site, accessCode, verifyCode);
            logResponse(null, begin, "", 0);
            return rtc;
        } catch (VDSException e) {
            String exMsg = e.toString();

            //do not log access & verify failures
            if (exMsg != null && !exMsg.contains("^")) {
                logResponse(null, begin, e.getMessage(), 0);
                LOGGER.error(e.getMessage(), e);
            }
            throw e;
        }
    }

    @WebMethod(operationName = "lookupUser")
    public User[] lookupUser(@WebParam(name = "site")
        Site site, @WebParam(name = "lookupstr")
        String lookupstr, @WebParam(name = "userId")
        String userId, @WebParam(name = "maxReturn") 
        int maxReturn) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            UserDataService providerService = new UserDataService();
            User[] rtc = providerService.lookupUser(site, lookupstr, userId, maxReturn);
            logResponse(null, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(null, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientImmunizations")
    public List<Immunization> getPatientImmunizations(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ImmunizationsDataService imService = new ImmunizationsDataService();
            List<Immunization> rtc = imService.getPatientImmunizations(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientMedications")
    public List<Medication> getPatientMedications(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            MedicationsDataService medsService = new MedicationsDataService();
            List<Medication> rtc = medsService.getPatientMedications(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }
    
    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientConsultRequests")
    public List<Consult> getPatientConsultRequests(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            List<Consult> rtc = notesService.getPatientConsultRequests(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientActiveConsultRequests")
    public List<Consult> getPatientActiveConsultRequests(@WebParam(name = "queryBean")
                                                   QueryBean queryBean) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            List<Consult> rtc = notesService.getPatientActiveConsultRequests(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientConsultRequest")
    public Consult getPatientConsultRequest(@WebParam(name = "queryBean")
                                                   QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            Consult rtc = notesService.getPatientConsultRequest(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getConsultReport")
    public List<FreeTextReport> getConsultReport(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            List<FreeTextReport> rtc = notesService.getConsultReport(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientDischargeSummaries")
    public List<ProgressNote> getPatientDischargeSummaries(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            List<ProgressNote> rtc = notesService.getPatientDischargeSummaries(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientProgressNotes")
    public List<ProgressNote> getPatientProgressNotes(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            List<ProgressNote> rtc = notesService.getPatientProgressNotes(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientOrders")
    public List<Order> getPatientOrders(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            OrdersDataService ordersService = new OrdersDataService();
            List<Order> rtc = ordersService.getPatientOrders(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientProblemList")
    public List<Problem> getPatientProblemList(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProblemsDataService problemService = new ProblemsDataService();
            List<Problem> rtc = problemService.getPatientProblemList(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProblemDetail")
    public ProblemDetail getProblemDetail(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProblemsDataService problemService = new ProblemsDataService();
            ProblemDetail rtc = problemService.getProblemDetail(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientRads")
    public List<RadiologyReport> getPatientRads(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            RadiologyDataService radService = new RadiologyDataService();
            List<RadiologyReport> rtc = radService.getPatientRads(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientVitals")
    public List<Vitals> getPatientVitals(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            VitalsDataService vitalsService = new VitalsDataService();
            List<Vitals> rtc = vitalsService.getPatientVitals(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientCurrentVitals")
    public List<Vitals> getPatientCurrentVitals(@WebParam(name = "queryBean")
        QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            VitalsDataService vitalsService = new VitalsDataService();
            List<Vitals> rtc = vitalsService.getPatientCurrentVitals(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getDRReferrals")
    public List<ReferralBean> getDRReferrals(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientDataService patientService = new PatientDataService();
            List<ReferralBean> rtc = patientService.getDRReferrals(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

//    @WebMethod(operationName = "getPrescriptionFills")
//    public Prescription getPrescriptionFills(@WebParam(name = "queryBean")
//            QueryBean queryBean) throws VDSException
//    {
//        try
//        {
//            MedicationsDataService medsService = new MedicationsDataService();
//            return medsService.getPrescriptionFills(queryBean);
//        }
//        catch (VDSException e)
//        {
//            logger.error(e.getMessage(), e);
//            throw e;
//        }
//    }

    @WebMethod(operationName = "getProgressNote")
    public FreeTextReport getProgressNote(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            FreeTextReport rtc = notesService.getProgressNote(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }


    @WebMethod(operationName = "getPatientDocumentList")
    public  List<VLERDocument> getPatientDocumentList(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            VLERDao vdao = new VLERDao();
            List<VLERDocument> rtc = vdao.getVLERDocumentList(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getPatientDocument")
    public  VLERDocument getPatientDocument(
            @WebParam(name = "vlerDoc") VLERDocument vlerDoc,
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            VLERDao vdao = new VLERDao();
            VLERDocument rtc = vdao.getVLERDocument(vlerDoc, queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderProfile")
    public User getProviderProfile(@WebParam(name = "site") Site site,
                                       @WebParam(name = "userIen") String userIen,
                                       @WebParam(name = "providerNpi") String providerNpi) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            UserDataService providerService = new UserDataService();
            User rtc = providerService.getProviderProfile(site, userIen, providerNpi);
            if (rtc == null) {
                logResponse(null, begin, "", 0);
            } else {
                logResponse(null, begin, "", 1);
            }
            return rtc;
        } catch (VDSException e) {
            logResponse(null, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderWorkload")
    public ProviderAppointmentBean[] getProviderWorkload(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            UserDataService providerService = new UserDataService();
            ProviderAppointmentBean[] rtc = providerService.getProviderWorkload(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getRadiologyReport")
    public RadiologyReport getRadiologyReport(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            RadiologyDataService radService = new RadiologyDataService();
            RadiologyReport rtc = radService.getRadiologyReport(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getUserAlerts")
    public Alert[] getUserAlerts(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            UserDataService providerService = new UserDataService();
            Alert[] rtc = providerService.getUserAlerts(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getVisitDiagnosis")
    public List<Diagnosis> getVisitDiagnosis(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientApptDataService patientApptService = new PatientApptDataService();
            List<Diagnosis> rtc = patientApptService.getVisitDiagnosis(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }
    @WebMethod(operationName = "getVisitNotes")
    public FreeTextReport getVisitNotes(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            gov.va.med.vds.patient.dataservice.NotesDataService notesService = new gov.va.med.vds.patient.dataservice.NotesDataService();
            FreeTextReport rtc = notesService.getVisitNotes(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getVisitProcedures")
    public List<Procedure> getVisitProcedures(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            PatientApptDataService patientApptService = new PatientApptDataService();
            List<Procedure> rtc = patientApptService.getVisitProcedures(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getMedicationDetail")
    public FreeTextReport getMedicationDetail(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            MedicationsDataService medsService = new MedicationsDataService();
            FreeTextReport rtc = medsService.getMedicationDetail(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getLabOrderResult")
    public List<LabResult> getLabOrderResult(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            LabsDataService labsService = new LabsDataService();
            List<LabResult> rtc = labsService.getLabOrderResult(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getPatientLabs")
    public List<LabOrder> getPatientLabs(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            LabsDataService labsService = new LabsDataService();
            List<LabOrder> rtc = labsService.getPatientLabs(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getPatientLabResults")
    public List<LabResult> getPatientLabResults(@WebParam(name = "queryBean")
                                         QueryBean queryBean) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            LabsDataService labsService = new LabsDataService();
            List<LabResult> rtc = labsService.getPatientLabResults(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }


    @WebMethod(operationName = "getPatientLabReport")
    public FreeTextReport getPatientLabReport(@WebParam(name = "queryBean")
                                         QueryBean queryBean) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            LabsDataService labsService = new LabsDataService();
            FreeTextReport rtc = labsService.getPatientLabReport(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getPatientLabTestResults")
    public LabResult[] getPatientLabTestResults(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            LabsDataService labsService = new LabsDataService();
            LabResult[] rtc = labsService.getPatientLabTestResults(queryBean);
            if (rtc == null) {
                logResponse(queryBean, begin, "", 0);
            } else {
                logResponse(queryBean, begin, "", rtc.length);
            }
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getOrderDetail")
    public FreeTextReport getOrderDetail(@WebParam(name = "queryBean")
            QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            OrdersDataService ordersService = new OrdersDataService();
            FreeTextReport rtc = ordersService.getOrderDetail(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderAppointments")
    public PatientAppointments[] getProviderAppointments(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderAppointmentsDataService appointmentsService =
                    new ProviderAppointmentsDataService();
            PatientAppointments[] rtc = appointmentsService.getProviderAppointments(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getWardAdmissions")
    public PatientAdmission[] getWardAdmissions(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderAdmissionsDataService admissionsService =
                    new ProviderAdmissionsDataService();
            PatientAdmission[] rtc = admissionsService.getWardAdmissions(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderAdmissions")
    public PatientAdmission[] getProviderAdmissions(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderAdmissionsDataService admissionsService =
                    new ProviderAdmissionsDataService();
            PatientAdmission[] rtc = admissionsService.getProviderAdmissions(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderConsultsRequested")
    public Consult[] getProviderConsultsRequested(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderConsultsDataService consultsService =
                    new ProviderConsultsDataService();
            Consult[] rtc = consultsService.getConsultsRequested(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderConsultsReceived")
    public Consult[] getProviderConsultsReceived(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderConsultsDataService consultsService =
                    new ProviderConsultsDataService();
            Consult[] rtc = consultsService.getConsultsReceived(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderUnsignedNotes")
    public ProgressNote[] getProviderUnsignedNotes(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderNotesDataService notesService =
                    new ProviderNotesDataService();
            ProgressNote[] rtc = notesService.getProviderUnsignedNotes(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderOrdersPending")
    public Order[] getProviderOrdersPending(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderOrdersDataService orderService =
                    new ProviderOrdersDataService();
            Order[] rtc = orderService.getOrdersPending(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderOrdersResulted")
    public Order[] getProviderOrdersResulted(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderOrdersDataService orderService =
                    new ProviderOrdersDataService();
            Order[] rtc = orderService.getOrdersResulted(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderOrderDetails")
    public Order getProviderOrderDetails(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderOrdersDataService orderService =
                    new ProviderOrdersDataService();
            Order rtc = orderService.getOrderDetail(queryBean);
            logResponse(queryBean, begin, "", 1);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    @WebMethod(operationName = "getProviderLabAbnormalResults")
    public LabResult[] getProviderLabAbnormalResults(
            @WebParam(name = "queryBean") QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProviderLabsDataService labService = new ProviderLabsDataService();
            LabResult[] rtc = labService.getAbnormalResults(queryBean);
            logResponse(queryBean, begin, "", rtc.length);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            String className = VistaData.class.getName();
            LoggerFactory.getLogger(className).error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientInsurance")
    public List<InsuranceBean> getPatientInsurance(@WebParam(name = "queryBean")
                                                   QueryBean queryBean) throws VDSException
    {
        long begin = System.currentTimeMillis();
        try {
            InsuranceDataService insuranceService = new InsuranceDataService();
            List<InsuranceBean> rtc = insuranceService.getPatientInsurance(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientQuestionnaires")
    public List<QuestionnaireBean> getPatientQuestionnaires(@WebParam(name = "queryBean")
                                                   QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();

        // This is a dummy call to get QuestionnaireBean into the vds data model.
        List<QuestionnaireBean> rtc = new ArrayList<QuestionnaireBean>();
        logResponse(queryBean, begin, "", rtc);
        return rtc;
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getDeploymentForms")
    public List<DeploymentForm> getDeploymentForms(@WebParam(name = "queryBean")
                                                   QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();

        // This is a dummy call to get DeploymentForm into the vds data model.
        List<DeploymentForm> rtc = new ArrayList<DeploymentForm>();
        logResponse(queryBean, begin, "", rtc);
        return rtc;
    }

    /**
     * Web service operation
     */
    @WebMethod(operationName = "getPatientProcedures")
    public List<Procedure> getPatientProcedures(@WebParam(name = "queryBean")
                                                QueryBean queryBean) throws VDSException {
        long begin = System.currentTimeMillis();
        try {
            ProceduresDataService proceduresService = new ProceduresDataService();
            List<Procedure> rtc = proceduresService.getPatientProcedures(queryBean);
            logResponse(queryBean, begin, "", rtc);
            return rtc;
        } catch (VDSException e) {
            logResponse(queryBean, begin, e.getMessage(), 0);
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Gets VistaDataService version: <MAJOR>.<MINOR>.<BUILD>
     * @return VistaDataService version
     */
    @WebMethod(operationName = "getVersion")
    public String getVersion() {
        return APP_CONFIG.getAppVersion();
    }

    @WebMethod(operationName = "resetStatus")
    public void resetStatus() {
        VistaDataStatus.resetStatus();
    }
    
    @WebMethod(operationName = "getStatus")
    public String getStatus() {
        return VistaDataStatus.getStatus();
    }

    @WebMethod(operationName = "getVistaStatusList")
    public List<VistaStatus> getVistasStatus(List<Site> siteList) {
        return new VistaStatusService().getVistaStatusList(siteList);
    }
    
//    private void logResponse(long begin, String err, int rcount) {
//        logResponse(null, begin, err, rcount);
//    }

    private void logResponse(QueryBean queryBean, long begin, String err, List<?> rtc) {
        if (rtc == null) {
            logResponse(queryBean, begin, err, 0);
        } else {
            logResponse(queryBean, begin, err, rtc.size());
        }
    }

    private void logResponse(QueryBean queryBean, long begin, String err, int rcount) {
        long end = System.currentTimeMillis();
        String patientId = "Patient ID was not set";
        String queryInfo = "";
        try {
            if (queryBean != null && queryBean.getPatient() != null) {
                patientId = queryBean.getPatient().getEDIPI();
                queryInfo = getKey(queryBean);
            }
            MessageContext msgctx = wsContext.getMessageContext();
            String operation = "" + msgctx.get(MessageContext.WSDL_OPERATION);
            String[] pieces = operation.split("\\}");
            if (pieces.length > 1) {
                operation = pieces[1];
            }

            String dsource = AppConfig.getInstance().getDataSource();
            ARMLog dbl = new ARMLog(dsource);
            dbl.logEvent(patientId, "VDS." + operation, err, end - begin, rcount, "", queryInfo);
            dbl.close();
            boolean error = false;
            if (err != null && err.length() > 0) {
                error = true;
            }
            VistaDataStatus.updateStatistics(operation, error, end - begin);
        } catch (Throwable t) {
            t = null;  // gobble up errors
        } 
    } 
              
    private String getKey(QueryBean queryBean) {
        StringBuffer rtc = new StringBuffer();
        if (queryBean != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss", Locale.getDefault());
            rtc.append(queryBean.getPatient().getEDIPI());
//            rtc.append('|');
////            rtc.append(type);
//            rtc.append('|');
            rtc.append("||");
            if (queryBean.getStartDate() != null) {
                rtc.append(sdf.format(queryBean.getStartDate().getTime()));
            }
            rtc.append('|');
            if (queryBean.getEndDate() != null) {
                rtc.append(sdf.format(queryBean.getEndDate().getTime()));
            }
            rtc.append('|');
            rtc.append(queryBean.getRecordSite());
            rtc.append('|');
            rtc.append(queryBean.getRequestingApp());
        }
        return rtc.toString();
    }
}
