/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice;

import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.provider.dataservice.UserDataService;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.apache.log4j.LogManager;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;

import java.util.Calendar;
import java.util.logging.Level;
import java.util.logging.Logger;

import static org.junit.Assert.fail;

/**
 *
 */
public class DataServiceBaseTest
{
    private static final Logger logger = Logger.getLogger(DataServiceBaseTest.class.getName());

    protected User user;
    protected Patient patient;
    protected String testPatientName;

    protected Site site;

    public DataServiceBaseTest()
    {
        site = TestSiteUtils.getInstance().getSite(TestConfig.get(TestConfig.PROVIDER_SITE_CODE));
    }

    @BeforeClass
    public static void setUpClass()
    {

    }

    @AfterClass
    public static void tearDownClass()
    {
    }

    @Before
    public void setUp()
    {
        LogManager.getRootLogger().setLevel(org.apache.log4j.Level.DEBUG);

        try
        {


            testPatientName = TestConfig.get(TestConfig.PATIENT_NAME);

            //select test patient
            PatientDataService patientService = new PatientDataService();
            QueryBean queryBean = new QueryBean();

            user = loginTestProvider();


            queryBean.setUser(user);
            queryBean.setMax(100);
            queryBean.setRecordSite(user.getHostSite());
            queryBean.setItemId(testPatientName);   // Admissions
            // Allergy
            // Clinical Reminders
            // Immunizations
            // Labs
            // Medications
            // Orders
            // Problems

            Patient[] patients = patientService.lookupPatient(queryBean);
            queryBean = new QueryBean();

            queryBean.setPatient(patients[0]);
            queryBean.setUser(user);

            if(!select(queryBean))
                fail("Failed to select patient");

            this.patient = patients[0];

        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }

    protected User loginTestProvider() throws VistaKernelHashCountLimitExceededException, VDSException {
        String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
        String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);

        UserDataService providerService = new UserDataService();
        User user = providerService.login(site, VistaKernelHash.encrypt(accessCode, false), VistaKernelHash.encrypt(verifyCode, false));

        if (user.getUserIen().contains("^"))
            throw new RuntimeException("Invalid user");

        return user;
    }

    protected User loginTestProvider(String userIen) throws VistaKernelHashCountLimitExceededException, VDSException {
        String accessCode = "janus";
        String verifyCode = "test::" + userIen;

        return loginTestProvider(accessCode, verifyCode, site);
    }

    protected User loginTestProvider(String userIen, Site site) throws VistaKernelHashCountLimitExceededException, VDSException {
        String accessCode = "janus";
        String verifyCode = "test::" + userIen;

        return loginTestProvider(accessCode, verifyCode, site);
    }

    protected User loginTestProvider(String accessCode, String verifyCode, Site site) throws VistaKernelHashCountLimitExceededException, VDSException {
        accessCode = VistaKernelHash.encrypt(accessCode, false);
        verifyCode = VistaKernelHash.encrypt(verifyCode, false);

        UserDataService userDataService = new UserDataService();
        User user = userDataService.login(site, accessCode, verifyCode);

        if (user.getUserIen().contains("^"))
            throw new RuntimeException("Invalid user");

        return user;
    }

    @After
    public void tearDown()
    {

    }

    protected boolean select(QueryBean qb) throws VDSException {
        PatientDataService patientService = new PatientDataService();
        return patientService.selectPatient(qb);
    }

    protected QueryBean setupQuery(User user, Patient patient, Calendar startDate, Calendar endDate)
    {
        QueryBean qb = new QueryBean();
        qb.setUser(user);
        qb.setPatient(patient);
        qb.setStartDate(startDate);
        qb.setEndDate(endDate);
        qb.setMax(new Integer(999));
        qb.setActive(new Integer(0));
        return qb;
    }

    protected QueryBean setupQuery(User user, Patient patient)
    {
        QueryBean qb = new QueryBean();
        qb.setUser(user);
        qb.setPatient(patient);
        qb.setMax(new Integer(999));
        qb.setActive(new Integer(0));
        return qb;
    }

    protected Patient selectPatient(User user, String patientName)
    {
        try
        {
            PatientDataService pds = new PatientDataService();
            QueryBean qb = new QueryBean();
            qb.setItemId(patientName);
            Patient[] patients = pds.lookupPatient(qb);
            if (patients != null && patients.length > 0)
            {
                qb = new QueryBean();
                qb.setPatient(patients[0]);
                qb.setUser(user);
                qb.setRecordSite(user.getHostSite());
                if(pds.selectPatient(qb))
                    return patients[0];
            }
        }
        catch (VDSException ex)
        {
            logger.log(Level.SEVERE, ex.getMessage(), ex);
        }
        return null;
    }

}
