/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.AllergyDataService;
import gov.va.med.vds.vistabeans.Allergy;
import gov.va.med.vds.vistabeans.AllergyDetail;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;
/**
 *
 *
 */
public class AllergyDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientAllergies()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            AllergyDataService allergyService = new AllergyDataService();
            List<Allergy> allergies = allergyService.getPatientAllergies(queryBean);
            assertTrue(allergies.size() > 0);
            for(Allergy allergy : allergies)
            {
                assertFalse(StringUtils.isEmpty(allergy.getSourceProtocol()));
                assertFalse(allergy.getSite() == null);
                assertFalse(allergy.getAllergyName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetAllergyDetail()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            AllergyDataService allergyService = new AllergyDataService();
            List<Allergy> allergies = allergyService.getPatientAllergies(queryBean);
            assertTrue(allergies.size() > 0);

            QueryBean qb2 = new QueryBean();
            qb2.setUser(user);
            qb2.setPatient(patient);
            qb2.setItemId(allergies.get(0).getAllergyId());
            qb2.setRecordSite(allergies.get(0).getSite());

            AllergyDetail allergyDetail = allergyService.getAllergyDetail(qb2);
            assertFalse(StringUtils.isEmpty(allergyDetail.getReportText()));
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
    
}
