/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.AppointmentsDataService;
import gov.va.med.vds.patient.dataservice.EncountersDataService;
import gov.va.med.vds.vistabeans.Encounter;
import gov.va.med.vds.vistabeans.PatientAppointments;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class EncounterDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientEncounters()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(1980, 0, 1);
            endDate.set(2014, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            EncountersDataService encountersService = new EncountersDataService();
            List<Encounter> encounters = encountersService.getPatientEncounters(queryBean);
            assertTrue(encounters.size() > 0);
            for(Encounter encounter : encounters)
            {
                assertFalse(StringUtils.isEmpty(encounter.getSourceProtocol()));
                assertFalse(encounter.getSite() == null);
                assertFalse(encounter.getClinic().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
