/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.LabsDataService;
import gov.va.med.vds.vistabeans.LabOrder;
import gov.va.med.vds.vistabeans.LabResult;
import gov.va.med.vds.vistabeans.QueryBean;
import gov.va.med.vds.vistabeans.Site;
import org.junit.Test;

import java.util.Calendar;
import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class LabsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientLabs()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 0, 1);
            endDate.set(2014, 9, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            LabsDataService labsService = new LabsDataService();
            List<LabOrder> labs = labsService.getPatientLabs(queryBean);

            
            assertTrue(labs.size() > 0);
            for(LabOrder lab : labs)
            {
                assertFalse(StringUtils.isEmpty(lab.getSourceProtocol()));
                assertFalse(lab.getSite() == null);
                assertFalse(lab.getTestName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientLabResults()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(1990, 0, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            LabsDataService labsService = new LabsDataService();
            List<LabResult> labs = labsService.getPatientLabResults(queryBean);

            assertTrue(labs.size() > 0);
            for(LabResult lab : labs)
            {
                assertFalse(StringUtils.isEmpty(lab.getSourceProtocol()));
                assertFalse(lab.getSite() == null);
                assertFalse(lab.getTestName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetLabOrderResults()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(1990, 0, 1);
            endDate.set(2013, 2, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            LabsDataService labsService = new LabsDataService();
            List<LabOrder> labs = labsService.getPatientLabs(queryBean);
            assertTrue(labs.size() > 0);

            int i = 0;
            boolean isMI = true;
            String accessionNumber = null;
            Site recordSite = null;
            Calendar collectionDate = null;

            while (labs.size() > i++ && isMI) {
                accessionNumber = labs.get(i).getAccessionNumber();
                recordSite = labs.get(i).getSite();
                collectionDate = labs.get(i).getCollectionDate();

                isMI = "MI".equalsIgnoreCase(labs.get(i).getSubscript());
            }
            queryBean.setItemId(accessionNumber);
//            queryBean.setItemId("CH 0317 234");
            queryBean.setRecordSite(recordSite);
            queryBean.setStartDate(collectionDate);

            List<LabResult> labResults = labsService.getLabOrderResult(queryBean);

            assertTrue(labResults.size() > 0);
            for(LabResult labResult : labResults)
            {
                assertFalse(StringUtils.isEmpty(labResult.getSourceProtocol()));
                assertFalse(labResult.getSite() == null);
                assertFalse(labResult.getTestName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetLabTestResults()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(1990, 0, 1);
            endDate.set(2013, 2, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            LabsDataService labsService = new LabsDataService();
            List<LabOrder> labs = labsService.getPatientLabs(queryBean);
            assertTrue(labs.size() > 0);

            int i = 0;
            boolean isMI = true;
            String accessionNumber = null;
            Site recordSite = null;
            Calendar collectionDate = null;

            while (labs.size() > i++ && isMI) {
                accessionNumber = labs.get(i).getAccessionNumber();
                recordSite = labs.get(i).getSite();
                collectionDate = labs.get(i).getCollectionDate();

                isMI = "MI".equalsIgnoreCase(labs.get(i).getSubscript());
            }
            queryBean.setItemId(accessionNumber);
//            queryBean.setItemId("CH 0317 234");
            queryBean.setRecordSite(recordSite);
            queryBean.setStartDate(collectionDate);

            List<LabResult> labResults = labsService.getLabOrderResult(queryBean);

            assertTrue(labResults.size() > 0);

            String testName = labResults.get(0).getTestName();
//            queryBean.setItemId(testName);
            queryBean.setItemId("GLUCOSE");
            queryBean.setStartDate(startDate);
            queryBean.setEndDate(endDate);

            LabResult[] labTestResults = labsService.getPatientLabTestResults(queryBean);
            assertTrue(labTestResults.length > 0);

            for(LabResult labResult : labTestResults)
            {
                assertFalse(StringUtils.isEmpty(labResult.getSourceProtocol()));
                assertFalse(labResult.getSite() == null);
                assertFalse(labResult.getTestName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

}
