/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.PatientDataService;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.PatientDemographics;
import gov.va.med.vds.vistabeans.PatientDemographicsDetail;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class PatientDataServiceTest extends DataServiceBaseTest
{
    @Override
    @Before
    public void setUp()
    {
        testPatientName = TestConfig.get(TestConfig.PATIENT_NAME);
    }

    @Test
    public void testLookupPatient()
    {
        try {
            PatientDataService patientService = new PatientDataService();
            QueryBean queryBean = setupQuery(loginTestProvider(), null, null, null);
            queryBean.setItemId(testPatientName);
            queryBean.setRecordSite(queryBean.getUser().getHostSite());
            Patient[] matches = patientService.lookupPatient(queryBean);
            assertTrue(matches.length > 0);
            assertTrue(matches[0].getName().equals(testPatientName));
        } catch (VDSException ex) {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }
    
    @Test
    public void testSelect()
    {
        try
        {
            String testPatientSSN = TestConfig.get(TestConfig.PATIENT_SSN);
            String testPatientICN = TestConfig.get(TestConfig.PATIENT_ICN);

            try {
                this.patient = selectTestPatient();
            } catch (VistaKernelHashCountLimitExceededException e) {
                fail(e.getMessage());
            }

            assertTrue(patient.getName().equals(testPatientName));
            assertTrue(patient.getSSN().equals(testPatientSSN));
            assertTrue(patient.getICN().equals(testPatientICN));

        } catch (VDSException ex) {
            fail(ex.getMessage());
        }
    }

    private Patient selectTestPatient() throws VistaKernelHashCountLimitExceededException, VDSException {
        //select test patient
        PatientDataService patientService = new PatientDataService();
        QueryBean queryBean = new QueryBean();

        user = loginTestProvider();
        queryBean.setUser(user);

        queryBean.setItemId(testPatientName);
        queryBean.setMax(100);
        queryBean.setRecordSite(queryBean.getUser().getHostSite());
        Patient[] patients = patientService.lookupPatient(queryBean);
        queryBean = new QueryBean();
        queryBean.setPatient(patients[0]);
        queryBean.setUser(user);

        if(select(queryBean))
            return patients[0];

        return null;
    }

    @Test
    public void testGetPatientDemographics()
    {
        try
        {
            try {
                this.patient = selectTestPatient();
            } catch (VistaKernelHashCountLimitExceededException e) {
                fail(e.getMessage());
            }

            QueryBean queryBean = setupQuery(user, patient, null, null);

            PatientDataService patientService = new PatientDataService();
            List<PatientDemographics> pDemos = patientService.getPatientDemographics(queryBean);
            assertTrue(pDemos.size() > 0);
            for(PatientDemographics demo : pDemos)
            {
                assertFalse(StringUtils.isEmpty(demo.getSourceProtocol()));
                assertFalse(demo.getSite() == null);
                assertFalse(demo.getName().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }


        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientDemographicDetails()
    {
        try
        {
            try {
                this.patient = selectTestPatient();
            } catch (VistaKernelHashCountLimitExceededException e) {
                fail(e.getMessage());
            }

            QueryBean queryBean = setupQuery(user, patient, null, null);

            PatientDataService patientService = new PatientDataService();
            List<PatientDemographicsDetail> pDemoDetails = patientService.getPatientDemographicsDetail(queryBean);
            assertTrue(pDemoDetails.size() > 0);
            for(PatientDemographicsDetail demo : pDemoDetails)
            {
                assertFalse(StringUtils.isEmpty(demo.getSourceProtocol()));
                assertFalse(demo.getSite() == null);
                assertFalse(demo.getReport().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }

        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testIsSensitivePatient()
    {
        try
        {
            try {
                this.patient = selectTestPatient();
            } catch (VistaKernelHashCountLimitExceededException e) {
                fail(e.getMessage());
            }

            QueryBean queryBean = setupQuery(user, patient, null, null);
            PatientDataService patientService = new PatientDataService();
            Boolean resp = patientService.isSensitivePatient(queryBean);

            assertTrue(resp);
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
