/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.patient;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.DataServiceBase;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.patient.dataservice.ProblemsDataService;
import gov.va.med.vds.vistabeans.Problem;
import gov.va.med.vds.vistabeans.ProblemDetail;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;

/**
 *
 *
 */
public class ProblemsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPatientProblems()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            queryBean.setStatus("ALL");
            ProblemsDataService problemService = new ProblemsDataService();
            List<Problem> problems = problemService.getPatientProblemList(queryBean);
            assertTrue(problems.size() > 0);
            for(Problem problem : problems)
            {
                assertFalse(StringUtils.isEmpty(problem.getSourceProtocol()));
                assertFalse(problem.getSite() == null);
                assertFalse(problem.getDescription().equals(DataServiceBase.CONNECTION_UNAVAILABLE));
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetProblemDetail()
    {
        try
        {
            QueryBean queryBean = setupQuery(user, patient, null, null);
            queryBean.setStatus("ALL");
            ProblemsDataService problemService = new ProblemsDataService();
            List<Problem> problems = problemService.getPatientProblemList(queryBean);
            assertTrue(problems.size() > 0);
            queryBean = setupQuery(user, patient, null, null);
            queryBean.setRecordSite(problems.get(0).getSite());
            queryBean.setItemId(problems.get(0).getId());
            ProblemDetail detail = problemService.getProblemDetail(queryBean);
            assertTrue(detail.getDescription().length() > 0);
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
