/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.provider;

import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.provider.dataservice.ProviderAdmissionsDataService;
import gov.va.med.vds.vistabeans.PatientAdmission;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.*;

/**
 * Gets the list of appointments for a specific user.
 */
public class AdmissionsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetWardAdmissions()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setItemId("235");

            ProviderAdmissionsDataService ads = new ProviderAdmissionsDataService();
            PatientAdmission[] admissions = ads.getWardAdmissions(queryBean);
            assertTrue(admissions.length > 0);
            for(PatientAdmission admission : admissions)
            {
                assertFalse(StringUtils.isEmpty(admission.getSourceProtocol()));
                assertFalse(admission.getSite() == null);
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetProviderAdmissions()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            ProviderAdmissionsDataService ads = new ProviderAdmissionsDataService();
            PatientAdmission[] admissions = ads.getProviderAdmissions(queryBean);
            assertTrue(admissions.length > 0);
            for(PatientAdmission admission : admissions)
            {
                assertFalse(StringUtils.isEmpty(admission.getSourceProtocol()));
                assertFalse(admission.getSite() == null);
            }
        }
        catch(VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
