/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.provider;

import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.provider.dataservice.ProviderConsultsDataService;
import gov.va.med.vds.vistabeans.Consult;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.*;

/**
 * Gets the list of appointments for a specific user.
 */
public class ConsultDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetConsultsRequested()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 2, 1);
            endDate.set(2010, 3, 1);

            user = loginTestProvider("janus", "test::453", site);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setUser(user);

            ProviderConsultsDataService cds = new ProviderConsultsDataService();
            Consult[] consults = cds.getConsultsRequested(queryBean);
            assertTrue(consults.length > 0);
            for (Consult consult : consults)
            {
                assertFalse(StringUtils.isEmpty(consult.getId()));
                assertFalse(StringUtils.isEmpty(consult.getSourceProtocol()));
                assertFalse(consult.getSite() == null);
            }
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testGetConsultsReceived()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 1, 1);
            endDate.set(2010, 12, 1);

            user = loginTestProvider("444", site);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            ProviderConsultsDataService cds = new ProviderConsultsDataService();
            Consult[] consults = cds.getConsultsReceived(queryBean);
            assertTrue(consults.length > 0);
            for (Consult consult : consults)
            {
                assertFalse(StringUtils.isEmpty(consult.getId()));
                assertFalse(StringUtils.isEmpty(consult.getSourceProtocol()));
                assertFalse(consult.getSite() == null);
            }
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }
}
