/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.provider;

import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.provider.dataservice.ProviderLabsDataService;
import gov.va.med.vds.vistabeans.LabResult;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.*;

public class LabsDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetAbnormalLabResults()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 2, 1);
            endDate.set(2010, 3, 1);

            user = loginTestProvider("59607", site);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            ProviderLabsDataService lds = new ProviderLabsDataService();
            LabResult labResults[] = lds.getAbnormalResults(queryBean);
            assertTrue(labResults.length > 0);
            for (LabResult labResult : labResults)
            {
                assertFalse(StringUtils.isEmpty(labResult.getSourceProtocol()));
                assertFalse(labResult.getSite() == null);
            }
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }
}
