/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.dataservice.provider;

import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.rpc.StringUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.dataservice.DataServiceBaseTest;
import gov.va.med.vds.provider.dataservice.ProviderOrdersDataService;
import gov.va.med.vds.vistabeans.Order;
import gov.va.med.vds.vistabeans.QueryBean;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.*;

/**
 * Gets the list of appointments for a specific user.
 */
public class OrdersDataServiceTest extends DataServiceBaseTest
{
    @Test
    public void testGetPendingOrders()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 1, 1);
            endDate.set(2011, 4, 1);

            user = loginTestProvider("59607", site);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            ProviderOrdersDataService ods = new ProviderOrdersDataService();
            Order orders[] = ods.getOrdersPending(queryBean);
            assertTrue(orders.length > 0);
            for (Order order : orders)
            {
                assertFalse(StringUtils.isEmpty(order.getSourceProtocol()));
                assertFalse(order.getSite() == null);
            }
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testGetResultedOrders()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2010, 2, 1);
            endDate.set(2010, 3, 1);

            user = loginTestProvider("59607", site);
            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);

            ProviderOrdersDataService ods = new ProviderOrdersDataService();
            Order[] orders = ods.getOrdersResulted(queryBean);
            assertTrue(orders.length > 0);
            for (Order order : orders)
            {
                assertFalse(StringUtils.isEmpty(order.getSourceProtocol()));
                assertFalse(order.getSite() == null);
            }
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        } catch (VistaKernelHashCountLimitExceededException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testGetOrderDetail()
    {
        try
        {
            Calendar startDate = Calendar.getInstance();
            Calendar endDate = Calendar.getInstance();
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            QueryBean queryBean = setupQuery(user, patient, startDate, endDate);
            queryBean.setItemId("2523448.1");

            ProviderOrdersDataService ods = new ProviderOrdersDataService();
            Order order = ods.getOrderDetail(queryBean);
            assertTrue(order.getOrderDetail().length() > 0);
            assertFalse(StringUtils.isEmpty(order.getSourceProtocol()));
            assertFalse(order.getSite() == null);
        }
        catch (VDSException ex)
        {
            fail(ex.getMessage());
        }
    }
}
