/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.provider.dao.cache.handler;

import com.intersys.objects.CacheException;
import gov.va.med.foundations.utilities.VistaKernelHash;
import gov.va.med.foundations.utilities.VistaKernelHashCountLimitExceededException;
import gov.va.med.vds.TestConfig;
import gov.va.med.vds.TestSiteUtils;
import gov.va.med.vds.base.dataservice.VDSException;
import gov.va.med.vds.common.AppConfig;
import gov.va.med.vds.provider.dao.cache.CacheSiteConnection;
import gov.va.med.vds.provider.dataservice.UserDataService;
import gov.va.med.vds.vistabeans.Patient;
import gov.va.med.vds.vistabeans.Site;
import gov.va.med.vds.vistabeans.User;
import org.junit.After;
import org.junit.Before;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Calendar;
import java.util.Map;

/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for Pacific Telehealth & Technology Hui/JITC
 */

public abstract class BaseHandlerTest
{
    private static final Logger logger = LoggerFactory.getLogger(BaseHandlerTest.class);

    private static final AppConfig APP_CONFIG = AppConfig.getInstance();

    protected CacheSiteConnection cacheConnection;
    protected String siteCode;
    protected User user;
    protected Patient patient;
    protected Calendar startDate;
    protected Calendar endDate;

    protected Site site;

    @Before
    public void setUp() throws VDSException, CacheException, VistaKernelHashCountLimitExceededException {
        siteCode = TestConfig.get(TestConfig.PROVIDER_SITE_CODE);

        site = TestSiteUtils.getInstance().getSite(siteCode);

        cacheConnection = createCacheConnection(site);
        cacheConnection.setPatientIEN(TestConfig.get(TestConfig.EHR_PATIENT_IEN));
    }

    @After
    public void tearDown()
    {

    }

    protected CacheSiteConnection createCacheConnection(Site site) throws CacheException
    {
        return new CacheSiteConnection(site);
    }

    protected User loginTestProvider() throws VistaKernelHashCountLimitExceededException, VDSException {
        String accessCode = TestConfig.get(TestConfig.PROVIDER_ACCESS_CODE);
        String verifyCode = TestConfig.get(TestConfig.PROVIDER_VERIFY_CODE);

        UserDataService providerService = new UserDataService();
        User user = providerService.login(site, VistaKernelHash.encrypt(accessCode, false), VistaKernelHash.encrypt(verifyCode, false));
        if (user.getUserIen().contains("^"))
            throw new RuntimeException("Invalid user");

        return user;
    }


}
