/*
 * Janus 4.0 (c)
 * Copyright (c) 2014 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCConnectionStrategy;
import gov.va.med.vds.patient.dao.rpc.ProceduresRPC;
import gov.va.med.vds.vistabeans.Procedure;
import org.junit.Test;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 *
 *
 */
public class ProceduresRPCTest
{
    private static final Logger logger = Logger.getLogger(ProceduresRPCTest.class.getName());

    @Test
    public void testParseProcedures() throws Exception
    {
        RPCConnection mockRPCConnection = mock(RPCConnection.class);
        when(mockRPCConnection.getConnectionStrategy()).thenReturn(RPCConnectionStrategy.NATIONAL);

        ProceduresRPC proceduresRPC = new ProceduresRPC(mockRPCConnection);
        String methodName = "parseProcedures";
        Class[] parameterTypes = new Class[1];
        parameterTypes[0] = String.class;
        Object[] parameters = new Object[1];
        Method method = proceduresRPC.getClass().getDeclaredMethod(methodName, parameterTypes);
        method.setAccessible(true);

        parameters[0] = PROCEDURES_VPR_JSON_RESPONSE;
        Procedure[] procedures = (Procedure[]) method.invoke(proceduresRPC, parameters);

        assertTrue(procedures.length == 4);
    }

    private static final String PROCEDURES_VPR_JSON_RESPONSE = "{\n" +
            "\t\"apiVersion\": \"1.01\",\n" +
            "\t\"data\": {\n" +
            "\t\t\"updated\": \"20141223111536\",\n" +
            "\t\t\"totalItems\": 4,\n" +
            "\t\t\"items\": [{\n" +
            "\t\t\t\"cptCode\": \"urn:cpt:85610\",\n" +
            "\t\t\t\"dateTimeEntered\": 200606012357,\n" +
            "\t\t\t\"encounterName\": \"LAB DIV 500 OOS ID 108 Jun 01, 2006\",\n" +
            "\t\t\t\"encounterUid\": \"urn:va:B362:8:visit:5035\",\n" +
            "\t\t\t\"facilityCode\": 888,\n" +
            "\t\t\t\"facilityName\": \"FT. LOGAN\",\n" +
            "\t\t\t\"localId\": 2478,\n" +
            "\t\t\t\"location\": \"LAB DIV 500 OOS ID 108\",\n" +
            "\t\t\t\"name\": \"PROTHROMBIN TIME\",\n" +
            "\t\t\t\"quantity\": 1,\n" +
            "\t\t\t\"type\": \"U\",\n" +
            "\t\t\t\"uid\": \"urn:va:B362:8:cpt:2478\"\n" +
            "\t\t},\n" +
            "\t\t{\n" +
            "\t\t\t\"cptCode\": \"urn:cpt:82565\",\n" +
            "\t\t\t\"dateTimeEntered\": 19961224101137,\n" +
            "\t\t\t\"encounterName\": \"LAB DIV 500 OOS ID 108 Dec 24, 1996\",\n" +
            "\t\t\t\"encounterUid\": \"urn:va:B362:8:visit:242\",\n" +
            "\t\t\t\"facilityCode\": 500,\n" +
            "\t\t\t\"facilityName\": \"CAMP MASTER\",\n" +
            "\t\t\t\"localId\": 163,\n" +
            "\t\t\t\"location\": \"LAB DIV 500 OOS ID 108\",\n" +
            "\t\t\t\"name\": \"ASSAY CREATININE\",\n" +
            "\t\t\t\"quantity\": 1,\n" +
            "\t\t\t\"type\": \"U\",\n" +
            "\t\t\t\"uid\": \"urn:va:B362:8:cpt:163\"\n" +
            "\t\t},\n" +
            "\t\t{\n" +
            "\t\t\t\"cptCode\": \"urn:cpt:99401\",\n" +
            "\t\t\t\"dateTimeEntered\": 19950718091835,\n" +
            "\t\t\t\"encounterName\": \"5 WEST PSYCH Jul 18, 1995\",\n" +
            "\t\t\t\"encounterUid\": \"urn:va:B362:8:visit:1797\",\n" +
            "\t\t\t\"facilityCode\": 888,\n" +
            "\t\t\t\"facilityName\": \"FT. LOGAN\",\n" +
            "\t\t\t\"localId\": 1299,\n" +
            "\t\t\t\"location\": \"5 WEST PSYCH\",\n" +
            "\t\t\t\"name\": \"Preventive Medicine Counseling and\\/or Risk Factor Reduction Intervention provided to an Individual (15 minutes) (CPT-4 99401)\",\n" +
            "\t\t\t\"quantity\": 1,\n" +
            "\t\t\t\"type\": \"U\",\n" +
            "\t\t\t\"uid\": \"urn:va:B362:8:cpt:1299\"\n" +
            "\t\t},\n" +
            "\t\t{\n" +
            "\t\t\t\"cptCode\": \"urn:cpt:90707\",\n" +
            "\t\t\t\"dateTimeEntered\": 19950718091835,\n" +
            "\t\t\t\"encounterName\": \"5 WEST PSYCH Jul 18, 1995\",\n" +
            "\t\t\t\"encounterUid\": \"urn:va:B362:8:visit:1797\",\n" +
            "\t\t\t\"facilityCode\": 888,\n" +
            "\t\t\t\"facilityName\": \"FT. LOGAN\",\n" +
            "\t\t\t\"localId\": 1300,\n" +
            "\t\t\t\"location\": \"5 WEST PSYCH\",\n" +
            "\t\t\t\"name\": \"MMR VACCINE, SC\",\n" +
            "\t\t\t\"quantity\": 1,\n" +
            "\t\t\t\"type\": \"U\",\n" +
            "\t\t\t\"uid\": \"urn:va:B362:8:cpt:1300\"\n" +
            "\t\t}]\n" +
            "\t}\n" +
            "}";

}
