/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.vds.rpc;

import gov.va.med.rpc.RPCConnection;
import gov.va.med.rpc.RPCException;
import gov.va.med.vds.provider.dao.rpc.UserRPC;
import gov.va.med.vds.patient.dao.rpc.VitalsRPC;
import gov.va.med.vds.vistabeans.Vitals;
import org.junit.Test;

import java.util.Calendar;

import static org.junit.Assert.assertTrue;

/**
 *
 *
 */
public class VitalsRPCTest extends BaseRPCTest
{
    @Test
    public void testGetVitals() throws RPCException {
        VitalsRPC vitalsRPC = new VitalsRPC(vistaCxn);
        Calendar startDate = Calendar.getInstance();
        Calendar endDate = Calendar.getInstance();
        startDate.set(2009, 0, 1);
        endDate.set(2010, 0, 1);

        Vitals[] vitals = vitalsRPC.getVitals(patientId, startDate, endDate);
        assertTrue(vitals.length > 0);
    }

    @Test
    public void testGetLatestVitals() throws RPCException
    {
        VitalsRPC vitalsRPC = new VitalsRPC(vistaCxn);
        Vitals[] vitals = vitalsRPC.getLatestVitals(patientId);
        assertTrue(vitals.length > 0);
    }

    @Test
    public void testGetVitalsVPR() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        VitalsRPC vitalsVPRRPC = new VitalsRPC(cxn);

        Calendar endDate = Calendar.getInstance();
        Calendar startDate = Calendar.getInstance();
        startDate.add(Calendar.YEAR, -5);
        Vitals[] vitals = vitalsVPRRPC.getVitalsVPR(patientId, startDate, endDate);

        assertTrue(vitals.length > 0);
    }

    @Test
    public void testGetLatestVitalsVPR() throws RPCException
    {
        RPCConnection cxn = new RPCConnection(vistaCxn.getSite());
        cxn.connect();
        UserRPC userRPC = new UserRPC(cxn);
        userRPC.visit(user, "VPR APPLICATION PROXY", true);

        VitalsRPC vitalsVPRRPC = new VitalsRPC(cxn);

        Vitals[] vitals = vitalsVPRRPC.getLatestVitalsVPR(patientId);

        assertTrue(vitals.length > 0);
    }
}
