package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.beans.PatientHistory;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.hdd.ConceptCodeMapper;
import gov.va.med.jmeadows.dao.util.BHIERelayBeanFactory;
import gov.va.med.jmeadows.dao.util.LocationHelper;
import gov.va.med.jmeadows.dao.util.MultiLocationQuery;
import gov.va.med.jmeadows.dao.util.MultiLocationQueryConfig;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.jmeadows.dao.util.PatientMultiLocQueryCfgBuilder;
import gov.va.med.jmeadows.smartbeans.patient.SmartHistory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: yidali
 * Date: 4/18/14
 * Time: 3:40 AM
 * To change this template use File | Settings | File Templates.
 */
public class HistoryDao extends DaoBase {

    /**
     * Retrieves a list of patient problems from available patient locations.
     *
     * @param queryBean Query parameters.
     *
     * {@link gov.va.med.vds.webservice.QueryBean} required parameters:
     *
     * {@link gov.va.med.vds.webservice.User}
     * {@link gov.va.med.vds.webservice.Patient}
     *
     * Optional:
     * startDate (defaults to T-3 months)
     * endDate   (defaults to T).
     *
     * @return A list of patient problems.
     * @throws gov.va.med.jmeadows.common.JMeadowsException if a serious error occurs.
     * @throws IllegalArgumentException if required parameters are missing or invalid
     */
    public List<PatientHistory> getPatientHistory(JMeadowsQuery queryBean) throws JMeadowsException
    {
        validateQueryBean(queryBean, DaoBase.QueryBeanParams.USER, DaoBase.QueryBeanParams.PATIENT);

        auditQuery("PatHistory", queryBean);

        List<PatientLocation> locs = LocationHelper.getPatientLocations(queryBean, true);

        //build QueryLocation configs
        List<MultiLocationQueryConfig> configs = new ArrayList<MultiLocationQueryConfig>();

        for (PatientLocation loc : locs) {
            if (SYSTEM_TYPE_DOD_ADAPTER.equalsIgnoreCase(loc.getSystemType())) {

                PatientLocation bhieLoc = LocationHelper.toBHIEPatientLocation(loc);

                List<String> includedSources = new ArrayList<String>();
                includedSources.add(APP_CONFIG.getBHIECustodianOidAHLTA());

                gov.va.med.bhie.QueryBean bhieQueryBean =
                        BHIERelayBeanFactory.create(queryBean, includedSources, null);

                //SB: Exclude data range to retrieve entire data set
                bhieQueryBean.setStartDate(null);
                bhieQueryBean.setEndDate(null);

                MultiLocationQueryConfig multiLocPatSocialHistoryConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientSocialHistory")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapHistoryFromBHIE")
                                .location(bhieLoc)
                                .build();


                MultiLocationQueryConfig multiLocPatFamilyHistoryConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientFamilyHistory")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapHistoryFromBHIE")
                                .location(bhieLoc)
                                .build();

                MultiLocationQueryConfig multiLocPatOtherHistoryConfig =
                        new PatientMultiLocQueryCfgBuilder()
                                .dataServiceHandler(sm_bhieRelayServiceHandler)
                                .dataServiceMethod("getPatientOtherHistory")
                                .dataServiceMethodArgs(bhieQueryBean)
                                .smartBeanMappingMethod("mapHistoryFromBHIE")
                                .location(bhieLoc)
                                .build();

                configs.add(multiLocPatSocialHistoryConfig);
                configs.add(multiLocPatFamilyHistoryConfig);
                configs.add(multiLocPatOtherHistoryConfig);

            }
//            else if (SYSTEM_TYPE_VISTA.equalsIgnoreCase(loc.getSystemType())) {
//                configs.add(
//                        new PatientMultiLocQueryCfgBuilder()
//                                .dataServiceHandler(sm_vistaHandler)
//                                .dataServiceMethod("getPatientImmunizations")
//                                .dataServiceMethodArgs(VistaBeanFactory.create(queryBean))
//                                .location(loc)
//                                .build()
//                );
//            }
        }

        //SmartHistory smartHistory = new SmartHistory();

        Object[] histObjs = MultiLocationQuery.execute(
                configs, new SmartHistory(), queryBean);

        List<PatientHistory> histories = null;

        if (histObjs != null) {
            histories = Arrays.asList((PatientHistory[]) histObjs);

            //map problem concept codes
            ConceptCodeMapper.getInstance().mapConceptCodes(hddDao, histories, CodingSystem.SNOMED);
        }

        return histories;
    }

}
