package gov.va.med.jmeadows.dao.util;

import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.DNS   .common.Utils;

import java.util.List;

public class BHIERelayBeanFactory {
    /**
     * Helper function that a VistA query bean to a BHIE query bean.
     *
     * @param qbean              Source query bean.
     * @param includedSources Included BHIE sources, optional parameter.
     * @param excludedSources Excluded BHIE sources, optional parameter.
     * @return BHIE QueryBean
     */
    public static gov.va.med.bhie.QueryBean create(JMeadowsQuery qbean,
                                                        List<String> includedSources, List<String> excludedSources) {

        gov.va.med.bhie.QueryBean bhieQueryBean = new gov.va.med.bhie.QueryBean();

        bhieQueryBean.setForceRefresh(qbean.isForceRefresh());

        String userIen = qbean.getUser().getUserIen();

        StringBuilder patientId = new StringBuilder();
        //todo: melcolm disable ahlta unit number
//        if(qb.getPatient().getAhltaUnitNumber() != null && !qb.getPatient().getAhltaUnitNumber().matches("")){
//            patientId.append(qb.getPatient().getAhltaUnitNumber());
//        }
//        else{
        patientId.append("EDIPI:");
        patientId.append(qbean.getPatient().getEDIPI());
//        }

        bhieQueryBean.setUserId(userIen);
        bhieQueryBean.setPatientId(patientId.toString());

        if (qbean.getStartDate() != null) {
            bhieQueryBean.setStartDate(Utils.toXMLGregorianCalendar(qbean.getStartDate().getTime()));
        }

        if (qbean.getEndDate() != null) {

            bhieQueryBean.setEndDate(Utils.toXMLGregorianCalendar(qbean.getEndDate().getTime()));
        }

        if (includedSources != null) {
            bhieQueryBean.getIncludedSources().addAll(includedSources);
        }
        if (excludedSources != null) {
            bhieQueryBean.getExcludedSources().addAll(excludedSources);
        }

        return bhieQueryBean;
    }
}
