/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.util;


import gov.va.med.jmeadows.dataservicehandler.DataServiceHandler;

public class MultiLocationQueryConfig {
    protected DataServiceHandler dataServiceHandler;
    protected String dataServiceMethod;
    protected Object[] dataServiceMethodArgs;
    protected boolean resultRequiresBeanMapping;
    protected String smartBeanMappingMethod;
    protected String connectionErrorMsg;
    protected boolean isSingleResultExpected;
    protected Location location;


    public MultiLocationQueryConfig(MultiLocQueryCfgBuilder builder) {
        super();
        this.dataServiceHandler = builder.dataServiceHandler;
        this.dataServiceMethod = builder.dataServiceMethod;
        this.dataServiceMethodArgs = builder.dataServiceMethodArgs;
        this.resultRequiresBeanMapping = builder.resultRequiresBeanMapping;
        this.smartBeanMappingMethod = builder.smartBeanMappingMethod;
        this.connectionErrorMsg = builder.connectionErrorMsg;
        this.isSingleResultExpected = builder.isSingleResultExpected;
    }

    public String toString() {
        StringBuilder sbuild = new StringBuilder();
        sbuild.append("{location: ");
        sbuild.append(location.toString());

        sbuild.append(", dataServiceMethod: ");
        sbuild.append(dataServiceMethod);

        sbuild.append(", dataServiceMethodArgs count: ");
        if (dataServiceMethodArgs != null) {
            sbuild.append(dataServiceMethodArgs.length);
        }

        sbuild.append(", resultRequiresBeanMapping: ");
        sbuild.append(resultRequiresBeanMapping);
        sbuild.append(",  smartBeanMappingMethod: ");
        sbuild.append(smartBeanMappingMethod);
        sbuild.append(", connectionErrorMsg: ");
        sbuild.append(connectionErrorMsg);
        sbuild.append(",  isSingleResultExpected: ");
        sbuild.append(isSingleResultExpected);
        sbuild.append("}");

        return sbuild.toString();
    }

    public boolean resultRequiresBeanMapping() {
        return resultRequiresBeanMapping;
    }

    public void setResultRequiresBeanMapping(boolean resultRequiresBeanMapping) {
        this.resultRequiresBeanMapping = resultRequiresBeanMapping;
    }

    public String getSmartBeanMappingMethod() {
        return smartBeanMappingMethod;
    }

    public void setSmartBeanMappingMethod(String smartBeanMappingMethod) {
        this.smartBeanMappingMethod = smartBeanMappingMethod;
    }
    
    public Object[] getWebserviceArgs() {
        Object[] rtc = null;
        if (dataServiceMethodArgs != null) {
            rtc = dataServiceMethodArgs.clone();
        }
        return rtc;
    }

    public void setWebserviceArgs(Object[] webserviceArgs) {
        if (webserviceArgs == null) {
            this.dataServiceMethodArgs = null;
        } else {
            this.dataServiceMethodArgs = webserviceArgs.clone();
        }
    }

    public String getWebserviceMethod() {
        return dataServiceMethod;
    }

    public void setWebserviceMethod(String webserviceMethod) {
        this.dataServiceMethod = webserviceMethod;
    }

    public DataServiceHandler getWebserviceHandler() {
        return dataServiceHandler;
    }

    public void setWebserviceHandler(DataServiceHandler webserviceHandler) {
        this.dataServiceHandler = webserviceHandler;
    }

    public boolean isSingleResultExpected() {
        return isSingleResultExpected;
    }

    public void setIsSingleResultExpected(boolean isSingleResultExpected) {
        this.isSingleResultExpected = isSingleResultExpected;
    }

    public DataServiceHandler getDataServiceHandler() {
        return dataServiceHandler;
    }

    public void setDataServiceHandler(DataServiceHandler dataServiceHandler) {
        this.dataServiceHandler = dataServiceHandler;
    }

    public String getDataServiceMethod() {
        return dataServiceMethod;
    }

    public void setDataServiceMethod(String dataServiceMethod) {
        this.dataServiceMethod = dataServiceMethod;
    }

    public Object[] getDataServiceMethodArgs() {
        Object[] rtc = null;
        if (dataServiceMethodArgs != null) {
            rtc = dataServiceMethodArgs.clone();
        }
        return rtc;
    }

    public void setDataServiceMethodArgs(Object[] dataServiceMethodArgs) {
        if (dataServiceMethodArgs == null) {
            this.dataServiceMethodArgs = null;
        } else {
            this.dataServiceMethodArgs = dataServiceMethodArgs.clone();
        }

    }

    public String getConnectionErrorMsg() {
        return connectionErrorMsg;
    }

    public void setConnectionErrorMsg(String connectionErrorMsg) {
        this.connectionErrorMsg = connectionErrorMsg;
    }

    public Location getLocation() {
        return location;
    }

    public void setLocation(Location location) {
        this.location = location;
    }
}