/*
 * Janus 4.0 (c)
 * Copyright (c) 2012 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.dataservicehandler;

/**
 * Created with IntelliJ IDEA.
 * User: DNS   ishihj
 * Date: 9/20/12
 * Time: 11:29 AM
 * To change this template use File | Settings | File Templates.
 */

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/*
 * This simple SOAPHandler will output the contents of incoming
 * and outgoing messages.
 */
public class SOAPLoggingHandler implements SOAPHandler<SOAPMessageContext> {

    private static final Logger LOGGER = LoggerFactory.getLogger(SOAPLoggingHandler.class);

    public Set<QName> getHeaders() {
        return null;
    }

    public boolean handleMessage(SOAPMessageContext smc) {
        logToSystemOut(smc);
        return true;
    }

    public boolean handleFault(SOAPMessageContext smc) {
        logToSystemOut(smc);
        return true;
    }

    public void close(MessageContext messageContext) {
        // nothing to clean up
    }

    /*
     * Check the MESSAGE_OUTBOUND_PROPERTY in the context
     * to see if this is an outgoing or incoming message.
     * Write a brief message to the print stream and
     * output the message. The writeTo() method can throw
     * SOAPException or IOException
     */
    private void logToSystemOut(SOAPMessageContext smc) {
        Boolean outboundProperty = (Boolean)
                smc.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY);

        if (outboundProperty.booleanValue()) {
            LOGGER.debug("Outbound message:");
        } else {
            LOGGER.debug("Inbound message:");
        }

        try {
            SOAPMessage message = smc.getMessage();

            ByteArrayOutputStream stream = new ByteArrayOutputStream();
            message.writeTo(stream);

            LOGGER.debug(stream.toString("UTF-8"));
        } catch (SOAPException e) {
            //logger.error("SOAPException: ", e );
            LOGGER.error(e.getMessage(), e);
        } catch (IOException e) {
            //logger.error("IOException: ", e );
            LOGGER.error(e.getMessage(), e);
        }
    }
}
