/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dataservicehandler;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.hdd.VLERSiteCache;
import gov.va.med.vds.webservice.Author;
import gov.va.med.vds.webservice.QueryBean;
import gov.va.med.vds.webservice.VistaData;
import gov.va.med.vds.webservice.VistaDataService;
import gov.va.med.vds.webservice.VlerDocument;

import java.util.List;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.Handler;


/**
 *
 * @author DNS   gotom
 */
public class VistaDataVLERServiceHandler extends DataServiceHandler {
    private VistaDataService m_VistaDataVLERService;

    public VistaDataVLERServiceHandler(String url) {
        super();

        serviceURL = url;
        serviceName = "VistaDataVLERService";
        serviceNamespace = "http://webservice.vds.med.DNS   /";
        qServiceName = new QName(serviceNamespace, serviceName);

        serviceConnectionTimeoutMS = APP_CONFIG.getVLERConnectionTimeoutMS();
        serviceRequestTimeoutMS = APP_CONFIG.getVLERRequestTimeoutMS();

        m_VistaDataVLERService = new VistaDataService();
    }

    private VistaData getVistaDataPort() {

        VistaData port = m_VistaDataVLERService.getVistaDataPort();

        //set port endpoint address
        BindingProvider prov = (BindingProvider) port;
        prov.getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceURL);
        prov.getRequestContext().put("com.sun.xml.internal.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.internal.ws.request.timeout", serviceRequestTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.connect.timeout", serviceConnectionTimeoutMS);
        prov.getRequestContext().put("com.sun.xml.ws.request.timeout", serviceRequestTimeoutMS);

        // SOAP message logging - Enable to show logging in output window
        if ("test".equalsIgnoreCase(APP_CONFIG.getAppEnvironment()) || "development".equalsIgnoreCase(APP_CONFIG.getAppEnvironment())) {
            prov.getBinding().getHandlerChain().add(new SOAPLoggingHandler());
            List<Handler> handlerList = prov.getBinding().getHandlerChain();
            handlerList.add(new SOAPLoggingHandler());
            prov.getBinding().setHandlerChain(handlerList);
        }

        return port;
    }

    public  List<VlerDocument> getVLERDocumentList(QueryBean queryBean) throws JMeadowsException {

        try {
            VistaData port = getVistaDataPort();
            List<VlerDocument> vdList = port.getPatientDocumentList(queryBean);
            for (VlerDocument vd : vdList) {
                if (vd.getHomeCommunityId() != null) {
                    Author author = new Author();
                    author.setInstitution(getVLERSiteName(vd.getHomeCommunityId()));
                    author.setName(vd.getHomeCommunityId());
                    vd.getAuthorList().add(0, author);
                }
            }
            return vdList;
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
    }

    public String getVLERSiteName(String hcid) {
        String output = "";
        String hci = hcid.substring(8, hcid.length());
        if (VLERSiteCache.getCode("", hci, "") != null) {
            output = VLERSiteCache.getCode("", hci, "").getDisplay();
        }
        return output;
    }

    public VlerDocument getVLERDocument(VlerDocument vDoc, QueryBean queryBean) throws JMeadowsException {
        VlerDocument returnVal = null;
        try {
            VistaData port = getVistaDataPort();
            returnVal =  port.getPatientDocument(vDoc, queryBean);
        } catch (Exception e) {
            throw new JMeadowsException(e);
        }
        return returnVal;
    }

}
