/*
 * Janus 4.0 (c)
 * Copyright (c) 2013 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.heartbeat;

import gov.va.med.jmeadows.common.JMeadowsException;
import net.sf.ehcache.constructs.blocking.CacheEntryFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;
import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

public class ServiceErrorsCacheEntryFactory implements CacheEntryFactory {
    private static final Logger LOGGER = LoggerFactory.getLogger(ServiceErrorsCacheEntryFactory.class);
    private DataSource dataSource;
    private String heartbeatCacheEntryKey;

    public ServiceErrorsCacheEntryFactory(DataSource dataSource, String heartbeatCacheEntryKey)
    {
        this.dataSource = dataSource;
        this.heartbeatCacheEntryKey = heartbeatCacheEntryKey;
    }

    public Object createEntry(Object key) throws RuntimeException, JMeadowsException {
        Object rtc = null;
        if (heartbeatCacheEntryKey.equals(key)) {
            rtc = getRecentQoSStatusesFromDB();
        }
        return rtc;
    }

    private String getRecentQoSStatusesFromDB() throws JMeadowsException {
        Connection sqlConn = null;

        CallableStatement cstmt = null;
        ResultSet rset = null;

        StringBuilder serviceErrors = new StringBuilder();

        try {
            sqlConn = dataSource.getConnection();
            cstmt = sqlConn.prepareCall("{call getRecentQoSServiceErrors}");
            rset = cstmt.executeQuery();
            while (rset.next()) {
                serviceErrors.append(rset.getString("service") + "^");
            }
        } catch (SQLException e) {
            LOGGER.error("getRecentQoSServiceErrors exception: " + e.getMessage());
            throw new JMeadowsException(e);
        } finally {
            try {
                if (rset != null) {
                    rset.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (cstmt != null) {
                    cstmt.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }

            try {
                if (sqlConn != null) {
                    sqlConn.close();
                }
            } catch (SQLException e) {
                LOGGER.error(e.getMessage());
            }
        }

        return serviceErrors.toString();
    }
}
