/*
 * Janus 4.0 (c)
 * Copyright (c) 2015 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.VLERCCDA;
import gov.va.med.jmeadows.dao.util.Location;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created with IntelliJ IDEA.
 * User: yidali
 * Date: 4/7/14
 * Time: 2:01 PM
 * To change this template use File | Settings | File Templates.
 */
public class SmartCCDA extends SmartBean implements Comparable {

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int DOCUMENTTITLE = 3;
    
    private static final Logger LOGGER = LoggerFactory.getLogger(SmartCCDA.class);

    public VLERCCDA m_Patient_ccda;

    public SmartCCDA() {
        super();
        m_Patient_ccda = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartCCDA(VLERCCDA ccda, String sort) {
        super();
        m_Patient_ccda = ccda;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartCCDA) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_Patient_ccda);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
    
            int size = array1.length;
            SmartCCDA[] temp = new SmartCCDA[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartCCDA newAll = new SmartCCDA((VLERCCDA) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            VLERCCDA[] ret = new VLERCCDA[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_ccda;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
//        SmartCCDA tmp = (SmartCCDA) obj;
        int retVal = 0;
//        if (this.m_sortBy == SITE && this.m_Patient_ccda.getSite().getName() != null && tmp.m_Patient_ccda.getSite().getName() != null) {
//            retVal = compareString(this.m_Patient_ccda.getSite().getName(), tmp.m_Patient_ccda.getSite().getName());
//        }
//        if (this.m_sortBy == DOCUMENTTITLE) {
//            retVal = compareString(this.m_Patient_ccda.getDocumentTitle(), tmp.m_Patient_ccda.getDocumentTitle());
//        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String sort) {
        String rtc = sort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (rtc != null) {
            rtc = setAscending(sort);
            if ("date".equalsIgnoreCase(rtc)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(rtc)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(rtc)) {
                m_sortBy = DOCUMENTTITLE;
            }
        }

    }



    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartCCDA[] temp = new SmartCCDA[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartCCDA newAll = new SmartCCDA((VLERCCDA) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartCCDA newAll = new SmartCCDA((VLERCCDA) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            VLERCCDA[] ret = new VLERCCDA[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_ccda;
            }
    
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String errorMsg) {
//        if (errorMsg == null) {
//            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
//        }

        VLERCCDA ccda = new VLERCCDA();
//        p.setDocumentTitle(errorMsg);
//        p.setFormDate(Utils.backDate().toGregorianCalendar());
        ccda.setSite(getConnectionUnavailableSite(location));
        return new VLERCCDA[]{ccda};
    }

    public VLERCCDA[] mapVLERCCDAFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection vlerResultCollection = (V4ResultCollection) resultCollection;
        List<VLERCCDA> vistaBeans = new ArrayList<VLERCCDA>();

        VLERCCDA vlerBean = new VLERCCDA();
        try {


            vlerBean.setNarratives(vlerResultCollection.getNarrativeList());
            vlerBean.setStatusList(vlerResultCollection.getStatusListv4());
            vlerBean.setErrorList(vlerResultCollection.getErrorListv4());
            vlerBean.setQueryComplete(vlerResultCollection.getQueryComplete());

            if (vlerResultCollection.getDataRecordList() != null) {
                if (vlerResultCollection.getDataRecordList().getDemographics() != null) {
                    SmartDemographics smartDemographics = new SmartDemographics();
                    vlerBean.setDemographics(Arrays.asList(smartDemographics.mapPatientDemographicsFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getDocumentInfo() != null) {
                    vlerBean.setDocumentInfo(vlerResultCollection.getDataRecordList().getDocumentInfo());
                }
                if (vlerResultCollection.getDataRecordList().getAllergy() != null) {
                    SmartAllergy smartAllergy = new SmartAllergy();
                    vlerBean.setAllergies(Arrays.asList(smartAllergy.mapAllergiesFromBHIEV4(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getEncounters() != null) {
                    //Encounters not currently part of DoD VLER deliverable
                    SmartEncounter smartEncounter = new SmartEncounter();
                    vlerBean.setEncounters(Arrays.asList(smartEncounter.mapEncountersFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getImmunizations() != null) {
                    SmartImmunization smartImmunization = new SmartImmunization();
                    vlerBean.setImmunizations(Arrays.asList(smartImmunization.mapImmunizationsFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getMedications() != null) {
                    //Medicatios not currently part of DoD VLER deliverable
                    SmartMedication smartMedication = new SmartMedication();
                    vlerBean.setMedications(Arrays.asList(smartMedication.getOutpatientMedicationsFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getExternalPartnerResults() != null) {
                    //Results not currently part of DoD VLER deliverable
                    SmartLabResult smartLabResult = new SmartLabResult();
                    vlerBean.setResults(Arrays.asList(smartLabResult.mapLabResultsFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getPayers() != null) {
                    SmartInsurance smartInsurance = new SmartInsurance();
                    vlerBean.setPayers(Arrays.asList(smartInsurance.mapInsurancesFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getProblems() != null) {
                    SmartProblem smartProblem = new SmartProblem();
                    vlerBean.setProblems(Arrays.asList(smartProblem.mapProblemListFromBHIE(vlerResultCollection)));
                }
                if (vlerResultCollection.getDataRecordList().getProcedures() != null) {
                    SmartOutpatientProcedure smartProcedure = new SmartOutpatientProcedure();
                    vlerBean.setProcedures(Arrays.asList(smartProcedure.mapOutpatientProceduresFromBHIE(vlerResultCollection)));

                }
                if (vlerResultCollection.getDataRecordList().getVitals() != null) {
                    SmartVitals smartVitals = new SmartVitals();
                    vlerBean.setVitals(Arrays.asList(smartVitals.mapVitalsFromBHIEV4(vlerResultCollection)));

                }

            }

            vistaBeans.add(vlerBean);

        } catch (Exception ex) {

            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }
        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(vlerResultCollection.getStatusListv4(),
                vlerResultCollection.getErrorListv4(), vlerResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (VLERCCDA[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new VLERCCDA[vistaBeans.size()]);
    }

}
