/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.ClinicalReminder;
import gov.va.med.DNS   .common.Utils;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

//import gov.va.med.DNS   .beans.ClinicalReminder;
//import gov.va.med.DNS   .beans.*;
//import gov.va.med.chcs.beans.*;
//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartClinicalReminder extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartClinicalReminder.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;
    public static final int LDATE = 4;

    public ClinicalReminder m_reminder;

    public SmartClinicalReminder() {
        super();
        m_reminder = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartClinicalReminder(ClinicalReminder reminder, String sort) {
        super();
        m_reminder = reminder;
        getSortValue(sort);
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartClinicalReminder[] temp = new SmartClinicalReminder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartClinicalReminder newAll = new SmartClinicalReminder((ClinicalReminder) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartClinicalReminder newAll = new SmartClinicalReminder((ClinicalReminder) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            ClinicalReminder[] ret = new ClinicalReminder[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_reminder;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartClinicalReminder[] temp = new SmartClinicalReminder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartClinicalReminder newAll = new SmartClinicalReminder((ClinicalReminder) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            ClinicalReminder[] ret = new ClinicalReminder[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_reminder;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartClinicalReminder) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_reminder);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartClinicalReminder tmp = (SmartClinicalReminder) obj;
        int retVal = 0;

        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_reminder.getDueDate(), tmp.m_reminder.getDueDate());
        }
        if (this.m_sortBy == SITE && this.m_reminder.getSite() != null && tmp.m_reminder.getSite() != null) {
            retVal = compareString(this.m_reminder.getSite().getSiteCode(), tmp.m_reminder.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_reminder.getDescription(), tmp.m_reminder.getDescription());
        }
        if (this.m_sortBy == LDATE) {
            retVal = compareDate(this.m_reminder.getLastOccurrenceDate(), tmp.m_reminder.getLastOccurrenceDate());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("dueDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
            if ("lastoccurrancedate".equalsIgnoreCase(sort)) {
                m_sortBy = LDATE;
            }
        }

    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        ClinicalReminder reminder = new ClinicalReminder();
        reminder.setDescription(errorMsg);
        reminder.setLastOccurrenceDate(Utils.backDate());
        reminder.setDueDate(Utils.backDate());
        reminder.setSite(getConnectionUnavailableSite(location));

        return new ClinicalReminder[]{reminder};
    }


}