/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.NoteV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Consult;
import gov.va.med.DNS   .common.Utils;
//import org.hui.chcs.NoteBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

//import gov.va.med.DNS   .beans.Consult;
//import gov.va.med.DNS   .beans.ProgressNote;
//import gov.va.med.chcs.beans.NoteBean;
//import gov.va.med.DNS   .beans.*;
//import gov.va.med.chcs.beans.*;
//import mil.osd.ha.citpo.jvi.webservice.*;

public class SmartConsult extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartConsult.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int LOC = 3;

    public Consult m_consult;

    public SmartConsult() {
        super();
        m_consult = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartConsult(Consult consult, String sort) {
        super();
        m_consult = consult;
        getSortValue(sort);
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartConsult[] temp = new SmartConsult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartConsult newAll = new SmartConsult((Consult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartConsult newAll = new SmartConsult((Consult) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Consult[] ret = new Consult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_consult;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartConsult[] temp = new SmartConsult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartConsult newAll = new SmartConsult((Consult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Consult[] ret = new Consult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_consult;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else  if (obj instanceof SmartConsult) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_consult);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartConsult tmp = (SmartConsult) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_consult.getRequestDate(), tmp.m_consult.getRequestDate());
        }
        if (this.m_sortBy == SITE && this.m_consult.getSite() != null && tmp.m_consult.getSite() != null) {
            retVal = compareString(this.m_consult.getSite().getSiteCode(), tmp.m_consult.getSite().getSiteCode());
        }
        if (this.m_sortBy == LOC) {
            retVal = compareString(this.m_consult.getService(), tmp.m_consult.getService());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }
        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("requestDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("service".equalsIgnoreCase(sort)) {
                m_sortBy = LOC;
            }
        }

    }

//    public Consult[] mapConsultsFromCHCS(Object[] dodBean) {
//        int size = dodBean.length;
//        Consult[] retBean = new Consult[size];
//        Consult bean = null;
//
//        for (int i = 0; i < dodBean.length; i++) {
//            /*
//             retBean[i].setId(dodBean[i].getItemId());
//             retBean[i].setProcedureConsult(dodBean[i].getNoteTitle());
//             retBean[i].setReport(dodBean[i].getCompleteNote());
//             retBean[i].setRequestDate(Utils.getDate(dodBean[i].getVerifiedDate()));
//             retBean[i].setService(dodBean[i].getNoteTitle());
//             retBean[i].setSiteCode(siteId);
//             */
//
//            bean = new Consult();
//
//            bean.setReport(((NoteBean) dodBean[i]).getCompleteNote());
//            bean.setRequestDate(Utils.getXMLCal(((NoteBean) dodBean[i]).getVerifiedDate()));
//
//            if (((NoteBean) dodBean[i]).getItemId() == null) {
//                bean.setStatus("Text");
//                bean.setService(((NoteBean) dodBean[i]).getNoteTitle());
//                bean.setProcedureConsult(((NoteBean) dodBean[i]).getNoteTypeName());
//            } else {
//                bean.setReport(((NoteBean) dodBean[i]).getItemId());
//                bean.setStatus(((NoteBean) dodBean[i]).getNoteFormat());
//                bean.setService(((NoteBean) dodBean[i]).getNoteTitle());
//                bean.setProcedureConsult(((NoteBean) dodBean[i]).getNoteTypeName());
//            }
//            bean.setId(((NoteBean) dodBean[i]).getNoteTypeCode());
//            retBean[i] = bean;
//
//            bean.setReport(((NoteBean) dodBean[i]).getCompleteNote());
//            bean.setRequestDate(Utils.getXMLCal(((NoteBean) dodBean[i]).getVerifiedDate()));
//
//            addCHCSSiteDataToVistaBean(bean);
//
////          if (((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getItemId() != null)
////          {
////              bean.setReport(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getItemId());
////              bean.setStatus(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getNoteFormat());
////              bean.setService(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getNoteTitle());
////              bean.setProcedureConsult(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getNoteTypeName());
////          }
////          else
////          {
////              bean.setStatus("Text");
////              bean.setService(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getNoteTitle());
////              bean.setProcedureConsult(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getNoteTypeName());
////          }
////          bean.setId(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getNoteTypeCode());
////          bean.setSiteCode(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getLocationCode());
////                  bean.setSiteName(((Mil_osd_ha_citpo_jvi_webservice_beans_NoteBean)dodBean[i]).getLocationName());
////          retBean[i]=bean;
//
//        }
//
//
//        return retBean;
//
//    }

    //Version 3
    public Consult[] mapConsultsFromBHIE(Object resultCollection) {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<Consult> vistaBeans = new ArrayList<Consult>();

        if (noteResultCollection.getDataRecordList() != null && noteResultCollection.getDataRecordList().getNotes() != null) {
            for (NoteV4 bhieNote : noteResultCollection.getDataRecordList().getNotes()) {
                //workaround until DoD adaptor returns patient consults
                if (!LOINC_CODE_NOTES.equals(bhieNote.getDataDomain().get(0).getCode()) || !NOTE_TYPE_CODE_CONSULT.contains(bhieNote.getType().get(0).getCode())) {
                    continue;
                }

                Consult bean = new Consult();
                if (bhieNote.getEnteredDate() != null && bhieNote.getEnteredDate().getStart() != null) {
                    bean.setRequestDate(bhieToXMLCal(bhieNote.getEnteredDate().getStart()));
                }
                if (bhieNote.getTitle() != null && !bhieNote.getTitle().isEmpty()) {
                    bean.setService(bhieNote.getTitle());
                }
                if (bhieNote.getType() != null && !bhieNote.getType().isEmpty()
                        && bhieNote.getType().get(0) != null) {
                    bean.setProcedureConsult(bhieNote.getType().get(0).getDisplay());
                }
                bean.setReport(bhieNote.getContent());
//                if (bhieNote.getEnteredBy() != null) {
//                    bean.setProvider(mapProvider(bhieNote.getEnteredBy()));
//                }
                if (bhieNote.getVerifiedBy() != null) {
                    bean.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                }
                if (bhieNote.getStatus() != null && !bhieNote.getStatus().isEmpty()
                        && bhieNote.getStatus().get(0) != null) {
                    bean.setStatus(bhieNote.getStatus().get(0).getDisplay());
                }
                bean.setComplexDataUrl(bhieNote.getComplexDataUrl());

//                addBHIESiteDataToVistaBean(bhieNote.getCustodianId(),
//                        bhieNote.getCustodianName(), bhieNote.getFacility(), bhieNote.getEventId(), bean);
//                addBHIESiteDataToVistaBean(bhieNote.getRepository().getId().get(0).getId(),
//                        bhieNote.getRepository().getName(), bhieNote.getFacility().getName(), bhieNote.getRecordId().getId(), bean);

                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null
                        && !bhieNote.getRepository().getIds().isEmpty()
                        && bhieNote.getRepository().getIds().get(0) != null) {
                    repositoryId = bhieNote.getRepository().getIds().get(0).getId();
                }
                if (bhieNote.getRepository() != null) {
                    repositoryName = bhieNote.getRepository().getName();
                }
                if (bhieNote.getFacility() != null) {
                    facilityName = bhieNote.getFacility().getName();
                }
                if (bhieNote.getRecordId() != null) {
                    eventId = bhieNote.getRecordId().getId();
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName, 
                        facilityName, eventId, bean);

                vistaBeans.add(bean);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(noteResultCollection.getStatusListv4(),
                noteResultCollection.getErrorListv4(), noteResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
            (Consult[]) generateConnectionUnavailableBean(
                    BHIE_STATUS_REPORT_LOCATION,
                    siteStatusReport)
        ));

        return vistaBeans.toArray(new Consult[vistaBeans.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        Consult consult = new Consult();
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }
        consult.setService(errorMsg);
        consult.setRequestDate(Utils.backDate());
        consult.setSite(getConnectionUnavailableSite(location));

        return new Consult[]{consult};
    }
}