/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.DemographicV4;
import gov.va.med.bhie.RecordIdentifierV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.foundations.utilities.Messages;
import gov.va.med.jmeadows.dao.beans.ResponsePatientQuery;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.util.PatientLocation;
import gov.va.med.vds.webservice.PatientDemographics;
import gov.va.med.vds.webservice.PatientDemographicsDetail;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

/**
 * @author DNS   brunos
 */
public class SmartDemographicsDetail extends SmartBean {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartDemographicsDetail.class);

    public static final int SITE = 1;

    public PatientDemographicsDetail m_demographicDetail;
    
    public SmartDemographicsDetail() {
        super();
        m_demographicDetail = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartDemographicsDetail(PatientDemographicsDetail demographicDetail, String sort) {
        super();
        m_demographicDetail = demographicDetail;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartDemographicsDetail) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_demographicDetail);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartDemographicsDetail[] temp = new SmartDemographicsDetail[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDemographicsDetail newAll = new SmartDemographicsDetail((PatientDemographicsDetail) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartDemographicsDetail newAll = new SmartDemographicsDetail((PatientDemographicsDetail) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientDemographicsDetail[] ret = new PatientDemographicsDetail[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_demographicDetail;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartDemographicsDetail[] temp = new SmartDemographicsDetail[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartDemographicsDetail newAll = new SmartDemographicsDetail((PatientDemographicsDetail) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientDemographicsDetail[] ret = new PatientDemographicsDetail[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_demographicDetail;
            }
            rtc = ret;
        }
        return rtc;
    }

//    public PatientDemographicsDetail[] mapPatientDemographicsDetailsFromCHCS(Object[] dodBean) {
//        PatientDemographicsDetail[] details = new PatientDemographicsDetail[dodBean.length];
//
//        StringBuilder reportBldr = new StringBuilder();
//        for (int i = 0; i < dodBean.length; i++) {
//            org.hui.chcs.DemographicsBean chcsBean = (org.hui.chcs.DemographicsBean) dodBean[i];
//            if (chcsBean != null) {
//                reportBldr.setLength(0);
//
//                reportBldr.append(reportCHCSItemStr("Name", chcsBean.getName()));
//                reportBldr.append(reportCHCSItemStr("Address1", chcsBean.getAddress1()));
//                reportBldr.append(reportCHCSItemStr("Address2", chcsBean.getAddress2()));
//                reportBldr.append(reportCHCSItemStr("City", chcsBean.getCity()));
//                reportBldr.append(reportCHCSItemStr("State", chcsBean.getState()));
//                reportBldr.append(reportCHCSItemStr("Zip code:", chcsBean.getZipCode()));
//                reportBldr.append(reportCHCSItemStr("SSN", chcsBean.getSSN()));
//                reportBldr.append(reportCHCSItemStr("PCM Dates", chcsBean.getPcmDates()));
//                reportBldr.append(reportCHCSItemStr("PCM", chcsBean.getPcm()));
//                reportBldr.append(reportCHCSItemStr("Clinic", chcsBean.getClinic()));
//                reportBldr.append(reportCHCSItemStr("Age", chcsBean.getAge()));
//                reportBldr.append(reportCHCSItemStr("DOB", chcsBean.getDob()));
//                reportBldr.append(reportCHCSItemStr("Sex", chcsBean.getSex()));
//                reportBldr.append(reportCHCSItemStr("Phone1", chcsBean.getPhone1()));
//                reportBldr.append(reportCHCSItemStr("Phone2", chcsBean.getPhone2()));
//                reportBldr.append(reportCHCSItemStr("Rank", chcsBean.getRank()));
//                reportBldr.append(reportCHCSItemStr("Sponsor", chcsBean.getSponsor()));
//                reportBldr.append(reportCHCSItemStr("Sponsor ID", chcsBean.getSponsorId()));
//                reportBldr.append(reportCHCSItemStr("Patient ID", chcsBean.getPatientId()));
//                reportBldr.append(reportCHCSItemStr("Enrolled", chcsBean.getEnrolled()));
//                reportBldr.append(reportCHCSItemStr("Unit", chcsBean.getUnit()));
//                reportBldr.append(reportCHCSItemStr("Organ Donor", chcsBean.getOrganDonor()));
//                reportBldr.append(reportCHCSItemStr("Clinic Phone", chcsBean.getClinicPhone()));
//                details[i] = new PatientDemographicsDetail();
//                addCHCSSiteDataToVistaBean(details[i]);
//                details[i].setReport(reportBldr.toString());
//            }
//        }
//
//        return details;
//    }

    public PatientDemographicsDetail[] mapPatientDemographicsDetailsFromPDWS(Object resultCollection) {
        PatientDemographicsDetail[] rtc = new PatientDemographicsDetail[1];
        rtc[0] = new PatientDemographicsDetail();
        StringBuilder reportBldr = new StringBuilder();
        reportBldr.setLength(0);

        ResponsePatientQuery response = (ResponsePatientQuery) resultCollection;

        if (response == null || response.getDemographics() == null) {
            List<PatientDemographicsDetail> details = new ArrayList<PatientDemographicsDetail>();
            PatientLocation bhieLoc = new PatientLocation();
            bhieLoc.setSystemType(SYSTEM_TYPE_DOD_ADAPTER);

            details.addAll(Arrays.asList(
                    (gov.va.med.vds.webservice.PatientDemographicsDetail[])
                            generateConnectionUnavailableBean(bhieLoc,
                                    Messages.get("demographics.dod.results.empty") + "\n\n"
                                    + Messages.get("demographics.dod.results.helpdesk") + "\n\n")));
            rtc = details.toArray(new PatientDemographicsDetail[details.size()]);
        } else {
            PatientDemographics patDemo = response.getDemographics();
            if (patDemo.getSponsorSSN() == null || patDemo.getSponsorSSN().length() == 0) {
                patDemo.setSponsorSSN(patDemo.getSSN());
            }
            if (patDemo.getSponsorName() == null || patDemo.getSponsorName().length() == 0) {
                patDemo.setSponsorName(patDemo.getName());
            }
            if (patDemo.getName() != null) {
                reportBldr.append(reportCHCSItemStr("Name", patDemo.getName()));
            } else {
                reportBldr.append(reportCHCSItemStr("Name", ""));
            }
            if (patDemo.getDob() != null) {
                reportBldr.append(reportCHCSItemStr("DOB", patDemo.getDob()));
            } else {
                reportBldr.append(reportCHCSItemStr("DOB", ""));
            }
            if (patDemo.getGender() != null) {
                reportBldr.append(reportCHCSItemStr("Sex", patDemo.getGender()));
            } else {
                reportBldr.append(reportCHCSItemStr("Sex", ""));
            }
            if (patDemo.getAddress1() != null && patDemo.getAddress1().length() > 0) {
                reportBldr.append(reportCHCSItemStr("Street 1", patDemo.getAddress1()));
            } else {
                reportBldr.append(reportCHCSItemStr("Street 1", ""));
            }
            if (patDemo.getAddress2() != null && patDemo.getAddress2().length() > 0) {
                reportBldr.append(reportCHCSItemStr("Street 2", patDemo.getAddress2()));
            } else {
                reportBldr.append(reportCHCSItemStr("Street 2", ""));
            }
            if (patDemo.getCity() != null) {
                reportBldr.append(reportCHCSItemStr("City", patDemo.getCity()));
            } else {
                reportBldr.append(reportCHCSItemStr("City", ""));
            }
            if (patDemo.getState() != null) {
                reportBldr.append(reportCHCSItemStr("State", patDemo.getState()));
            } else {
                reportBldr.append(reportCHCSItemStr("State", ""));
            }
            if (patDemo.getZipCode() != null) {
                reportBldr.append(reportCHCSItemStr("Zip Code", patDemo.getZipCode()));
            } else {
                reportBldr.append(reportCHCSItemStr("Zip Code", ""));
            }
            if (patDemo.getPhone1() != null) {
                reportBldr.append(reportCHCSItemStr("Phone1", patDemo.getPhone1()));
            } else {
                reportBldr.append(reportCHCSItemStr("Phone1", ""));
            }
            if (patDemo.getPhone2() != null) {
                reportBldr.append(reportCHCSItemStr("Phone2", patDemo.getPhone1()));
            } else {
                reportBldr.append(reportCHCSItemStr("Phone2", ""));
            }
            if (patDemo.getSSN() != null) {
                reportBldr.append(reportCHCSItemStr("SSN", patDemo.getSSN()));
            } else {
                reportBldr.append(reportCHCSItemStr("SSN", ""));
            }
            if (patDemo.getMaritalStatus() != null) {
                reportBldr.append(reportCHCSItemStr("Marital Status", patDemo.getMaritalStatus()));
            } else {
                reportBldr.append(reportCHCSItemStr("Marital Status", ""));
            }
            if (patDemo.getReligion() != null && !patDemo.getReligion().isEmpty() && patDemo.getReligion().get(0) != null && patDemo.getReligion().get(0).length() > 0) {
                reportBldr.append(reportCHCSItemStr("Religion", patDemo.getReligion().get(0)));
            } else {
                reportBldr.append(reportCHCSItemStr("Religion", ""));
            }
            if (patDemo.getRace() != null && !patDemo.getRace().isEmpty()
                    && patDemo.getRace().get(0) != null
                    && patDemo.getRace().get(0).getDisplay() != null) {
                reportBldr.append(reportCHCSItemStr("Race", patDemo.getRace().get(0).getDisplay()));
            } else {
                reportBldr.append(reportCHCSItemStr("Race", ""));
            }
            if (patDemo.getEthnicity() != null && !patDemo.getEthnicity().isEmpty()
                    && patDemo.getEthnicity().get(0) != null
                    && patDemo.getEthnicity().get(0).getDisplay() != null) {
                reportBldr.append(reportCHCSItemStr("Ethnicity", patDemo.getEthnicity().get(0).getDisplay()));
            } else {
                reportBldr.append(reportCHCSItemStr("Ethnicity", ""));
            }
            if (patDemo.getSponsorSSN() != null && !patDemo.getSponsorSSN().isEmpty()) {
                reportBldr.append(reportCHCSItemStr("Sponsor SSN", patDemo.getSponsorSSN()));
            } else {
                reportBldr.append(reportCHCSItemStr("Sponsor SSN", ""));
            }
            if (patDemo.getSponsorName() != null && !patDemo.getSponsorName().isEmpty()) {
                reportBldr.append(reportCHCSItemStr("Sponsor Name", patDemo.getSponsorName()));
            } else {
                reportBldr.append(reportCHCSItemStr("Sponsor Name", ""));
            }
            if (patDemo.getEligibility() != null && !patDemo.getEligibility().isEmpty()) {
                reportBldr.append(reportCHCSItemStr("Deers Eligibility", patDemo.getEligibility()));
            } else {
                reportBldr.append(reportCHCSItemStr("Deers Eligibility", ""));
            }
            if (patDemo.getEligibilityStatus() != null && !patDemo.getEligibilityStatus().isEmpty()) {
                reportBldr.append(reportCHCSItemStr("Deers Eligibility Status", patDemo.getEligibilityStatus()));
            } else {
                reportBldr.append(reportCHCSItemStr("Deers Eligibility Status", ""));
            }
            if (patDemo.getRank() != null && !patDemo.getRank().isEmpty()) {
                reportBldr.append(reportCHCSItemStr("Rank", patDemo.getRank()));
            } else {
                reportBldr.append(reportCHCSItemStr("Rank", ""));
            }
            rtc[0].setReport(reportBldr.toString());
            addBHIESiteDataToVistaBean("", "PDWS", "PDWS", "", rtc[0]);
        }
        return rtc;
    }
    
    public PatientDemographicsDetail[] mapPatientDemographicsDetailsFromBHIE(Object resultCollection) {
        V4ResultCollection patDemoResultCollection = (V4ResultCollection) resultCollection;
        List<PatientDemographicsDetail> details = new ArrayList<PatientDemographicsDetail>();

        //If data record is empty, inform user
        if (patDemoResultCollection.getDataRecordList() == null || patDemoResultCollection.getDataRecordList().getDemographics() == null) {
            PatientLocation bhieLoc = new PatientLocation();
            bhieLoc.setSystemType(SYSTEM_TYPE_DOD_ADAPTER);

            details.addAll(Arrays.asList(
                    (gov.va.med.vds.webservice.PatientDemographicsDetail[])
                            generateConnectionUnavailableBean(bhieLoc,
                                    Messages.get("demographics.dod.results.empty") + "\n\n"
                                    + Messages.get("demographics.dod.results.helpdesk") + "\n\n")));
        } else {
            StringBuilder reportBldr = new StringBuilder();
            SimpleDateFormat sdf = new SimpleDateFormat("dd MMM yyyy", Locale.getDefault());
            if (patDemoResultCollection.getDataRecordList().getDemographics() != null) {
                for (DemographicV4 patDemo : patDemoResultCollection.getDataRecordList().getDemographics()) {
                    //PatientDemographics patDemo = dataRecord.getPatientDemographics();
                    reportBldr.setLength(0);
                    if (patDemo.getName() != null) {
                        reportBldr.append(reportCHCSItemStr("Name", patDemo.getName().getLast() + "," + patDemo.getName().getFirst() + " " + patDemo.getName().getMiddle()));
                        reportBldr.append(reportCHCSItemStr("Suffix", patDemo.getName().getSuffix()));
                    }

                    if (patDemo.getBirthDate() != null && patDemo.getBirthDate().getStart() != null) {
                        reportBldr.append(reportCHCSItemStr("DOB", sdf.format(bhieToCal(patDemo.getBirthDate().getStart()).getTime())));
                    }
                    if (patDemo.getSex() != null && patDemo.getSex().get(0) != null) {
                        reportBldr.append(reportCHCSItemStr("Sex", patDemo.getSex().get(0).getCode()));
                    }
                    if (patDemo.getAddresses() != null && !patDemo.getAddresses().isEmpty()) {
                        if (patDemo.getAddresses().get(0).getStreet() != null && !patDemo.getAddresses().get(0).getStreet().isEmpty()) {
                            reportBldr.append(reportCHCSItemStr("Street 1", patDemo.getAddresses().get(0).getStreet().get(0)));
                            if (patDemo.getAddresses().get(0).getStreet().size() > 1) {
                                reportBldr.append(reportCHCSItemStr("Street 2", patDemo.getAddresses().get(0).getStreet().get(1)));
                            }
                        }
                        reportBldr.append(reportCHCSItemStr("City", patDemo.getAddresses().get(0).getCity()));
                        reportBldr.append(reportCHCSItemStr("State", patDemo.getAddresses().get(0).getState()));
                        reportBldr.append(reportCHCSItemStr("Zip Code", patDemo.getAddresses().get(0).getPostalCode()));
                        reportBldr.append(reportCHCSItemStr("County", patDemo.getAddresses().get(0).getCounty()));
                        reportBldr.append(reportCHCSItemStr("Country", patDemo.getAddresses().get(0).getCountry()));
                    }
                    if (patDemo.getTelecoms() != null && !patDemo.getTelecoms().isEmpty()) {
                        reportBldr.append(reportCHCSItemStr("Phone", patDemo.getTelecoms().get(0).getValue()));
                    }
                    reportBldr.append(reportCHCSItemStr("Email", patDemo.getEmail()));
//                reportBldr.append(reportCHCSItemStr("SSN", patDemo.getSocialSecurityNumber()));
                    String ssn = "";
                    if (patDemo.getPatientId() != null) {
                        Iterator<RecordIdentifierV4> iter = patDemo.getPatientId().iterator();
                        while (iter.hasNext()) {
                            RecordIdentifierV4 recordid = iter.next();
                            if ("2.16.840.1.113883.3.42.126.100001.2".equals(recordid.getAssigningAuthority())) {
                                ssn = recordid.getId();
                                break;
                            }
//                    if ("2.16.840.1.113883.3.42.10001.100001.12".equals(id.getAssigningAuthority())) {
//                        String edi = id.getId();
//                        if (edi != null && edi.startsWith("D") && edi.length() > 1) {
//                            edi = edi.substring(1);
//                        }
//                        if (edi != null && edi.length() > 0 && PDWSBlackList.getInstance().isBlackListed(edi)) {
//                            reportBldr.append(reportCHCSItemStr("Sensitive", "true"));
//                        }
//                    }
                        }
                    }
                    reportBldr.append(reportCHCSItemStr("SSN", ssn));
                    reportBldr.append(reportCHCSItemStr("Marital Status", getFirstSpdcV4(patDemo.getMaritalStatus()).getDisplay()));
                    reportBldr.append(reportCHCSItemStr("Religion", getFirstSpdcV4(patDemo.getReligion()).getDisplay()));
                    reportBldr.append(reportCHCSItemStr("Race", getFirstSpdcV4(patDemo.getRace()).getDisplay()));
                    if (patDemo.getEnrollmentLocation() != null) {
                        reportBldr.append(reportCHCSItemStr("Enrollment Facility", patDemo.getEnrollmentLocation().getName()));
                    }
// fmp mia                reportBldr.append(reportCHCSItemStr("FMP", patDemo.getFmp()));
                    if (patDemo.getSponsor() != null && patDemo.getSponsor().getSsn() != null && patDemo.getSponsor().getSsn().length() > 0) {
                        reportBldr.append(reportCHCSItemStr("FMP", patDemo.getFmp() + "/" + patDemo.getSponsor().getSsn()));
                    } else {
                        reportBldr.append(reportCHCSItemStr("FMP", patDemo.getFmp()));
                    }
                    if (patDemo.getAssignedUnit() != null) {
                        reportBldr.append(reportCHCSItemStr("UIC", patDemo.getAssignedUnit().getName()));
                    }
                    reportBldr.append(reportCHCSItemStr("Patient Category", getFirstSpdcV4(patDemo.getPatientCategory()).getDisplay()));
                    reportBldr.append(reportCHCSItemStr("Deers Eligibility", getFirstSpdcV4(patDemo.getDeersEligibility()).getDisplay()));
                    reportBldr.append(reportCHCSItemStr("Rank", getFirstSpdcV4(patDemo.getRank()).getDisplay()));
                    reportBldr.append(reportCHCSItemStr("Branch of Service", getFirstSpdcV4(patDemo.getBranch()).getDisplay()));
                    if (patDemo.getSponsor() != null) {
                        reportBldr.append(reportCHCSItemStr("Sponsor Rank", getFirstSpdcV4(patDemo.getSponsor().getRank()).getDisplay()));
                        reportBldr.append(reportCHCSItemStr("Sponsor Branch of Service", getFirstSpdcV4(patDemo.getSponsor().getBranch()).getDisplay()));
//                reportBldr.append(reportCHCSItemStr("Sponsor SSN", patDemo.getSponsorSsn()));
                        reportBldr.append(reportCHCSItemStr("Sponsor SSN", patDemo.getSponsor().getSsn()));
                        if (patDemo.getSponsor().getAssignedUnit() != null) {
                            reportBldr.append(reportCHCSItemStr("Sponsor UIC", patDemo.getSponsor().getAssignedUnit().getName()));
                        }
                    }
                    PatientDemographicsDetail patDetail = new PatientDemographicsDetail();

//                addBHIESiteDataToVistaBean(patDemo.getCustodianId(),
//                        patDemo.getCustodianName(), getFirstSpdc(patDemo.getFacility()).getDisplay(), patDemo.getEventId(), patDetail);
//                addBHIESiteDataToVistaBean(patDemo.getRepository().getId().get(0).getId(),
//                        patDemo.getRepository().getName(), patDemo.getFacility().getName(), patDemo.getRecordId().getId(), patDetail);
                    String repositoryId = "";
                    String repositoryName = "";
                    String facilityName = "";
                    String eventId = "";
                    if (patDemo.getRepository() != null && patDemo.getRepository().getIds() != null && !patDemo.getRepository().getIds().isEmpty()) {
                        repositoryId = patDemo.getRepository().getIds().get(0).getId();
                    }
                    if (patDemo.getRepository() != null) {
                        repositoryName = patDemo.getRepository().getName();
                    }
                    if (patDemo.getFacility() != null) {
                        facilityName = patDemo.getFacility().getName();
                    }
                    if (patDemo.getRecordId() != null) {
                        eventId = patDemo.getRecordId().getId();
                    }
                    if (repositoryId == null || repositoryId.length() == 0) {
                        repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                    }
                    if (repositoryName == null || repositoryName.length() == 0) {
                        repositoryName = "ahlta";
                    }
                    addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                            facilityName, eventId, patDetail);

                    patDetail.setReport(reportBldr.toString());

                    details.add(patDetail);
                }
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(patDemoResultCollection.getStatusListv4(),
                patDemoResultCollection.getErrorListv4(), patDemoResultCollection.isVler());

        details.addAll(Arrays.asList(
            (PatientDemographicsDetail[]) generateConnectionUnavailableBean(
                    BHIE_STATUS_REPORT_LOCATION,
                    siteStatusReport)
        ));

        return details.toArray(new PatientDemographicsDetail[details.size()]);
    }

    //Version 2
//    public PatientDemographics[] mapPatientDemographicsFromBHIE(Object resultCollection)
//    {
//       PatientDemographicsResultCollection patDemoResultCollection = (PatientDemographicsResultCollection) resultCollection;
//       List<PatientDemographics> vistaBeans = new ArrayList<PatientDemographics>();
//
//       //If data record is empty, inform user.
//       if (patDemoResultCollection.getDataRecordList() == null || patDemoResultCollection.getDataRecordList().isEmpty())
//       {
//           PatientLocation bhieLoc = new PatientLocation();
//           bhieLoc.setSystemType(SYSTEM_TYPE_DOD_ADAPTER);
//
//           vistaBeans.addAll(Arrays.asList(
//                   (PatientDemographics[]) generateBHIEPatRegErrorBean(bhieLoc)));
//       }
//       else
//       {
//           SimpleDateFormat df = new SimpleDateFormat("dd MMM yyyy");
//           for (PatientDemographicsDataRecord dataRecord : patDemoResultCollection.getDataRecordList())
//           {
//
//               if (dataRecord == null || dataRecord.getPatientDemographics() == null) continue;
//
//               gov.va.med.bhie.PatientDemographics bhiePat = dataRecord.getPatientDemographics();
//               PatientDemographics pat = new PatientDemographics();
//               pat.setAddress1(bhiePat.getStreet());
//               pat.setAddress2(bhiePat.getStreet2());
//               //pat.setAge(chcsPat.getAge());
//               pat.setCity(bhiePat.getCity());
//               //pat.setClinic(bhiePat.get);
//               pat.setDob(df.format(bhieToCal(bhiePat.getDateOfBirth()).getTime()));
//               pat.setGender(bhiePat.getSex());
////               pat.getRace().addAll(getCodes(bhiePat.getRace()));
//try {
//               pat.getRace().clear();
//               if (bhiePat.getRace() != null && !bhiePat.getRace().isEmpty()) {
//                   pat.getRace().addAll(Arrays.asList(getCodes(bhiePat.getRace())));
//System.out.println("RACE HAS DATA!!!");
//               } else {
//System.out.println("RACE IS EMPTY!!!");
//}
//} catch (Exception ex) {
//ex.printStackTrace();
//System.out.println("RACE ERROR "+ex.getMessage());
//}
//               //pat.setPatientId(chcsPat.getPatientId());
//               pat.setName(bhiePat.getLastName() + ", " + bhiePat.getFirstName() + " " + bhiePat.getMiddleName());
//               //pat.setPrimaryProvider(chcsPat.getPcm());
//               pat.setPhone1(bhiePat.getPhoneNumber());
//               //pat.setPhone2(chcsPat.getPhone2());
//               pat.setSSN(bhiePat.getSocialSecurityNumber());
//               pat.setState(bhiePat.getState());
//               pat.setZipCode(bhiePat.getPostCode());
//               pat.setCodeGreen("0");
//
//               addBHIESiteDataToVistaBean(bhiePat.getCustodianId(),
//                       bhiePat.getCustodianName(), getFirstSpdc(bhiePat.getFacility()).getDisplay(), bhiePat.getEventId(), pat);
//
//               vistaBeans.add(pat);
//           }
//       }
//
//       //include site status report
//       String siteStatusReport = generateBhieStatusReport(patDemoResultCollection.getStatusList(),
//               patDemoResultCollection.getErrorList());
//
//       vistaBeans.addAll(Arrays.asList(
//           (PatientDemographics[]) generateConnectionUnavailableBean(
//                   BHIE_STATUS_REPORT_LOCATION,
//                   siteStatusReport)
//       ));
//
//       return vistaBeans.toArray(new PatientDemographics[0]);
//    }

    

    private String reportCHCSItemStr(String label, String value) {
        StringBuilder str = new StringBuilder("");
        str.append(label);
        str.append(": ");
        if (value != null) {
            str.append(value);
        }
        str.append("\n");

        return str.toString();
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientDemographicsDetail detail = new PatientDemographicsDetail();

        detail.setReport(errorMsg);
        detail.setSite(getConnectionUnavailableSite(location));

        return new PatientDemographicsDetail[]{detail};
    }

    public int compareTo(Object obj) {
        SmartDemographicsDetail tmp = (SmartDemographicsDetail) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_demographicDetail.getSite() != null && tmp.m_demographicDetail.getSite() != null) {
            retVal = compareString(this.m_demographicDetail.getSite().getSiteCode(), tmp.m_demographicDetail.getSite().getSiteCode());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    //TODO: sort is always site
    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = SITE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }
    }
}
