/*
 * Janus 4.0 (c)
 * Copyright (c) 2015 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *      Honorable Senator Daniel K. Inouye
 *      VA Pacific Islands Health Care System
 *      Tripler Army Medical Center
 */

package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.NoteV4;
import gov.va.med.bhie.QuestionnaireV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.Form;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.ProgressNote;
import gov.va.med.vds.webservice.QuestionnaireBean;
import gov.va.med.DNS   .common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Created with IntelliJ IDEA.
 * User: yidali
 * Date: 4/7/14
 * Time: 2:01 PM
 * To change this template use File | Settings | File Templates.
 */
public class SmartForm extends SmartBean implements Comparable {

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int DOCUMENTTITLE = 3;

    private static final Logger LOGGER = LoggerFactory.getLogger(SmartForm.class);

    public Form m_Patient_Form;

    public SmartForm() {
        super();
        m_Patient_Form = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartForm(Form form, String sort) {
        super();
        m_Patient_Form = form;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartForm) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_Patient_Form);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartForm[] temp = new SmartForm[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartForm newAll = new SmartForm((Form) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Form[] ret = new Form[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_Form;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartForm tmp = (SmartForm) obj;
        int retVal = 0;
        if (this.m_sortBy == SITE && this.m_Patient_Form.getSite().getName() != null && tmp.m_Patient_Form.getSite().getName() != null) {
            retVal = compareString(this.m_Patient_Form.getSite().getName(), tmp.m_Patient_Form.getSite().getName());
        }
        if (this.m_sortBy == DOCUMENTTITLE) {
            retVal = compareString(this.m_Patient_Form.getDocumentTitle(), tmp.m_Patient_Form.getDocumentTitle());
        }

        if (this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("date".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = DOCUMENTTITLE;
            }
        }

    }


    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartForm[] temp = new SmartForm[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartForm newAll = new SmartForm((Form) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartForm newAll = new SmartForm((Form) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Form[] ret = new Form[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_Patient_Form;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Form form = new Form();
        form.setDocumentTitle(errorMsg);
        form.setFormDate(Utils.backDate().toGregorianCalendar());
        form.setSite(getConnectionUnavailableSite(location));
        return new Form[]{form};
    }

    public Form[] mapQuestionnairesFromBHIE(Object resultCollection) {
        List<Form> formBeans = new ArrayList<Form>();

//        List<QuestionnaireBean> vistaBeans = new ArrayList<QuestionnaireBean>();
        V4ResultCollection questionnaireResultCollection = (V4ResultCollection) resultCollection;

        if (questionnaireResultCollection.getDataRecordList() != null && questionnaireResultCollection.getDataRecordList().getQuestionnaires() != null) {

            for (QuestionnaireV4 questionnaire : questionnaireResultCollection.getDataRecordList().getQuestionnaires()) {
                Form formBean = new Form();

                if (questionnaire == null) {
                    continue;
                }

                QuestionnaireBean bean = new QuestionnaireBean();

                bean.setComments(questionnaire.getComments());
                if (questionnaire.getResponseDate() != null) {
                    bean.setResponseDate(bhieToXMLCalV4(questionnaire.getResponseDate()));
                    formBean.setFormDate(bhieToXMLCalV4(questionnaire.getResponseDate()).toGregorianCalendar());
                }

                if (questionnaire.getResponses() != null && !questionnaire.getResponses().isEmpty()) {
                    for (gov.va.med.bhie.QuestionnaireDetailsV4 responses : questionnaire.getResponses()) {
                        gov.va.med.vds.webservice.QuestionnaireDetails det = new gov.va.med.vds.webservice.QuestionnaireDetails();
                        det.setQuestion(responses.getQuestion());
                        det.setSequence(responses.getSequenceNumber());
                        if (responses.getAnswers() != null && !responses.getAnswers().isEmpty()) {
                            for (gov.va.med.bhie.QuestionnaireDetailAnswersV4 answer : responses.getAnswers()) {
                                gov.va.med.vds.webservice.QuestionnaireDetailAnswers ans = new gov.va.med.vds.webservice.QuestionnaireDetailAnswers();
                                ans.setAnswer(answer.getAnswer());
                                ans.setAnswerType(answer.getAnswerType());
                                ans.setChoice(answer.getChoice());
                                ans.setCorrectAnswer(answer.getCorrectAnswer());
                                det.getAnswers().add(ans);
                            }
                        }
                        if (responses.getChoices() != null && !responses.getChoices().isEmpty()) {
                            for (gov.va.med.bhie.QuestionnaireDetailChoicesV4 choice : responses.getChoices()) {
                                gov.va.med.vds.webservice.QuestionnaireDetailChoices ichoice = new gov.va.med.vds.webservice.QuestionnaireDetailChoices();
                                ichoice.setAnswerType(choice.getAnswerType());
                                ichoice.setChoice(choice.getChoice());
                                ichoice.setChoiceText(choice.getText());
                                ichoice.setCorrectAnswer(choice.getCorrectAnswer());
                                ichoice.setSelected(choice.isSelected());
                                det.getChoices().add(ichoice);
                            }
                        }
                        if (responses.getComments() != null && !responses.getComments().isEmpty()) {
                            for (gov.va.med.bhie.QuestionnaireDetailCommentsV4 comment : responses.getComments()) {
                                gov.va.med.vds.webservice.QuestionnaireDetailComments cmt = new gov.va.med.vds.webservice.QuestionnaireDetailComments();
                                cmt.setCommentBy(comment.getEnteredBy().getName());
                                cmt.setDateEntered(comment.getEnteredDate().getStart());
                                cmt.setNoteNumber(comment.getCommentNumber());
                                cmt.setNoteText(comment.getComment());
                                cmt.setSeqNumber(comment.getSequenceNumber());
                                det.getComments().add(cmt);
                            }
                        }
                        bean.getResponses().add(det);
                    }
                }
                bean.setEnteredBy(mapProvider(questionnaire.getEnteredBy()));
                bean.setEnteredDate(Utils.formatDisplayDateTime(bhieToXMLCalV4(questionnaire.getEnteredDate())));
                if (questionnaire.getRecordId() != null) {
                    bean.setRecordId(questionnaire.getRecordId().getId());
                }
                bean.setName(questionnaire.getName());
                bean.setType(questionnaire.getType());
                bean.setVersion(questionnaire.getVersion());
                if (questionnaire.getResponseComments() != null && !questionnaire.getResponseComments().isEmpty()) {
                    for (gov.va.med.bhie.QuestionnaireResponseCommentsV4 comment : questionnaire.getResponseComments()) {
                        gov.va.med.vds.webservice.QuestionnaireResponseComments cmt = new gov.va.med.vds.webservice.QuestionnaireResponseComments();
                        cmt.setComment(comment.getComment());
                        cmt.setDateEntered(comment.getEnteredDate().getStart());
                        cmt.setEnteredBy(mapProvider(comment.getEnteredBy()));
                        cmt.setNumber(comment.getNumber());
                        cmt.setSequence(comment.getSequence());
                        bean.getResponseComments().add(cmt);
                    }
                }


                String repositoryId = "";
                String repositoryName = "";
                String facilityName = "";
                String eventId = "";
                if (questionnaire.getRepository() != null && questionnaire.getRepository().getIds() != null && !questionnaire.getRepository().getIds().isEmpty()) {
                    repositoryId = questionnaire.getRepository().getIds().get(0).getId();
                }
                if (questionnaire.getRepository() != null) {
                    repositoryName = questionnaire.getRepository().getName();
                }
                if (questionnaire.getFacility() != null) {
                    facilityName = questionnaire.getFacility().getName();
                }
                if (questionnaire.getRecordId() != null) {
                    eventId = questionnaire.getRecordId().getId();
                    formBean.setFormId(eventId);
                }
                if (repositoryId == null || repositoryId.length() == 0) {
                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                }
                if (repositoryName == null || repositoryName.length() == 0) {
                    repositoryName = "ahlta";
                }
                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                        facilityName, eventId, bean);

//                if(questionnaire.getDataDomain()!= null && !questionnaire.getDataDomain().isEmpty()){
//                    for(SpdcBean spdc : questionnaire.getDataDomain()){
//                        Code code = new Code();
//                        code.setSystem("LOINC");
//                        code.setCode(spdc.getCode());
//                        code.setDisplay(spdc.getDisplay());
//
//                        formBean.getCodes().add(code);
//                    }
//                }

                Code code = new Code();
                code.setSystem("LOINC");
                code.setCode("74465-6");
                code.setDisplay("Questionnaire response section Document");

                formBean.getCodes().add(code);

                if (questionnaire.getEnteredBy() != null
                        && questionnaire.getEnteredBy().getName() != null) {
                    formBean.setEnteredBy(questionnaire.getEnteredBy().getName());
                }
                formBean.setDocType("Q");
                formBean.setFacilityName(facilityName);
                formBean.setDocumentTitle(questionnaire.getName());
                formBean.setQuestionnaireBean(bean);
                addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, formBean);


                formBeans.add(formBean);
            }
        }

        String siteStatusReport = generateBhieStatusReportV4("Questionnaires", questionnaireResultCollection.getStatusListv4(),
                questionnaireResultCollection.getErrorListv4());

        formBeans.addAll(Arrays.asList(
                (Form[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return formBeans.toArray(new Form[formBeans.size()]);
    }


//    public Form[] mapDeploymentFormsFromBHIE(Object resultCollection) {
//        List<Form> formBeans = new ArrayList<Form>();
//
//        //List<gov.va.med.vds.webservice.DeploymentForm> vistaBeans = new ArrayList<gov.va.med.vds.webservice.DeploymentForm>();
//        DeploymentResultCollection deploymentResultCollection = (DeploymentResultCollection) resultCollection;
//
//        if (deploymentResultCollection.getDataRecordList() != null) {
//
//            for (DeploymentDataRecord dataRecord : deploymentResultCollection.getDataRecordList())
//            {
//                Form formBean = new Form();
//
//                if (dataRecord == null || dataRecord.getDeploymentForm() == null) continue;
//
//                gov.va.med.vds.webservice.DeploymentForm bean = new gov.va.med.vds.webservice.DeploymentForm();
//                gov.va.med.bhie.DeploymentForm form = dataRecord.getDeploymentForm();
//                bean.setNoteId(form.getEventId());
//                bean.setCompleteNote(form.getCompleteNote());
//                bean.setCustodianId(form.getCustodianId());
//                bean.setEnteredBy(form.getEnteredBy());
//                bean.setEnteredByDate(form.getEnteredByDate());
//                bean.setFacility(form.getFacility());
//                bean.setLoincCode(form.getLoincCode());
//                bean.setNoteFormat(form.getNoteFormat());
//                bean.setNoteTitle(form.getNoteTitle());
//                bean.setNoteTypeCode(form.getNoteTypeCode());
//                bean.setNoteTypeName(form.getNoteTypeName());
//                bean.setOtherIdentifier(form.getOtherIdentifier());
//                bean.setRepositoryId(form.getRepositoryId());
//                bean.setVerifiedBy(form.getVerifiedBy());
//                bean.setVerifiedDate(form.getVerifiedDate());
//
//
//                String repositoryId = "";
//                String repositoryName = "ahlta";
//                String facilityName = "";
//                String eventId = "";
//
//                repositoryId = form.getRepositoryId();
//                facilityName = form.getFacility();
//
//                if (repositoryId == null || repositoryId.length() == 0) {
//                    repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
//                }
//                if (facilityName == null || facilityName.length() == 0) {
//                    facilityName = "ahlta";
//                }
//                addBHIESiteDataToVistaBean(repositoryId, repositoryName,
//                        facilityName, eventId, bean);
//
//                if(StringUtils.isNotBlank(form.getNoteTypeCode())){
//                    Code code = new Code();
//                    code.setSystem(CodingSystem.DOD_MEDCIN.getDisplayName());
//                    code.setCode(form.getNoteTypeCode());
//                    code.setDisplay(null);
//
//                    formBean.getCodes().add(code);
//
//                }
//
//                Code code = new Code();
//                code.setSystem("LOINC");
//                code.setCode("64284-3");
//                code.setDisplay("Readiness for duty assessment");
//
//                formBean.getCodes().add(code);
//
//                formBean.setFormId(form.getEventId());
//                formBean.setEnteredBy(form.getEnteredBy());
//                if(form.getEnteredByDate()!=null)
//                    formBean.setFormDate(Utils.getCal(form.getEnteredByDate()));
//                formBean.setDocType("F");
//                formBean.setFacilityName(facilityName);
//                formBean.setDocumentTitle(form.getNoteTitle());
//                formBean.setDeploymentForm(bean);
//                addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, formBean);
//
//                formBeans.add(formBean);
//            }
//        }
//
//        String siteStatusReport = generateBhieStatusReport("DeploymentForms", deploymentResultCollection.getStatusList(),
//                deploymentResultCollection.getErrorList());
//
//        formBeans.addAll(Arrays.asList(
//                (Form[]) generateConnectionUnavailableBean(
//                        BHIE_STATUS_REPORT_LOCATION,
//                        siteStatusReport)
//        ));
//
//        return formBeans.toArray(new Form[0]);
//    }

    // Version 4
    public Form[] mapDeploymentFormsFromBHIE(Object resultCollection) throws JMeadowsException {
        V4ResultCollection noteResultCollection = (V4ResultCollection) resultCollection;
        List<Form> vistaBeans = new ArrayList<Form>();
        try {

            if (noteResultCollection != null && noteResultCollection.getDataRecordList() != null && noteResultCollection.getDataRecordList().getNotes() != null) {
                //BDA is currently sending us a null id, so let's create one for JLV cache
                int formId = 0;

                for (NoteV4 bhieNote : noteResultCollection.getDataRecordList().getNotes()) {
                    if (bhieNote == null) {
                        continue;
                    }

                    Form form = new Form();
                    ProgressNote bean = new ProgressNote();

                    if (bhieNote.getTitle() != null && !bhieNote.getTitle().isEmpty()) {
                        bean.setNoteTitle(bhieNote.getTitle());
                    }
                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty()
                            && bhieNote.getType().get(0) != null) {
                        bean.setNoteType(bhieNote.getType().get(0).getDisplay());
                    }
                    bean.setSensitive(bhieNote.getSensitive());
                    if (bhieNote.getContentType() != null && !bhieNote.getContentType().isEmpty()) {
                        bean.setStatus(bhieNote.getContentType().get(0).getCode());
                    }
                    bean.setComplexDataUrl(bhieNote.getComplexDataUrl());
                    String custodian = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        custodian = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getEnteredDate() != null) {
                        bean.setNoteDate(bhieToXMLCalV4(bhieNote.getEnteredDate()));
                        form.setFormDate(bhieToXMLCalV4(bhieNote.getEnteredDate()).toGregorianCalendar());
                    } else if (APP_CONFIG.getBHIECustodianOidSHARE().equalsIgnoreCase(custodian)) {
                        bean.setNoteDate(bhieToXMLCalV4(bhieNote.getVerifiedDate()));
                        form.setFormDate(bhieToXMLCalV4(bhieNote.getVerifiedDate()).toGregorianCalendar());
                    }
                    String clinic = "";
                    if (bhieNote.getFacility() != null) {
                        clinic = bhieNote.getFacility().getName();
                    }

                    //dod adapter facility format: location^clinic
                    if (StringUtils.isNotBlank(clinic) && clinic.contains("^")) {
                        clinic = clinic.substring(clinic.indexOf('^') + 1);
                    }

                    bean.setLocation(clinic);

                    //                 ALTHA currrently does not send CDA and uses complexURL
                    //                if ("CDA".equalsIgnoreCase(bhieNote.getNoteFormat())) {
                    //                    //may complete complete note and event id
                    //                    mapCDAToProgressNote(bhieNote, bean);
                    //                } else

                    bean.setNoteText(bhieNote.getContent());
//                    if(bhieNote.getRepository() != null
//                            && bhieNote.getRepository().getId() != null
//                            && !bhieNote.getRepository().getId().isEmpty()
//                            && bhieNote.getRepository().getId().get(0).getAssigningAuthority() != null
//                            && APP_CONFIG.getBHIECustodianOidSHARE().matches(bhieNote.getRepository().getId().get(0).getAssigningAuthority())
//                            && bhieNote.getContentType() != null
//                            && !bhieNote.getContentType().isEmpty()
//                            && bhieNote.getContentType().get(0).getCode() != null
//                            && "CDA".equalsIgnoreCase(bhieNote.getContentType().get(0).getCode()))
//                        //TODO:populate provider from BHIERelay
//                        bean.setProvider(mapProvider(bhieNote.getEnteredBy()));
                    if (bhieNote.getVerifiedBy() != null) {
                        bean.setProvider(mapProvider(bhieNote.getVerifiedBy()));
                    }

                    if (bhieNote.getType() != null && !bhieNote.getType().isEmpty() && StringUtils.isNotBlank(bhieNote.getType().get(0).getCode())) {
                        Code code = new Code();
                        code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                        code.setCode(bhieNote.getType().get(0).getCode());
                        bean.getCodes().add(code);
                    }

                    if (bhieNote.getDataDomain() != null && !bhieNote.getDataDomain().isEmpty() && StringUtils.isNotBlank(bhieNote.getDataDomain().get(0).getCode())) {
                        Code code = new Code();
                        code.setSystem(CodingSystem.DOD_NOTES.getDisplayName());
                        code.setCode(bhieNote.getDataDomain().get(0).getCode());
                        bean.getCodes().add(code);
                    }

                    //                addBHIESiteDataToVistaBean(bhieNote.getCustodianId(),
                    //                        bhieNote.getCustodianName(), bhieNote.getFacility(), bhieNote.getEventId(), bean);
                    //                addBHIESiteDataToVistaBean(bhieNote.getRepository().getId().get(0).getId(),
                    //                        bhieNote.getRepository().getName(), bhieNote.getFacility().getName(), bhieNote.getRecordId().getId(), bean);
                    String repositoryId = "";
                    String repositoryName = "";
                    String facilityName = "";
                    String eventId = "";
                    if (bhieNote.getRepository() != null && bhieNote.getRepository().getIds() != null && !bhieNote.getRepository().getIds().isEmpty()) {
                        repositoryId = bhieNote.getRepository().getIds().get(0).getId();
                    }
                    if (bhieNote.getRepository() != null) {
                        repositoryName = bhieNote.getRepository().getName();
                    }
                    if (bhieNote.getFacility() != null) {
                        facilityName = bhieNote.getFacility().getName();
                    }
                    if (bhieNote.getRecordId() != null) {
                        eventId = bhieNote.getRecordId().getId();
                    }
                    if (repositoryId == null || repositoryId.length() == 0) {
                        repositoryId = "2.16.840.1.113883.3.42.126.100001.13";
                    }
                    if (repositoryName == null || repositoryName.length() == 0) {
                        repositoryName = "ahlta";
                    }
                    addBHIESiteDataToVistaBean(repositoryId, repositoryName,
                            facilityName, eventId, bean);

                    Code code = new Code();
                    code.setSystem("LOINC");
                    code.setCode("64284-3");
                    code.setDisplay("Readiness for duty assessment");

                    form.getCodes().add(code);
                    if (bhieNote.getRecordId() != null && bhieNote.getRecordId().getId() != null) {
                        form.setFormId(bhieNote.getRecordId().getId());
                    } else {
                        form.setFormId("deploymentForm_" + formId);
                        formId++;
                    }
                    form.setEnteredBy(form.getEnteredBy());
//                    if(bhieNote.getEnteredDate()!=null)
//                        form.setFormDate(Utils.getCal(bhieNote.getEnteredDate()));
                    form.setDocType("F");
                    form.setFacilityName(facilityName);
                    if (bhieNote.getTitle() != null) {
                        form.setDocumentTitle(bhieNote.getTitle());
                    } else {
                        form.setDocumentTitle("Pre/Post - Deployment Health Assessment Form");
                    }

                    addBHIESiteDataToVistaBean(repositoryId, repositoryName, facilityName, eventId, form);

                    form.setDeploymentForm(bean);

                    form.setDeploymentForm(bean);
                    vistaBeans.add(form);
                }
            }
        } catch (Exception ex) {
            LOGGER.error("EXCEPTION: " + ex.getLocalizedMessage(), ex);
            throw new JMeadowsException(ex);
        }
        //include site status report
        String siteStatusReport = "";
        if (noteResultCollection != null) {
            siteStatusReport = generateBhieStatusReportV4(noteResultCollection.getStatusListv4(), 
                noteResultCollection.getErrorListv4(), noteResultCollection.isVler());
        }

        vistaBeans.addAll(Arrays.asList(
                (Form[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));
        return vistaBeans.toArray(new Form[vistaBeans.size()]);
    }
}
