/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */

package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.PayerV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.InsuranceBean;
import gov.va.med.vds.webservice.Telecom;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartInsurance extends SmartBean implements Comparable {
//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartInsurance.class);

    public static final int DATE = 0;
    public static final int SITE = 1;

    public InsuranceBean m_insurance;

    public SmartInsurance() {
        super();
        m_insurance = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartInsurance(InsuranceBean insurance, String sort) {
        super();
        m_insurance = insurance;
        getSortValue(sort);
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartInsurance[] temp = new SmartInsurance[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartInsurance newAll = new SmartInsurance((InsuranceBean) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartInsurance newAll = new SmartInsurance((InsuranceBean) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            InsuranceBean[] ret = new InsuranceBean[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_insurance;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartInsurance[] temp = new SmartInsurance[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartInsurance newAll = new SmartInsurance((InsuranceBean) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            InsuranceBean[] ret = new InsuranceBean[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_insurance;
            }
            rtc = ret;
        }
        return rtc;
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (!(obj instanceof SmartInsurance)) {
            rtc = false;
        } else {
            rtc = compareTo(obj) == 0;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_insurance);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public int compareTo(Object obj) {
        SmartInsurance tmp = (SmartInsurance) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_insurance.getEffectiveDate(), tmp.m_insurance.getEffectiveDate());
        }
        if (this.m_sortBy == SITE && this.m_insurance.getSite() != null && tmp.m_insurance.getSite() != null) {
            retVal = compareString(this.m_insurance.getSite().getSiteCode(), tmp.m_insurance.getSite().getSiteCode());
        }

        if (!m_bAscending) {
            retVal = -retVal;
        }

        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("effectiveDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
        }

    }

    public InsuranceBean[] mapInsurancesFromBHIE(Object resultCollection) {
        V4ResultCollection insuranceResultCollection = (V4ResultCollection) resultCollection;
        List<InsuranceBean> vistaBeans = new ArrayList<InsuranceBean>();

        if (insuranceResultCollection.getDataRecordList() != null) {
            for (PayerV4 bhieInsurance : insuranceResultCollection.getDataRecordList().getPayers()) {
                if (bhieInsurance == null || bhieInsurance.getPlan() == null) {
                    continue;
                }

                InsuranceBean bean = new InsuranceBean();

                // insuranceType (BHIE) -> insuranceType (VistaBean) -> Health Plan Type (GUI)
                if (bhieInsurance.getPlan().getType() != null) {
                    SpdcBeanV4 planType = getPrimarySpdcV4(bhieInsurance.getPlan().getType());
                    if (planType != null) {
                        bean.setInsuranceType(planType.getDisplay());

                        //todo: melcolm add check for coding system from VLER
                        // Standardized Insurance Type
                        Code code = new Code();
                        code.setSystem(CodingSystem.DOD_MEDCIN.getDisplayName());
                        code.setCode(planType.getCode());
                        bean.getCodes().add(code);
                    }

                }
                // insuranceName (BHIE) -> companyName (VistaBean) -> Health Plan Name (GUI)

                if (bhieInsurance.getPlan().getInsurance() != null) {
                    bean.setCompanyName(bhieInsurance.getPlan().getInsurance().getName());
                }
                bean.setNarrativeId(bhieInsurance.getNarrativeId());
                // planName (BHIE) -> planName (VistaBean) -> Plan Name (GUI)
                bean.setPlanName(bhieInsurance.getPlan().getName());
                // planEffectiveTime (BHIE) -> effectiveDate (VistaBean) -> Plan Effective Date (GUI)
                if (bhieInsurance.getPlan().getEffectiveDate() != null) {
                    bean.setEffectiveDate(bhieToXMLCalV4(bhieInsurance.getPlan().getEffectiveDate()));
                }
                // planExpirationTime (BHIE) -> expirationDate (VistaBean) -> Plan Expiration Date (GUI)
                if (bhieInsurance.getPlan().getExpirationDate() != null) {
                    bean.setExpirationDate(bhieToXMLCalV4(bhieInsurance.getPlan().getExpirationDate()));
                }
                // groupNumber (BHIE) -> groupNumber (VistaBean) -> Group Number (GUI)
                bean.setGroupNumber(bhieInsurance.getGroup());
                // subscriberUnitNumber (BHIE) -> memberId (VistaBean) -> Member ID (GUI)
                bean.setMemberId(bhieInsurance.getPolicy());

                // policyNumber (BHIE) -> subscriberId (VistaBean) -> Subscriber ID (GUI)
                if (bhieInsurance.getSubscriber() != null){

                    if(bhieInsurance.getSubscriber().getIds() != null
                        && bhieInsurance.getSubscriber().getIds().size() > 0) {
                        bean.setSubscriberId(bhieInsurance.getSubscriber().getIds().get(0).getId());
                    }
                    if (bhieInsurance.getSubscriber().getBirthDate() != null) {
                        bean.setSubscriberDOB(bhieToXMLCalV4(bhieInsurance.getSubscriber().getBirthDate()));
                    }

                    // relationship (BHIE) -> relationship (VistaBean) -> Subscriber's Relationship to Insurer (GUI)
                    if (bhieInsurance.getSubscriber().getRelationship() != null && getPrimarySpdcV4(bhieInsurance.getSubscriber().getRelationship()) != null) {
                        bean.setRelationShip(getPrimarySpdcV4(bhieInsurance.getSubscriber().getRelationship()).getDisplay());
                    }
                }

                // Health Plan Mailing Address (GUI)
                if (bhieInsurance.getPlan().getAddress() != null
                        && bhieInsurance.getPlan().getAddress().getStreet() != null
                        && bhieInsurance.getPlan().getAddress().getStreet().size() > 0) {
                    if (bhieInsurance.getPlan().getAddress().getStreet().get(0) != null) {
                        bean.setCompanyAddressLine1(bhieInsurance.getPlan().getAddress().getStreet().get(0));
                    }
                    if (bhieInsurance.getPlan().getAddress().getStreet().size() > 1) {
                        if (bhieInsurance.getPlan().getAddress().getStreet().get(1) != null) {
                            bean.setCompanyAddressLine2(bhieInsurance.getPlan().getAddress().getStreet().get(1));
                        }
                        if (bhieInsurance.getPlan().getAddress().getStreet().size() > 2
                                && bhieInsurance.getPlan().getAddress().getStreet().get(2) != null) {
                            bean.setCompanyAddressLine3(bhieInsurance.getPlan().getAddress().getStreet().get(2));
                        }
                    }
                    bean.setCompanyCity(bhieInsurance.getPlan().getAddress().getCity());
                    bean.setCompanyPostalCode(bhieInsurance.getPlan().getAddress().getPostalCode());
                    if (bhieInsurance.getPlan().getAddress().getState() == null) {
                        bean.setCompanyStateProvince(bhieInsurance.getPlan().getAddress().getCounty());
                    } else {
                        bean.setCompanyStateProvince(bhieInsurance.getPlan().getAddress().getState());
                    }
                }

                // areaCode + localNumber (BHIE) -> companyTelecom (VistaBean) -> Health Plan Contact Information (GUI)
                Telecom telecom = new Telecom();
                String phNum = "";
                if (bhieInsurance.getPlan().getTelecom() != null) {
                    phNum = bhieInsurance.getPlan().getTelecom().getValue();
                }
                telecom.setValue(phNum);
                bean.getCompanyTelecom().add(telecom);

                bean.setComment(bhieInsurance.getComment());
                bean.setRxBin(bhieInsurance.getRxBin());
                bean.setRxPcn(bhieInsurance.getRxPcn());
                bean.setTextDescription(bhieInsurance.getTextDescription());

                // Site
                String repositoryId = "";
                String repositoryName = "";
                String recordId = "";
                String facilityName = "";

                if (bhieInsurance.getRepository() != null
                        && bhieInsurance.getRepository().getIds() != null
                        && !bhieInsurance.getRepository().getIds().isEmpty()) {
                    repositoryId = bhieInsurance.getRepository().getIds().get(0).getId();
                    repositoryName = bhieInsurance.getRepository().getName();
                }
                if (bhieInsurance.getFacility() != null) {
                    facilityName = bhieInsurance.getFacility().getName();
                }
                if (bhieInsurance.getRecordId() != null) {
                    recordId = bhieInsurance.getRecordId().getId();
                }

                if (insuranceResultCollection.isVler()) {
                    // copy subscriberId to memberId for VLER data
                    bean.setMemberId(bean.getSubscriberId());

                    addBHIESiteDataToVistaBean(repositoryId,
                            "VLER", facilityName, recordId, bean);
                } else {
                    addBHIESiteDataToVistaBean(repositoryId,
                            repositoryName, facilityName, recordId, bean);
                }

                vistaBeans.add(bean);
            }
        }

        String siteStatusReport = generateBhieStatusReportV4(insuranceResultCollection.getStatusListv4(),
                insuranceResultCollection.getErrorListv4(), insuranceResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (InsuranceBean[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION, siteStatusReport)
        ));

        return vistaBeans.toArray(new InsuranceBean[vistaBeans.size()]);
    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        InsuranceBean ins = new InsuranceBean();
        ins.setPlanName(errorMsg);
        ins.setSite(getConnectionUnavailableSite(location));

        return new InsuranceBean[]{ins};
    }
}
