/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */


package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.ProblemV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.Problem;
import gov.va.med.vds.webservice.ProblemDetail;
import gov.va.med.vds.webservice.ProblemNote;
import gov.va.med.DNS   .common.Utils;
import org.apache.commons.lang3.StringUtils;
//import org.hui.chcs.ProblemBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.*;

//import gov.va.med.DNS   .beans.*;
//import gov.va.med.chcs.beans.*;

public class SmartProblem extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartProblem.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;

//    private static final GregorianCalendar CAL1809 = new GregorianCalendar(1809, 0, 1);

    public Problem m_problem;

    public SmartProblem() {
        super();
        m_problem = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartProblem(Problem prob, String sort) {
        super();
        m_problem = prob;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartProblem) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_problem);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartProblem[] temp = new SmartProblem[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProblem newAll = new SmartProblem((Problem) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartProblem newAll = new SmartProblem((Problem) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Problem[] ret = new Problem[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_problem;
            }

            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartProblem[] temp = new SmartProblem[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartProblem newAll = new SmartProblem((Problem) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Problem[] ret = new Problem[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_problem;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartProblem tmp = (SmartProblem) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            Date date1 = null;
            Date date2 = null;
            if (this.m_problem.getLastModifiedDate() != null) {
                date1 = this.m_problem.getLastModifiedDate().toGregorianCalendar().getTime();
            }
            if (tmp.m_problem.getLastModifiedDate() != null) {
                date2 = tmp.m_problem.getLastModifiedDate().toGregorianCalendar().getTime();
            }
            retVal = compareDate(date1, date2);
        }
        if (this.m_sortBy == SITE && this.m_problem.getSite() != null && tmp.m_problem.getSite() != null) {
            String site1 = null;
            String site2 = null;
            if (this.m_problem.getSite() != null) {
                site1 = this.m_problem.getSite().getSiteCode();
            }
            if (tmp.m_problem.getSite() != null) {
                site2 = tmp.m_problem.getSite().getSiteCode();
            }
            retVal = compareString(site1, site2);
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_problem.getDescription(), tmp.m_problem.getDescription());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("lastmodifiedDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
        }


    }

//    public static Problem[] mapProblemListFromCHCS(Object[] dodBean) {
//        ProblemBean[] chcsBean = (ProblemBean[]) dodBean;
//        int size = chcsBean.length;
//        Problem[] vistaBean = new Problem[size];
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            Problem bean = new Problem();
//            String acuity = chcsBean[i].getAcuity();
//            if (acuity != null) {
//                if ("A".equals(acuity)) {
//                    acuity = "Acute";
//                }
//                if ("C".equals(acuity)) {
//                    acuity = "Chronic";
//                }
//            }
//            bean.setAcuity(acuity);
//            bean.setDescription(chcsBean[i].getLongDescription());
//            bean.setIcdCode(chcsBean[i].getIcdCode());
//            if (chcsBean[i].getLastModifiedDate() != null) {
//                bean.setLastModifiedDate(Utils.getXMLCal(chcsBean[i].getLastModifiedDate()));
//            } else if (chcsBean[i].getOnsetDate() == null) {
//                bean.setLastModifiedDate(Utils.toXMLGregorianCalendar(CAL1809.getTime()));
//            } else {
//                bean.setLastModifiedDate(Utils.getXMLCal(chcsBean[i].getOnsetDate()));
//            }
//            if (chcsBean[i].getOnsetDate() == null) {
//                bean.setOnsetDate(Utils.toXMLGregorianCalendar(CAL1809.getTime()));
//            } else {
//                bean.setOnsetDate(Utils.getXMLCal(chcsBean[i].getOnsetDate()));
//            }
//
//            bean.setStatus(chcsBean[i].getStatus());
//            bean.setId(chcsBean[i].getProblemId());
//
//            addCHCSSiteDataToVistaBean(bean);
//
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }

//    public Problem[] mapProblemListFromBHIE(Object resultCollection) {
//        ProblemResultCollectionV3 problemResultCollection = (ProblemResultCollectionV3) resultCollection;
//        List<Problem> vistaBeans = new ArrayList<Problem>();
//
//        if (problemResultCollection.getDataRecordList() != null) {
//
//            for (ProblemDataRecordV3 dataRecord : problemResultCollection.getDataRecordList()) {

    public Problem[] mapProblemListFromBHIE(Object resultCollection) {
        V4ResultCollection problemResultCollection = (V4ResultCollection) resultCollection;
        List<Problem> vistaBeans = new ArrayList<Problem>();
        boolean hasMedcin;
        boolean excludeMedcin;

        if (problemResultCollection.getDataRecordList() != null) {

            for (ProblemV4 bhieProblem : problemResultCollection.getDataRecordList().getProblems()) {

                if (bhieProblem == null) {
                    continue;
                }
                //remove problems with a negative status "-" ; located in otherIdentifier field.
                if (bhieProblem.getNegationInd() != null && ("-".equalsIgnoreCase(bhieProblem.getNegationInd()) || "true".equalsIgnoreCase(bhieProblem.getNegationInd()))) {
                    continue;
                }

                ProblemDetail bean = new ProblemDetail();

                String acuity = "";
                if (bhieProblem.getChronicity() != null && getPrimarySpdcV4(bhieProblem.getChronicity()) != null) {
                    acuity = getPrimarySpdcV4(bhieProblem.getChronicity()).getDisplay();
                }
                if (acuity != null && acuity.length() > 0) {
                    if ("A".equals(acuity)) {
                        acuity = "Acute";
                    }
                    if ("C".equals(acuity)) {
                        acuity = "Chronic";
                    }
                }

                bean.setAcuity(acuity);

                if (bhieProblem.getProblem() != null) {
                    hasMedcin = false;
                    excludeMedcin = false;

                    for (SpdcBeanV4 problem : bhieProblem.getProblem()) {
                        if (problem != null && problem.getSystem() != null) {
                            if (CodingSystem.ICD10_CM.getOID().equalsIgnoreCase(problem.getSystem())) {
                                bean.setIcdCode(problem.getCode());
                                excludeMedcin = true;
                                //bean.setDescription(problem.getDisplay());
                            } else if (CodingSystem.ICD9_CM_DISEASES.getOID().equalsIgnoreCase(problem.getSystem())
                                    && (bean.getIcdCode() == null || bean.getIcdCode().isEmpty())) {

                                bean.setIcdCode(problem.getCode());
                                //bean.setDescription(problem.getDisplay());
                            }

                            if (CodingSystem.DOD_MEDCIN.getOID().equalsIgnoreCase(problem.getSystem())) {
                                Code code = new Code();
                                code.setSystem(CodingSystem.DOD_MEDCIN.getDisplayName());
                                code.setCode(problem.getCode());
                                bean.getCodes().add(code);
                                hasMedcin = true;
                            }
                        }
                    }

                    // If ICD-10 code exists, exclude/remove MEDCIN from code list, if exists
                    if (hasMedcin && excludeMedcin) {
                        Code code;
                        for (Iterator<Code> codeIterator = bean.getCodes().iterator(); codeIterator.hasNext(); ) {
                            code = codeIterator.next();
                            if (code != null && CodingSystem.DOD_MEDCIN.getDisplayName().equalsIgnoreCase(code.getSystem())) {
                                codeIterator.remove();
                            }
                        }
                    }

                    if (getPrimarySpdcV4(bhieProblem.getProblem()) != null) {
                        bean.setDescription(getPrimarySpdcV4(bhieProblem.getProblem()).getDisplay());
                    }

                }

                if (StringUtils.isNotBlank(bhieProblem.getLastModifiedDate())) {
                    bean.setLastModifiedDate(bhieToXMLCal(bhieProblem.getLastModifiedDate()));
                } else {
                    //use onset date as last modified date if last modified date is null
                    bean.setLastModifiedDate(bhieToXMLCalV4(bhieProblem.getModifiedDate()));
                }

                if (bhieProblem.getOnsetDate() != null && bhieToXMLCalV4(bhieProblem.getOnsetDate()) != null) {
                    String onsetDate = "";
                    Calendar onsetDateCal = bhieToXMLCalV4(bhieProblem.getOnsetDate()).toGregorianCalendar();
                    onsetDate = Utils.formatDisplayDate(onsetDateCal);
                    bean.setOnsetDate(onsetDate);
                }

                if (bhieProblem.getStatus() != null && getPrimarySpdcV4(bhieProblem.getStatus()) != null) {
                    bean.setStatus(getPrimarySpdcV4(bhieProblem.getStatus()).getDisplay());
                }
                if (bhieProblem.getEnteredBy() != null) {
                    bean.setEnteredBy(bhieProblem.getEnteredBy().getName());
                }
                bean.setEnteredDate(bhieToXMLCalV4(bhieProblem.getEnteredDate()));
                if (bhieProblem.getProvider() != null) {
                    bean.setResponsibleProvider(bhieProblem.getProvider().getName());
                }
                if (bhieProblem.getFacility() != null) {
                    bean.setHospitalLocation(bhieProblem.getFacility().getName());
                }
                bean.setId(bhieProblem.getProblemId());
                if (bhieProblem.getNarrativeId() != null) {
                    bean.setNarrativeId(bhieProblem.getNarrativeId());
                }

                String facilityName = "";
                String repositoryName = "";
                String repositoryId = "";
                String recordId = "";

                if (bhieProblem.getFacility() != null && bhieProblem.getFacility().getName() != null) {
                    facilityName = bhieProblem.getFacility().getName();
                }

                if (bhieProblem.getRepository() != null && bhieProblem.getRepository().getName() != null) {
                    repositoryName = bhieProblem.getRepository().getName();
                    if (bhieProblem.getRepository().getIds() != null && bhieProblem.getRepository().getIds().get(0) != null) {
                        repositoryId = bhieProblem.getRepository().getIds().get(0).getId();
                    }
                }

                if (bhieProblem.getRecordId() != null) {
                    recordId = bhieProblem.getRecordId().getId();
                }

                if (problemResultCollection.isVler()) {
                    addBHIESiteDataToVistaBean(repositoryId,
                            "VLER", facilityName, recordId, bean);
                } else {
                    addBHIESiteDataToVistaBean(repositoryId,
                            repositoryName, facilityName, recordId, bean);
                }

                if (bhieProblem.getComment() != null && bhieProblem.getComment().length() > 0) {
                    ArrayList<ProblemNote> problemNotes = new ArrayList<ProblemNote>();
                    ProblemNote problemNote = new ProblemNote();
                    problemNote.setNoteText(bhieProblem.getComment());

                    if (problemResultCollection.isVler()) {
                        addBHIESiteDataToVistaBean(repositoryId,
                                "VLER", facilityName, recordId, problemNote);
                    } else {
                        //SB: Using hospital location instead of facility since facility field comes back empty
                        addBHIESiteDataToVistaBean(repositoryId,
                                repositoryName, facilityName, recordId, problemNote);
                    }
                    problemNotes.add(problemNote);
                    bean.getNotes().addAll(problemNotes);
                }

                vistaBeans.add(bean);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(problemResultCollection.getStatusListv4(), problemResultCollection.getErrorListv4(), problemResultCollection.isVler());
        vistaBeans.addAll(Arrays.asList(
                (Problem[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new Problem[vistaBeans.size()]);
    }

    /*
  public static Problem[] getProblemsFromNHIN(commondatalayerservice.wsdl.nhinc.fha.hhs.gov.ProblemBeanType nhinBean[], String siteId)
  {
      int size = nhinBean.length;
      Problem vistaBean[] = new Problem[size];

      for (int i=0; i < nhinBean.length ; i++)
      {
        Problem bean = new Problem(siteId);
        String acuity = nhinBean[i].getAcuity();
        if (acuity != null )
        {
           if (acuity.equals("A")) { acuity="Acute";}
           if (acuity.equals("C")) { acuity="Chronic";}
        }
        bean.setAcuity(acuity);
        bean.setDescription(nhinBean[i].getLongDescription());
        bean.setIcdCode(nhinBean[i].getIcdCode());
        if (nhinBean[i].getLastModifiedDate() != null)
        {
           bean.setLastModifiedDate(nhinBean[i].getLastModifiedDate().getTime());
        }
        else
        {
          bean.setLastModifiedDate(nhinBean[i].getOnsetDate().getTime());
        }
        bean.setOnsetDate(nhinBean[i].getOnsetDate().getTime());
        bean.setStatus(nhinBean[i].getStatus());
        bean.setId(nhinBean[i].getProblemId());
        vistaBean[i]=bean;

      }
      return vistaBean;
  }
   * */
//    public static ProblemDetail getProblemDetailFromCHCS(ProblemBean[] chcsBean, String siteId, String itemId) {
//
//        ProblemDetail bean = null;
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            if (chcsBean[i].getProblemId().equals(itemId)) {
//                bean = new ProblemDetail();
//                String acuity = chcsBean[i].getAcuity();
//                if (acuity != null && acuity.length() > 0) {
//                    if ("A".equals(acuity)) {
//                        acuity = "Acute";
//                    }
//                    if ("C".equals(acuity)) {
//                        acuity = "Chronic";
//                    }
//                }
//                bean.setAcuity(acuity);
//                bean.setDescription(chcsBean[i].getLongDescription());
//                bean.setIcdCode(chcsBean[i].getIcdCode());
//                if (chcsBean[i].getLastModifiedDate() != null) {
//                    bean.setLastModifiedDate(Utils.getXMLCal(chcsBean[i].getLastModifiedDate()));
//                } else {
//                    bean.setLastModifiedDate(Utils.getXMLCal(chcsBean[i].getOnsetDate()));
//                }
//                bean.setOnsetDate(Utils.getXMLCal(chcsBean[i].getOnsetDate()));
//                bean.setStatus(chcsBean[i].getStatus());
//                bean.setId(chcsBean[i].getProblemId());
//                bean.setEnteredBy(chcsBean[i].getEnteredBy());
//                bean.setEnteredDate(Utils.getXMLCal(chcsBean[i].getEnteredDate()));
//                bean.setHospitalLocation(chcsBean[i].getHospitalLocation());
//
//                addCHCSSiteDataToVistaBean(bean);
//
//                if (chcsBean[i].getComment() != null) {
//                    ProblemNote note = new ProblemNote();
//                    note.setNoteText(chcsBean[i].getComment());
//                    //bean.getNotes().add(note);
//
//                    addCHCSSiteDataToVistaBean(note);
//
//                    ArrayList<ProblemNote> notes = new ArrayList<ProblemNote>();
//                    notes.add(note);
//                    bean.getNotes().addAll(notes);
//                }
//            }
//        }
//        return bean;
//    }


    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Problem prob = new Problem();
        prob.setDescription(errorMsg);
        prob.setLastModifiedDate(Utils.backDate());
        prob.setSite(getConnectionUnavailableSite(location));
        return new Problem[]{prob};
    }
}