/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 * Honorable Senator Daniel K. Inouye
 * VA Pacific Islands Health Care System
 * Tripler Army Medical Center
 */

package gov.va.med.jmeadows.smartbeans.patient;

import gov.va.med.bhie.AmendmentV4;
import gov.va.med.bhie.RadiologyReportV4;
import gov.va.med.bhie.SpdcBeanV4;
import gov.va.med.bhie.V4ResultCollection;
import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.hdd.CodingSystem;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.site.Sites;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.RadiologyReport;
import gov.va.med.vds.webservice.Site;
import gov.va.med.DNS   .common.Utils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class SmartRadiologyExam extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartRadiologyExam.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int EXAM = 3;
    public static final int STATUS = 4;

    public RadiologyReport m_exam;

    public SmartRadiologyExam() {
        super();
        m_exam = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartRadiologyExam(RadiologyReport exam, String sort) {
        super();
        m_exam = exam;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartRadiologyExam) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_exam);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartRadiologyExam[] temp = new SmartRadiologyExam[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartRadiologyExam newAll = new SmartRadiologyExam((RadiologyReport) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartRadiologyExam newAll = new SmartRadiologyExam((RadiologyReport) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            RadiologyReport[] ret = new RadiologyReport[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_exam;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartRadiologyExam[] temp = new SmartRadiologyExam[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartRadiologyExam newAll = new SmartRadiologyExam((RadiologyReport) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            RadiologyReport[] ret = new RadiologyReport[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_exam;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartRadiologyExam tmp = (SmartRadiologyExam) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_exam.getExamDate(), tmp.m_exam.getExamDate());
        }
        if (this.m_sortBy == SITE && this.m_exam.getSite() != null && tmp.m_exam.getSite() != null) {
            retVal = compareString(this.m_exam.getSite().getSiteCode(), tmp.m_exam.getSite().getSiteCode());
        }
        if (this.m_sortBy == EXAM) {
            retVal = compareString(this.m_exam.getStudy(), tmp.m_exam.getStudy());
        }
        if (this.m_sortBy == STATUS) {
            retVal = compareString(this.m_exam.getStatus(), tmp.m_exam.getStatus());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }
        return retVal;

    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("examdate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("Study".equalsIgnoreCase(sort)) {
                m_sortBy = EXAM;
            }
            if ("status".equalsIgnoreCase(sort)) {
                m_sortBy = STATUS;
            }
        }

    }

//    public RadiologyReport[] mapRadiologyExamsFromCHCS(Object[] dodBean) {
//        RadExamBean[] chcsBean = (RadExamBean[]) dodBean;
//        int size = chcsBean.length;
//        RadiologyReport[] vistaBean = new RadiologyReport[size];
//
//        for (int i = 0; i < chcsBean.length; i++) {
//            RadiologyReport bean = new RadiologyReport();
//            bean.setStudy(chcsBean[i].getProcedure());
//            bean.setExamDate(Utils.getXMLCal(chcsBean[i].getExamDateTime()));
//            bean.setStatus(chcsBean[i].getExamStatus());
//            bean.setExamId(chcsBean[i].getExamId());
//            bean.setReasonForOrder(chcsBean[i].getReasonForOrder());
//            bean.setAccessionNumber(chcsBean[i].getExamNumber());
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//
//        }
//        return vistaBean;
//    }

    public RadiologyReport[] mapRadiologyExamFromBHIE(Object resultCollection) {
        V4ResultCollection radiologyResultCollection = (V4ResultCollection) resultCollection;
        List<RadiologyReport> vistaBeans = new ArrayList<RadiologyReport>();


        if (radiologyResultCollection.getDataRecordList() != null
                && radiologyResultCollection.getDataRecordList().getRadiologyReports() != null) {
            StringBuilder textSB = new StringBuilder();
            for (RadiologyReportV4 bhieRad : radiologyResultCollection.getDataRecordList().getRadiologyReports()) {
                if (bhieRad == null) {
                    continue;
                }

                RadiologyReport bean = new RadiologyReport();

                if (bhieRad.getExam() != null && bhieRad.getExam().getType() != null
                        && !bhieRad.getExam().getType().isEmpty()) {

                    if (getPrimarySpdcV4(bhieRad.getExam().getType()) != null) {
                        bean.setStudy(getPrimarySpdcV4(bhieRad.getExam().getType()).getDisplay());
                    }
                    for (SpdcBeanV4 spdc : bhieRad.getExam().getType()) {
                        Code code = new Code();
//                        if (CodingSystem.LOINC.getOID().equalsIgnoreCase(spdc.getSystem())) {
//                            code.setSystem(CodingSystem.LOINC.getDisplayName());
//                        } else
                        if (CodingSystem.DOD_NCID.getOID().equalsIgnoreCase(spdc.getSystem())) {
                            code.setSystem(CodingSystem.DOD_NCID.getDisplayName());
                        }
                        code.setCode(spdc.getCode());
                        bean.getCodes().add(code);
                    }
                    bean.setExamDate(bhieToXMLCalV4(bhieRad.getExam().getDate()));
                    if (bhieRad.getExam().getExamNumber() != null) {
                        bean.setExamId(bhieRad.getExam().getExamNumber().getId());
                    }
                }
                if (bhieRad.getStatus() != null && !bhieRad.getStatus().isEmpty()
                        && getPrimarySpdcV4(bhieRad.getStatus()) != null) {
                    bean.setStatus(getPrimarySpdcV4(bhieRad.getStatus()).getDisplay());
                    bean.setReportStatus(getPrimarySpdcV4(bhieRad.getStatus()).getDisplay());
                }

                bean.setReasonForOrder(bhieRad.getReason());

                if (bhieRad.getOrder() != null && bhieRad.getOrder().getAccession() != null) {
                    bean.setAccessionNumber(bhieRad.getOrder().getAccession().getId());

                }

                if (bhieRad.getOrder() != null && bhieRad.getOrder().getOrderNumber() != null) {
                    bean.setOrderNumber(bhieRad.getOrder().getOrderNumber().getId());
                }

                String facilityName = "";
                String repositoryName = "";
                String repositoryId = "";
                String recordId = "";

                if (bhieRad.getFacility() != null && bhieRad.getFacility().getName() != null) {
                    facilityName = bhieRad.getFacility().getName();
                }

                if (bhieRad.getRepository() != null && bhieRad.getRepository().getName() != null) {
                    repositoryName = bhieRad.getRepository().getName();
                    if (bhieRad.getRepository().getIds() != null && bhieRad.getRepository().getIds().get(0) != null) {
                        repositoryId = bhieRad.getRepository().getIds().get(0).getId();
                    }
                }

                if (bhieRad.getRecordId() != null) {
                    recordId = bhieRad.getRecordId().getId();
                }

                addBHIESiteDataToVistaBean(repositoryId,
                        repositoryName, facilityName, recordId, bean);

                //using DMIS ID, include endpoints from Site DB. This is needed for medweb support.
//                String facilityName = bhieRad.getFacility();
                Site site = null;
                if (facilityName != null && facilityName.contains("/")) {
                    int startIdx = facilityName.indexOf('/') + 1;
                    if (startIdx < facilityName.length()) {
                        String dmisId = facilityName.substring(startIdx);
                        site = Sites.getInstance().getSiteByDmisID(dmisId);

                        if (site != null) {
                            bean.getSite().getEndpoints().addAll(site.getEndpoints());
                            bean.getSite().setSiteCode(site.getSiteCode());
                            bean.getSite().setDmisId(site.getDmisId());
                        }
                    }
                }
                if (bhieRad.getApprovedBy() != null) {
                    bean.setApprovedBy(bhieRad.getApprovedBy().getName());
                }

                bean.setImpressionText("");
                if (bhieRad.getInterpretedBy() != null) {
                    bean.setInterpretingHCP(bhieRad.getInterpretedBy().getName());
                }
                if (bhieRad.getPriority() != null) {
                    bean.setPriority(bhieRad.getPriority());
                }

                if (bhieRad.getResult() != null && !bhieRad.getResult().isEmpty()
                        && getPrimarySpdcV4(bhieRad.getResult()) != null) {
                    bean.setResultCode(getPrimarySpdcV4(bhieRad.getResult()).getCode());
                    bean.setAbnormal(getPrimarySpdcV4(bhieRad.getResult()).getDisplay());
                }

                bean.setTranscribeDate(bhieToXMLCalV4(bhieRad.getTranscriptionDate()));
                textSB.setLength(0);
                textSB.append(bhieRad.getContent());

                if (bhieRad.getAmendments() != null && bhieRad.getAmendments().size() > 0) {
                    int lastIndex = bhieRad.getAmendments().size() - 1;
                    AmendmentV4 lastAmendment = bhieRad.getAmendments().get(lastIndex);
                    if (bean.getStatus() != null && !bean.getStatus().toLowerCase().contains("amended")) {
                        StringBuilder statusText = new StringBuilder();
                        statusText.append(bean.getStatus());
                        statusText.append(" (Amended)");
                        bean.setStatus(statusText.toString());
                        bean.setReportStatus(statusText.toString());
                    }

                    bean.setAmendedDate(bhieToXMLCalV4(lastAmendment.getVerifiedDate()));
                    if (lastAmendment.getResult() != null && !lastAmendment.getResult().isEmpty()
                            && getPrimarySpdcV4(lastAmendment.getResult()) != null) {
                        bean.setResultCode(getPrimarySpdcV4(lastAmendment.getResult()).getCode());
                        bean.setAbnormal(getPrimarySpdcV4(lastAmendment.getResult()).getDisplay());
                    }
                }
                bean.setReportText(textSB.toString());

                vistaBeans.add(bean);
            }
        }

        //include site status report
        String siteStatusReport = generateBhieStatusReportV4(radiologyResultCollection.getStatusListv4(),
                radiologyResultCollection.getErrorListv4(), radiologyResultCollection.isVler());

        vistaBeans.addAll(Arrays.asList(
                (RadiologyReport[]) generateConnectionUnavailableBean(
                        BHIE_STATUS_REPORT_LOCATION,
                        siteStatusReport)
        ));

        return vistaBeans.toArray(new RadiologyReport[vistaBeans.size()]);
    }

//    public static RadiologyReport getRadReportFromCHCS(RadiologyResultBundleBean coll, String siteId) {
//
//        RadExamBean chcsExam = coll.getExam();
//        RadiologyResultBean chcsResult = (RadiologyResultBean) coll.getResult();
//
//        RadiologyReport vistaBean = new RadiologyReport();
//        vistaBean.setApprovedBy(chcsResult.getApprovedBy());
//        vistaBean.setImpressionText("");
//        vistaBean.setInterpretingHCP(chcsResult.getInterpretedBy());
//        vistaBean.setPriority(chcsExam.getPriority());
//        vistaBean.setExamDate(Utils.getXMLCal(chcsExam.getExamDateTime()));
//        vistaBean.setExamId(chcsExam.getExamNumber());
//        vistaBean.setReasonForOrder(chcsExam.getReasonForOrder());
//        vistaBean.setReportStatus(chcsResult.getReportStatus());
//        vistaBean.setResultCode(chcsResult.getResultCode());
//        vistaBean.setStudy(chcsExam.getProcedure());
//        vistaBean.setTranscribeDate(Utils.getXMLCal(chcsResult.getTranscriptionDate()));
//        vistaBean.setAccessionNumber(chcsExam.getExamNumber());
//        addCHCSSiteDataToVistaBean(vistaBean);
//        StringBuilder textSB = new StringBuilder(chcsResult.getResultText());
//
//        if (chcsResult.isHasAmendments()) {
//            int size = chcsResult.getAmendments().size();
//
//            RadAmendmentBean[] amends = null;
//            amends = chcsResult.getAmendments().toArray(new RadAmendmentBean[0]);
//
//            for (int i = 0; i < size; i++) {
//                RadAmendmentBean amend = (RadAmendmentBean) amends[i];
//                textSB.append("\n<b>Amendment</b>");
//                Date aDate = Utils.getDate(amend.getDateTime());
//                String iHcp = amend.getInterpretingProvider();
//                String rpt = amend.getReportText();
//                String code = amend.getResultCode();
//                textSB.append("\n<b>Amendment Date/Time: </b>");
//                textSB.append(Utils.formatDisplayDate(aDate));
//                textSB.append("\n<b>Interpreting User: </b>");
//                textSB.append(iHcp);
//                textSB.append("\n<b>Result Code: </b>");
//                textSB.append(code);
//                textSB.append("\n<b>Amended Report: </b>\n");
//                textSB.append(rpt);
//            }
//        }
//        vistaBean.setReportText(textSB.toString());
//
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        RadiologyReport radrpt = new RadiologyReport();
        radrpt.setStudy(errorMsg);
        radrpt.setExamDate(Utils.backDate());
        radrpt.setSite(getConnectionUnavailableSite(location));

        return new RadiologyReport[]{radrpt};
    }

    public RadiologyReport[] generateRadReportConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        RadiologyReport radrpt = new RadiologyReport();
        radrpt.setStudy(errorMsg);
        radrpt.setExamDate(Utils.backDate());
        radrpt.setSite(getConnectionUnavailableSite(location));

        return new RadiologyReport[]{radrpt};
    }
}