/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.provider;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.smartbeans.patient.SmartBean;
import gov.va.med.vds.webservice.PatientAppointments;
import gov.va.med.DNS   .common.Utils;
//import org.hui.chcs.ProviderAppointmentBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class SmartAppointment extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartAppointment.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;
    public static final int LOC = 4;
    public static final int HCP = 5;
    public static final int STAT = 6;

    public PatientAppointments m_appointment;

    public SmartAppointment() {
        super();
        m_appointment = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartAppointment(PatientAppointments appt, String sort) {
        super();
        m_appointment = appt;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartAppointment) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_appointment);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartAppointment[] temp = new SmartAppointment[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAppointment newAll = new SmartAppointment((PatientAppointments) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartAppointment newAll = new SmartAppointment((PatientAppointments) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientAppointments[] ret = new PatientAppointments[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_appointment;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartAppointment[] temp = new SmartAppointment[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartAppointment newAll = new SmartAppointment((PatientAppointments) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            PatientAppointments[] ret = new PatientAppointments[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_appointment;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartAppointment tmp = (SmartAppointment) obj;
        int retVal = 0;

        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_appointment.getApptDate(), tmp.m_appointment.getApptDate());
        }
        if (this.m_sortBy == SITE && this.m_appointment.getSite() != null && tmp.m_appointment.getSite() != null) {
            retVal = compareString(this.m_appointment.getSite().getSiteCode(), tmp.m_appointment.getSite().getSiteCode());
        }
        if (this.m_sortBy == TYPE) {
            retVal = compareString(this.m_appointment.getApptType(), tmp.m_appointment.getApptType());
        }
        if (this.m_sortBy == LOC) {
            retVal = compareString(this.m_appointment.getClinic(), tmp.m_appointment.getClinic());
        }
        if (this.m_sortBy == HCP) {
            retVal = compareString(this.m_appointment.getProvider().getName(), tmp.m_appointment.getProvider().getName());
        }
        if (this.m_sortBy == STAT) {
            retVal = compareString(this.m_appointment.getStatus(), tmp.m_appointment.getStatus());
        }

        if (!this.m_bAscending) {
            retVal = -retVal;
        }
        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("apptDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("apptType".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
            if ("clinic".equalsIgnoreCase(sort)) {
                m_sortBy = LOC;
            }
            if ("providerName".equalsIgnoreCase(sort)) {
                m_sortBy = HCP;
            }
            if ("status".equalsIgnoreCase(sort)) {
                m_sortBy = STAT;
            }
        }
    }

//    public PatientAppointments[] mapPatientAppointmentsFromCHCS(Object[] dodBean) {
//        int size = dodBean.length;
//        PatientAppointments[] vistaBean = new PatientAppointments[size];
//        ProviderAppointmentBean[] chcsBeans = (ProviderAppointmentBean[]) dodBean;
//
//        for (int i = 0; i < size; i++) {
//            PatientAppointments bean = new PatientAppointments();
//            bean.setApptDate(chcsBeans[i].getApptDateTime());
//            bean.setApptId(chcsBeans[i].getEncounterId());
//            bean.setApptType(chcsBeans[i].getApptType());
//            bean.setClinic(chcsBeans[i].getClinicName());
//            Provider provider = new Provider();
//            provider.setName(chcsBeans[i].getProviderName());
//            bean.setProvider(provider);
//            bean.setReason(chcsBeans[i].getApptReason());
//            bean.setStatus(chcsBeans[i].getApptStatus());
//            bean.setPatientId(chcsBeans[i].getPatientId());
//            bean.setPatientName(chcsBeans[i].getPatientName());
//            bean.setProviderClass("");
//            bean.setUserIen("");
//            bean.setClinicId("");
//            bean.setProviderPager("");
//            bean.setProviderPhone("");
//            addCHCSSiteDataToVistaBean(bean);
//            vistaBean[i] = bean;
//        }
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        PatientAppointments apt = new PatientAppointments();
        apt.setClinic(errorMsg);
        apt.setApptDate(Utils.backDate());
        apt.setSite(getConnectionUnavailableSite(location));

        return new PatientAppointments[]{apt};
    }
}