/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.provider;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.smartbeans.patient.SmartBean;
import gov.va.med.vds.webservice.LabResult;
import gov.va.med.DNS   .common.Utils;
//import org.hui.chcs.LabResultBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class SmartLabResult extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartLabResult.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int NAME = 3;

    public LabResult m_result;

    public SmartLabResult() {
        super();
        m_result = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartLabResult(LabResult result, String sort) {
        super();
        m_result = result;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartLabResult) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_result);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartLabResult[] temp = new SmartLabResult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartLabResult newAll = new SmartLabResult((LabResult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartLabResult newAll = new SmartLabResult((LabResult) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            LabResult[] ret = new LabResult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_result;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartLabResult[] temp = new SmartLabResult[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartLabResult newAll = new SmartLabResult((LabResult) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            LabResult[] ret = new LabResult[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_result;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        SmartLabResult tmp = (SmartLabResult) obj;
        int retVal = 0;
        if (this.m_sortBy == DATE) {
            retVal = compareDate(this.m_result.getResultDate(), tmp.m_result.getResultDate());
        }
        if (this.m_sortBy == SITE && this.m_result.getSite() != null && tmp.m_result.getSite() != null) {
            retVal = compareString(this.m_result.getSite().getSiteCode(), tmp.m_result.getSite().getSiteCode());
        }
        if (this.m_sortBy == NAME) {
            retVal = compareString(this.m_result.getTestName(), tmp.m_result.getTestName());
        }
        if (!this.m_bAscending) {
            retVal = -retVal;
        }

        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;

        if (sort != null) {
            sort = setAscending(sort);
            if ("resultDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("description".equalsIgnoreCase(sort)) {
                m_sortBy = NAME;
            }
        }
    }

//    public LabResult[] mapLabResultsFromCHCS(Object[] dodBean) {
//        LabResultBean[] lrb = (LabResultBean[]) dodBean;
//        LabResult[] labResults = new LabResult[lrb.length];
//
//        for (int i = 0; i < lrb.length; i++) {
//            LabResult lresult = new LabResult();
//            lresult.setOrderId(lrb[i].getOrderId());
//            lresult.setPatientId(lrb[i].getPatientId());
//            lresult.setPatientName(lrb[i].getPatientName());
//            lresult.setResultDate(Utils.getXMLCal(lrb[i].getResultedDate()));
//            lresult.setTestName(lrb[i].getDescription());
//            lresult.setResult(lrb[i].getResultId());
//            addCHCSSiteDataToVistaBean(lresult);
//            labResults[i] = lresult;
//        }
//
//        return labResults;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        LabResult lresult = new LabResult();
        lresult.setTestName(errorMsg);
        lresult.setResultDate(Utils.backDate());
        lresult.setSite(getConnectionUnavailableSite(location));

        return new LabResult[]{lresult};
    }
}