/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.smartbeans.provider;

import gov.va.med.foundations.utilities.HashCodeUtil;
import gov.va.med.jmeadows.dao.util.Location;
import gov.va.med.jmeadows.smartbeans.patient.SmartBean;
import gov.va.med.vds.webservice.Order;
import gov.va.med.DNS   .common.Utils;
//import org.hui.chcs.OrderBean;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.util.Arrays;

public class SmartOrder extends SmartBean implements Comparable {

//    private static final Logger LOGGER = LoggerFactory.getLogger(SmartOrder.class);

    public static final int DATE = 1;
    public static final int SITE = 2;
    public static final int TYPE = 3;
    public static final int STATUS = 4;

    public Order m_order;

    public SmartOrder() {
        super();
        m_order = null;
        m_sortBy = 1;
        m_bAscending = false;
    }

    public SmartOrder(Order order, String sort) {
        super();
        m_order = order;
        getSortValue(sort);
    }

    @Override
    public boolean equals(Object obj) {
        boolean rtc;
        if (this == obj) {
            rtc = true;
        } else if (obj instanceof SmartOrder) {
            rtc = compareTo(obj) == 0;
        } else {
            rtc = false;
        }
        return rtc;
    }

    @Override
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, m_order);
        result = HashCodeUtil.hash(result, m_bAscending);
        result = HashCodeUtil.hash(result, m_sortBy);

        return result;
    }

    @Override
    public Object[] combineData(Object[] array1, Object[] array2, String sort) {
        Object[] rtc;
        if (array2 == null) {
            rtc = sort(array1, sort);
        } else if (array1 == null) {
            rtc = sort(array2, sort);
        } else {
            int size = array1.length + array2.length;
            SmartOrder[] temp = new SmartOrder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartOrder newAll = new SmartOrder((Order) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            for (int i = 0; i < array2.length; i++) {
                SmartOrder newAll = new SmartOrder((Order) array2[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Order[] ret = new Order[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_order;
            }
            rtc = ret;
        }
        return rtc;
    }

    public Object[] sort(Object[] array1, String sort) {
        Object[] rtc;
        if (array1 == null) {
            rtc = null;
        } else {
            int size = array1.length;
            SmartOrder[] temp = new SmartOrder[size];
            int cnt = 0;
            for (int i = 0; i < array1.length; i++) {
                SmartOrder newAll = new SmartOrder((Order) array1[i], sort);
                temp[cnt] = newAll;
                cnt++;
            }
            Arrays.sort(temp);
            Order[] ret = new Order[temp.length];
            for (int i = 0; i < temp.length; i++) {
                ret[i] = temp[i].m_order;
            }
            rtc = ret;
        }
        return rtc;
    }

    public int compareTo(Object obj) {
        int retVal = 0;
        if (obj != null) {
            SmartOrder tmp = (SmartOrder) obj;
            if (this.m_sortBy == DATE) {
                retVal = compareDate(this.m_order.getOrderDate(), tmp.m_order.getOrderDate());
            }
            if (this.m_sortBy == SITE && this.m_order.getSite() != null && tmp.m_order.getSite() != null) {
                retVal = compareString(this.m_order.getSite().getSiteCode(), tmp.m_order.getSite().getSiteCode());
            }
            if (this.m_sortBy == TYPE) {
                retVal = compareString(this.m_order.getType(), tmp.m_order.getType());
            }
            if (this.m_sortBy == STATUS) {
                retVal = compareString(this.m_order.getStatus(), tmp.m_order.getStatus());
            }

            if (!this.m_bAscending) {
                retVal = -retVal;
            }
        }
        return retVal;
    }

    private void getSortValue(String insort) {
        String sort = insort;
        m_sortBy = DATE;
        m_bAscending = false;
        if (sort != null) {
            sort = setAscending(sort);
            if ("OrderDate".equalsIgnoreCase(sort)) {
                m_sortBy = DATE;
            }
            if ("siteCode".equalsIgnoreCase(sort)) {
                m_sortBy = SITE;
            }
            if ("type".equalsIgnoreCase(sort)) {
                m_sortBy = TYPE;
            }
            if ("status".equalsIgnoreCase(sort)) {
                m_sortBy = STATUS;
            }
        }

    }

//    public Order[] mapOrdersFromCHCS(Object[] dodBean) {
//        OrderBean[] orderBeans = (OrderBean[]) dodBean;
//        Order[] vistaBean = new Order[dodBean.length];
//
//        for (int i = 0; i < dodBean.length; i++) {
//            Order bean = new Order();
//            bean.setOrderid(orderBeans[i].getOrderid());
//            bean.setPatientId(orderBeans[i].getPatientId());
//            bean.setPatientName(orderBeans[i].getPatientName());
//            bean.setOrderDate(orderBeans[i].getTaskDateTime());
//            bean.setEnteredDate(orderBeans[i].getOrderDate());
//            bean.setCompletedDate(orderBeans[i].getCompletedDate());
//            bean.setDescription(orderBeans[i].getDescription());
//            bean.setType(orderBeans[i].getType());
//            bean.setStatus(orderBeans[i].getStatus());
//            bean.setOrderResult(orderBeans[i].getResultId());
//            addCHCSSiteDataToVistaBean(bean);
//
//            vistaBean[i] = bean;
//        }
//
//        return vistaBean;
//    }

    @Override
    public Object[] generateConnectionUnavailableBean(Location location, String inerrorMsg) {
        String errorMsg = inerrorMsg;
        if (errorMsg == null) {
            errorMsg = ERROR_MSG_CONNECTION_UNAVAILABLE;
        }

        Order order = new Order();

        order.setDescription(errorMsg);
        order.setOrderDate(Utils.backDate());
        order.setSite(getConnectionUnavailableSite(location));

        return new Order[]{order};
    }

}
