package dodadapter;

import gov.va.med.foundations.utilities.EncryptUtil;
import gov.va.med.foundations.utilities.SSLDebugUtilities;
import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.dataservicehandler.BHIERelayServiceHandler;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.vds.webservice.Patient;
import gov.va.med.vds.webservice.Site;
import gov.va.med.vds.webservice.User;
import gov.va.med.DNS   .common.Utils;
import org.junit.Before;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

public class BaseTest
{
    protected User user;
    protected Patient patient;

    protected Calendar startDate;
    protected Calendar endDate;

    protected static final AppConfig appConfig = AppConfig.getInstance();

    protected BHIERelayServiceHandler bhieRelayServiceHandler;

    protected List<String> excludedSources;
    protected List<String> includedSources;

    @Before
    public void setup()
    {
        startDate = Calendar.getInstance();
        endDate = Calendar.getInstance();

        startDate.set(2011, 0, 1);

        user = new User();
        patient = new Patient();

        user.setName("");
        user.setUserIen("56435");


        user.setEncryptedFederatedUid(EncryptUtil.encrypt("000003000"));
        user.setAgency("VA");

        Site hostSite = new Site();
        hostSite.setAgency("VA");
        hostSite.setMoniker("HON");
        hostSite.setSiteCode("459");
        hostSite.setName("Honolulu, HI");

        user.setHostSite(hostSite);

        patient.setEDIPI("0000000002");
        patient.setName("BCMA,EIGHT");

        bhieRelayServiceHandler = new BHIERelayServiceHandler(appConfig.getBHIERelayServiceURL());

        excludedSources = new ArrayList<String>();
        excludedSources.add(appConfig.getBHIECustodianOidVA());
        includedSources = new ArrayList<String>();
        includedSources.add(appConfig.getBHIECustodianOidAHLTA());

        SSLDebugUtilities.trustAllHostnames();
        SSLDebugUtilities.trustAllHttpsCertificates();

    }

    protected JMeadowsQuery setupQuery(User user, Patient patient, Calendar startDate, Calendar endDate)
    {
        JMeadowsQuery qb = new JMeadowsQuery();
        qb.setUser(user);
        qb.setPatient(patient);
        if (startDate != null) qb.setStartDate(Utils.getCalendar(startDate.getTime()));
        if (endDate != null) qb.setEndDate(Utils.getCalendar(endDate.getTime()));
        qb.setMax(999);
        qb.setActive(0);
        return qb;
    }

    protected gov.va.med.bhie.QueryBean toBHIEQueryBean(JMeadowsQuery qb,
                                                        List<String> includedSources, List<String> excludedSources) {

        gov.va.med.bhie.QueryBean bhieQueryBean = new gov.va.med.bhie.QueryBean();

        String userIen = qb.getUser().getUserIen();

        String patientId = "EDIPI:" + qb.getPatient().getEDIPI();

        //String patientId = "DOD1:HA.OSD.MIL/A0101:" + qb.getPatient().getEDIPI();

        bhieQueryBean.setUserId(userIen);
        bhieQueryBean.setPatientId(patientId);

        if (qb.getStartDate() != null) {
            bhieQueryBean.setStartDate(Utils.toXMLGregorianCalendar(qb.getStartDate().getTime()));
        }

        if (qb.getEndDate() != null) {
            bhieQueryBean.setEndDate(Utils.toXMLGregorianCalendar(qb.getEndDate().getTime()));
        }

        if (includedSources != null) {
            bhieQueryBean.getIncludedSources().addAll(includedSources);
        }
        if (excludedSources != null) bhieQueryBean.getExcludedSources().addAll(excludedSources);

        return bhieQueryBean;
    }

}
