package dodadapter;

import gov.va.med.bhie.*;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.PatientEncounter;
import gov.va.med.jmeadows.smartbeans.patient.*;
import gov.va.med.vds.webservice.*;
import gov.va.med.vds.webservice.Encounter;
import gov.va.med.vds.webservice.Medication;
import gov.va.med.vds.webservice.PatientDemographics;
import gov.va.med.vds.webservice.Problem;
import gov.va.med.vds.webservice.Vitals;
import org.apache.commons.lang3.StringUtils;
import org.junit.Test;

import static junit.framework.Assert.assertTrue;

public class DODAdapterTest extends BaseTest {

    //complete
    @Test
    public void testAppointments() throws JMeadowsException {


        V4ResultCollection patientAppointmentsRC =
                bhieRelayServiceHandler.getPatientAppointments(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));

        SmartAppointment smartAppointment = new SmartAppointment();
        PatientAppointments[] patientAppointments = smartAppointment.mapAppointmentsFromBHIE(patientAppointmentsRC);

        assertTrue(patientAppointments.length > 0);

        for(PatientAppointments appt : patientAppointments)
        {
            assertTrue(appt.getApptDate() != null);


            Site site = appt.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }
    //complete
    @Test
    public void testEncounters() throws JMeadowsException {
        V4ResultCollection patientEncountersRC = bhieRelayServiceHandler.getPatientEncounters(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
        SmartEncounter smartEncounter = new SmartEncounter();
        Encounter[] patientEncounters = smartEncounter.mapEncountersFromBHIE(patientEncountersRC);

        assertTrue(patientEncounters.length > 0);

        for(Encounter e : patientEncounters)
        {
            assertTrue(e.getEnteredByDate() != null);


            Site site = e.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }

    //complete
    @Test
    public void testPatientDemographics() throws JMeadowsException
    {
        V4ResultCollection patientDemographicsResultCollection =
                bhieRelayServiceHandler.getPatientDemographics(toBHIEQueryBean(setupQuery(user, patient, null, null), null, excludedSources));
        SmartDemographics smartDemographics = new SmartDemographics();
        PatientDemographics[] patientDemographics = smartDemographics.mapPatientDemographicsFromBHIE(patientDemographicsResultCollection);

        assertTrue(patientDemographics.length > 0);
        assertTrue(patientDemographics[0].getSite() != null);

    }

    //complete
    @Test
    public void testPatientDemographicDetails() throws JMeadowsException
    {
        V4ResultCollection patientDemographicsResultCollection =
                bhieRelayServiceHandler.getPatientDemographics(toBHIEQueryBean(setupQuery(user, patient, null, null), null, excludedSources));
        SmartDemographicsDetail smartDemographicDetail = new SmartDemographicsDetail();
        PatientDemographicsDetail[] patientDemographicsDetails = smartDemographicDetail.mapPatientDemographicsDetailsFromBHIE(patientDemographicsResultCollection);

        assertTrue(patientDemographicsDetails.length > 0);
        assertTrue(patientDemographicsDetails[0].getSite() != null);

    }

    //complete
    @Test
    public void testDischargeSummaries() throws JMeadowsException
    {
        DischargeNoteResultCollection resultCollection =
                bhieRelayServiceHandler.getPatientDischargeSummaries(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));

        SmartProgressNote smartProgressNote = new SmartProgressNote();
        ProgressNote[] dischargeSummaries = smartProgressNote.getDischargeNoteFromBHIE(resultCollection);

        assertTrue(dischargeSummaries.length > 0);

        for(ProgressNote note : dischargeSummaries)
        {
            assertTrue(note.getNoteDate() != null);

            Site site = note.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }

    }

    @Test
    public void testPatientConsults() throws JMeadowsException
    {
        V4ResultCollection patientConsults = bhieRelayServiceHandler.getPatientConsults(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null,  excludedSources));
        SmartConsult smartConsult = new SmartConsult();
        Consult[] consults = smartConsult.mapConsultsFromBHIE(patientConsults);

        assertTrue(consults.length > 0);
        for(Consult c : consults)
        {
            assertTrue(c.getRequestDate() != null);

            Site site = c.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }
    //complete
    @Test
    public void testPatientOutpatientMeds() throws JMeadowsException
    {
        V4ResultCollection resultCollection = bhieRelayServiceHandler.getPatientMedications(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
        SmartMedication smartMedication = new SmartMedication();
        Medication[] outPatMeds = smartMedication.getOutpatientMedicationsFromBHIE(resultCollection);

        assertTrue(outPatMeds.length > 0);

        for(Medication med : outPatMeds)
        {
            assertTrue(med.getLastFilledDate() != null);
            assertTrue(med.getPrescriptionFills().size() > 0);

            Site site = med.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }
    //complete
    @Test
    public void testPatientInpatientMeds() throws JMeadowsException
    {
        V4ResultCollection resultCollection = bhieRelayServiceHandler.getPatientMedications(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
        SmartMedication smartMedication = new SmartMedication();
        Medication[] inpatMeds = smartMedication.getInpatientMedicationsFromBHIE(resultCollection);

        assertTrue(inpatMeds.length > 0);

        for(Medication med : inpatMeds)
        {
            assertTrue(med.getLastFilledDate() != null);
            assertTrue(med.getPrescriptionFills().size() > 0);

            Site site = med.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }

    //complete
    @Test
    public void testProblemsList() throws JMeadowsException
    {
        V4ResultCollection resultCollection = bhieRelayServiceHandler.getPatientProblemList(toBHIEQueryBean(setupQuery(user, patient, null, null), null, excludedSources));
        SmartProblem smartProblem = new SmartProblem();
        Problem[] problems = smartProblem.mapProblemListFromBHIE(resultCollection);

        assertTrue(problems.length > 0);

        for(Problem problem : problems)
        {
            assertTrue(problem.getLastModifiedDate() != null);
            assertTrue(problem.getOnsetDate() != null);

            Site site = problem.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }

    }

    //complete
    @Test
    public void testProgressNotes() throws JMeadowsException {

        V4ResultCollection noteResultCollection = bhieRelayServiceHandler.getPatientNotes(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));

        SmartProgressNote smartProgressNote = new SmartProgressNote();
        ProgressNote[] progressNotes = smartProgressNote.getProgressNotesFromBHIE(noteResultCollection);
        assertTrue(progressNotes.length > 0);

        for(ProgressNote note : progressNotes)
        {
            assertTrue(note.getNoteDate() != null);

            Site site = note.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }

    //complete
    @Test
    public void testVitals() throws JMeadowsException
    {
        VitalsResultCollection vitalsResultCollection = bhieRelayServiceHandler.getPatientVitals(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
        SmartVitals smartVitals = new SmartVitals();
        Vitals[] vitals = smartVitals.mapVitalsFromBHIE(vitalsResultCollection);
        assertTrue(vitals.length > 0);

        for(Vitals vital : vitals)
        {
            assertTrue(vital.getDateTimeTaken() != null);

            Site site = vital.getSite();
            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
            assertTrue(StringUtils.isNotBlank(site.getName()));
        }
    }

    @Test
    public void testRadiology() throws JMeadowsException
    {
//        RadiologyResultCollection resultCollection = bhieRelayServiceHandler.getPatientRadiologyReports(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
//        SmartRadiologyExam smartRadiologyExam = new SmartRadiologyExam();
//        RadiologyReport[] radiologyReports = smartRadiologyExam.mapRadiologyExamFromBHIE(resultCollection);
//
//        assertTrue(radiologyReports.length > 0);
//
//        for(RadiologyReport rad : radiologyReports)
//        {
//            assertTrue(rad.getExamDate() != null);
//
//            Site site = rad.getSite();
//            assertTrue(StringUtils.isNotBlank(site.getSiteCode()));
//            assertTrue(StringUtils.isNotBlank(site.getMoniker()));
//            assertTrue(StringUtils.isNotBlank(site.getName()));
//        }

    }

    @Test
    public void testLabs() throws JMeadowsException
    {
//        LabChemistryResultCollection labChemRC = bhieRelayServiceHandler.getPatientLabChemistries(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
//        LabMicrobiologyResultCollection labMicroRC = bhieRelayServiceHandler.getPatientLabMicrobiologies(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
//        LabAnatomicPathologyResultCollection labAnaPathRC = bhieRelayServiceHandler.getPatientLabAnatomicPathologies(toBHIEQueryBean(setupQuery(user, patient, startDate, endDate), null, excludedSources));
//
//        SmartLabOrder smartLabOrder = new SmartLabOrder();
//        LabOrder[] chemLabs = smartLabOrder.mapChemistryLabsFromBHIE(labChemRC);
//        LabOrder[] microLabs = smartLabOrder.mapMicrobiologyLabsFromBHIE(labMicroRC);
//        LabOrder[] anaPathLabs = smartLabOrder.mapAnatomicPathologyLabsFromBHIE(labAnaPathRC);

//        assertTrue(chemLabs.length > 0);
//        assertTrue(microLabs.length > 0);
//        assertTrue(anaPathLabs.length > 0);



    }

}
