package gov.va.med.jmeadows.dao.hdd;


import com.mchange.v2.c3p0.ComboPooledDataSource;
import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vds.webservice.Code;
import gov.va.med.vds.webservice.DataBean;
import org.junit.Before;
import org.junit.Test;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class ConceptCodeMapperIntegrationTest
{
    private HDDDao hddDao;

    @Before
    public void setUp()
    {
        DataSource dataSource;

        if (AppConfig.getInstance().useC3PO()) {
            //setup c3po db connection pools
            try {
                ComboPooledDataSource c3poSQLConnectionPool = new ComboPooledDataSource();
                c3poSQLConnectionPool.setDriverClass(AppConfig.getInstance().getSQLDriver());
                c3poSQLConnectionPool.setJdbcUrl(AppConfig.getInstance().getSQLURL());
                dataSource = c3poSQLConnectionPool;
            } catch (PropertyVetoException ex) {
                throw new RuntimeException(ex);
            }
        } else {
            try {

                dataSource = InitialContext.doLookup(AppConfig.getInstance().getJDBC_JNDILookup());
            } catch (NamingException ex) {
                throw new RuntimeException(ex);
            }
        }

        hddDao = new HDDDao(dataSource);
    }

    @Test
    public void testConceptCodeMapper()
    {
        ConceptCodeMapper conceptCodeMapper = ConceptCodeMapper.getInstance();

        List<DataBean> dataBeanList = new ArrayList<DataBean>();

        List<Code> codes = new ArrayList<Code>();

        Code codeAA = new Code();
        codeAA.setCode("AA");
        codeAA.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAA.setDisplay("Aortic aneurysm");
        codes.add(codeAA);

        Code codeAMI = new Code();
        codeAMI.setCode("AMI");
        codeAMI.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAMI.setDisplay("Myocardial Infarction");
        codes.add(codeAMI);

        Code codeCHF = new Code();
        codeCHF.setCode("CHF");
        codeCHF.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeCHF.setDisplay("Congestive heart failure");
        codes.add(codeCHF);

        for(Code code : codes)
        {
            DataBean dataBean = new DataBean();
            dataBean.getCodes().add(code);
            dataBeanList.add(dataBean);
        }

        try {

            conceptCodeMapper.mapConceptCodes(hddDao, dataBeanList, CodingSystem.ICD9_CM_DISEASES);

            assertTrue(dataBeanList.size() == 3);

            DataBean aaBean = dataBeanList.get(0);
            assertTrue(aaBean.getCodes().size() == 2);
            assertTrue(aaBean.getCodes().get(0).getCode().equals(codeAA.getCode()));
            assertTrue(aaBean.getCodes().get(1).getCode().equals("441.9"));
            assertTrue(aaBean.getCodes().get(1).getSystem().equals(CodingSystem.ICD9_CM_DISEASES.getDisplayName()));

            DataBean amiBean = dataBeanList.get(1);
            assertTrue(amiBean.getCodes().size() == 2);
            assertTrue(amiBean.getCodes().get(0).getCode().equals(codeAMI.getCode()));
            assertTrue(amiBean.getCodes().get(1).getCode().equals("410.90"));
            assertTrue(amiBean.getCodes().get(1).getSystem().equals(CodingSystem.ICD9_CM_DISEASES.getDisplayName()));

            DataBean chfBean = dataBeanList.get(2);
            assertTrue(chfBean.getCodes().size() == 2);
            assertTrue(chfBean.getCodes().get(0).getCode().equals(codeCHF.getCode()));
            assertTrue(chfBean.getCodes().get(1).getCode().equals("428.0"));
            assertTrue(chfBean.getCodes().get(1).getSystem().equals(CodingSystem.ICD9_CM_DISEASES.getDisplayName()));

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testConceptCodeMapperFailure()
    {
        ConceptCodeMapper conceptCodeMapper = ConceptCodeMapper.getInstance();

        List<DataBean> dataBeanList = new ArrayList<DataBean>();

        List<Code> codes = new ArrayList<Code>();

        Code codeAA = new Code();
        codeAA.setCode("AA");
        codeAA.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAA.setDisplay("Aortic aneurysm");
        codes.add(codeAA);

        Code codeAMI = new Code();
        codeAMI.setCode("AMI");
        codeAMI.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAMI.setDisplay("Myocardial Infarction");
        codes.add(codeAMI);

        //add unmapped code
        Code codeUNK = new Code();
        codeUNK.setCode("UNK");
        codeUNK.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeUNK.setDisplay("UNKNOWN");
        codes.add(codeUNK);

        for(Code code : codes)
        {
            DataBean dataBean = new DataBean();
            dataBean.getCodes().add(code);
            dataBeanList.add(dataBean);
        }

        try {

            conceptCodeMapper.mapConceptCodes(hddDao, dataBeanList, CodingSystem.ICD9_CM_DISEASES);

            assertTrue(dataBeanList.size() == 3);

            DataBean aaBean = dataBeanList.get(0);
            assertTrue(aaBean.getCodes().size() == 2);
            assertTrue(aaBean.getCodes().get(0).getCode().equals(codeAA.getCode()));
            assertTrue(aaBean.getCodes().get(1).getCode().equals("441.9"));
            assertTrue(aaBean.getCodes().get(1).getSystem().equals(CodingSystem.ICD9_CM_DISEASES.getDisplayName()));

            DataBean amiBean = dataBeanList.get(1);
            assertTrue(amiBean.getCodes().size() == 2);
            assertTrue(amiBean.getCodes().get(0).getCode().equals(codeAMI.getCode()));
            assertTrue(amiBean.getCodes().get(1).getCode().equals("410.90"));
            assertTrue(amiBean.getCodes().get(1).getSystem().equals(CodingSystem.ICD9_CM_DISEASES.getDisplayName()));

            //no CHF ICD mappings
            DataBean unkBean = dataBeanList.get(2);
            assertTrue(unkBean.getCodes().size() == 1);
            assertTrue(unkBean.getCodes().get(0).getCode().equals(codeUNK.getCode()));

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testMatchingSourceAndTargetCodes()
    {
        ConceptCodeMapper conceptCodeMapper = ConceptCodeMapper.getInstance();

        List<DataBean> dataBeanList = new ArrayList<DataBean>();

        List<Code> codes = new ArrayList<Code>();

        Code codeAA = new Code();
        codeAA.setCode("AA");
        codeAA.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAA.setDisplay("");
        codes.add(codeAA);

        Code codeAMI = new Code();
        codeAMI.setCode("AMI");
        codeAMI.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeAMI.setDisplay("");
        codes.add(codeAMI);

        Code codeCHF = new Code();
        codeCHF.setCode("CHF");
        codeCHF.setSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName());
        codeCHF.setDisplay("");
        codes.add(codeCHF);

        for(Code code : codes)
        {
            DataBean dataBean = new DataBean();
            dataBean.getCodes().add(code);
            dataBeanList.add(dataBean);
        }

        try {

            conceptCodeMapper.mapConceptCodes(hddDao, dataBeanList, CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET);

            assertTrue(dataBeanList.size() == 3);

            DataBean aaBean = dataBeanList.get(0);
            assertTrue(aaBean.getCodes().get(0).getCode().equals(codeAA.getCode()));
            assertTrue(aaBean.getCodes().get(0).getCode().equals("AA"));
            assertTrue(aaBean.getCodes().get(0).getSystem().equals(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName()));
            assertTrue(!aaBean.getCodes().get(0).getDisplay().isEmpty());

            for(int i = 1; i < aaBean.getCodes().size(); i++)
            {
                Code code = aaBean.getCodes().get(i);
                assertTrue(!"AA".equals(code.getCode()));
            }

            DataBean amiBean = dataBeanList.get(1);
            assertTrue(amiBean.getCodes().get(0).getCode().equals(codeAMI.getCode()));
            assertTrue(amiBean.getCodes().get(0).getCode().equals("AMI"));
            assertTrue(amiBean.getCodes().get(0).getSystem().equals(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName()));
            assertTrue(!amiBean.getCodes().get(0).getDisplay().isEmpty());

            for(int i = 1; i < amiBean.getCodes().size(); i++)
            {
                Code code = amiBean.getCodes().get(i);
                assertTrue(!"AMI".equals(code.getCode()));
            }


            DataBean chfBean = dataBeanList.get(2);
            assertTrue(chfBean.getCodes().get(0).getCode().equals(codeCHF.getCode()));
            assertTrue(chfBean.getCodes().get(0).getCode().equals("CHF"));
            assertTrue(chfBean.getCodes().get(0).getSystem().equals(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getDisplayName()));
            assertTrue(!chfBean.getCodes().get(0).getDisplay().isEmpty());

            for(int i = 1; i < chfBean.getCodes().size(); i++)
            {
                Code code = chfBean.getCodes().get(i);
                assertTrue(!"CHF".equals(code.getCode()));
            }


        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

}
