package gov.va.med.jmeadows.dao.hdd;


import com.mchange.v2.c3p0.ComboPooledDataSource;
import gov.va.med.jmeadows.common.AppConfig;
import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.vds.webservice.Code;
import org.junit.Before;
import org.junit.Test;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;
import java.beans.PropertyVetoException;
import java.util.List;
import java.util.Map;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

public class HDDDaoTest
{
    private HDDDao hddDao;

    @Before
    public void setUp()
    {
        DataSource dataSource;

        if (AppConfig.getInstance().useC3PO()) {
            //setup c3po db connection pools
            try {
                ComboPooledDataSource c3poSQLConnectionPool = new ComboPooledDataSource();
                c3poSQLConnectionPool.setDriverClass(AppConfig.getInstance().getSQLDriver());
                c3poSQLConnectionPool.setJdbcUrl(AppConfig.getInstance().getSQLURL());
                dataSource = c3poSQLConnectionPool;
            } catch (PropertyVetoException ex) {
                throw new RuntimeException(ex);
            }
        } else {
            try {

                dataSource = InitialContext.doLookup(AppConfig.getInstance().getJDBC_JNDILookup());
            } catch (NamingException ex) {
                throw new RuntimeException(ex);
            }
        }

        hddDao = new HDDDao(dataSource);
    }

    @Test
    public void testMapConceptCodes()
    {
        String targetSystemId = "2.16.840.1.113883.9.1.BOGUS";
        CodeMappingList list = new CodeMappingList();

        CodeMappingItem item = new CodeMappingItem();
        item.setCustomMap("D6");
        item.setCode("12014");
        item.setSourceSystem("2.16.840.1.113883.7.1.BOGUS");
        item.setTargetSystem("2.16.840.1.113883.9.1.BOGUS");
        list.getCodeMappingItems().add(item);

        CodeMappingItem item2 = new CodeMappingItem();
        item2.setCustomMap("6");
        item2.setCode("4019595");
        item2.setSourceSystem("2.16.840.1.113883.5.1.BOGUS");
        item2.setTargetSystem("2.16.840.1.113883.9.1.BOGUS");
        list.getCodeMappingItems().add(item2);

        CodeMappingItem item3 = new CodeMappingItem();
        item3.setCustomMap("D6");
        item3.setCode("1033712000");
        item3.setSourceSystem("2.16.840.1.113883.7.1.BOGUS");
        item3.setTargetSystem("2.16.840.1.113883.9.1.BOGUS");
        list.getCodeMappingItems().add(item3);

        try {
            Map<String,List<Code>> codeMap = hddDao.mapConceptCodes(list, targetSystemId);

            String itemMapKey = item.getSourceSystem()+"^"+item.getCode();
            assertTrue(!codeMap.containsKey(itemMapKey));
            //assertTrue("441.9".equals(codeMap.get(itemMapKey).get(0).getCode()));

            String item2MapKey = item2.getSourceSystem()+"^"+item2.getCode();
            assertTrue(codeMap.containsKey(item2MapKey));
            assertTrue("C0001924".equals(codeMap.get(item2MapKey).get(0).getCode()));

            String item3MapKey = item3.getSourceSystem()+"^"+item3.getCode();
            assertTrue(codeMap.containsKey(item3MapKey));
            assertTrue("C0049065".equals(codeMap.get(item3MapKey).get(0).getCode()));

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testMapConceptCodes_old()
    {
        String targetSystemId = CodingSystem.ICD9_CM_DISEASES.getOID();

        CodeMappingList codeMappingList = new CodeMappingList();

        CodeMappingItem itemAA = new CodeMappingItem();
        itemAA.setCode("AA");
        itemAA.setSourceSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID());
        itemAA.setTargetSystem(targetSystemId);

        CodeMappingItem itemAMI = new CodeMappingItem();
        itemAMI.setCode("AMI");
        itemAMI.setSourceSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID());
        itemAMI.setTargetSystem(targetSystemId);

        CodeMappingItem itemCHF = new CodeMappingItem();
        itemCHF.setCode("CHF");
        itemCHF.setSourceSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID());
        itemCHF.setTargetSystem(targetSystemId);

        codeMappingList.getCodeMappingItems().add(itemAA);
        codeMappingList.getCodeMappingItems().add(itemAMI);
        codeMappingList.getCodeMappingItems().add(itemCHF);


        try {
            Map<String,List<Code>> codeMap = hddDao.mapConceptCodes(codeMappingList, targetSystemId);

            String itemAAMapKey = itemAA.getSourceSystem()+"^"+itemAA.getCode();
            assertTrue(codeMap.containsKey(itemAAMapKey));
            assertTrue("441.9".equals(codeMap.get(itemAAMapKey).get(0).getCode()));

            String itemAMIMapKey = itemAMI.getSourceSystem()+"^"+itemAMI.getCode();
            assertTrue(codeMap.containsKey(itemAMIMapKey));
            assertTrue("410.90".equals(codeMap.get(itemAMIMapKey).get(0).getCode()));

            String itemCHFMapKey = itemCHF.getSourceSystem()+"^"+itemCHF.getCode();
            assertTrue(codeMap.containsKey(itemCHFMapKey));
            assertTrue("428.0".equals(codeMap.get(itemCHFMapKey).get(0).getCode()));

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testMapConceptCodeFailure()
    {
        String targetSystemId = CodingSystem.ICD9_CM_DISEASES.getOID();

        CodeMappingList codeMappingList = new CodeMappingList();

        CodeMappingItem itemUNK = new CodeMappingItem();
        itemUNK.setCode("UNK");
        itemUNK.setSourceSystem(CodingSystem.HAHS_EMERGENCY_DEPARTMENT_DIAGNOSIS_CODESET.getOID());
        itemUNK.setTargetSystem(targetSystemId);

        codeMappingList.getCodeMappingItems().add(itemUNK);


        try {
            Map<String,List<Code>> codeMap = hddDao.mapConceptCodes(codeMappingList, targetSystemId);

            String itemUNKMapKey = itemUNK.getSourceSystem()+"^"+itemUNK.getCode();
            assertTrue(!codeMap.containsKey(itemUNKMapKey));
            assertTrue(codeMap.isEmpty());

        } catch (JMeadowsException e) {
            fail(e.getMessage());
        }
    }
}
