/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.vds.webservice.Allergy;
import gov.va.med.vds.webservice.AllergyDetail;
import org.apache.axis.utils.StringUtils;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;

/**
 *
 * @author DNS   brunos
 */
public class AllergyDaoTest extends DaoBaseTest
{
    private AllergyDao allergyDao;

    public AllergyDaoTest() throws JMeadowsException
    {
        allergyDao = new AllergyDao();
    }

    @Test
    public void testGetPatientAllergies()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            qb.setIncludeVLER(true);
            List<Allergy> allergies = allergyDao.getPatientAllergies(qb);
            assertTrue(allergies.size() > 0);
            testForBeanMeta(allergies);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetDODVLERPatientAllergies()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            List<Allergy> allergies = allergyDao.getDODVLERPatientAllergies(qb);
            assertTrue(allergies.size() > 0);
            testForBeanMeta(allergies);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetAllergyDetail()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            List<Allergy> allergies = allergyDao.getPatientAllergies(qb);
            assertTrue(allergies.size() > 0);

            Allergy allergy = null;
            for(Allergy alg : allergies)
            {
                if ("VA".equalsIgnoreCase(alg.getSite().getAgency()))
                {
                    allergy = alg;
                    break;
                }
            }

            assertNotNull(allergy);

            JMeadowsQuery qb2 = new JMeadowsQuery();
            qb2.setUser(user);
            qb2.setPatient(patient);
            if (allergy != null)
            {
                qb2.setItemId(allergy.getAllergyId());
                qb2.setRecordSiteCode(allergy.getSite().getSiteCode());
            }

            AllergyDetail allergyDetail = allergyDao.getAllergyDetail(qb2);

            assertNotNull(allergyDetail);

            assertFalse(StringUtils.isEmpty(allergyDetail.getReportText()));

        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }
}
