/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.vds.webservice.ClinicalReminder;
import gov.va.med.vds.webservice.FreeTextReport;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author DNS   brunos
 */
public class ClinicalRemindersDaoTest extends DaoBaseTest
{
    private ClinicalRemindersDao clinicalRemindersDao;

    public ClinicalRemindersDaoTest() throws JMeadowsException
    {
        clinicalRemindersDao = new ClinicalRemindersDao();
    }

    @Test
    public void testGetPatientClinicalReminders()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            List<ClinicalReminder> reminders = clinicalRemindersDao.getPatientClinicalReminders(qb);
            assertTrue(reminders.size() > 0);
            testForBeanMeta(reminders);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetClinicalReminderDetail()
    {
        try
        {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            List<ClinicalReminder> reminders = clinicalRemindersDao.getPatientClinicalReminders(qb);

            setUp();
            
            qb = setupQuery(user, patient, startDate, endDate);
            qb.setItemId(reminders.get(0).getId());
            qb.setRecordSiteCode(reminders.get(0).getSite().getSiteCode());
            FreeTextReport reminderDetail = clinicalRemindersDao.getClinicalReminderDetail(qb);
            assertTrue(reminderDetail != null && reminderDetail.getReportText().length() > 1);
        }
        catch(JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }
}
