/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.patient;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.beans.VLERCCDA;
import gov.va.med.vds.webservice.VlerDocument;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 *
 * @author DNS   gotom
 */
public class VlerDaoTest extends DaoBaseTest
{
    private VLERDao vlerDao;

    public VlerDaoTest() throws JMeadowsException
    {
        vlerDao = new VLERDao();
    }

    @Test
    public void testGetPatientVLERDocList()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

            patient.setDob("1/2/1982");
            patient.setSSN("666100002");
            patient.setICN("1012638925V204624");
            patient.setGender("M");
            patient.setName("NWHINZZZTESTPATIENT,NWHINTWO");
            List<VlerDocument> docList = vlerDao.getVLERDocumentList(qb);
            assertTrue(docList.size() > 0);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetPatientVLERDoc()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

            patient.setDob("1/2/1982");
            patient.setSSN("666100002");
            patient.setICN("1012638925V204624");
            patient.setGender("M");
            patient.setName("NWHINZZZTESTPATIENT,NWHINTWO");
            VlerDocument vDoc = new VlerDocument();
            vDoc.setHomeCommunityId("urn:oid:1.3.6.1.4.1.26580.10");
            vDoc.setDocumentUniqueId("122bd2c8-77fc-4727-b6c7-2a7994112058");
            vDoc.setRepositoryUniqueId("1.2.840.114350.1.13.111.3.7.2.688879");

            VlerDocument outputDoc = vlerDao.getVLERDocument(vDoc, qb);
            assertTrue(outputDoc.getName() != null);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetDODVLERPatientCCDA()
    {
        try {
            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            List<VLERCCDA> vlerCCDAs = vlerDao.getDODVLERPatientCCDA(qb);
            assertTrue(vlerCCDAs.size() > 0);
            testForBeanMeta(vlerCCDAs);
        } catch (JMeadowsException ex) {
            fail(ex.getMessage());
        }
    }

}
