/**
 * Janus 4.0 (c)
 * Copyright (c) 2011 Hawaii Resource Group LLC. All Rights Reserved.
 * Developed for the Pacific Telehealth & Technology Hui and the Pacific Joint Information Technology Center
 * Contributors:
 *     Honorable Senator Daniel K. Inouye
 *     VA Pacific Islands Health Care System
 *     Tripler Army Medical Center
 */
 
 
package gov.va.med.jmeadows.dao.provider;

import gov.va.med.jmeadows.common.JMeadowsException;
import gov.va.med.jmeadows.dao.beans.JMeadowsQuery;
import gov.va.med.jmeadows.dao.patient.DaoBaseTest;
import gov.va.med.vds.webservice.Consult;
import org.junit.Test;

import java.util.List;

import static org.junit.Assert.*;

/**
 *
 * @author DNS   brunos
 */
public class ProviderConsultsDaoTest extends DaoBaseTest
{
    private ProviderConsultsDao consultsDao;
    
    public ProviderConsultsDaoTest() throws JMeadowsException
    {
        super();
        consultsDao = new ProviderConsultsDao();
    }

    @Test
    public void testGetProviderConsultsRequested()
    {
        try 
        {
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);
            
            mockProviderBasedQuery(qb.getUser(), "1522", "258");

            qb.setStartDate(null);  // Default to T-1 month
            qb.setEndDate(null);    // Default to T+1 month

            List<Consult> consultsRequested = consultsDao.getProviderConsultsRequested(qb);
            assertTrue(consultsRequested.size() > 0);
            if(consultsRequested.size() == 1)
            {
                String consultError = consultsRequested.get(0).getService();
                assertFalse(ERROR_MSG_CONNECTION_UNAVAILABLE.equalsIgnoreCase(consultError));
            }
            logger.debug("# of Consults Requested: " + consultsRequested.size());
            logger.debug("---");
            for(Consult c : consultsRequested)
            {
                logger.debug("ConsultId: " + c.getId());
                logger.debug("PatientId: " + c.getPatientId());
                logger.debug("PatientName: " + c.getPatientName());
                logger.debug("ConsultService: " + c.getService());
                logger.debug("ConsultType: " + c.getConsultType());
                logger.debug("ConsultStatus: " + c.getStatus());
                logger.debug("---");
            }
            testForProviderBeanMeta(consultsRequested);
        } 
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }

    @Test
    public void testGetProviderConsultsReceived()
    {
        try
        {
            startDate.set(2011, 2, 1);
            endDate.set(2011, 3, 1);

            JMeadowsQuery qb = setupQuery(user, patient, startDate, endDate);

            mockProviderBasedQuery(qb.getUser(), "1522", "258");
            qb.setStartDate(null);  // Default to T-1 month
            qb.setEndDate(null);    // Default to T+1 month

            List<Consult> consultsReceived = consultsDao.getProviderConsultsReceived(qb);
            assertTrue(consultsReceived.size() > 0);

            if(consultsReceived.size() == 1)
            {
                String consultError = consultsReceived.get(0).getService();
                assertFalse(ERROR_MSG_CONNECTION_UNAVAILABLE.equalsIgnoreCase(consultError));
            }

            logger.debug("# of Consults Received: " + consultsReceived.size());
            logger.debug("---");
            for(Consult c : consultsReceived)
            {
                logger.debug("ConsultId: " + c.getId());
                logger.debug("PatientId: " + c.getPatientId());
                logger.debug("PatientName: " + c.getPatientName());
                logger.debug("ConsultService: " + c.getService());
                logger.debug("ConsultType: " + c.getConsultType());
                logger.debug("ConsultStatus: " + c.getStatus());
                logger.debug("---");
            }
            testForProviderBeanMeta(consultsReceived);
        }
        catch (JMeadowsException ex)
        {
            fail(ex.getMessage());
        }
    }
}
